<?php
define('TABLE_PREFIX', 'archive_');
define('MAIN_TABLE', 'heslo');
define('PK_NAME', 'id');
define('IGNORE', serialize(array(
    'cache', 'stat_jinych_cis_hodnot', 'archive_heslo', 'hesla_rel',
    'hesla_v_seznamu', 'hesla_online', 'track_ukladani_hesel',
    'track_navstevy_hesel', 'zpracovatel', 'korektura',
    'propojeni_vyznamu', 'uzivatel'
)));
define('DB', 'neoled');

function logarch($str)
{
    file_put_contents('logarch.txt', $str . "\n", FILE_APPEND);
}


/**
 * Seznam cizích klíčů definovaných na sloupcích tabulky $table
 * @param $table
 * @return mixed
 */
function get_keys($table)
{
    $keys = dibi::fetchAll('
		SELECT COLUMN_NAME, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, CONSTRAINT_NAME
		FROM information_schema.KEY_COLUMN_USAGE
		WHERE TABLE_SCHEMA=%s AND TABLE_NAME=%s AND REFERENCED_TABLE_NAME IS NOT NULL
		ORDER BY REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME
	', DB, $table);
    return $keys;
}


/**
 * Seznam tabulek a sloupců s definovaným FK, které odkazují na tabulku $table
 * @param $table
 * @return mixed
 */
function get_children($table)
{
    $ref = dibi::fetchAll('
		SELECT TABLE_NAME, COLUMN_NAME FROM information_schema.KEY_COLUMN_USAGE
		WHERE REFERENCED_TABLE_NAME=%s
	', $table);
    return $ref;
}


/**
 * Vrací informace o cizím klíči na sloupci $column v tabulce $table
 * @param $table
 * @param $column
 * @return mixed
 */
function check_key($table, $column)
{
    $key = dibi::fetch('
		SELECT REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, CONSTRAINT_NAME
		FROM information_schema.KEY_COLUMN_USAGE
		WHERE TABLE_SCHEMA=%s AND TABLE_NAME=%s AND COLUMN_NAME=%s AND REFERENCED_TABLE_NAME IS NOT NULL
		ORDER BY REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME
	', DB, $table, $column);
    return $key;
}

//zálohuje relace na úrovni nižší než heslo
function get_child_rels($hid)
{
    $return = array();
    $vyzs = dibi::fetchPairs('SELECT id FROM vyznamy WHERE hid = %i', $hid);
    $return['propojeni_vyznamu_from'] = dibi::fetchAll('SELECT * FROM propojeni_vyznamu WHERE vyz_from IN %in', $vyzs);
    $return['propojeni_vyznamu_to'] = dibi::fetchAll('SELECT * FROM propojeni_vyznamu WHERE vyz_to IN %in', $vyzs);

    return $return;
}

/**
 * Archivuje data z 1 tabulky
 * $table = název aktuální tabulky
 * $column = název sloupce (FK) pro filtr z aktuální tabulky
 * $value = hodnota sloupce pro filtr z aktuální tabulky
 * $parent = nadřazený záznam (FK v archivu)
 */
function archive(&$gdata, $table, $column, $value, $parent = False, $uid = False, $user_save = 0, $comment = '')
{
    if (in_array($table, unserialize(IGNORE))) {
        return;
    }
    $archive_pk = False;
//    všechna data z aktuální tabulky, kde záznamy odpovídají výběru $column = $value
    $data = dibi::fetchAll('SELECT * FROM %n WHERE %n = %i', $table, $column, $value);
    if (!$data) return;
//    pro každý relevantní záznam z aktuální tabulky
    foreach ($data as $row) {
//        uchováme primární klíč záznamu z ostré tabulky
        $pk = $row[PK_NAME];
//        a odstraníme ho z pole $row, protože se do archivní tabulky nebude zapisovat do sloupce PK_NAME (=id)
        unset($row[PK_NAME]);
//        pokud existuje nadřazený záznam, který již je v archivu, zapíšeme jeho id z archivu
//        a uložíme též referenci na ostrý záznam
        if ($parent) {
            $row[$column] = $parent;
            $row['main_id'] = $pk;
        } else {
//            nadřazený záznam v archivu není, jedná se tedy o archivaci vlastní tabulky heslo, uložíme si id ostrého hesla
            $row['main_heslo'] = $value;
//            zkontrolujeme, jestli již neexistuje tabulka archive_heslo, pokud ne, vytvoříme ji
//            a přidáme sloupce pro referenci na ostré heslo
            $main_exists = dibi::fetchSingle('SHOW TABLES LIKE %s', TABLE_PREFIX . $table);
            if (!$main_exists) {
                dibi::query('CREATE TABLE %n LIKE %n', TABLE_PREFIX . $table, $table);
                dibi::query('
					ALTER TABLE %n
						ADD COLUMN `main_heslo` INT NOT NULL,
						ADD INDEX (`main_heslo`),
						ADD COLUMN `f_deleted` BOOLEAN NOT NULL DEFAULT 0
				', TABLE_PREFIX . $table);
                //dibi::query('ALTER TABLE `version` ADD FOREIGN KEY (id) REFERENCES %n (`id`) ON UPDATE CASCADE ON DELETE CASCADE', TABLE_PREFIX.$table);
            }
        }
//        archivace vlastních dat, $processed je příznak toho, jestli už se podařilo data zarchivovat
//        někdy je nutné provést kroky před archivací (create table nebo alter table), než se to podaří
        $processed = False;
        while ($processed == False) {
            try {
//                zkusíme vložit záznam do archivu, pokud se podaří, jeho PK z archivu se uloží pro případné potomky
                dibi::query('INSERT INTO %n', TABLE_PREFIX . $table, $row);
                $archive_pk = dibi::insertId();
                if (!$parent) {
                    dibi::query('INSERT INTO `version`', array('id' => $archive_pk, 'uid' => $uid, 'user_save' => $user_save, 'comment' => $comment));
                }
                $processed = True;
                if (isset($gdata['translate'][$table])) {
                    $gdata['translate'][$table][$pk] = $archive_pk;
                }
            } catch (DibiException $e) {
//                vložení záznamu se nepodařilo, zde se zjišťuje důvod
//                tabulka neexistuje: archivujeme část hesla, která je v nové tabulce, ta se musí vytvořit
//                sloupec neexistuje: do tabulky byl přidán nový sloupec, je potřeba ho přidat i do archivní
//                (toto se může opakovat víckrát)
//                zároveň se musí navíc vytvořit i FK vazby
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) { //tabulka neexistuje
                    dibi::query('CREATE TABLE %n LIKE %n', TABLE_PREFIX . $table, $table);
                    dibi::query('ALTER TABLE %n ADD COLUMN `main_id` INT NULL DEFAULT NULL', TABLE_PREFIX . $table);
                    $keys = get_keys($table);
                    foreach ($keys as $k) {
                        dibi::query('ALTER TABLE %n ADD INDEX (%n)', TABLE_PREFIX . $table, $k['COLUMN_NAME']);
                        if (strpos($k['REFERENCED_TABLE_NAME'], 'cis_') === 0) {
                            dibi::query('ALTER TABLE %n ADD FOREIGN KEY (%n) REFERENCES %n (%n) ON UPDATE CASCADE ON DELETE CASCADE', TABLE_PREFIX . $table, $k['COLUMN_NAME'], $k['REFERENCED_TABLE_NAME'], $k['REFERENCED_COLUMN_NAME']);
                        } else {
                            dibi::query('ALTER TABLE %n ADD FOREIGN KEY (%n) REFERENCES %n (%n) ON UPDATE CASCADE ON DELETE CASCADE', TABLE_PREFIX . $table, $k['COLUMN_NAME'], TABLE_PREFIX . $k['REFERENCED_TABLE_NAME'], $k['REFERENCED_COLUMN_NAME']);
                        }
                    }
                } elseif ($e->getCode() == MYSQL_UNKNOWN_COLUMN) { //sloupec neexistuje
                    $msg = $e->getMessage();
                    $col = substr($msg, strpos($msg, '\'') + 1);
                    $col = substr($col, 0, strpos($col, '\'')); //$col teď obsahuje název sloupce
                    $col_info = dibi::fetch('SELECT COLUMN_TYPE, IS_NULLABLE FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=%s AND TABLE_NAME=%s AND COLUMN_NAME=%s', DB, $table, $col);
                    dibi::query('ALTER TABLE %n ADD COLUMN %n %sql %sql', TABLE_PREFIX . $table, $col, $col_info['COLUMN_TYPE'], ($col_info['IS_NULLABLE'] == 'YES' ? 'NULL' : 'NOT NULL'));
                    $key = check_key($table, $col);
                    if ($key) {
                        dibi::query('ALTER TABLE %n ADD INDEX (%n)', TABLE_PREFIX . $table, $col);
                        dibi::query('ALTER TABLE %n ADD FOREIGN KEY (%n) REFERENCES %n (%n) ON UPDATE CASCADE ON DELETE CASCADE', TABLE_PREFIX . $table, $col, TABLE_PREFIX . $key['REFERENCED_TABLE_NAME'], $key['REFERENCED_COLUMN_NAME']);
                    }
                } else {
//                    pokud nastala jiná chyba, proces končí
                    return;
                }
            }
        }

//        po archivování dat se najdou potomci archivovaného záznamu a archivují se
//        jako filtr se použije sloupec, který obsahuje FK v tabulce potomka, a hodnota primárního klíče
//        nadřazeného záznamu
        $ref = get_children($table);
        foreach ($ref as $r) {
            archive($gdata, $r['TABLE_NAME'], $r['COLUMN_NAME'], $pk, $archive_pk);
        }
    }
    return $archive_pk;
}

/**
 * @param $table - ostrá tabulka, která se aktuálně zpracovává
 * @param $column , $value - sloupec a jeho hodnota pro určení prvků v tabulce
 * @param $value
 * @param bool $parent - id nadřazeného prvku v ostré tabulce
 * @param bool $archive_id - id archivovaného záznamu
 */
function get_from_archive($table, $column, $value, $parent = False, $archive_id = False)
{
    if (in_array($table, unserialize(IGNORE))) { //přeskočíme ignorované tabulky
        return;
    }
    $exists = dibi::fetchSingle('SHOW TABLES LIKE %s', TABLE_PREFIX . $table);
    if (!$exists) { //přeskočíme tabulky, které ještě v archivu neexistují
        return;
    }
    //data z archivní tabulky
    if ($archive_id) { //pouze pro hlavní tabulku
        $data = dibi::fetchAll('SELECT * FROM %n WHERE %n = %i AND %n=%i', TABLE_PREFIX . $table, $column, $value, PK_NAME, $archive_id);
        $existing = False;
    } else { //pro všechny ostatní tabulky - data z archivní tabulky
        //dibi::test('SELECT * FROM %n WHERE %n = %i', TABLE_PREFIX.$table, $column, $value);
        $data = dibi::fetchAll('SELECT * FROM %n WHERE %n = %i', TABLE_PREFIX . $table, $column, $value);
        //dibi::test('SELECT %n FROM %n WHERE %n = %i', PK_NAME, $table, $column, $parent);
        $existing = dibi::fetchPairs('SELECT %n FROM %n WHERE %n = %i', PK_NAME, $table, $column, $parent);
        $existing = array_flip($existing);
    }
    if (!$data) return;
    // existující záznamy v ostré tabulce
    foreach ($data as $row) {
        // PK z archivní tabulky pro načtení potomků
        $pk = $row[PK_NAME];
        // kopie dat
        if ($parent) { //má rodiče = není to hlavní tabulka
            $main_pk = $row['main_id'];
            unset($row[PK_NAME], $row['main_id']);
            $row[$column] = $parent;
            try {
                dibi::begin();
                if (isset($existing[$main_pk])) { //záznam existuje v ostré tabulce
                    //dibi::test('UPDATE %n SET %a WHERE %n = %i', $table, $row, PK_NAME, $main_pk);
                    dibi::query('UPDATE %n SET %a WHERE %n = %i', $table, $row, PK_NAME, $main_pk);
                    $new_pk = $main_pk;
                    unset($existing[$main_pk]);
                } else { //záznam neexistuje v ostré tabulce
                    //dibi::test('INSERT INTO %n', $table, $row);
                    dibi::query('INSERT INTO %n', $table, $row);
                    $ref_id = dibi::insertId();
                    //dibi::test('UPDATE %n SET `main_id` = %i WHERE %n = %i', TABLE_PREFIX.$table, $ref_id, PK_NAME, $pk);
                    dibi::query('UPDATE %n SET `main_id` = %i WHERE %n = %i', TABLE_PREFIX . $table, $ref_id, PK_NAME, $pk);
                    $new_pk = $ref_id; //nové PK z ostré tabulky pro potomky
                }
                dibi::commit();
            } catch (DibiException $e) {
                var_dump($e);
                dibi::rollback();
                return;
            }
        } else {
            unset($row['main_heslo'], $row['f_deleted'], $row[PK_NAME]);
            try {
                dibi::begin();
                //dibi::test('UPDATE %n SET %a WHERE %n = %s', MAIN_TABLE, $row, PK_NAME, $value);
                dibi::query('UPDATE %n SET %a WHERE %n = %s', MAIN_TABLE, $row, PK_NAME, $value);
                dibi::commit();
            } catch (DibiException $e) {
                dibi::rollback();
                var_dump($e);
                return;
            }
            $new_pk = $value;
        }
        //najít relace
        $ref = get_children($table);
        foreach ($ref as $r) {
            get_from_archive($r['TABLE_NAME'], $r['COLUMN_NAME'], $pk, $new_pk);
        }
    }
    if ($existing) {
        foreach ($existing as $id => $nothing) {
            //dibi::test('DELETE FROM %n WHERE %n = %i', $table, PK_NAME, $id);
            dibi::query('DELETE FROM %n WHERE %n = %i', $table, PK_NAME, $id);
        }
    }
}

function archive_rels($hid, $archid, $data)
{
    $exist = dibi::fetchSingle('SHOW TABLES LIKE %s', TABLE_PREFIX . 'hesla_rel');
    if (!$exist) {
        dibi::query('CREATE TABLE %n LIKE %n', TABLE_PREFIX . 'hesla_rel', 'hesla_rel');
        dibi::query('
					ALTER TABLE %n
						ADD COLUMN `main_id` INT NOT NULL,
						ADD COLUMN `archived` VARCHAR(5) NOT NULL,
						ADD COLUMN `archived_id` INT NOT NULL,
						ADD FOREIGN KEY (archived_id) REFERENCES %n (`id`) ON UPDATE CASCADE ON DELETE CASCADE
				', TABLE_PREFIX . 'hesla_rel', TABLE_PREFIX . MAIN_TABLE);

        dibi::query('CREATE TABLE %n LIKE %n', TABLE_PREFIX . 'hesla_rel_places', 'hesla_rel_places');
        dibi::query('
					ALTER TABLE %n
					ADD FOREIGN KEY (hesla_rel_id) REFERENCES %n (`id`) ON UPDATE CASCADE ON DELETE CASCADE
				', TABLE_PREFIX . 'hesla_rel_places', TABLE_PREFIX . 'hesla_rel');

        dibi::query('CREATE TABLE %n LIKE %n', TABLE_PREFIX . 'hesla_rel_slave_places', 'hesla_rel_slave_places');
        dibi::query('
					ALTER TABLE %n
					ADD FOREIGN KEY (hesla_rel_id) REFERENCES %n (`id`) ON UPDATE CASCADE ON DELETE CASCADE
				', TABLE_PREFIX . 'hesla_rel_slave_places', TABLE_PREFIX . 'hesla_rel');

        dibi::query('CREATE TABLE %n LIKE %n', TABLE_PREFIX . 'hesla_rel_link_type_vyber', 'hesla_rel_link_type_vyber');
        dibi::query('
            ALTER TABLE %n
            ADD FOREIGN KEY (`hesla_rel_id`) REFERENCES %n (`id`) ON DELETE CASCADE ON UPDATE NO ACTION,
            ADD FOREIGN KEY (`cis_id`) REFERENCES `cis_hesla_rel_link_type` (`id`) ON DELETE CASCADE ON UPDATE CASCADE
        ', TABLE_PREFIX . 'hesla_rel_link_type_vyber', TABLE_PREFIX . 'hesla_rel');
    }
    $all_rels_main = dibi::fetchAll('
        SELECT id, rel_type, main_hid, main_str, slave_hid, slave_str, sortorder, komentar, poznamka, sortorder2, spojeno
        FROM hesla_rel WHERE main_hid = %i
    ', $hid);

    $all_rels_slave = dibi::fetchAll('
        SELECT id, rel_type, main_hid, main_str, slave_hid, slave_str, sortorder, komentar, poznamka, sortorder2, spojeno
        FROM hesla_rel WHERE slave_hid = %i
    ', $hid);

    if ($all_rels_main) {
// odkazy z: vyznam, exmplifikace, ant, syn, heslo
        foreach ($all_rels_main as $r) {
            $id = $r['id'];
            $r['archived'] = 'main';
            $r['main_id'] = $id;
            $r['archived_id'] = $archid;
            unset($r['id']);
            dibi::query('
                INSERT INTO %n
            ', TABLE_PREFIX . 'hesla_rel', $r);
            $rid = dibi::insertId();

            $ltype = dibi::fetchAll('
                SELECT cis_id, sortorder, jine
                FROM hesla_rel_link_type_vyber
                WHERE hesla_rel_id = %i
            ', $id);

            foreach ($ltype as $lt) {
                $lt['hesla_rel_id'] = $rid;
                dibi::query('INSERT INTO %n', TABLE_PREFIX . 'hesla_rel_link_type_vyber', $lt);
            }

            $places = dibi::fetchAll('
                SELECT place, place_id
                FROM hesla_rel_places
                WHERE hesla_rel_id = %i
            ', $id);
            foreach ($places as $p) {
                $p['hesla_rel_id'] = $rid;
                if ($p['place'] == 'heslo') {
                    $p['place_id'] = $archid;
                } elseif ($p['place'] == 'exemplifikace') {
                    $p['place_id'] = $data['exemplifikace'][$p['place_id']];
                } else { // vyznam, ant, syn
                    $p['place_id'] = $data['vyznamy'][$p['place_id']];
                }
                dibi::query('
                    INSERT INTO %n
                ', TABLE_PREFIX . 'hesla_rel_places', $p);
            }
        }

    }

    if ($all_rels_slave) {
// odkazy na: vyznam, heslo
        foreach ($all_rels_slave as $r) {
            $id = $r['id'];
            $r['archived'] = 'slave';
            $r['main_id'] = $id;
            $r['archived_id'] = $archid;
            unset($r['id']);
            dibi::query('
                INSERT INTO %n
            ', TABLE_PREFIX . 'hesla_rel', $r);
            $rid = dibi::insertId();

            $ltype = dibi::fetchAll('
                SELECT cis_id, sortorder, jine
                FROM hesla_rel_link_type_vyber
                WHERE hesla_rel_id = %i
            ', $id);

            foreach ($ltype as $lt) {
                $lt['hesla_rel_id'] = $rid;
                dibi::query('INSERT INTO %n', TABLE_PREFIX . 'hesla_rel_link_type_vyber', $lt);
            }
            $places = dibi::fetchAll('
                SELECT place, place_id
                FROM hesla_rel_slave_places
                WHERE hesla_rel_id = %i
            ', $id);
            foreach ($places as $p) {
                $p['hesla_rel_id'] = $rid;
                if ($p['place'] == 'heslo') {
                    $p['place_id'] = $archid;
                } else { // vyznam
                    $p['place_id'] = $data['vyznamy'][$p['place_id']];
                }
                dibi::query('
                    INSERT INTO %n
                ', TABLE_PREFIX . 'hesla_rel_slave_places', $p);
            }

        }
    }
}

function archive_default($hid, $is_user = False, $comment = 'Automatická záloha')
{
    try {
        $data = array(
            'translate' => array(
                'heslo' => array(),
                'vyznamy' => array(),
                'exemplifikace' => array()
            )
        );
        archive($data, 'heslo', 'id', $hid, False, get_logged_user(), $is_user, $comment);
        archive_rels($hid, $data['translate']['heslo'][$hid], $data['translate']);
    } catch (Exception $e) {
        return False;
    }
    return True;
}