<?php
/**
 * Soubor abs.base.php obsahujici definici abstraktni tridy CDO_Base.
 */

/**
 * Abstraktni trida, ze ktere se odvozuji vsechny ostatni tridy pro CDO.
 */
abstract class CDO_Base
{
    /**
     * @var int $input typ vstupu
     */
    protected $input;
    /**
     * @var array $data datova slozka
     */
    public $data;
    /**
     * @var array $colsStructured potomci - strukturovane elementy
     */
    protected $colsStructured = array();
    /**
     * @var array $colsSimple potomci - nestrukturovane elementy
     */
    protected $colsSimple = array();
    /**
     * @var array $corr korektury
     */
    public $corr = array();
    /** @var string sloupec s odkazem na nadrazeny zaznam */
    protected $parentFk;
    /**
     * @var int $parentVal id nadrazeneho zaznamu
     */
    protected $parentVal;
    /**
     * @var string $domStr retezec v DOM id
     */
    protected $domStr = '';
    /**
     * @var string $baseDom DOM id bez id polozky
     */
    protected $baseDom = '';
    /**
     * @var string $parentDomId id DOM elementu nadrazeneho zaznamu
     */
    protected $parentDomId = '';
    /**
     * @var array $domId id DOM elementu
     */
    public $domId = array();
    /**
     * @var array $maximized rozbalene elementy
     */
    protected $maximized;
    /**
     * @var string $xmlWrapper nazev obaloveho xml elementu
     */
    protected $xmlWrapper;

    /** @var string $xmlElem nazev obaloveho xml elementu pro jednu polozku */
    protected $xmlElem;

    /**
     * @var string $tbl nazev mysql tabulky
     */
    protected $tbl;
    /** @var int $hid id hesla */
    protected $hid;
    /** @var string $alias alias pro hromadne query */
    protected $alias;

    function __clone()
    {
        $this->corr = array();
    }

    /**
     * Setter pro parentVal.
     * @param $parentVal
     */
    public function setParentVal($parentVal)
    {
        $this->parentVal = $parentVal;
    }

    /**
     * Setter pro parentDomId.
     * @param $domId
     */
    public function setParentDomId($domId)
    {
        $this->parentDomId = $domId;
        $this->baseDom = ($this->domStr ? $this->parentDomId . '--' . $this->domStr : $this->parentDomId);
    }

    /**
     * Nastavi hid
     * @param $hid
     */
    public function setHid($hid)
    {
        $this->hid = $hid;
    }

    /**
     * Znovuvytvoreni dom ids.
     * @param int $hid
     */
    public function recreateDomIds($hid = 0)
    {
        $dataIds = array_keys($this->data);
        $this->setHid($hid);
        unset($this->domId);
        $class = get_class($this);
        if ($class == 'CDO_Ciselnik') {
            $this->domId = $this->baseDom;
        } elseif ($class == 'CDO_Semafor') {
            $this->setParentDomId('heslo-' . $hid);
            $this->baseDom = $this->parentDomId . '--' . 'semafor';
            $this->domId = $this->baseDom;
        } elseif ($class == 'CDO') {
            foreach ($this->data as $key => $entry) {
                if (get_parent_class($entry) == 'CDO_Base') {
                    $entry->setHid($hid);
                    $entry->setParentDomId($this->domId);
                    $entry->recreateDomIds($hid);
                    $entry->setParentVal($hid);
                }
            }
        } else {
            foreach ($dataIds as $id) {
                $this->domId[$id] = $this->baseDom . '-' . $id;
                foreach ($this->data[$id] as $key => $entry) {
                    if (get_parent_class($entry) == 'CDO_Base') {
                        $entry->setParentVal($id);
                        $entry->setParentDomId($this->domId[$id]);
                        $entry->recreateDomIds($hid);
                        $entry->setHid($hid);
                    }
                }
            }
        }
    }

    /**
     * "Hluboké" naklonování objektu.
     * @param bool $top
     * @param bool $topId
     * @return CDO_Base
     */
    public function getClone($top = False, $topId = False)
    {
        $class = get_class($this);
        if ($class == 'CDO_Ciselnik' || $class == 'CDO_Semafor') {
            return clone $this;
        }
        $copy = clone $this;
        $transl = array();
        $copy->data = array();
        if ($top) {
            if ($topId) {
                $copy->hid = $topId;
            } else {
                $copy->hid = $this->newId();
            }
            foreach ($this->data as $key => $entry) {
                if (get_parent_class($entry) == 'CDO_Base') {
                    $cl = $entry->getClone();
                    $copy->data[$key] = $cl;
                } else {
                    $copy->data[$key] = $entry;
                }
            }
        } else {
            $ids = $topId ? array($topId) : array_keys($this->data);
            foreach ($ids as $id) {
                $nid = $this->newId();
                $transl[$id] = $nid;
                $copy->data[$nid] = new DibiRow(array());
            }
            foreach ($ids as $id) {
                foreach ($this->data[$id] as $key => $entry) {
                    if (get_parent_class($entry) == 'CDO_Base') {
                        $cl = $entry->getClone();
                        $copy->data[$transl[$id]]->{$key} = $cl;
                    } else {
                        $copy->data[$transl[$id]]->{$key} = $entry;
                    }
                }
                $copy->data[$transl[$id]]->id = $transl[$id];
            }
        }
        return $copy;
    }

    /**
     * Tlacitko pro nahrani z archivu.
     * @param $domId
     * @param bool $archive
     * @return string
     */
    protected function cloneFromArchiveBtn($domId, $archive = False)
    {
        $html = '
            <button type="button" onclick="fromArchive(\'' . $domId . '\')">Nahrát</button>
        ';
        return $archive ? $html : '';
    }

    /**
     * Nastaveni zdroje dat.
     * @param int $input
     */
    public function setInput($input = CDOin_DB)
    {
        $this->input = $input;
    }

    /**
     * Nacteni dat do objektu dle zdroje dat.
     */
    public function load()
    {
        switch ($this->input) {
            case CDOin_DB:
                return $this->loadFromDB();
                break;
            case CDOin_ARCHIVE:
                return $this->loadFromDB(True);
                break;
            default:
                return BAD_INPUT_TYPE;
        }
    }

    /**
     * Ulozeni dat z objektu do specifikovaneho vystupu.
     * @param int $output typ vystupu
     * @param array $originalData puvodni data k prepsani
     * @return Response navratova hodnota
     */
    public function save($output = CDOout_DB, $originalData = array())
    {
        switch ($output) {
            case CDOout_DB:
                return $this->saveToDB(False, $originalData);
                break;
            case CDOout_ARCHIVE:
                return $this->saveToDB(True, $originalData);
                break;
            default:
                return new Response(BAD_OUTPUT_TYPE);
                break;
        }
    }

    /**
     * Odebrani polozky dle daneho id
     * @param int $id id
     * @return Response navratova hodnota
     */
    public function remove($id)
    {
        if (isset($this->data[$id])) {
            unset($this->data[$id]);
            return new Response(OK, $id);
        } else {
            return new Response(WRONG_ID_GIVEN, $id);
        }
    }

    /**
     * Prepise data.
     * @param int $id id polozky
     * @param string $field pole
     * @param array $data nova data
     * @param bool $data2
     * @return Response
     */
    public function update($id, $field, $data, $data2 = False)
    {
        if (isset($this->data[$id][$field]) || is_null($this->data[$field])) {
            $this->data[$id][$field] = trim($data);
            return new Response(OK);
        } else {
            return new Response(WRONG_ID_GIVEN, array('id' => $id, 'field' => $field), 'Položka neexistuje.');
        }
    }

    /**
     * Zmeni poradi polozek.
     * @param $list
     */
    public function reorder($list)
    {
        $list = array_flip($list);
        uksort($this->data, function ($k1, $k2) use ($list) {
            return $list[$k1] > $list[$k2];
        });
    }

    /**
     * Posune polozku "nahoru".
     * @param $id
     * @return Response
     */
    public function moveUp($id)
    {
        $list = array_keys($this->data);
        $pos = array_search($id, $list);
        if ($pos > 0 && count($list) > 1) {
            $tmp = $list[$pos];
            $list[$pos] = $list[$pos - 1];
            $list[$pos - 1] = $tmp;
            $this->reorder($list);
            return new Response(OK, $list);
        } else {
            return new Response(NOT_OK, $list);
        }
    }

    /**
     * Posune polozku "dolu".
     * @param $id
     * @return Response
     */
    public function moveDown($id)
    {
        $list = array_keys($this->data);
        $pos = array_search($id, $list);
        if ($pos < count($list) - 1 && count($list) > 1) {
            $tmp = $list[$pos];
            $list[$pos] = $list[$pos + 1];
            $list[$pos + 1] = $tmp;
            $this->reorder($list);
            return new Response(OK, $list);
        } else {
            return new Response(NOT_OK, $list);
        }
    }

    /**
     * Konverze pro vystup.
     * @param $str
     * @return string
     */
    protected function convert($str)
    {
        return $str;
//        return iconv('UTF-8', 'Windows-1250', $str);
    }

    /**
     * Nacte seznam korektur dle seznamu DOM id.
     * @param array $list
     * @param array $aliases
     * @return array korektury
     */
    protected function loadCorrectionsByDom($list, $aliases = array())
    {
        if ($aliases) {
            $l = array();
            foreach ($list as $itm) {
                if (isset($aliases[$itm])) {
                    array_push($l, $aliases[$itm]);
                } else {
                    array_push($l, $itm);
                }
            }
        } else {
            $l = $list;
        }
        $tmp = dibi::query('
          SELECT k1.field_DOMid, k1.id, k1.field, IF(COUNT(ks.id) = 0, 1, 0) AS nova
				FROM korektura k1 LEFT JOIN korektura k2
							ON (k1.field_DOMid = k2.field_DOMid AND k1.id < k2.id)
						LEFT JOIN korektura_stavy ks ON ks.kor_id=k1.id
				WHERE k2.id IS NULL AND k1.field_DOMid IN %in
				GROUP BY k1.id
        ', $l)->fetchAssoc('field_DOMid');
        if ($aliases) {
            $corr = array();
            $aflip = array_flip($aliases);
            foreach ($tmp as $dom => $k) {
                if (isset($aflip[$dom])) {
                    $corr[$aflip[$dom]] = $k;
                } else {
                    $corr[$dom] = $k;
                }

            }
        } else {
            $corr = $tmp;
        }

        return $corr;
    }

    /**
     * Upozorneni na aktivni korekturu.
     * @return string
     */
    function aktCorrAlert()
    {
        return 'alert(\'Pole obsahuje korekturu! Pro editace pole, prosím, vraťte korekturu korektorovi, nebo ji aplikujte.\'); return false;';
    }

    public function disableCorrection($domId)
    {
        $this->corr[$domId]->nova = 0;
    }

    /**
     * Vygeneruje standalone input.
     * @param $domId
     * @param $archive
     * @param $value
     * @param array $classes
     * @param string $onclick
     * @param array $attrs
     * @return string
     */
    protected function printInput($domId, $archive, $value, $classes = array(), $onclick = '', $attrs = array())
    {
        $corr = $this->displayCorrection($this->corr[$domId]['field_DOMid'], $this->corr[$domId]['id'], $this->corr[$domId]['field'], $this->corr[$domId]['nova']);
        if ($corr['class']) {
            array_push($classes, $corr['class']);
        }
        return '
            <span id="' . $domId . '" class="correctableFieldContainer">
                ' . $corr['flag'] . '
                <input id="' . $domId . '-input"
                type="text" value="' . $value . '"
                class="' . implode(' ', $classes) . '"
                ' . ($archive ? ' readonly="readonly"' : '') . '
                onclick="' . $onclick . '"
                onblur="' . $this->intputOnBlur($domId, $archive) . '" />
            </span>
        ';
    }

    /**
     * Vygeneruje standalone textarea.
     * @param $domId
     * @param $archive
     * @param $value
     * @param array $classes
     * @param string $onclick
     * @param array $attrs
     * @return string
     */
    protected function printTextarea($domId, $archive, $value, $classes = array(), $onclick = '', $attrs = array())
    {
        $corr = $this->displayCorrection($this->corr[$domId]['field_DOMid'], $this->corr[$domId]['id'], $this->corr[$domId]['field'], $this->corr[$domId]['nova']);
        if ($corr['class']) {
            array_push($classes, $corr['class']);
        }
        return '
            <span id="' . $domId . '" class="correctableFieldContainer">
                ' . $corr['flag'] . '
                <textarea class="' . implode(' ', $classes) . '"' . ($archive ? ' readonly="readonly"' : '') . '
                    onclick="' . $onclick . '"
                    onblur="' . $this->intputOnBlur($domId, $archive) . '">' . $value . '</textarea>
            </span>
        ';
    }

    /**
     * @param $domId
     * @param $archive
     * @param $value
     * @param $src
     * @param array $classes
     * @return string
     */
    protected function printCheckboxImg($domId, $archive, $value, $src, $classes = array())
    {
        $corr = $this->displayCorrection($domId, $this->corr[$domId]['id'], $this->corr[$domId]['field'], $this->corr[$domId]['nova']);
        if ($corr['class']) {
            array_push($classes, $corr['class']);
        }
        $src = str_replace('%val', $value, $src);
        $onChange = $archive ? '' : ' onChange="save_input(this, \'' . $domId . '\')"';
        return '
                <span id="' . $domId . '" class="correctableFieldContainer">
                    ' . $corr['flag'] . '
                    <input type="hidden" value="' . $value . '"' . $onChange . ' />
                    <img class="' . implode(' ', $classes) . '" data-type="img" src="css/images/' . $src . '" alt="' . $value . '" />
                </span>
            ';
    }


    /**
     * Vygeneruje standalone checkbox.
     * @param $domId
     * @param $archive
     * @param $value
     * @param array $classes
     * @return string
     */
    protected function printCheckbox($domId, $archive, $value, $classes = array())
    {
        $corr = $this->displayCorrection($domId, $this->corr[$domId]['id'], $this->corr[$domId]['field'], $this->corr[$domId]['nova']);
        if ($corr['class']) {
            array_push($classes, $corr['class']);
        }
        $classes[] = 'relative';
        return '
                <span id="' . $domId . '" class="correctableFieldContainer">
                    ' . $corr['flag'] . '
                    <input type="checkbox"
                    ' . (intval($value) ? ' checked="checked"' : '') . '
                    class="' . implode(' ', $classes) . '"
                    ' . ($archive ? ' disabled="disabled"' : '') . '
                    onchange="' . $this->intputOnBlur($domId, $archive) . '" />
                </span>
            ';
    }

    /**
     * Vygeneruje skupinu radio buttonu.
     * @param $domId
     * @param $archive
     * @param $valText
     * @param $value
     * @param bool $printInRow
     * @return string
     */
    protected function printRadioGroup($domId, $archive, $valText, $value, $printInRow = True)
    {
        $corr = $this->displayCorrection($domId, $this->corr[$domId]['id'], $this->corr[$domId]['field'], $this->corr[$domId]['nova']);
        if ($corr['class']) {
            $corr['class'] = ' ' . $corr['class'];
        }
        $html = '
            <div class="floatLeft">
            <span id="' . $domId . '" class="correctableFieldContainer">
                    ' . $corr['flag'] . '
            </span>
        ';
        $onChange = $archive ? 'disabled="disabled"' : ' onChange="save_input(this, \'' . $domId . '\')"';
        if ($this->corr[$domId]['nova']) {
            $onChange .= ' disabled="disabled"';
        }
        $el = $printInRow ? 'span' : 'div';
        foreach ($valText as $val => $t) {
            $ch = ($val == $value ? ' checked="checked"' : '');
            $html .= '
                <' . $el . '>
                    <input id="' . $domId . '-' . $val . '" class="' . $domId . $corr['class'] . '" type="radio" name="' . $domId . '" value="' . $val . '"' . $ch . $onChange . ' />
                    <label for="' . $domId . '-' . $val . '" style="font-weight: normal; position: relative; top: -4px;">' . $t . '</label>
                </' . $el . '>
            ';
        }
        $html .= '
            </div>
        ';
        return $html;

    }

    /**
     * Vygeneruje atribut on blur.
     * @param $domId
     * @param bool|False $archive
     * @return string
     */
    protected function intputOnBlur($domId, $archive = False)
    {
        return $archive ? '' : 'save_input(this, \'' . $domId . '\')';
    }

    /**
     * Vygeneruje ikonku ke korekture.
     * @param $form_id string $domId id DOM elementu
     * @param int $kor_id id posledni korektury
     * @param string $type typ pole
     * @param bool $nova je korektura nevyrizena
     * @param int $fleft odsazeni flagu zleva
     * @return array flag a class
     */
    public function displayCorrection($form_id, $kor_id, $type, $nova, $fleft = 0)
    {
        $corrected_flag = '';
        $corrected_cls = '';
        $onClick = ' onClick="redakce_popup(\'hist\', \'' . $form_id . '\')"';
        if ($nova) {
            $corrected_flag = '
                <img src="css/images/corr_label.png" alt="korektura" class="corrected clickable" kor="' . $kor_id . '" viewel="' . $form_id . '" type="' . $type . '"' . ($fleft ? 'style="left:' . $fleft . 'px"' : '') . ' />
                <span class="clickable correcteddiv" style="display:none;"><img src="css/images/empty.png" alt="korektura"' . $onClick . ' /></span>
            ';
            $corrected_cls = 'corrected';
        } else {
            if ($kor_id) {
                $corrected_flag = '
                <span class="clickable correcteddiv" ' . ($fleft ? 'style="position:absolute;left:' . $fleft . 'px"' : '') . '>
                    <img src="css/images/history.png"  alt="korektura"' . $onClick . ' />
                </span>';
            }
        }

        return array('flag' => $corrected_flag, 'class' => $corrected_cls, 'nova' => $nova);
    }

    /**
     * Defaultni hodnoty pro pole s korekturami.
     * @return array
     */
    protected function emptyCorrectionArray()
    {
        return array(
            'flag' => '',
            'class' => '',
            'nova' => False
        );
    }

    /**
     *
     */
    protected function setCorrStatus()
    {

    }

    /**
     * Vygeneruje tlacitko pro pridani nove polozky.
     * @param $dom
     * @param $label
     * @return string
     */
    protected function addFieldsBtn($dom, $label)
    {
        $html = '
            <div
            id="' . $dom . '_addButt"
            onClick="add_fields(\'' . $dom . '\');"
            class="addRemFieldsButt whiteBG butt floatLeft">
                + přidat ' . $label . '
            </div>';

        return $html;
    }

    /**
     * Vygeneruje tlacitko pro odebrani polozky.
     * @param $dom
     * @return string
     */
    protected function rmFieldsBtn($dom)
    {
        $html = '
            <div class="removeRemFieldsButt center redBG butt"
            onClick="rm_field(\'' . $dom . '\')" >-</div>';

        return $html;
    }

    /**
     * Id pro novou polozku.
     * @param bool $isDB
     * @return string
     */
    protected function newId($isDB = False)
    {
        return $isDB ? dibi::insertId() : 'n' . rand(500, 100000);
    }

    /**
     * Vygeneruje kod pro inicializaci TinyMCE.
     * @param $dom
     * @param bool $archive
     * @param bool $type
     * @param bool $id
     * @return string
     */
    protected function initTinyMCE($dom, $archive = False, $type = False, $id = False)
    {
        $ro = $archive ? 1 : 0;
        return '
            <script type="text/javascript">
                CDO_initTinyMCE(\'#' . $dom . '\', ' . $ro . ', \'' . $type . '\', \'' . $id . '\');
            </script>
        ';
    }

    /**
     * Vygeneruje varovani pro upravu prevzatych informaci.
     * @param array $list
     * @return string
     */
    public function getVarovaniSkup($list = array())
    {
        if (!$list) {
            return '';
        }
        $arr = '';
        foreach ($list as $l) {
            $arr[] = getVyznamNum($l, True, True, True);
        }
        $html = '
            <div class="varovani_skupina">
                <h3>Varování</h3>
                Změny se projeví také v heslech: ' . implode(', ', $arr) . '
            </div>
        ';

        return $html;
    }

    /**
     * @param $tbl
     * @param $atbl
     * @param $sqlData
     * @return Response
     */
    protected function archiveInsert($tbl, $atbl, $sqlData)
    {
        $processed = False;
        $dbid = False;
        while ($processed == False) {
            try {
                dibi::begin();
//                dibi::test('INSERT INTO %n', $atbl, $sqlData);
                dibi::query('INSERT INTO %n', $atbl, $sqlData);
                $dbid = dibi::insertId();
                dibi::commit();
                $processed = True;
            } catch (DibiException $e) {
                /* ERROR HANDLING */
                $resp = handle_archive_error($e, $tbl);
                dibi::rollback();
                if ($resp->status != OK) {
                    return $resp;
                }
            }
        }
        return new Response(OK, $dbid);
    }

    /**
     * @param $data
     * @param $rownum
     * @param $row
     * @param $len
     * @return array
     */
    protected function insertTabRow(&$data, $rownum, $row, $len)
    {
        if (isset($data[$rownum])) {
            $data[$rownum] = array_merge($data[$rownum], $row);
        } else {
            $data[$rownum] = array_merge(array_fill(0, $len, ''), $row);
        }
    }

    protected function getDomOverride($class, $field)
    {
        return $field;
    }

    protected function getDomId($id = False, $field = False)
    {
        if (!$id && !$field) {
            return $this->domId;
        }
        $field = $this->getDomOverride(get_class($this), $field);
        if (is_array($this->domId)) {
            $dom = $this->domId[$id];
        } else {
            $dom = $this->domId;
        }

        return $dom . '--' . $field;
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $headers = array();
        $i = 0;
        $len = count($data[0]);
        foreach ($this->data as $id => $d) {
            $tmp = array();
            foreach ($fields as $c => $j) {
                if (isset($this->data[$id][$c])) {
                    if (get_parent_class($this->data[$id][$c]) != 'CDO_Base') {
                        $headers[] = $c;
                        $tmp[] = wrap_output($this->getDomId($id, $c), $this->data[$id][$c], $wrap);
                    }
                }
            }
            $this->insertTabRow($data, $i, $tmp, $len);
            $i++;
            foreach ($fields as $c => $j) {
                if (get_parent_class($this->data[$id][$c]) == 'CDO_Base') {
                    $headers[] = $c;
                    $response = $this->data[$id]->{$c}->tabExport($data, $fields[$c], $wrap);
                }
            }
        }

        return new Response(OK, array('headers' => $headers));
    }

    protected function getSQLFields($fields)
    {
        return $fields;
    }

    public function getSQL($fields = False, $parentTable = False)
    {
        // doplňková pole
        $fields = $this->getSQLFields($fields);
        // vždy načteme z DB alespoň ID
        $return = array(
            'columns' => array('[' . $this->tbl . '].[id] AS [' . $this->alias . '--id]'),
        );
        // pro každý vybraný sloupec (jednoduché string sloupce)
        foreach ($this->colsSimple as $c) {
            if ($fields === False || isset($fields[$c])) {
                if ($c == 'vystup' && get_class($this) == 'CDO_Exempl') {
                    $return['columns'][] = '[' . $this->tbl . '].[' . $c . '] AS [' . $this->alias . '--vystup_exemplifikace]';
                } else {
                    $return['columns'][] = '[' . $this->tbl . '].[' . $c . '] AS [' . $this->alias . '--' . $c . ']';
                }
            }
        }
        $firstId = array_keys($this->data);
        $firstId = $firstId[0];
        $childTbl = array();
        // pro každý vybraný strukturovaný sloupec (subclass CDO_Base)
        foreach ($this->colsStructured as $c) {
            if ($fields === False || isset($fields[$c])) {
                $childFields = isset($fields[$c]) ? $fields[$c] : False;
                $resp = $this->data[$firstId][$c]->getSQL($childFields, $this->tbl);
                $return['columns'] = array_merge($return['columns'], $resp->data['columns']);
                $childTbl[] = $resp->data['tables'];
            }
        }
        // struktura, která se pak využije v JOINech - spojíme přes id nadřazené tabulky a FK aktuální tabulky
        $return['tables'] = array(
            '[' . $this->tbl . ']' => array(
                'main' => array('[' . $parentTable . '].[id]', '[' . $this->tbl . '].[' . $this->parentFk . ']'),
                'child' => $childTbl
            )
        );

        return new Response(OK, $return);
    }

    protected function addTabData($id)
    {
        return array();
    }

    /**
     * Formatovani dat do "flat" tabulky.
     * @param $data
     * @param array $fields
     * @param string $parentStr
     * @return array
     */
    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        $rows = array();
        $i = 0;
        foreach ($data as $did => $entry) {
            //$did == 'heslo-6508' např.
            $id = explode('-', $did)[1];
            $rows[$i] = $this->addTabData($id);
            if ($parentStr) {
                $str = $parentStr . '--' . $did;
            } else {
                $str = $did;
            }
            foreach ($entry as $col => $val) {
                if (get_parent_class($this->data['temp'][$col]) == 'CDO_Base') {
                    $class = get_class($this->data['temp'][$col]);
                    if (in_array($class, array('CDO_Podhesli', 'CDO_Exempl'))) {
                        $tmp = $this->data['temp'][$col]->tabFormat($data[$did][$col], $fields[$col], $str);
                        $rows[$i][$col] = $tmp[0];
                        $lnt = count($tmp);
                        for ($j = 1; $j < $lnt; $j++) {
                            $rows[$i + $j][$col] = $tmp[$j];
                        }
                    } else {
                        $childData = $this->data['temp'][$col]->tabFormat($data[$did][$col], $fields[$col], $str);
                        $rows[$i][$col] = $childData;
                    }
                } else {
                    if ($col == 'id') continue;
                    if ($col == 'vystup' || $col == 'vystup_vyznam' || $col == 'vystup_hesla' || $col == 'vystup_exemplifikace') {
                        $val = getVystup($val);
                    }
                    $rows[$i][$col] = $val;
                }
            }
            $i = count($rows);
        }
        return $rows;
    }


    /**
     * Nacteni dat z databaze do objektu.
     * @param bool|False $archive nacist z archivu
     * @return int navratova hodnota
     */
    abstract protected function loadFromDB($archive = False);

    /**
     * Ulozeni dat z objektu do DB.
     * @param bool|False $archive ulozit do archivu
     * @param array $originalData puvodni data v DB
     * @return int navratova hodnota
     */
    abstract protected function saveToDB($archive = False, $originalData = array());

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return mixed
     */
    abstract public function saveToArchive($archiveParentVal, $archiveHid = 0);

    /**
     * XML export
     * @return string
     */
    abstract public function printXML();

    /**
     * Export do pole pro html tabulku / csv
     * @param $data
     * @param array $fields pole pro export
     * @return Response
     */
}