<?php
/**
 * Soubor ciselnik.php obsahujici definici tridy CDO_Ciselnik.
 */

/**
 * Trida pro ciselnik.
 */
class CDO_Ciselnik extends CDO_Base
{
    /** @var int $type typ ciselniku */
    private $type;

    /** @var int $parent typ nadrazeneho zaznamu */
    private $parent;

    /** @var string $enumTable nazev tabulky ciselniku moznych hodnot */
    private $enumTable;

    /** @var string $enumCol nazev sloupce v ciselniku moznych hodnot, ktery obsahuje text polozky */
    private $enumCol;

    /** @var string $choiceTable nazev tabulky, ktera obsahuje vybrane hodnoty ciselniku */
    private $choiceTable;

    /** @var string $defTbl */
    private $defTbl;

    /** @var string nazev sloupce, ktery obsahuje odkaz na nadrazeny zaznam */
    private $choiceParentFk;

    /** @var string $choiceEnumFk nazev sloupce, ktery obsahuje odkaz na ciselnik moznych hodnot */
    private $choiceEnumFk;

    /** @var bool $hasJine je mozne mit ve vyberu hodnotu "jine" */
    private $hasJine;

    /** @var int $parentLinked je nadrazeny zaznam dedeny */
    private $parentLinked;

    /** @var string $linkOld puvodni link na vyznam */
    private $linkOld;

    /**
     * Konstruktor tridy CDO_Ciselnik.
     * @param int $input
     * @param bool|False $type
     * @param bool|False $parent
     * @param bool|False $parentVal
     * @param bool $parentDomId
     * @param int $linked
     */
    function __construct($input = CDOin_DB, $type = False, $parent = False, $parentVal = False, $parentDomId = False, $linked = 0)
    {
        $this->input = $input;
        $this->domId = '';
        $this->parent = $parent;
        $this->parentLinked = $linked;
        $this->linkOld = $linked;
        $this->parentDomId = $parentDomId;
        $this->parentVal = $parentVal;
        $this->type = $type;
        $this->xmlWrapper = 'choices';
        $this->xmlElem = 'choice';
        if ($type && $parent && $parentVal) {
            $this->translateQuery($type);
        }
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        }
    }

    /**
     * Setter pro parentLinked.
     * @param int $parentLinked
     */
    public function setParentLinked($parentLinked)
    {
        $this->parentLinked = $parentLinked;
    }

    /**
     * Vyplni informace o zdrojove DB dle typu ciselniku.
     * @param int $q typ ciselniku
     */
    private function translateQuery($q)
    {
        $this->hasJine = True;
        $this->choiceEnumFk = 'cis_id';
        $this->alias = '';
        switch ($q) {
            case ZPRACOVATELE:
                $this->choiceParentFk = 'hid';
                $this->enumTable = 'uzivatel';
                $this->enumCol = 'login';
                $this->choiceTable = 'zpracovatel';
                $this->choiceEnumFk = 'uid';
                $this->hasJine = False;
                $this->domStr = 'zpracovatele';
                $this->xmlWrapper = 'authors';
                $this->alias = 'zpracovatele';
                break;
            case TYP_HESLA:
                $this->choiceParentFk = 'vid';
                $this->enumTable = 'cis_typ_hesla';
                $this->enumCol = 'typ_hesla';
                $this->choiceTable = 'typ_hesla_var_vyber';
                $this->domStr = 'typ_hesla';
                $this->xmlWrapper = 'lemma-type';
                $this->alias = 'varianta-typ_hesla';
                break;
            case SLOV_DRUHY:
                switch ($this->parent) {
                    case VYZNAM:
                        $this->choiceParentFk = 'vyz_slov_druhy_vyber_rel_id';
                        $this->choiceTable = 'slov_druhy_vyber_vyz';
                        $this->alias = 'vyznam-slov_druh-vyber';
                        break;
                    case VARIANTA:
                        $this->choiceParentFk = 'var_slov_druhy_vyber_rel_id';
                        $this->choiceTable = 'slov_druhy_vyber';
                        $this->alias = 'varianta-slov_druh-vyber';
                        break;
                    case PODHESLI:
                        $this->choiceParentFk = 'podh_rel_id';
                        $this->choiceTable = 'podh_vyz_slov_druhy';
                        $this->alias = 'vyznam-podhesli-slov_druh-vyber';
                        break;
                }
                $this->enumTable = 'cis_slov_druhy';
                $this->enumCol = 'slovny_druh';
                $this->domStr = '';
                break;
            case TVAR_CHAR:
                switch ($this->parent) {
                    case VYZNAM:
                        $this->choiceParentFk = 'vyz_tvar_char_vyber_rel_id';
                        $this->choiceTable = 'tvar_char_vyber_vyz';
                        $this->alias = 'vyznam-tvar_char-vyber';
                        break;
                    case VARIANTA:
                        $this->choiceParentFk = 'var_tvar_char_vyber_rel_id';
                        $this->choiceTable = 'tvar_char_vyber';
                        $this->alias = 'varianta-tvar_char-vyber';
                        break;
                    case PODHESLI:
                        $this->choiceParentFk = 'podh_tvch_rel_id';
                        $this->choiceTable = 'podh_tvar_char_vyber';
                        $this->alias = 'vyznam-podhesli-tvar_char-vyber';
                        break;
                }
                $this->enumTable = 'cis_tvar_char';
                $this->enumCol = 'tvar_char';
                $this->domStr = '';
                break;
            case TVAR_CHAR_SL:
                switch ($this->parent) {
                    case VYZNAM:
                        $this->choiceParentFk = 'vyz_tvar_char_sl_vyber_rel_id';
                        $this->choiceTable = 'tvar_char_sl_vyber_vyz';
                        $this->alias = 'vyznam-tvar_char_sl-vyber';
                        break;
                    case VARIANTA:
                        $this->choiceParentFk = 'var_tvar_char_sl_vyber_rel_id';
                        $this->choiceTable = 'tvar_char_sl_vyber';
                        $this->alias = 'varianta-tvar_char_sl-vyber';
                        break;
                    case PODHESLI:
                        $this->choiceParentFk = 'podh_tvch_sl_rel_id';
                        $this->choiceTable = 'podh_tvar_char_sl_vyber';
                        $this->alias = 'vyznam-podhesli-tvar_char_sl-vyber';
                        break;
                }
                $this->enumTable = 'cis_tvar_char_sl';
                $this->enumCol = 'tvar_char_sl';
                $this->domStr = '';
                break;
            case STYL_KVAL:
                switch ($this->parent) {
                    case VYZNAM:
                        $this->choiceParentFk = 'vyz_id';
                        $this->choiceTable = 'styl_kval_vyz_vyber';
                        $this->alias = 'vyznam-styl_kval';
                        break;
                    case VARIANTA:
                        $this->choiceParentFk = 'vid';
                        $this->choiceTable = 'styl_kval_var_vyber';
                        $this->alias = 'varianta-styl_kval';
                        break;
                    case PODHESLI:
                        $this->choiceParentFk = 'podh_id';
                        $this->choiceTable = 'styl_kval_podh_vyber';
                        $this->alias = 'vyznam-podhesli-styl_kval';
                        break;
                }
                $this->xmlWrapper = 'styl-qualifier';
                $this->enumTable = 'cis_styl_kval';
                $this->enumCol = 'styl_kval';
                $this->domStr = 'styl_kval';
                break;
            case GRAM_INFO:
                switch ($this->parent) {
                    case VYZNAM:
                        $this->choiceParentFk = 'vyz_id';
                        $this->choiceTable = 'gram_info_vyz_vyber';
                        $this->alias = 'vyznam-gram_info';
                        break;
                    case VARIANTA:
                        $this->choiceParentFk = 'vid';
                        $this->choiceTable = 'gram_info_var_vyber';
                        $this->alias = 'varianta-gram_info';
                        break;
                    case PODHESLI:
                        $this->choiceParentFk = 'podh_id';
                        $this->choiceTable = 'podh_vyz_gram_info';
                        $this->alias = 'vyznam-podhesli-gram_info';
                        break;
                }
                $this->enumTable = 'cis_gram_info';
                $this->enumCol = 'gram_info';
                $this->domStr = 'gram_info';
                $this->xmlWrapper = 'gram-info';
                break;
            case TEMATICKE_OKRUHY:
                switch ($this->parent) {
                    case VYZNAM:
                        $this->choiceParentFk = 'vyz_id';
                        $this->choiceTable = 'tematicke_okruhy_vyz_vyber';
                        $this->alias = 'vyznam-tematicke_okruhy';
                        break;
                    case VARIANTA:
                        $this->choiceParentFk = 'vid';
                        $this->choiceTable = 'tematicke_okruhy_var_vyber';
                        $this->alias = 'varianta-tematicke_okruhy';
                        break;
                    case PODHESLI:
                        $this->choiceParentFk = 'podh_id';
                        $this->choiceTable = 'tematicke_okruhy_podh_vyber';
                        $this->alias = 'vyznam-podhesli-tematicke_okruhy';
                        break;
                }
                $this->enumTable = 'cis_tematicke_okruhy';
                $this->enumCol = 'tematicke_okruhy';
                $this->domStr = 'tematicke_okruhy';
                $this->xmlWrapper = 'them-area';
                break;
            case PUVOD:
                $this->choiceParentFk = 'vid';
                $this->enumTable = 'cis_puvod';
                $this->enumCol = 'puvod';
                $this->choiceTable = 'puvod_var_vyber';
                $this->domStr = 'puvod';
                $this->xmlWrapper = 'origin';
                $this->alias = 'varianta-puvod';
                break;
            case OBOR_UZITI:
                switch ($this->parent) {
                    case VYZNAM:
                        $this->choiceParentFk = 'vyz_id';
                        $this->choiceTable = 'obor_uziti_vyz_vyber';
                        $this->alias = 'vyznam-obor_uziti';
                        break;
                    case VARIANTA:
                        $this->choiceParentFk = 'vid';
                        $this->choiceTable = 'obor_uziti_var_vyber';
                        $this->alias = 'varianta-obor_uziti';
                        break;
                    case PODHESLI:
                        $this->choiceParentFk = 'podh_id';
                        $this->choiceTable = 'obor_uziti_podh_vyber';
                        $this->alias = 'vyznam-podhesli-obor_uziti';
                        break;
                }
                $this->enumTable = 'cis_obor_uziti';
                $this->enumCol = 'obor_uziti';
                $this->domStr = 'obor_uziti';
                $this->xmlWrapper = 'field-of-use';
                break;
            case METAVYKLAD1:
                if ($this->parentLinked) {
                    $this->choiceTable = 'metavyklad1_vyz_vyber_m';
                    $this->domStr = 'metavyklad1_master';
                    $this->defTbl = 'metavyklad1_vyz_vyber';
                } else {
                    $this->choiceTable = 'metavyklad1_vyz_vyber';
                    $this->domStr = 'metavyklad1';
                }
                $this->alias = 'vyznam-metavyklad1';
                $this->choiceParentFk = 'vyz_id';
                $this->enumTable = 'cis_metavyklad1';
                $this->enumCol = 'metavyklad1';
                $this->xmlWrapper = 'meta-meaning1';
                break;
            case TYP_LINKOVANI:
                $this->choiceParentFk = 'hesla_rel_id';
                $this->enumTable = 'cis_hesla_rel_link_type';
                $this->enumCol = 'link_type';
                $this->choiceTable = 'hesla_rel_link_type_vyber';
                $this->domStr = '';
                $this->xmlWrapper = 'link-type';
                $this->alias = 'typl';
                break;
            case PROPRII:
                $this->choiceParentFk = 'id';
                $this->enumTable = 'heslare';
                $this->enumCol = 'label';
                $this->choiceTable = 'heslo';
                $this->hasJine = False;
                $this->choiceEnumFk = 'heslar_id';
                $this->domStr = 'heslar';
                $this->xmlWrapper = 'proprii';
                $this->alias = 'propr';
                break;
            case KONEC_ZAHLAVI:
                $this->choiceParentFk = 'hid';
                $this->enumTable = 'cis_konec_zahlavi';
                $this->enumCol = 'konec_zahlavi';
                $this->choiceTable = 'konec_zahlavi_vyber';
                $this->domStr = 'konec_zahlavi';
                $this->xmlWrapper = 'origin';
                $this->alias = 'heslo-konec_zahlavi';
                break;
        }
        if (!isset($this->defTbl)) {
            $this->defTbl = $this->choiceTable;
        }
        $this->baseDom = ($this->domStr ? $this->parentDomId . '--' . $this->domStr : $this->parentDomId);
        $this->domId = $this->baseDom;
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array($this->domId);
        $transl = array();
        if ($this->parentLinked) {
            $transl[$this->domId] = str_replace($this->parentVal, $this->parentLinked, $this->domId);
        }
        $this->corr = $this->loadCorrectionsByDom($domList, $transl);
    }

    /**
     * Prepne mezi dedenym/nededenym
     * @param $linked
     * @param bool $load
     */
    public function switchLinked($linked, $load = False)
    {
        $this->parentLinked = $linked;
        $this->translateQuery($this->type);
        $this->loadCorrections();
        if ($load) {
            $this->loadFromDB();
        }
    }

    /**
     * Seznam vybranych hodnot ciselniku.
     * @param int $number pocet prvnich hodnot
     * @param string $glue oddelovac
     * @return string
     */
    public function getValsList($number = 0, $glue = '<br />')
    {
        if ($number) {
            return implode($glue, array_slice(array_map('text_html_preprocess', $this->data), 0, $number));
        } else {
            return implode($glue, array_map('text_html_preprocess', $this->data));
        }
    }

    /**
     * Prepise data ciselniku.
     * @param array $ids pole id polozek vyberu
     * @return Response
     */
    public function updateCis($ids)
    {
        $this->data = array();
        if(!is_null($ids)){
            foreach ($ids as $id) {
                if (is_numeric($id)) {
                    $this->data[$id] = dibi::fetchSingle('
                        SELECT %n FROM %n WHERE id = %i
                    ', $this->enumCol, $this->enumTable, $id);
                } else {
                    $this->data['jine'] = $this->convert($id);
                }
            }
        }
        return new Response(OK, $this->getValsList(2));
    }

    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if ($this->parentLinked && !is_numeric($this->parentLinked)) {
            $tbl = $this->defTbl;
            $fk = substr($this->parentLinked, 3);
        } else {
            $tbl = $this->choiceTable;
            $fk = $this->parentLinked ? $this->parentLinked : $this->parentVal;
        }
        if ($this->hasJine) {
            try {
                $this->data = dibi::fetchPairs('
				    SELECT IFNULL(%n, "jine"), IF(%n IS NULL, jine, c.%n)
                    FROM %n v LEFT JOIN %n c ON v.%n=c.id
                    WHERE v.%n = %i
                    ORDER BY v.sortorder ASC
			    ', $this->choiceEnumFk, $this->choiceEnumFk, $this->enumCol,
                    $prefix . $tbl, $this->enumTable, $this->choiceEnumFk,
                    $this->choiceParentFk, $fk);
            } catch (DibiException $e) {
                if ($archive) {
                    if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                        $this->data = array();
                    }
                } else {
                    return handle_db_error($e);
                }
            }
        } else {
            try {
                if ($this->type == PROPRII) {
                    $this->data = dibi::fetchPairs('
                    SELECT v.%n, c.%n
                    FROM %n v LEFT JOIN %n c ON v.%n=c.id
                    WHERE v.%n = %i 
                ', $this->choiceEnumFk, $this->enumCol,
                        $prefix . $this->choiceTable, $this->enumTable, $this->choiceEnumFk,
                        $this->choiceParentFk, $fk);
                } else {
                    $this->data = dibi::fetchPairs('
                    SELECT v.%n, c.%n
                    FROM %n v LEFT JOIN %n c ON v.%n=c.id
                    WHERE v.%n = %i
                    ORDER BY v.sortorder ASC
                ', $this->choiceEnumFk, $this->enumCol,
                        $prefix . $this->choiceTable, $this->enumTable, $this->choiceEnumFk,
                        $this->choiceParentFk, $fk);
                }
            } catch (DibiException $e) {
                if ($archive) {
                    if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                        $this->data = array();
                    }
                } else {
                    return handle_db_error($e);
                }
            }

        }

        return new Response(OK);
    }

    protected function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        // je ve skupině a předtím nebyl
        $newlyLinked = $this->parentLinked && !$this->linkOld;
        // není ve skupině a předtím byl
        $newlyUnlinked = !$this->parentLinked && $this->linkOld;
        if (!$newlyLinked && !$newlyUnlinked && CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        $ord = 1;
        try {
            dibi::begin();
            // pokud dojde k přidání / odebrání ze skupiny, vymažeme data z neskupinové tabulky
            if ($newlyUnlinked || $newlyLinked) {
                $originalData = array();
                dibi::query('DELETE FROM %n WHERE %n = %i', $this->defTbl, $this->choiceParentFk, $this->parentVal);
                if ($newlyLinked) {
                    // jako original data musíme vzít data ze skupiny
                    $originalData = dibi::fetchPairs('
                        SELECT %n, 1 AS tmp FROM %n WHERE %n = %i
                    ', $this->choiceEnumFk, $prefix . $this->choiceTable, $this->choiceParentFk, $this->parentLinked);
                }
            }

            foreach ($this->data as $cid => $text) {
                $sqlData = array(
                    'sortorder' => $ord++
                );
                if ($cid === 'jine') {
                    $sqlData[$this->choiceEnumFk . '%sql'] = 'NULL';
                    $sqlData['jine'] = $text;
                } else {
                    $sqlData[$this->choiceEnumFk] = $cid;
                    $sqlData['jine%sql'] = 'NULL';
                }
                if (!$this->hasJine) {
                    unset($sqlData['jine'], $sqlData['jine%sql']);
                }
                if (isset($originalData[$cid])) {
                    // update
                    $where = array(
                        $this->choiceParentFk => $this->parentLinked ? $this->parentLinked : $this->parentVal,
                        $this->choiceEnumFk . '%iN' => intval($cid)
                    );
                    dibi::query('
                      UPDATE %n SET %a WHERE %and
                    ', $prefix . $this->choiceTable, $sqlData, $where);
                    unset($originalData[$cid]);
                } else {
                    // insert
                    $sqlData[$this->choiceParentFk] = $this->parentLinked ? $this->parentLinked : $this->parentVal;
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . $this->choiceTable, $sqlData);
                }
            }
            foreach ($originalData as $cid => $text) {
                //delete
                $where = array(
                    $this->choiceParentFk => $this->parentLinked ? $this->parentLinked : $this->parentVal,
                    $this->choiceEnumFk . '%iN' => intval($cid)
                );
                dibi::query('
                    DELETE FROM %n WHERE %and
                ', $prefix . $this->choiceTable, $where);
            }

            dibi::commit();
        } catch (DibiException $e) {
            dibi::rollback();
            return handle_db_error($e);
        }

        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '';
        if ($archive) {
            $nahled = $this->getValsList(False, '; ');
        } else {
            $nahled = $this->getValsList(2);
        }
        $pocet = count($this->data);
        if ($archive) {
            $corr = $this->emptyCorrectionArray();
        } else {
            $corr = $this->displayCorrection($this->corr[$this->domId]['field_DOMid'], $this->corr[$this->domId]['id'], $this->corr[$this->domId]['field'], $this->corr[$this->domId]['nova']);
        }
        if ($this->type == METAVYKLAD1 && $this->parentLinked) {
            $onClick = 'onClick="show_editor_ciselniku(\'' . $this->domId . '\', this, ' . intval($archive) . ');varovaniSkup(this, \'' . $this->parentVal . '\');"';
        } else {
            $onClick = 'onClick="show_editor_ciselniku(\'' . $this->domId . '\', this, ' . intval($archive) . ')"';
        }
        $html .= '
            <div id="' . $this->domId . '_popUp" title="Výběr hodnot z číselníku" class="popUp"
                data-form_id="' . $this->domId . '"
                data-cis="' . $this->type . '">
            </div>
            <table class="w100 ciselnik"><tr>
                <td class="border padd flatBox cisView ' . $corr['class'] . '" id="' . $this->domId . '--list">
                    <div>' . $nahled . '</div>';
        if (strpos($corr['flag'], 'corr_label.png') !== False) {
            $html .= $corr['flag'];
        }
        $html .= '</td><th>';
        if (strpos($corr['flag'], 'history.png') !== False) {
            $html .= $corr['flag'];
        }
        if (!$archive) {
            $html .= '
                <table class="floatLeft" style="border-spacing: 0;">
                    <tr><td id="' . $this->domId . '--cisView" class="center bold yellowBG border butt sqr' . ($corr['nova'] ? ' is_corrected' : '') . '" ' . $onClick . '>E</td></tr>
                    <tr><td id="' . $this->domId . '--valsNum" class="center bold blueBG border sqr">' . $pocet . '</td></tr>
                </table>
            ';
        } else {
            $html .= '
                <table class="floatLeft" style="border-spacing: 0;">
                    <tr><td class="center bold blueBG border sqr"></td></tr>
                    <tr><td class="center bold blueBG border sqr">' . $pocet . '</td></tr>
                </table>
            ';
        }
        $html .= '
            </th>
            </table>
        ';
        return $html;
    }

    public function printHeslarVyber($archive = false)
    {
        if ($archive) {
            $corr = $this->emptyCorrectionArray();
        } else {
            $corr = $this->displayCorrection($this->corr[$this->domId]['field_DOMid'], $this->corr[$this->domId]['id'], $this->corr[$this->domId]['field'], $this->corr[$this->domId]['nova']);
        }
        $cisData = dibi::fetchPairs('
              SELECT id, %n FROM %n 
              ORDER BY %n ASC
            ', $this->enumCol, $this->enumTable, 'id');
        $onChange = $archive ? 'disabled="disabled"' : ' onChange="save_input(this, \'' . $this->domId . '\')"';
        $html = '
            <div class="floatLeft correctableFieldContainer ' . $corr['class'] . '" id="' . $this->domId . '--heslar---helperIdCpy">
                ' . $corr['flag'] . '
                <div class="floatLeft label"> typ hesláře: </div>
                <div class="floatLeft searchThisFieldOnClick" id="' . $this->domId . '">';
        foreach ($cisData as $id => $type) {
            if ($id == array_keys($this->data)[0]) {
                $html .= '<input type="radio" name="typ_heslare" value="' . $id . '" class="noBorder" checked ' . $onChange . ' /> <label class="thinLabel"> ' . $type . '</label> <br />';
            } else {
                $html .= '<input type="radio" name="typ_heslare" value="' . $id . '" class="noBorder" ' . $onChange . ' /> <label class="thinLabel"> ' . $type . '</label> <br />';
            }
        }
        $html .= '
                </div>
                <div class="clear"></div>
            </div>
        ';
        return $html;
    }

    /**
     * Vygeneruje editacni dialog.
     * @param bool|False $archive
     * @return string
     */
    public function printEditor($archive = False)
    {
        // sloupec, podle kterého se má řadit
        if ($this->type == ZPRACOVATELE) {
            $sortcol = 'login';
        } elseif ($this->type == PROPRII) {
            $sortcol = 'id';
        } else {
            $sortcol = 'sortorder';
        }

        // načtení dat pro výběr - načteme jen ty položky číselníku, které nejsou ve výběru
        $z = "";
        if ($this->data) {    
            if($this->type == ZPRACOVATELE){
                $z = " AND role = ".HLAVNI_REDAKTOR." OR role = ".UZIVATEL;
            }
            $cisData = dibi::fetchPairs('
              SELECT id, %n FROM %n
              WHERE id NOT IN %in '.$z.'
              ORDER BY %n ASC
            ', $this->enumCol, $this->enumTable, array_keys($this->data), $sortcol);
        } else {
            if($this->type == ZPRACOVATELE){
                $z = " WHERE role = ".HLAVNI_REDAKTOR." OR role = ".UZIVATEL;
            }
            $cisData = dibi::fetchPairs('
              SELECT id, %n FROM %n '.$z.'
              ORDER BY %n ASC
            ', $this->enumCol, $this->enumTable, $sortcol);
        }
        $jine = isset($this->data['jine']) ? $this->data['jine'] : '';
        $id_ciselnik = $this->domId . '--cis';
        $id_cisVyber = $this->domId . '--cisVyber';
        $html['sb_ciselnik'] = '<select multiple="multiple" class="sbCiselnik" id="' . $id_ciselnik . '">';
        foreach ($cisData as $id => $d) {
            $html['sb_ciselnik'] .= '<option value="' . $id . '">' . text_html_preprocess($d) . '</option>';
        }

        // pokud ve výběru není hodnota "jiné", pak se zobrazí v číselníku
        if ($this->hasJine && !isset($this->data['jine'])) {
            $html['sb_ciselnik'] .= '<option value="jine">jiné</option>';
        }
        $html['sb_ciselnik'] .= '</select>';

        $html['sb_cisVyber'] = '<select multiple="multiple" class="sbCisVyber" id="' . $id_cisVyber . '">';
        foreach ($this->data as $id => $d) {
            if ($id == 'jine') $d = 'jiné';
            $html['sb_cisVyber'] .= '<option value="' . $id . '">' . text_html_preprocess($d) . '</option>';
        }
        $html['sb_cisVyber'] .= '</select>';

        $html['field_extraOpt'] = $this->hasJine ? '<input type="text" id="' . $this->domId . '--jine" value="' . text_html_preprocess($jine) . '" maxlength="45" style="width:190px"/>' : '';
        $html['butt_submit'] = '';
        $html['arr_up'] = '<input type="button" class="sortUp_butt" value="&nbsp;Up&nbsp;" onClick="Selectbox.moveOptionUp(document.getElementById(\'' . $id_cisVyber . '\'))">';
        $html['arr_down'] = '<input type="button" class="sortDown_butt" value="&nbsp;Down&nbsp;" onClick="Selectbox.moveOptionDown(document.getElementById(\'' . $id_cisVyber . '\'))">';
        $html['arr_toVyber'] = '
           <input id="B1"
                  type="button"
                  name="right"
                  value="&gt;&gt;"
                  onClick="Selectbox.moveSelectedOptions(document.getElementById(\'' . $id_ciselnik . '\'), document.getElementById(\'' . $id_cisVyber . '\'), false)"
           >';
        $html['arr_toCis'] = '
           <input type="button"
                  name="right"
                  value="&lt;&lt;"
                  onClick="Selectbox.moveSelectedOptions(document.getElementById(\'' . $id_cisVyber . '\'), document.getElementById(\'' . $id_ciselnik . '\'), false)"
           >';

        return form_ciselniku_TPL($html);
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';
        foreach ($this->data as $id => $c) {
            $xml .= '
                <' . $this->xmlElem . ' id="' . $id . '">' . $c . '</' . $this->xmlElem . '>
            ';
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        if ($this->type == PROPRII) {
            return new Response(OK);
        }
        $tbl = $this->defTbl;
        $atbl = ARCHIVE_PREFIX . $tbl;
        $ord = 1;
        foreach ($this->data as $cid => $text) {
            $sqlData = array(
                'sortorder' => $ord++
            );
            if ($cid === 'jine') {
                $sqlData[$this->choiceEnumFk . '%sql'] = 'NULL';
                $sqlData['jine'] = $text;
            } else {
                $sqlData[$this->choiceEnumFk] = $cid;
                $sqlData['jine%sql'] = 'NULL';
            }
            if (!$this->hasJine) {
                unset($sqlData['jine'], $sqlData['jine%sql']);
            }

            // insert
            $sqlData[$this->choiceParentFk] = $archiveParentVal;
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }

        }
        return new Response(OK);
    }

    public function isPresent($cis_id)
    {
        return isset($this->data[$cis_id]) ? $this->data[$cis_id] : '';
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $data[count($data) - 1][] = wrap_output($this->getDomId(), $this->getValsList(0, ', '), $wrap);
        $headers = array($this->choiceTable);
        return new Response(OK, array('headers' => $headers));
    }

    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        $linked = False;
        $tmp = array();
        $domtmp = array();
        if ($this->type == METAVYKLAD1) {
            $domtmp = explode('-', $parentStr);
            $vyz_id = $domtmp[count($domtmp) - 1];
            unset($domtmp[count($domtmp) - 1]);
            $linked = dibi::fetchSingle('
              SELECT vyklad_vyznamu_link FROM vyznamy
              WHERE vyklad_vyznamu_link IS NOT NULL AND metavyklad1_link = 1 AND id = %i
            ', $vyz_id);
        }
        if ($linked) {
            $data = dibi::query('
                SELECT CONCAT("metavyklad1-", v.id) AS id, c.metavyklad1 AS vyber
                FROM metavyklad1_vyz_vyber_m v INNER JOIN cis_metavyklad1 c ON v.cis_id = c.id
                WHERE v.vyz_id = %i
                ORDER BY v.sortorder ASC
            ', $linked)->fetchAssoc('id');
            $parentStr = implode('-', $domtmp) . '-' . $linked;
        }
        foreach ($data as $did => $entry) {
            //$did == 'heslo-6508' např.
            if (isset($entry['jine']) && !is_null($entry['jine']) && $entry['jine']) {
                $tmp[] = $entry['jine'];
            } else {
                $tmp[] = $entry['vyber'];
            }
        }
        return wrap_output($parentStr . '--' . $this->domStr, implode(', ', $tmp), WRAP_CORRECTION);
    }

    public function getSQL($fields = False, $parentTable = False)
    {
        $enumTableAlias = $this->enumTable . $this->parent;
        $return = array(
            'columns' => array(
                '[' . $this->choiceTable . '].[id] AS [' . $this->alias . '--id]',
                '[' . $this->choiceTable . '].[' . $this->choiceEnumFk . '] AS [' . $this->alias . '--vyber_id]',
                '[' . $enumTableAlias . '].[' . $this->enumCol . '] AS [' . $this->alias . '--vyber]'),
            'tables' => array(
                '([' . $this->choiceTable . '] LEFT JOIN [' . $this->enumTable . '] AS [' . $enumTableAlias . ']
                ON [' . $this->choiceTable . '].[' . $this->choiceEnumFk . '] = [' . $enumTableAlias . '].[id])' => array(
                    'main' => array('[' . $parentTable . '].[id]', '[' . $this->choiceTable . '].[' . $this->choiceParentFk . ']'),
                    'child' => array()
                )
            )
        );
        if ($this->hasJine) {
            $return['columns'][] = '[' . $this->choiceTable . '].[jine] AS [' . $this->alias . '--jine]';
        }

        return new Response(OK, $return);
    }
}

