<?php
/**
 * Soubor exempl.php obsahujici definici tridy CDO_Exempl.
 */

/**
 * Trida pro skupinu exemplifikaci
 */
class CDO_Exempl extends CDO_Base
{
    /**
     * @var int $parent typ nadrazeneho zaznamu
     */
    private $parent;

    /**
     * Konstruktor tridy CDO_ExtPozn.
     * @param int $input
     * @param int $parent
     * @param bool|False $parentVal
     * @param bool|False $parentDomId
     * @param bool|False $hid
     */
    function __construct($input = CDOin_DB, $parent = VYZNAM, $parentVal = False, $parentDomId = False, $hid = False)
    {
        $this->input = $input;
        $this->parent = $parent;
        $this->parentVal = $parentVal;
        $this->parentDomId = $parentDomId;
        $this->hid = $hid;
        $this->alias = 'vyznam-exemplifikace';
        $this->xmlWrapper = 'exemplifications';
        $this->colsSimple = array('exemplifikace', 'vystup');
        $this->colsStructured = array('propojeni');
        if ($parent && $parentVal) {
            $this->translateQuery();
        }
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {
            $this->data = array();
        }
    }

    /**
     * Vyplni informace o zdrojove DB.
     */
    private function translateQuery()
    {
        $this->tbl = 'exemplifikace';
        $this->parentFk = 'vyz_id';
        $this->domStr = 'exemplifikacia';
        $this->baseDom = $this->parentDomId . '--exemplifikacia';
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $this->corr = $this->loadCorrectionsByDom($this->domId);
    }

    public function add($exemplifikace = '', $vystup = DEF_EXEMPL_VYSTUP)
    {
        $newId = $this->newId();
        $this->domId[$newId] = $this->parentDomId . '--exemplifikacia-' . $newId;
        $this->data[$newId] = new DibiRow(array(
            'exemplifikace' => $exemplifikace,
            'exemplifikace_bz' => strip_tags($exemplifikace),
            'vystup' => $vystup,
            'propojeni' => new CDO_Link(CDOin_EMPTY, HREL_TYPE_pripojovani, true, $newId, $this->domId[$newId], $this->hid, 'exemplifikace', count($this->domId))
        ));
        return new Response(OK, $newId);
    }

    public function update($id, $field, $data, $data2 = False)
    {
        if ($field == 'exemplifikace') {
            $this->data[$id]->exemplifikace = trim($data) ;
            $this->data[$id]->exemplifikace_bz = bz( trim($data) , 'exemplifikace');
            return new Response(OK);
        } else {
            return parent::update($id, $field, $data);
        }
    }

    public function remove($id)
    {
        if ($this->data[$id]->propojeni->hasData()) {
            return new Response(NOT_OK, array(), 'Exemplifikace obsahuje odkazy.');
        } else {
            return parent::remove($id);
        }
    }


    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::query('
                SELECT id, exemplifikace, exemplifikace_bz, vystup
                FROM %n
                WHERE %n = %i
                ORDER BY sortorder ASC
            ', $prefix . $this->tbl, $this->parentFk, $this->parentVal)->fetchAssoc('id');

        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }

        $counter = 0;
        foreach ($this->data as $id => $ex) {
            $counter++;
            $this->domId[$id] = $this->baseDom . '-' . $id;
            $this->data[$id]['propojeni'] = new CDO_Link($this->input, HREL_TYPE_pripojovani, true, $id, $this->domId[$id], $this->hid, "exemplifikace", $counter);
        }
        return new Response(OK);
    }

    protected function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        $ord = 1;
        try {
            dibi::begin();
            $sqlData = array(
                $this->parentFk => $this->parentVal,
            );
            foreach ($this->data as $id => $d) {
                $sqlData['sortorder'] = $ord++;
                $sqlData['exemplifikace'] = $d['exemplifikace'];
                $sqlData['exemplifikace_bz'] = bz($d['exemplifikace'], 'exemplifikace');
                $sqlData['vystup'] = $d['vystup'];
                $d['propojeni']->saveToDB($archive, $originalData[$id]['propojeni']->data);
                if (isset($originalData[$id])) {
                    // update
                    dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . $this->tbl, $sqlData, $id);
                    unset($originalData[$id]);
                } else {
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . $this->tbl, $sqlData);
                }
            }

            foreach ($originalData as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . $this->tbl, $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }
        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param int $id
     * @param bool $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        $vystupArr = array(NEZARAZENO => 'nezařazeno', TISTENY => 'tištěný + elektronický', ELEKTRONICKY => 'elektronický');
        $dom = $this->domId[$id];
        if ($archive) {
            $corr = $this->emptyCorrectionArray();
        } else {
            $corr = $this->displayCorrection($dom, $this->corr[$dom]['id'], $this->corr[$dom]['field'], $this->corr[$dom]['nova']);
        }
        $html = '
            <div id="' . $dom . '" class="multiInput">
                ' . ($archive ? '' : $this->rmFieldsBtn($this->domId[$id])) . '
                <div>
                    <div class="floatLeft label">Exemplifikace</div>
                    <div class="floatRight">
                        <span class="label floatLeft">do výstupu:</span>
                        ' . $this->printRadioGroup($dom . '--vystup', $archive, $vystupArr, $this->data[$id]->vystup) . '
                    </div>
                </div>
                <div class="clear"></div>
                <div id="' . $dom . '--helperIdCpy" class="correctableFieldContainer">
                    ' . $corr['flag'] . '
                    <textarea id="' . $dom . '--text" class="greenBG mceEditor_italic cdoEditor ' . $corr['class'] . '" rows="3" style="width:797px;height:100px;">
                        ' . $this->data[$id]->exemplifikace . '
                    </textarea>
                    ' . $this->initTinyMCE($dom . '--text', $archive, EXEMPLIFIKACE, $id) . '
                </div>
        ';
        if (!$archive) {
            $html .= '
                <div class="sortingArrows">
                    <a onClick="moveItem(\'' . $this->baseDom . '\', \'' . $id . '\', \'up\')" class="moveUp"><img src="css/images/arrow_up.png" alt="Posunout výše" /></a><br />
                    <a onClick="moveItem(\'' . $this->baseDom . '\', \'' . $id . '\', \'down\')" class="moveDown"><img src="css/images/arrow_down.png" alt="Posunout níže" /></a>
                </div>
            ';
        }
        $html .= '<div id="' . $dom . '-propojeni_wrapper">' . $this->data[$id]->propojeni->printForm($archive) . '</div>
                <div class="hspace"></div>
            </div>
        ';
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = ' <div id="' . $this->baseDom . '" class="multiContainer margin4addButt margin4orderArrows"> ';
        if (!$this->data) {
            $this->add();
        }
        foreach ($this->data as $id => $d) {
            $html .= $this->printOne($id, $archive);
        }
        $html .= '</div> ';
        if (!$archive) {
            $html .= '
                <div class="tlacitkoRemField" >
                    ' . $this->addFieldsBtn($this->baseDom, 'exemplifikaci') . '
                </div>
            ';
        }
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<exemplifications>';
        foreach ($this->data as $id => $d) {
            $xml .= '
                <exemplification id="' . $id . '">
                    <exemplification-text-html><![CDATA[' . $d->exemplifikace . ']]></exemplification-text-html>
                    <exemplification-text>' . $d->exemplifikace_bz . '</exemplification-text>
                </exemplification>
            ';
        }
        $xml .= '</exemplifications>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $tbl = $this->tbl;
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        $ord = 1;
        $sqlData = array(
            $this->parentFk => $archiveParentVal
        );
        foreach ($this->data as $id => $d) {
            $sqlData['sortorder'] = $ord++;
            $sqlData['exemplifikace'] = $d['exemplifikace'];
            $sqlData['exemplifikace_bz'] = bz($d['exemplifikace'], 'exemplifikace');
            $sqlData['vystup'] = $d['vystup'];
            if (is_numeric($id)) {
                $sqlData['main_id'] = $id;
            }
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
            $dbid = $tmp->data;
            foreach ($this->colsStructured as $col) {
                $this->data[$id][$col]->saveToArchive($dbid, $archiveHid);
            }

        }
        return new Response(OK);
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $strArr = array();
        foreach ($this->data as $id => $d) {
            $strArr[] = wrap_output($this->getDomId($id, 'exemplifikace'), $d['exemplifikace'], $wrap);
        }
        $data[count($data) - 1][] = implode(', ', $strArr);
        $headers = array('exemplifikace');
        return new Response(OK, array('headers' => $headers));
    }

    protected function getSQLFields($fields)
    {
        $f = array('exemplifikace' => 1);
        if (isset($fields['vystup'])) {
            $f['vystup'] = 1;
        }
        return $f;
    }

//    public function tabFormat($data, $fields = array(), $parentStr = '')
//    {
//        $tmp = array();
//        foreach ($data as $did => $entry) {
//            $tmp[] = wrap_output($parentStr . '--' . $this->domStr . '-' . $entry['id'], $entry['exemplifikace'], WRAP_CORRECTION);
//        }
//        return implode(', ', $tmp);
//    }
}