<?php
/**
 * Soubor extpozn.php obsahujici definici tridy CDO_ExtPozn.
 */

/**
 * Trida pro skupinu externich poznamek
 */
class CDO_ExtPozn extends CDO_Base
{
    /**
     * @var int $parent typ nadrazeneho zaznamu
     */
    private $parent;
    /** @var array $cisEnum ciselnik typu */
    private $typEnum;

    /**
     * Konstruktor tridy CDO_ExtPozn.
     * @param int $input
     * @param bool|False $parent
     * @param bool|False $parentVal
     * @param bool|False $parentDomId
     */
    function __construct($input = CDOin_DB, $parent = False, $parentVal = False, $parentDomId = False)
    {
        $this->input = $input;
        $this->parent = $parent;
        $this->parentVal = $parentVal;
        $this->parentDomId = $parentDomId;
        $this->xmlWrapper = 'ext-remarks';
        $this->colsSimple = array('poznamka', 'cis_id');
        $this->typEnum = dibi::fetchPairs('SELECT id, typ FROM cis_typ_ext_pozn');
        if ($parent && $parentVal) {
            $this->translateQuery();
        }
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {
            $this->data = array();
        }
    }

    /**
     * Vyplni informace o zdrojove DB.
     */
    private function translateQuery()
    {
        $this->alias = '';
        if ($this->parent == HESLO) {
            $this->tbl = 'ext_poznamky_hes';
            $this->parentFk = 'hid';
            $this->alias = 'heslo-ext_pozn';
        } elseif ($this->parent == VYZNAM) {
            $this->tbl = 'ext_poznamky_vyz';
            $this->parentFk = 'vyz_id';
            $this->alias = 'vyznam-ext_pozn';
        }
        $this->domStr = 'extPozn';
        $this->baseDom = $this->parentDomId . '--extPozn';
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array();
        foreach ($this->domId as $id => $dom) {
            array_push($domList, $dom . '--text');
        }
        $this->corr = $this->loadCorrectionsByDom($domList);
    }

    /**
     * Prida novou externi poznamku.
     * @param string $poznamka
     * @param int $cis_id
     * @return Response
     */
    public function add($poznamka = '', $cis_id = DEF_EXT_POZN_TYP)
    {
        $newId = $this->newId();
        $this->domId[$newId] = $this->parentDomId . '--extPozn-' . $newId;
        $this->data[$newId] = new DibiRow(array(
            'cis_id' => $cis_id,
            'poznamka' => $poznamka,
            'typ' => dibi::fetchSingle('SELECT typ FROM cis_typ_ext_pozn WHERE id = %i', $cis_id)
        ));
        return new Response(OK, $newId);
    }

    /**
     * Prepise data externi poznamky.
     * @param int $id
     * @param string $field
     * @param array $pozn
     * @param bool $typ
     * @return Response
     */
    public function update($id, $field, $pozn, $typ = True)
    {
        $this->data[$id]->poznamka = $this->convert($pozn);
        if ($typ !== True) {
            if (!$pozn) {
                $typ = EXT_POZN_EMPTY;
            } elseif (!$typ) {
                $typ = DEF_EXT_POZN_TYP;
            }
            $this->data[$id]->cis_id = $typ;
            $this->data[$id]->typ = dibi::fetchSingle('SELECT typ FROM cis_typ_ext_pozn WHERE id = %i', $typ);
        }
        return new Response(OK, $this->data[$id]->typ);
    }

    /**
     * Nacteni externich poznamek z DB.
     * @param bool $archive
     * @return Response
     */
    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::query('
                SELECT e.id, e.cis_id, e.poznamka, c.typ
                FROM %n e INNER JOIN cis_typ_ext_pozn c ON e.cis_id = c.id
                WHERE %n = %i
                ORDER BY id ASC
            ', $prefix . $this->tbl, $this->parentFk, $this->parentVal)->fetchAssoc('id');

        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }

        foreach ($this->data as $id => $ex) {
            $this->domId[$id] = $this->baseDom . '-' . $id;
        }

        return new Response(OK);
    }

    /**
     * Ulozeni externich poznamek do DB.
     * @param bool $archive
     * @param array $originalData
     * @return Response
     */
    protected function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        try {
            dibi::begin();
            $sqlData = array(
                $this->parentFk => $this->parentVal,
            );
            foreach ($this->data as $id => $d) {
                $sqlData['poznamka'] = utf8_trim($d['poznamka']);
                $sqlData['cis_id'] = $d['cis_id'];
                if ($sqlData['poznamka'] == '') {
                    $sqlData['cis_id'] = EXT_POZN_EMPTY;
                } else if ($sqlData['cis_id'] == EXT_POZN_EMPTY) {
                    $sqlData['cis_id'] = DEF_EXT_POZN_TYP;
                }

                if (isset($originalData[$id])) {
                    // update
                    dibi::query('
                          UPDATE %n SET %a WHERE id = %i
                      ', $prefix . $this->tbl, $sqlData, $id);
                    unset($originalData[$id]);
                } else {
                    dibi::query('
                          INSERT INTO %n
                        ', $prefix . $this->tbl, $sqlData);
                }
            }

            foreach ($originalData as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . $this->tbl, $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }
        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param int $id
     * @param bool $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        $dom = $this->domId[$id];
        $corrDom = $dom . '--text';
        if ($archive) {
            $corr = $this->emptyCorrectionArray();
        } else {
            $corr = $this->displayCorrection($corrDom, $this->corr[$corrDom]['id'], $this->corr[$corrDom]['field'], $this->corr[$corrDom]['nova']);
        }
        $typstr = ($this->data[$id]->cis_id == EXT_POZN_EMPTY ? '&nbsp;' : $this->data[$id]->typ);
        $html = '
            <div id="' . $dom . '_popUp" title="Editor externí poznámky" class="popUp"></div>
            <div id="' . $dom . '" class="multiInput">
                ' . ($archive ? '' : $this->rmFieldsBtn($this->domId[$id])) . '
                <div class="vspace">&nbsp;</div>
                ' . $corr['flag'] . '
                <div id="' . $dom . '--textDisp" class="border flatBox floatLeft inputFont padd extPozn correctableFieldContainer ' . $corr['class'] . '" style="width: 720px; min-height: 18px" >
                    ' . $this->data[$id]->poznamka . '
                </div>
                <div class="vspace">&nbsp;</div>
                <div id="' . $dom . '--typeDisp" class="border flatBox paddS floatLeft extPozn" style="width: 50px">' . $typstr . '</div>
                <div class="vspace">&nbsp;</div>
                ' . ($archive ? '' : '<div id="' . $dom . '--expView"
                    class="center bold butt sqr yellowBG border flatBox floatLeft' . ($corr['nova'] ? ' is_corrected' : '') . '"
                    onClick="show_editor_extPozn(\'' . $dom . '\', this)">E</div>') . '
                <div class="hspace"></div>
            </div>
        ';
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '
            <div class="label">externí poznámka k ' . ($this->parent == VYZNAM ? 'významu' : 'heslu') . ':</div>
            <div class="' . ($this->parent == VYZNAM ? '' : 'orangeBox') . '">
            <div id="' . $this->baseDom . '" class="multiContainer margin4addButt">
        ';
        if (!$this->data) {
            $this->add();
        }
        foreach ($this->data as $id => $d) {
            $html .= $this->printOne($id, $archive);
        }
        $html .= '</div>';
        if (!$archive) {
            $html .= '
                <div class="tlacitkoRemField" >
                    ' . $this->addFieldsBtn($this->baseDom, 'poznámku') . '
                </div>
            ';
        }
        $html .= '</div>';
        return $html;
    }

    /**
     * Vygeneruje editacni dialog.
     * @param $id
     * @return string
     */
    public function printEditor($id)
    {
        $dom = $this->domId[$id];
        $sb = dibi::fetchPairs('SELECT id, typ FROM cis_typ_ext_pozn ORDER BY sortorder ASC');
        $html = '
            <textarea id="' . $dom . '--text" style="width:690px" rows="6">' . $this->data[$id]->poznamka . '</textarea>
            <div><select id="' . $dom . '--type">
        ';
        foreach ($sb as $i => $t) {
            $sel = ($i == $this->data[$id]->cis_id ? ' selected="selected"' : '');
            $html .= '<option value="' . $i . '"' . $sel . '>' . $t . '</option>';
        }
        $html .= '</select></div>';
        return $html;
    }

    /**
     * XML export.
     * @return string
     */
    public function printXML()
    {
        $xml = '<ext-remarks>';
        foreach ($this->data as $id => $d) {
            $xml .= '
                <ext-remark id="' . $id . '">
                    <remark>' . strip_tags($d->poznamka) . '</remark>
                    <remark-html><![CDATA[' . $d->poznamka . ']]></remark-html>
                    <type id="' . $d->cis_id . '">' . $d->typ . '</type>
                </ext-remark>
            ';
        }
        $xml .= '</ext-remarks>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return Response
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $tbl = $this->tbl;
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        $sqlData = array(
            $this->parentFk => $archiveParentVal,
        );
        foreach ($this->data as $id => $d) {
            $sqlData['poznamka'] = $d['poznamka'];
            $sqlData['cis_id'] = $d['cis_id'];
            if (is_numeric($id)) {
                $sqlData['main_id'] = $id;
            }
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
        }
        return new Response(OK);
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $strArr = array();
        foreach ($this->data as $id => $d) {
            // wrap_output($this->getDomId($id, 'exemplifikace'), $d['exemplifikace'], $wrap)
            $strArr[] = wrap_output($this->getDomId($id, 'poznamka'), $d['poznamka'], $wrap) .
                '(' . wrap_output($this->getDomId($id, 'typ'), $d['typ'], $wrap) . ')';
        }
        $data[count($data) - 1][] = implode(', ', $strArr);
        $headers = array('ext. pozn.');
        return new Response(OK, array('headers' => $headers));
    }

    protected function getSQLFields($fields)
    {
        return array('poznamka' => 1, 'cis_id' => 1);
    }

    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        $tmp = array();
        foreach ($data as $did => $entry) {
            $pozn = strip_tags($entry['poznamka']);
            if ($entry['cis_id']) {
                $tstr = ' (' . $this->typEnum[$entry['cis_id']] . ')';
            } else {
                $tstr = '';
            }
            $tmp[] = wrap_output($parentStr . '--' . $this->domStr . '-' . $entry['id'], $pozn, WRAP_CORRECTION) . $tstr;
        }
        return implode(', ', $tmp);
    }
}