<?php
/**
 * Soubor heslo.php obsahujici definici tridy CDO
 */

/**
 * Trida pro cele heslo.
 */
class CDO extends CDO_Base
{
    /**
     * @var array $podhesliVyz seznam podhesli vyznamu v hesle
     */
    public $podhesliVyz = array();
    /**
     * @var int $prevHid predchozi heslo
     */
    private $prevHid;
    /**
     * @var int $nextHid nasledujici heslo
     */
    private $nextHid;
    /**
     * @var array $resolvedCorrections vyresene korektury
     */
    private $resolvedCorrections = array();
    /** @var int $mainHid hid pokud jsou data z archivu */
    private $mainHid;

    /**
     * Konstruktor tridy CDO.
     * @param int $input
     * @param $hid
     */
    function __construct($input = CDOin_DB, $hid)
    {
        $this->input = $input;
        $this->hid = $hid;
        $this->mainHid = $hid;
        $this->alias = 'heslo';
        $this->tbl = 'heslo';
        $this->domId = 'heslo-' . $this->hid;
        $this->colsStructured = array('zpracovatele', 'varianta', 'ext_pozn', 'vyznam', 'propojeni_s', 'propojeni', 'synonyma_slave', 'antonyma_slave', 'semafor', "konec_zahlavi");
        $this->colsSimple = array('stav', 'vystup', 'lexgraf_pozn', 'pozn_odbornik', 'pid', 'redukce', 'heslar_id');
        // TODO: prev hid a next hid
        $this->prevHid = $hid - 1;
        $this->nextHid = $hid + 1;
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->mainHid = dibi::fetchSingle('SELECT main_heslo FROM archive_heslo WHERE id = %i', $this->hid);
            $this->prevHid = $this->prev_arch_entry();
            $this->nextHid = $this->next_arch_entry();
            $this->loadFromDB(True);
        } elseif ($this->input === CDOin_SQL) {
            $this->loadStructured();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = new DibiRow(array());
        }
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array(
            $this->domId . '--do_vystupu',
            $this->domId . '--stav',
            $this->domId . '--lexikograficka_poznamka',
            $this->domId . '--poznamka_odbornik'
        );
        $this->corr = $this->loadCorrectionsByDom($domList);
    }

    public function update($id, $field, $data, $data2 = False)
    {
        if (isset($this->data[$field]) || is_null($this->data[$field])) {
            $this->data[$field] = trim($data);
            if ($field == 'pid') {
                $this->data['pidvar'] = getHlavniVarianta($this->data['pid']);
            }
            return new Response(OK);
        } else {
            return new Response(WRONG_ID_GIVEN, array('id' => $id, 'field' => $field), 'Položka neexistuje.');
        }
    }

    public function setHid($hid)
    {
        parent::setHid($hid);
        $this->update($hid, 'id', $hid);
    }

    public function setPrevNextHid($prev, $next)
    {
        $this->prevHid = $prev;
        $this->nextHid = $next;
    }

    public function next_arch_entry()
    {
        return dibi::fetchSingle('
            SELECT MIN(id) FROM archive_heslo
            WHERE main_heslo = %i AND id > %i
        ', $this->mainHid, $this->hid);
    }

    public function prev_arch_entry()
    {
        return dibi::fetchSingle('
            SELECT MAX(id) FROM archive_heslo
            WHERE main_heslo = %i AND id < %i
        ', $this->mainHid, $this->hid);
    }


    private function loadStructured()
    {
        $this->data['semafor'] = new CDO_Semafor($this->input, $this->hid, $this->domId, $this->data['created_by']);

        $this->data['synonyma_slave'] = new CDO_Link($this->input, HREL_TYPE_syn, false, $this->hid, $this->domId, $this->hid, 'heslo', 0);
        $this->data['antonyma_slave'] = new CDO_Link($this->input, HREL_TYPE_ant, false, $this->hid, $this->domId, $this->hid, 'heslo', 0);

        $this->data['varianta'] = new CDO_Varianta($this->input, $this->hid);
        $this->data['vyznam'] = new CDO_Vyznam($this->input, $this->hid);
        $this->data['zpracovatele'] = new CDO_Ciselnik($this->input, ZPRACOVATELE, HESLO, $this->hid, $this->domId);
        $this->data['ext_pozn'] = new CDO_ExtPozn($this->input, HESLO, $this->hid, $this->domId);

        $this->data['propojeni_s'] = new CDO_Link($this->input, HREL_TYPE_pripojovani, false, $this->hid, $this->domId, $this->hid, 'heslo', 0);
        $this->data['propojeni'] = new CDO_Link($this->input, HREL_TYPE_pripojovani, true, $this->hid, $this->domId, $this->hid, 'heslo', 0);
        $this->data['odk_slave'] = new CDO_Link($this->input, HREL_TYPE_odkazy, false, $this->hid, $this->domId, $this->hid, 'heslo', 0);
        $this->data['odk_main'] = new CDO_Link($this->input, HREL_TYPE_odkazy, true, $this->hid, $this->domId, $this->hid, 'heslo', 0);
        $this->data['konec_zahlavi'] = new CDO_Ciselnik($this->input, KONEC_ZAHLAVI, HESLO, $this->hid, $this->domId);
    }


    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        $this->data = dibi::fetch('
				SELECT id, cts, uts, stav, vystup, lexgraf_pozn, pozn_odbornik, pid, redukce, heslar_id, created_by, updated_by
				FROM %n WHERE id = %i
			', $prefix . 'heslo', $this->hid);
        if ($this->data['pid']) {
            $this->data['pidvar'] = getHlavniVarianta($this->data['pid']);
        } else {
            $this->data['pidvar'] = '';
        }
        $this->podhesliVyz = dibi::fetchPairs('
            SELECT CAST(p.id AS CHAR), v.id FROM podhesli_vyznamu p INNER JOIN vyznamy v ON p.vyz_id = v.id WHERE v.hid = %i
        ', $this->hid);

        if (!isset($_SESSION['podh_vyz'])) {
            $_SESSION['podh_vyz'] = array();
        }

        foreach ($this->podhesliVyz as $p => $v) {
            $_SESSION['podh_vyz'][(string)$p] = (string)$v;
        }
        $this->loadStructured();
    }

    protected function saveToDB($archive = False, $originalData = null)
    {
        if (!$archive && is_numeric($this->hid)) {
            track_ulozeni_hesla($this->hid, get_logged_user());
        }
        if($this->data['vystup']!=TISTENY){
            $manager = new CacheManager();
            $manager->removeFromWeb($this->hid);
        } 
        if (CDO_compare($this->data, $originalData)) {
            $this->saveResolvedCorrections();
            return new Response(OK);
        }

        if (!$this->data['pid']) {
            $this->data['pid'] = null;
        }
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            dibi::begin();
            $sqlData = array();
            foreach ($this->colsSimple as $col) {
                $sqlData[$col] = $this->data[$col];
            }
            $sqlData['redukce'] = intval($this->data['redukce']);
            $sqlData['uts%sql'] = 'NOW()';
            if ($originalData) {
                // update
                dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . 'heslo', $sqlData, $this->hid);
                foreach ($this->colsStructured as $col) {
                    $this->data[$col]->saveToDB($archive, $originalData[$col]->data);
                }
                $dbid = $this->hid;
                $resp = $this->data['odk_slave']->saveToDB($archive, $originalData['odk_slave']->data);
                $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                if ($resp->data != false) {
                    foreach ($resp->data as $leva => $prava) {
                        if (!empty($this->data['odk_main']->data[$prava])) {
                            $this->data['odk_slave']->data[$leva] = $this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]];
                            unset($this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]]);
                            $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                            $this->data['odk_main']->data[$prava]["spojeno"] = $leva;
                        }
                    }
                }
                $resp = $this->data['odk_main']->saveToDB($archive, $originalData['odk_main']->data);
                $originalData['odk_main']->data = $this->data['odk_main']->data;
                if ($resp->data != false) {
                    foreach ($resp->data as $leva => $prava) {
                        if (!empty($this->data['odk_slave']->data[$prava])) {
                            $this->data['odk_main']->data[$leva] = $this->data['odk_main']->data[$this->data['odk_slave']->data[$prava]["spojeno"]];
                            unset($this->data['odk_main']->data[$this->data['odk_slave']->data[$prava]["spojeno"]]);
                            $originalData['odk_main']->data = $this->data['odk_main']->data;
                            $this->data['odk_slave']->data[$prava]["spojeno"] = $leva;
                        }
                    }
                    $resp = $this->data['odk_slave']->saveToDB($archive, $originalData['odk_slave']->data);;
                    $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                }
                if ($resp->data != false) {
                    foreach ($resp->data as $leva => $prava) {
                        if (!empty($this->data['odk_main']->data[$prava])) {
                            $this->data['odk_slave']->data[$leva] = $this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]];
                            unset($this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]]);
                            $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                            $this->data['odk_main']->data[$prava]["spojeno"] = $leva;
                        }
                    }
                    $this->data['odk_main']->saveToDB($archive, $originalData['odk_main']->data);
                }
            } else {
                dibi::query('
                        INSERT INTO %n
                    ', $prefix . 'heslo', $sqlData);
                $dbid = $this->newId(True);
                foreach ($this->colsStructured as $col) {
                    $this->data[$col]->setParentVal($dbid);
                    $this->data[$col]->saveToDB($archive);
                }
                $this->data['odk_slave']->setParentVal($dbid);
                $this->data['odk_main']->setParentVal($dbid);
                $resp = $this->data['odk_slave']->saveToDB($archive, $originalData['odk_slave']->data);
                $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                if ($resp->data != false) {
                    foreach ($resp->data as $leva => $prava) {
                        if (!empty($this->data['odk_main']->data[$prava])) {
                            $this->data['odk_slave']->data[$leva] = $this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]];
                            unset($this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]]);
                            $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                            $this->data['odk_main']->data[$prava]["spojeno"] = $leva;
                        }
                    }
                }
                $resp = $this->data['odk_main']->saveToDB($archive, $originalData['odk_main']->data);
                $originalData['odk_main']->data = $this->data['odk_main']->data;
                if ($resp->data != false) {
                    foreach ($resp->data as $leva => $prava) {
                        if (!empty($this->data['odk_slave']->data[$prava])) {
                            $this->data['odk_main']->data[$leva] = $this->data['odk_main']->data[$this->data['odk_slave']->data[$prava]["spojeno"]];
                            unset($this->data['odk_main']->data[$this->data['odk_slave']->data[$prava]["spojeno"]]);
                            $originalData['odk_main']->data = $this->data['odk_main']->data;
                            $this->data['odk_slave']->data[$prava]["spojeno"] = $leva;
                        }
                    }
                    $resp = $this->data['odk_slave']->saveToDB($archive, $originalData['odk_slave']->data);
                    $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                }
                if ($resp->data != false) {
                    foreach ($resp->data as $leva => $prava) {
                        if (!empty($this->data['odk_main']->data[$prava])) {
                            $this->data['odk_slave']->data[$leva] = $this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]];
                            unset($this->data['odk_slave']->data[$this->data['odk_main']->data[$prava]["spojeno"]]);
                            $originalData['odk_slave']->data = $this->data['odk_slave']->data;
                            $this->data['odk_main']->data[$prava]["spojeno"] = $leva;
                        }
                    }
                    $this->data['odk_main']->saveToDB($archive, $originalData['odk_main']->data);
                }
            }
            $this->saveResolvedCorrections();
            dibi::commit();
            return new Response(OK, $dbid);
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    { 
        if (!$archive) {
            track_navsteva_hesla($this->hid, get_logged_user());
        }
        $html = '';
        $antSynMaster = !(empty($this->data->synonyma_slave->data) && empty($this->data->antonyma_slave)) ? ("Významy, u nichž je toto heslo uvedeno jako synonymum/antonymum") : ("Tento heslo není připojeno jako synonymum k žádnému významu");

        if (is_err()) $html .= print_errs();
        $html .= print_new_corrections_box($this->hid);
        $html .= '
            <form action="formular_hesla_save.php" method="post" name="hlavni_formular_hesla" id="hlavniFormularHesla"
                data-hid="' . $this->hid . '" data-needConfirm="0" data-archive="' . intval($archive) . '" data-checked="0">
            <div>
                <input type="hidden" name="hid" value="' . $this->hid . '"/>
                ' . $this->printStickyHeader($archive) . '
                ' . $this->printHeader($archive) . '
                <div class="hspaceXXL"></div>
                <div class="main_rels">
                    <div class="show_main_rels clickable label">
                        <img src="css/images/roll_out.png" alt="Rozbalit" />Ukaž hesla, která odkazují na toto heslo, resp. ke kterým je připojeno/přihnízdováno toto heslo
                    </div>
                    <div class="main_rels_boxes" style="display: none">
                        <div id="slave_pripojeni">
                            ' . $this->data['propojeni_s']->printForm($archive) . '
                        </div>
                        <div class="hspace"></div><div class="hspace"></div>
                        <div id="slave_odkazy">
                            ' . $this->data['odk_slave']->printForm($archive) . '
                        </div>
                        <div class="hspace"></div>
                        <div class="hide_main_rels label clickable"><img src="css/images/roll_in.png" alt="Skrýt" />Skrýt</div>
                    </div>
                </div>
                <div class="hspaceXXL"></div>
                ' . $this->data['varianta']->printForm($archive) . '
                <div class="hspace"></div>' .
            $this->printKonecZahlavi($archive)
            . '<div class="hspace"></div>
                ' . $this->data['ext_pozn']->printForm($archive) . '
                <div class="hspaceXXL"></div>
                <div id="' . $this->domId . '-as" colspan="2" class="clickable label" onclick="show_ant_syn(\'' . $this->domId . '\')">
                            <img alt="Rozbalit" src="css/images/roll_out.png" >
                            ' . $antSynMaster . '
                </div>
                <div colspan="2" id="' . $this->domId . '-ash" style="display:none">
                    <div  id="' . $this->domId . '-s_syn_wrapper" class="w50 floatLeft">
                    ' . $this->data['synonyma_slave']->printForm($archive) . '
                    </div>
                    <div id="' . $this->domId . '-s_ant_wrapper" class="w50 floatLeft">
                    ' . $this->data['antonyma_slave']->printForm($archive) . '
                    </div>
                    <div  class="clickable label"  onclick="hide_ant_syn(\'' . $this->domId . '\')">
                        <img alt="Skrýt" src="css/images/roll_in.png" onclick="$(\'.hide_ant_syn\').show">
                        Skrýt
                    </div>
                </div>
                <div class="hspaceXXL"></div>
                ' . $this->data['vyznam']->printForm($archive) . '
                <div class="hspace"></div>
                <div id="heslo-' . $this->hid . '-propojeni_wrapper">
                ' . $this->data['propojeni']->printForm($archive) . '
                </div>
                <div class="hspace"></div>
                <div id="main_odkazy">
                ' . $this->data['odk_main']->printForm($archive) . '
                </div>
        ';
        if (!$archive) {
            $html .= '
                <input type="submit" value="uložit" class="blueButtBox buttBox floatRight" />
                <input type="button" value="uložit jako kopii" class="blueButtBox buttBox floatRight" onclick="show_heslo_copy_form()" />
            ';
        }
        $html .= '
            </div>
            <div id="varovani-wrapper"></div>
            </form>
        ';
        if (!$archive) {
            $html .= '
                <div id="kopieHesla_popUp" title="Vytvoření kopie hesla" class="popUp"></div>
                <script type="text/javascript">
                    var hidOnlineUpdInterval;
                    $(function(){
                        hidOnlineUpdInterval = setInterval(function(){ update_heslo_block_uts(' . $this->hid . ') }, 30000);
                    });
                </script>
            ';
        }

        return $html;
    }

    /**
     * Hlavni varianta hesla
     * @param bool $homonymie
     * @return string
     */
    public function getHlavniVarianta($homonymie = False)
    {
        $vkeys = array_keys($this->data['varianta']->data);
        $var = $this->data['varianta']->data[$vkeys[0]]['varianta'];
        if ($homonymie && $this->data['varianta']->data[$vkeys[0]]['homonymie']) {
            $var .= ' [' . $this->data['varianta']->data[$vkeys[0]]['homonymie'] . ']';
        }
        return $var;
    }

    /**
     * Nastavi hlavni variantu hesla
     * @param $varianta
     * @param bool|string $homonymie
     * @return string
     */
    public function setHlavniVarianta($varianta, $homonymie = '')
    {
        $vkeys = array_keys($this->data['varianta']->data);
        $this->data['varianta']->update($vkeys[0], 'varianta', $varianta);
        $this->data['varianta']->update($vkeys[0], 'homonymie', $homonymie);
        return new Response(OK);
    }

    /**
     * Vygeneruje hlavicku formulare
     * @param bool $archive
     * @return string
     */
    private function printHeader($archive = False)
    {
        $vystupArr = array(
            NEZARAZENO => 'nezařazeno',
            NEZARAZENO_FRAZ => 'nezařazeno - hyperlemma',
            TISTENY => 'tištěný + elektronický',
            ELEKTRONICKY => 'elektronický'
        );
        $stavDom = $this->domId . '--stav';
        $corrStav = $this->displayCorrection($this->corr[$stavDom]['field_DOMid'], $this->corr[$stavDom]['id'], $this->corr[$stavDom]['field'], $this->corr[$stavDom]['nova']);
        if ($archive) {
            $stav_onclick = '';
        } else {
            $stav_onclick = ' onClick="JS_update_stav_hesla(' . $this->hid . ', this); save_input(false, \'' . $stavDom . '\')"';
        }
        $stavArr = array(
            NOVE => 'nové',
            ROZPRACOVANE => 'rozpracované',
            DOKONCENE => 'dokončené'
        );
        if ($this->data['stav'] == NOVE) {
            $stav = $stavArr[NOVE];
            $stavClass = 'noveHeslo';
        } elseif ($this->data['stav'] == ROZPRACOVANE) {
            $stav = $stavArr[ROZPRACOVANE];
            $stavClass = 'rozpracHeslo';
        } elseif ($this->data['stav'] == DOKONCENE) {
            $stav = $stavArr[DOKONCENE];
            $stavClass = 'dokoncHeslo';
        } else {
            $stav = '';
            $stavClass = '';
        }
        if ($corrStav['class']) {
            $stavClass .= ' is_corrected';
        }
        if ($archive) {
            $pidBlock = '
            <input type="text" value="' . $this->data['pidvar'] . '" size="58" readonly="readonly" />
            ';
        } else {
            $pidBlock = '
                <input id="parent_heslo" type="text" value="' . $this->data['pidvar'] . '" class="ac_heslo_field" size="58"
                    onBlur="save_input($(\'#parent_heslo-hid\'), \'' . $this->domId . '--pid\')" />
                <input id="parent_heslo-hid" type="hidden" value="' . $this->data['pid'] . '" />
            ';
        }
        $heslarArr = dibi::fetchPairs('SELECT Id, Label FROM heslare ORDER BY sortorder ASC');
        $html = '
            <div class="orangeBox">
                <table class="w100">
                    <tr>
                        <td style="width: 65%">
                            <h1 class="floatLeft">' . $this->getHlavniVarianta(True) . '</h1>
                            <div class="floatLeft" style="margin-left: 60px;">
        ';
        if ($this->prevHid) {
            if ($archive) {
                $html .= '<a href="formular_hesla_new.php?hid=' . $this->mainHid . '&amp;archid=' . $this->prevHid . '" title="předchozí heslo"><img src="css/images/prev.png" alt="&lt;&lt;"/></a>';
            } else {
                $html .= '<a href="formular_hesla_new.php?hid=' . $this->prevHid . '" title="předchozí heslo"><img src="css/images/prev.png" alt="&lt;&lt;"/></a>';
            }
        }
        if ($this->nextHid) {
            if ($archive) {
                $html .= '<a href="formular_hesla_new.php?hid=' . $this->mainHid . '&amp;archid=' . $this->nextHid . '" title="další heslo"><img src="css/images/next.png" alt="&gt;&gt;"/></a>';
            } else {
                $html .= '<a href="formular_hesla_new.php?hid=' . $this->nextHid . '" title="další heslo"><img src="css/images/next.png" alt="&gt;&gt;"/></a>';
            }
        }
        $html .= '
                            </div>
                            <div class="clear"></div>
                            <div style="margin-bottom: 20px;">
                                <span class="label">Toto heslo podřadit (jako variantu frazému) pod: </span>
                                <br />
                                ' . $pidBlock . '
                            </div>
                            <div class="floatLeft correctableFieldContainer" id="' . $this->domId . '--do_vystupu---helperIdCpy">
                                <div class="floatLeft label">výstup: </div>
                                ' . $this->printRadioGroup($this->domId . '--do_vystupu', $archive, $vystupArr, $this->data['vystup'], False) . '
                            </div>
                            <div class="floatLeft correctableFieldContainer" id="' . $this->domId . '--heslar---helperIdCpy">
                                <div class="floatLeft label">typ hesláře: </div>
                                ' . $this->printRadioGroup($this->domId . '--heslar', $archive, $heslarArr, $this->data['heslar_id'], False) . '
                            </div>
                        </td>
                        <td class="lightBG padd smallFont flatBox" style="width: 35%">
                            <div class="floatLeft">
                                <div>
                                    Zpracovatel:&nbsp;' . $this->data['zpracovatele']->getValsList(0, ', ') . '
                                 </div>
                                 <div id="' . $stavDom . '---helperIdCpy" class="correctableFieldContainer" style="position: relative;">
                                    ' . $corrStav['flag'] . '
                                    Stav: <span id="stav" class="' . $corrStav['class'] . '">' . $stav . '</span>
                                 </div>
                                 <div>Vytvořeno: ' . $this->data['cts']->format('j. n. Y') . '</div>
                                 <div>Změněno:   ' . $this->data['uts']->format('j. n. Y') . '</div>
                            </div>
                            ' . $this->data['semafor']->printForm($archive) . '
                        </td>
                    </tr>
                </table>
                <div class="hspace"></div>
                <div>
                    <label>lexikografická poznámka:</label> <br />
                    ' . $this->printTextarea($this->domId . '--lexikograficka_poznamka', $archive, $this->data['lexgraf_pozn'], array('lightBG', 'w100fix', 'padd', 'flatBox', 'autosize')) . '
                </div>
                <div class="pozn_odb_wrap">
                    <div class="pozn_odb_show clickable label"><img src="css/images/roll_out.png" alt="Rozbalit" />poznámka pro odborníka:</div>
                    <div class="pozn_odb_block" style="display:none;">
                        <label>poznámka pro odborníka:</label>
                        ' . $this->printTextarea($this->domId . '--poznamka_odbornik', $archive, $this->data['pozn_odbornik'], array('lightBG', 'w100fix', 'padd', 'flatBox', 'autosize')) . '
                        <div class="pozn_odb_hide label clickable"><img src="css/images/roll_in.png" alt="Skrýt" />Skrýt</div>
                    </div>
                </div>
                <div class="hspace"></div>
                <div class="label">zpracovatel:</div>
                <div class="floatLeft" style="width:360px">
                    ' . $this->data['zpracovatele']->printForm($archive) . '
                </div>
        ';
        if (!$archive) {
            $html .= '
               <div>
                   <div class="redukce">
                        <label>Redukovat duplicity v PDF náhledu</label>
                        ' . $this->printCheckbox($this->domId . '--redukce', $archive, $this->data['redukce']) . '
                    </div>
                    <a href="archiv.php?hid=' . $this->hid . '" class="archive_view_icon" target="_blank" title="Zobrazit archiv verzí">&nbsp;</a>
                    <a class="archive_save_icon" data-hid="' . $this->hid . '" title="Uložit tuto verzi do archivu" onClick="toArchive()">&nbsp;</a>
                    <span class="live_butt" title="Zobrazit/skrýt živý náhled">&nbsp;</span>
                    <a href="xml_export.php?hid=' . $this->hid . '&amp;action=xml" class="xmlExport_butt" target="_blank" title="XML výstup">&nbsp;</a>
                    <a href="tisk_hesel_new.php?hid=' . $this->hid . '&amp;page=heslo_varianty&amp;action=preview" class="preview_butt" target="_blank" title="Tiskový výstup (Word)">&nbsp;</a>
                    <a href="tisk_hesel_new.php?hid=' . $this->hid . '&amp;page=heslo_varianty&amp;action=export" id="pdf_export" target="_blank" class="exportHesla_butt" title="Tiskový výstup (PDF)">&nbsp;</a>
                    <a href="tisk_hesel_new.php?hid=' . $this->hid . '&amp;page=heslo_varianty" target="_blank" class="nahledHesla_butt" title="Tiskový náhled (HTML) s redakčním modulem">&nbsp;</a>
                    <a href="korektury_historie.php?hid=' . $this->hid . '" class="history_butt" title="Historie korektur hesla" target="_blank">&nbsp;</a>
                    <a href="heslar.php" class="heslar_butt" title="Heslář">&nbsp;</a>
                    <div class="qSearch_butt clickable" title="Rychlé vyhledávání"></div>
               </div>
            ';
        } elseif ($this->input == CDOin_ARCHIVE) {
            $html .= '
                <div>
                    <button type="button" onclick="fromArchiveFull()">Nahrát</button>
                </div>
            ';
        }

        $html .= '
            <div class="clear"></div>
            </div>
        ';

        return $html;
    }


    /**
     * Vygeneruje editor pro zadani posledni informace zahlavi
     * @param bool $archive
     * @return string
     */
    private function printKonecZahlavi($archive = False)
    {
        $html = '
                <div class="label">poslední místo záhlaví:</div>
                <div class="orangeBox" style="height: 50px;">
                   <div class="floatLeft multiInput" style="min-width:150px;">
                       ' . $this->data['konec_zahlavi']->printForm($archive) . '
                   </div>
                </div>
       ';
        return $html;
    }


    /**
     * Vytiskne plovouci hlavicku.
     * @param bool $archive
     * @return string
     */
    private function printStickyHeader($archive = False)
    {
        $html = '
            <script type="text/javascript">
            $(function(){
                // Check the initial Poistion of the Sticky Header
                var stickyHeaderTop = $("#stickyheader").offset().top - 40;
                var stickyHeaderW   = $("#stickyheader").parent().width() - 40; //40px = 2*(borderWidth+padding)
                $("#stickyheader").css({position: "static", top: "0px", display: "none"});
                $(window).scroll(function(){
                    if( $(window).scrollTop() > stickyHeaderTop ) {
                        $("#stickyheader").css({position: "fixed", top: "0px", display: "block", visibility: "visible", width: stickyHeaderW+"px"});
                    } else {
                        $("#stickyheader").css({position: "static", top: "0px", display: "none"});
                    }
                });
            });
            </script>
            <div id="stickyheader" class="orangeBox">
                <table style="width: 100%">
                <tr>
                    <td>
                        <h1 class="floatLeft">' . $this->getHlavniVarianta(True) . '</h1>   <br/>
                        <div class="floatLeft" style="margin-left: 40px;">
                          <a href="formular_hesla_new.php?hid=' . $this->prevHid . '" title="předchozí frazém"><img src="css/images/prev.png" alt="&lt;&lt;"/></a>
                          <a href="formular_hesla_new.php?hid=' . $this->nextHid . '" title="další frazém"><img src="css/images/next.png" alt="&gt;&gt;" /></a>
                        </div>
                    </td>
                    <td style="text-align: right;">
        ';
        if (!$archive) {
            $html .= '
                <input type="submit" value="uložit" class="blueButtBox buttBox floatRight" />
                <!--<span class="blueButtBox buttBox floatRight" id="preview_butt" />Preview</span>-->
                <a href="archiv.php?hid=' . $this->hid . '" class="archive_view_icon" target="_blank" title="Zobrazit archiv verzí">&nbsp;</a>
                  <a class="archive_save_icon" data-hid="' . $this->hid . '" title="Uložit tuto verzi do archivu">&nbsp;</a>
                <span class="live_butt" title="Zobrazit/skrýt živý náhled">&nbsp;</span>
                <a href="xml_export.php?hid=' . $this->hid . '&amp;action=xml" class="xmlExport_butt" target="_blank" title="XML výstup">&nbsp;</a>
                <a href="tisk_hesel_new.php?hid=' . $this->hid . '&amp;page=heslo_varianty&amp;action=preview" class="preview_butt" target="_blank" title="Tiskový výstup (Word)">&nbsp;</a>
                <a href="tisk_hesel_new.php?hid=' . $this->hid . '&amp;page=heslo_varianty&amp;action=export" target="_blank" class="exportHesla_butt" title="Tiskový výstup (PDF)">&nbsp;</a>
                <a href="tisk_hesel_new.php?hid=' . $this->hid . '&amp;page=heslo_varianty" target="_blank" class="nahledHesla_butt" title="Tiskový náhled (HTML) s redakčním modulem">&nbsp;</a>
                <a href="heslar.php" class="heslar_butt" title="Návrat do hesláře">&nbsp;</a>
                <div class="qSearch_butt clickable" title="Rychlé vyhledávání"></div>
            ';
        }
        $html .= '
                </td>
            </tr>
            </table>
        </div>
        ';
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<lemma id="' . $this->hid . '">';
        foreach ($this->colsStructured as $col) {
            $xml .= $this->data->{$col}->printXML();
        }
        $xml .= '</lemma>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        $sqlData = array();
        foreach ($this->colsSimple as $col) {
            $sqlData[$col] = $this->data[$col];
        }
        $sqlData['uts'] = $this->data['uts'];
        $sqlData['created_by'] = $this->data['created_by'];
        $sqlData['updated_by'] = $this->data['updated_by'];
        $sqlData['main_heslo'] = $this->hid;
        $sqlData['redukce'] = intval($this->data['redukce']);
        $tmp = $this->archiveInsert($this->tbl, $atbl, $sqlData);
        if ($tmp->status !== OK) {
            return $tmp;
        }
        $dbid = $tmp->data;
        dibi::query('INSERT INTO `version`', array('id' => $dbid, 'uid' => get_logged_user(), 'user_save' => 1, 'comment' => $archiveHid));

        foreach ($this->colsStructured as $col) {
            $this->data[$col]->saveToArchive($dbid, $dbid);
        }
        return new Response(OK);
    }

    public function kopieHeslaPopup()
    {
        $html = '
            <form id="kopieHesla_form" method="post" action="formular_hesla_save.php">
                <div>
                    <input type="text" name="heslo" value="" maxlength="255" />
                    <input type="hidden" name="hid" value="' . $this->hid . '" />
                    <input type="hidden" name="heslo_copy" value="1" />
                </div>
            </form>
        ';
        return $html;
    }

    /**
     * Pridani vyresene nebo zamitnute korektury.
     * @param $data
     * @return Response
     */
    public function addResolvedCorrection($data)
    {
        $this->resolvedCorrections[] = $data;
        return new Response(OK, $this->resolvedCorrections);
    }

    /**
     * Getter pro resolvedCorrections.
     * @return array
     */
    public function getResolvedCorrections()
    {
        return $this->resolvedCorrections;
    }

    /**
     * Ulozi stavy vyresenych / zamitnutych korektur.
     */
    public function saveResolvedCorrections()
    {
        require_once(dirname(__DIR__) . '/redakce.class.php');
        $RED = new Redakce();
        foreach ($this->resolvedCorrections as $r) {
            $resp = $RED->set_status($r['act'], $r['kor_id'], $r['note']);
        }
        $this->resolvedCorrections = array();
        unset($RED);
    }

    /**
     * Kontrola hlavni varianty hesla.
     * @return Response
     */
    public function checkHlavniVarianta()
    {
        $vkeys = array_keys($this->data['varianta']->data);
        $var = trim($this->data['varianta']->data[$vkeys[0]]['varianta']);
        $f_var = $this->data['varianta']->data[$vkeys[0]]['f_varianta'];
        if (strlen($var) > 0 && $f_var == 1) {
            return new Response(OK);
        } else {
            return new Response(NOT_OK, array(), '1. varianta nesmí být podheslí a zároveň musí mít vyplněné pole "heslo (var.)")');
        }
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $headers = array('Heslo');
        $data = array(
            array($this->getHlavniVarianta(True))
        );

        foreach ($this->colsStructured as $col) {
            if (isset($fields[$col])) {
                $response = $this->data->{$col}->tabExport($data, $fields[$col], $wrap);
                $headers = array_merge($headers, $response->data['headers']);
            }
        }
        return new Response(OK, array('headers' => $headers, 'data' => $data));
    }

    public function getSQL($fields = False, $parentTable = False)
    {
        $return = array(
            'columns' => array('[heslo].[id] AS [' . $this->alias . '--id]'),
        );
        foreach ($this->colsSimple as $c) {
            if ($fields === False || isset($fields[$c])) {
                if ($c == 'vystup') {
                    $return['columns'][] = '[' . $this->tbl . '].[' . $c . '] AS [' . $this->alias . '--vystup_hesla]';
                } else {
                    $return['columns'][] = '[' . $this->tbl . '].[' . $c . '] AS [' . $this->alias . '--' . $c . ']';
                }
            }
        }
        $childTbl = array();

        if ($fields === False || isset($fields['varianta'])) {
            $f = $fields === False ? False : $fields['varianta'];
            $resp = $this->data['varianta']->getSQL($f, $this->tbl);
            $return['columns'] = array_merge($return['columns'], $resp->data['columns']);
            $childTbl[] = $resp->data['tables'];
        }
        if ($fields === False || isset($fields['vyznam'])) {
            $f = $fields === False ? False : $fields['vyznam'];
            $resp = $this->data['vyznam']->getSQL($f, $this->tbl);
            $return['columns'] = array_merge($return['columns'], $resp->data['columns']);
            $childTbl[] = $resp->data['tables'];
        }
        if ($fields === False || isset($fields['ext_pozn'])) {
            $f = $fields === False ? False : $fields['ext_pozn'];
            $resp = $this->data['ext_pozn']->getSQL($f, $this->tbl);
            $return['columns'] = array_merge($return['columns'], $resp->data['columns']);
            $childTbl[] = $resp->data['tables'];
        }
        $return['tables'] = array(
            '[' . $this->tbl . ']' => array(
                'main' => array(),
                'child' => $childTbl
            )
        );

        return new Response(OK, $return);
    }

    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        $rows = array();
        $i = 0;
        foreach ($data as $did => $entry) {
            //$did == 'heslo-6508' např.
            $hid = explode('-', $did)[1];
            $rows[$i]['heslo'] = getHlavniVarianta($hid, True);
            if ($parentStr) {
                $str = $parentStr . '--' . $did;
            } else {
                $str = $did;
            }
            foreach ($entry as $col => $val) {
                if (get_parent_class($this->data[$col]) == 'CDO_Base') {
                    /*
                     * význam, varianta, extpozn, ...
                     */
                    $tmp = $this->data[$col]->tabFormat($data[$did][$col], $fields[$col], $str);
                    if (get_class($this->data[$col]) == 'CDO_Varianta' || get_class($this->data[$col]) == 'CDO_Vyznam') {
                        $rows[$i][$col] = $tmp[0];
                        $lnt = count($tmp);
                        for ($j = 1; $j < $lnt; $j++) {
                            $rows[$i + $j][$col] = $tmp[$j];
                        }
                    } else {
                        $rows[$i][$col] = $tmp;
                    }
                } else {
                    if ($col == 'id') continue;
                    if ($col == 'vystup_hesla') {
                        $val = getVystup($val);
                    }
                    $rows[$i][$col] = $val;
                }
            }
            $i = count($rows);
        }
        return $rows;
    }

    /**
     * Getter pro semafor
     * @return CDO_Semafor
     */
    public function getSemafor()
    {
        if (isset($this->data['semafor'])) {
            return $this->data['semafor'];
        } else {
            $this->data['semafor'] = new CDO_Semafor(CDOin_DB, $this->hid, $this->domId, $this->data['created_by']);
            return $this->data['semafor'];
        }
    }
}