<?php
/**
 * Soubor link.php obsahujici definici tridy CDO_Link.
 */
/** TODO: dodelat cisla vyznamu */

/**
 * Trida pro skupinu odkazu.
 */
class CDO_Link extends CDO_Base
{
    /**
     * @var int $parent typ nadrazeneho zaznamu
     */
    private $parent;
    /**
     * @var int $parentOrder poradove cislo rodice elementu
     */
    private $parentOrder;
    /**
     * @var int $type cislo typu linku
     */
    private $type;
    /**
     * @var int $type cislo typu linku
     */
    private $typeString;
    /**
     * @var bool $master prepinani master/slave
     */
    private $master;

    /** @var array $localDomId */
    private $localDomId = array();

    /** @var string $localDomStr */
    private $localDomStr = '';

    /**
     * Konstruktor tridy CDO_Link.
     * @param int $input
     * @param int $type
     * @param bool $master
     * @param bool $parentVal
     * @param bool $parentDomId
     * @param int $hid
     * @param bool $parent
     * @param bool $parentOrder
     */
    function __construct($input = CDOin_DB, $type, $master, $parentVal = False, $parentDomId = False, $hid = 0, $parent = False, $parentOrder = False)
    {
        $this->input = $input;
        $this->parentDomId = $parentDomId;
        $this->parentVal = $parentVal;
        $this->hid = $hid;
        $this->parent = $parent;
        $this->parentOrder = $parentOrder;
        $this->xmlWrapper = 'links';
        $this->master = $master;
        switch ($type) {
            case HREL_TYPE_syn:
                $this->type = [HREL_TYPE_syn, HREL_TYPE_syn_przn];
                $this->typeString = "syn";
                $this->localDomStr = $this->master ? 'synonyma_main' : 'synonyma_slave';
                break;
            case HREL_TYPE_ant:
                $this->type = [HREL_TYPE_ant, HREL_TYPE_ant_przn];
                $this->typeString = "ant";
                $this->localDomStr = $this->master ? 'antonyma_main' : 'antonyma_slave';
                break;
            case HREL_TYPE_odkazy:
                $this->type = [HREL_TYPE_odkazy];
                $this->typeString = "odk";
                $this->localDomStr = $this->master ? 'odk_main' : 'odk_slave';
                break;
            case HREL_TYPE_pripojovani:
                $this->type = [HREL_TYPE_pripojovani, HREL_TYPE_prihnizdovani];
                $this->typeString = "prip";
                $this->localDomStr = $this->master ? 'propojeni' : 'propojeni_s';
                break;
        }
        $this->alias = $this->parent . '-' . $this->localDomStr;
        $this->localDomStr = $this->parentDomId . '--' . $this->localDomStr;
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        }
    }

    public function hasData()
    {
        return !empty($this->data);
    }

    public function add()
    {

    }

    public function addSlavePlace($id, $place, $place_id)
    {
        $newId = $this->newId();
        $this->data[$id]->slave_places[$newId] = new DibiRow(array(
            'id' => $newId,
            'place' => $place,
            'place_id' => $place_id
        ));
    }

    public function removeSlavePlace($id, $slplace_id)
    {
        unset($this->data[$id]->slave_places[$slplace_id]);
    }

    public function updateSlavePlace($id, $slplace_id, $place, $place_id)
    {
        $this->data[$id]->slave_places[$slplace_id]['place'] = $place;
        $this->data[$id]->slave_places[$slplace_id]['place_id'] = $place_id;
    }

    public function addMainPlace($id, $place, $place_id)
    {
        $newId = $this->newId();
        $this->data[$id]->main_places[$newId] = new DibiRow(array(
            'id' => $newId,
            'place' => $place,
            'place_id' => $place_id
        ));
    }

    public function removeMainPlace($id, $mplace_id)
    {
        unset($this->data[$id]->main_places[$mplace_id]);
    }

    public function updateMainPlace($id, $mplace_id, $place, $place_id)
    {
        $this->data[$id]->main_places[$mplace_id]['place'] = $place;
        $this->data[$id]->main_places[$mplace_id]['place_id'] = $place_id;
    }

    function CDO_print_hid_rel_ant_syn_main($multiContainerDOMid, $hidRel_id=0, $rel_data=false, $type=NULL)
    {
        $rnd = rand(0, 10000);
        if ($hidRel_id == 0) $hidRel_id = "new$rnd";
        $main_heslo_str = "";
        $places2sel = array();
        $slavePlaces2sel = array();
        $form_name = "hidRel[$hidRel_id]";

        if (!empty($rel_data['slave_places'])) {
            foreach ($rel_data['slave_places'] as $id => $data) {
                $place_slave_str = $data['place'];
                $place_slave_str .= "-" . $data['place_id'];
                $slavePlaces2sel[$id] = $place_slave_str;
            }
        }

        $corr_hidRel_id = $this->getCorrectionId($hidRel_id);
        $corr = $this->displayCorrection($this->corr[$corr_hidRel_id]['field_DOMid'], $this->corr[$corr_hidRel_id]['id'], $this->corr[$corr_hidRel_id]['field'], $this->corr[$corr_hidRel_id]['nova']);

        //if(is_numeric($hidRel_id) && $hidRel_id > 0)
        //{
        if (!empty($rel_data['komentar'])) $slave_heslo_kom = text_html_preprocess($rel_data['komentar']);
        if (!empty($rel_data['main_hid'])) {
            $hl_var_id = get_hlavni_varianta_hesla($rel_data['main_hid']);
            $hl_var_str = print_varianta_str($hl_var_id);
            $main_heslo_str = text_html_preprocess($hl_var_str);
        } else {
            if (!empty($rel_data['main_str'])) $main_heslo_str = text_html_preprocess($rel_data['main_str']);
        }
        $slave_hid = $this->hid;


        if (!empty($rel_data['main_places'])) {
            foreach ($rel_data['main_places'] as $id => $data) {
                $place_str = $data['place'];
                $place_str .= "-" . $data['place_id'];
                $places2sel[$id] = $place_str;
            }
        }
        $rel_type = $rel_data['rel_type'];
        if (empty($rel_data['rel_type'])) $rel_type = $this->type[0];
        //}
        if(empty($type)){
            if (strpos(array_values($places2sel)[0],'ant') !== false)
            {
                $rel_name='ant';
            }
            else
            {
                $rel_name='syn';
            }
        }else{
            $rel_name=$type;
        }
        $multiHtml = '';
        $multiHtml .= '<table class="multiInput hidRel floatLeft clear correctableFieldContainer" id="' . $multiContainerDOMid . '-' . $hidRel_id . '" ><tr>';
        $multiHtml .= '<th>' . remFields_butt_ajax() . '</th>';

        $multiHtml .= '<td style="display:none">
                          <i style="font-size: 12px">místo připojeni v tomto hesle \'' . get_db_values('varianty', 'varianta', get_hlavni_varianta_hesla($slave_hid)) . '\'</i><br />
                          ' . print_hidRel_placeSelector_ant_syn($slave_hid, $multiContainerDOMid, $form_name, $slavePlaces2sel, true, true, $corr["class"]) . '
                     </td>';
        $multiHtml .= '<td>';
        $multiHtml .= $rel_name . '.:<input id="' . $multiContainerDOMid . '-' . $hidRel_id . '--main_str"  type="text"   name="' . $form_name . '[main_str]" value="' . $main_heslo_str . '" class="textField ac_heslo_field loadHidRelPlaces_ant_syn  ' . $corr["class"] . '" data-hid="' . $slave_hid . '" data-form_id="' . $multiContainerDOMid . '" data-form_name="' . $form_name . '" data-only_vyznamy="1" />&nbsp;';
        $multiHtml .= '<input id="' . $multiContainerDOMid . '-' . $hidRel_id . '--main_str-hid"  type="hidden" name="' . $form_name . '[main_hid]" value="' . $rel_data['main_hid'] . '" />';
        $multiHtml .= '</td><td>kom.:<input id="' . $multiContainerDOMid . '-' . $hidRel_id . '--slave_kom" type="text" name="' . $form_name . '[slave_kom]" value="' . $slave_heslo_kom . '" class="textField slave_kom ' . $corr["class"] . '"/>&nbsp;';
        if (!empty($rel_data['main_hid'])) $multiHtml .= '<br/><a href="formular_hesla_new.php?hid=' . $rel_data['main_hid'] . '" class="label"><u>' . text_html_preprocess($rel_data['main_str']) . '</u>&nbsp;&gt;&gt;</a>';
        else {
            //vytvorit nove heslo - ale este pred tym asi bude treba urobit nejaku kontrolu ci nahodou uz vytvarane heslo neexistuje
        }
        $multiHtml .= '</td><td>';
        $multiHtml .= '<input type="hidden" name="' . $form_name . '[slave_hid]" value="' . $slave_hid . '" />';
        $multiHtml .= '<input type="hidden" class="rel_type" name="' . $form_name . '[rel_type]" value="' . $rel_type . '" class="rel_type"/></td></tr><tr>';
        $multiHtml .= '<th>&nbsp</th><th>' . print_hidRel_placeSelector_ant_syn($rel_data['main_hid'], $multiContainerDOMid, $form_name, $places2sel, true, false, $corr["class"]) . '</th>';
        $multiHtml .= '<th>' . print_typ_syn_ant_propoj_selector($multiContainerDOMid, $form_name, $rel_type, $corr["class"]) . '</th>';

        $multiHtml .= "</tr></table>";
        return $multiHtml;
    }

    function CDO_print_hid_rel_ant_syn_slave($multiContainerDOMid, $hidRel_id = 0, $rel_data = false)
    {
        $rnd = rand(0, 10000);
        $slave_heslo_str = $slave_heslo_hlvar = $slave_heslo_hom = "";
        $places2sel = array();
        $slave_hid = 0;


        if (!empty($rel_data['main_places'])) {
            foreach ($rel_data['main_places'] as $id => $data) {
                $place_str = $data['place'];
                if ($place_str != "heslo") $place_str .= "-" . $data['place_id'];
                $places2sel[$id] = $place_str;
            }
        }
        if (!is_numeric($hidRel_id) || $hidRel_id != 0)//upravit pred nasazenim CDO
        {
            $rel_type = $rel_data['rel_type'];
            $place_str = $this->typeString;
            if (!empty($rel_data['slave_hid'])) {
                $slave_hid = $rel_data['slave_hid'];
                $hl_var_id = get_hlavni_varianta_hesla($rel_data['slave_hid']);
                $hl_var_str = print_varianta_str($hl_var_id);
                $slave_heslo_str = text_html_preprocess($hl_var_str);
            } else {
                if (!empty($rel_data['slave_str'])) $slave_heslo_str = text_html_preprocess($rel_data['slave_str']);
            }
            if (!empty($rel_data['komentar'])) $slave_heslo_kom = text_html_preprocess($rel_data['komentar']);
            //miesto vo VEDLAJSOM HESLE
            if (!empty($rel_data['slave_places'])) {
                foreach ($rel_data['slave_places'] as $id => $data) {
                    $place_slave_str = $data['place'];
                    if ($place_slave_str != "heslo") $place_slave_str .= "-" . $data['place_id'];
                    $slave_places2sel[$id] = $place_slave_str;
                }
            }
        } else {
            $rel_type = $this->type[0];
            $hidRel_id = "new$rnd";
            $place_str = $this->typeString;
        }

        $form_name = "hidRel[$hidRel_id]";
        $multiHtml = "\r\n";
        $multiHtml .= "\r\n<table class='multiInput hidRel floatLeft clear' id='$multiContainerDOMid-$hidRel_id'><tr>";
        $multiHtml .= "\r\n<th>" . remFields_butt_ajax() . "</th>";
        $selector_vyznamov = true;
        $multiHtml .= "\r\n<td  style='display:none' >" . print_hidRel_placeSelector_ant_syn($rel_data["main_hid"], $multiContainerDOMid, $form_name, $places2sel, $selector_vyznamov) . "</td>";

        $corr_hidRel_id = $this->getCorrectionId($hidRel_id);
        $corr = $this->displayCorrection($this->corr[$corr_hidRel_id]['field_DOMid'], $this->corr[$corr_hidRel_id]['id'], $this->corr[$corr_hidRel_id]['field'], $this->corr[$corr_hidRel_id]['nova']);

        $multiHtml .= "\r\n<td>";
        $tmp_slave_heslo_str = ($slave_heslo_str);
        if ($tmp_slave_heslo_str != false) {
            $slave_heslo_str = $tmp_slave_heslo_str;
        }
        $multiHtml .= explode("-", $place_str)[0] . ".:<span id='odkaz-synonymum-$hidRel_id--tvar' class='correctableFieldContainer'><input type='text' id='$multiContainerDOMid-$hidRel_id--slave_str' name='" . $form_name . "[slave_str]' value='$slave_heslo_str' class='textField ac_heslo_field loadHidRelSlavePlaces_ant_syn " . $corr["class"] . "' data-hid='$rel_data->main_hid' data-form_id='$multiContainerDOMid' data-form_name='$form_name' data-only_vyznamy='1' /></span>&nbsp;
     <input id='$multiContainerDOMid-$hidRel_id--slave_str-hid' type='hidden' name='" . $form_name . "[slave_hid]' value='$slave_hid' />";
        $multiHtml .= "</td>\r\n<td>kom.:<span id='odkaz-synonymum-$hidRel_id--komentar' class='correctableFieldContainer'><input type='text' id='$multiContainerDOMid-$hidRel_id--slave_kom' name='" . $form_name . "[slave_kom]' value='$slave_heslo_kom' class='textField slave_kom " . $corr["class"] . "'/></span>&nbsp;";
        $multiHtml .= "</td>";
        $multiHtml .= "\r\n<td style='vertical-align: bottom'><table cellspacing='0' class='small_updown_arrows floatLeft'><tr><td class='up clickable' onClick='javascript: return JS_moveUp(this, \"multiInput\")'></td></tr><tr><td class='down clickable' onClick='javascript: return JS_moveDown(this, \"multiInput\")'></td></tr></tbody></table></td>";
        $multiHtml .= "</tr>\r\n<tr>";
        $slave_place_selector = true;
        $multiHtml .= " \r\n<th>&nbsp;</th>
                       \r\n<td>místo:
                         " . print_hidRel_placeSelector_ant_syn($slave_hid, $multiContainerDOMid, $form_name, $slave_places2sel, $selector_vyznamov, $slave_place_selector, $corr["class"]) . "
                       </td>";
        $multiHtml .= "\r\n<td>" . print_typ_syn_ant_propoj_selector($multiContainerDOMid, $form_name, $rel_type, $corr["class"]) . "</td>";
        $multiHtml .= "<input type='hidden' name='" . $form_name . "[main_hid]' value='" . $rel_data["main_hid"] . "' />";
        $multiHtml .= "<input type='hidden' class='rel_type' name='" . $form_name . "[rel_type]' value='$rel_type' />";
        $multiHtml .= "\r\n</tr>\r\n</table>";

        return $multiHtml;
    }

    public function printPopup()
    {
        $multiContainerDOMid = $this->parentDomId . '--hidRel-' . $this->typeString . '-' . (($this->master) ? 'main' : 'slave') . '--' . $this->hid . '--' . $this->parent;
        $html = '';
        $html .= $this->master ? '' : '<table class="w100 AZsortOuter"><tr><td></td><td class="AZsort clickable"></td></tr></table>';

        $html.='<div id="'.$multiContainerDOMid.'" class="multiContainer floatLeft ">';
        if(empty($this->data))
        {
            $html.=$this->master?$this->CDO_print_hid_rel_ant_syn_slave($multiContainerDOMid,0,0):$this->CDO_print_hid_rel_ant_syn_main($multiContainerDOMid,0,0,$this->typeString);
        }
        else {
            foreach ($this->data as $hidRel_id=>$rel_data) {
                $html .= $this->master?$this->CDO_print_hid_rel_ant_syn_slave($multiContainerDOMid,$hidRel_id,$rel_data):$this->CDO_print_hid_rel_ant_syn_main($multiContainerDOMid,$hidRel_id,$rel_data);
            }
        }
        $html .= "</div><div class='tlacitkoRemField'>";
        if ($this->master) {
            $html .= addFields_butt_ajax('hidRel', $multiContainerDOMid, array('hidRel_type' => 'CDO_ant_syn_slave', 'main_hid' => $this->hid, 'place' => $this->typeString . "onymum", 'place_id' => $this->parentVal));
        } else {
            $html .= addFields_butt_ajax('hidRel', $multiContainerDOMid, array('hidRel_type' => 'CDO_ant_syn_main', 'slave_hid' => $this->hid, 'place' => $this->typeString . "onymum", 'place_id' => $this->parentVal));
        }
        $html .= "</div>";
        return $html;
    }

    public function print_hid_rel_odkazy_mains($hid)
    {
        $multiContainerDOMid = "hidRel-odkazy-mains--$hid";
        $html = "";
        $html .= "<div id='$multiContainerDOMid' class='multiContainer floatLeft'>";

        if (empty($this->data)) {
            $html .= CDO_print_hid_rel_odkazy_main($multiContainerDOMid, $hid, 0, 0);
        } else {
            foreach ($this->data as $hidRel_id => $data) {
                $html .= CDO_print_hid_rel_odkazy_main($multiContainerDOMid, $hid, $hidRel_id, $data);
            }
        }

        $html .= "</div>";
        $html .= "<div class='tlacitkoRemField'>";
        $html .= addFields_butt_ajax('hidRel', $multiContainerDOMid, array('hidRel_type' => 'CDO_odkaz_main', 'slave_hid' => $hid));
        $html .= "</div>";
        return $html;
    }

    public function print_hid_rel_odkazy_slaves($hid)
    {
        $multiContainerDOMid = "hidRel-odkazy-slaves--$hid";
        $html = "";
        $html .= "<table class='w100 AZsortOuter'><tr><td></td><td class='AZsort clickable'></td></tr></table>";
        $html .= "<div id='$multiContainerDOMid' class='multiContainer floatLeft'>";
        if (empty($this->data)) {
            $html .= CDO_print_hid_rel_odkazy_slave($multiContainerDOMid, $hid, 0, 0);
        } else {
            foreach ($this->data as $hidRel_id => $data) {
                $html .= CDO_print_hid_rel_odkazy_slave($multiContainerDOMid, $hid, $hidRel_id, $data, false, $this);
            }
        }

        $html .= "</div>";
        $html .= "<div class='tlacitkoRemField'>";
        $html .= addFields_butt_ajax('hidRel', $multiContainerDOMid, array('hidRel_type' => 'CDO_odkaz_slave', 'main_hid' => $hid));
        $html .= "</div>";
        return $html;
    }

    public function print_hid_rel_pripojeni_mains($slave_hid, $archive = false)
    {
        $multiContainerDOMid = "hidRel-propojeni-mains";
        $html = "";
        $html .= "<div id='$multiContainerDOMid' class='multiContainer'>";
        $master = true;
        if (empty($this->data)) {
            $html .= CDO_print_hid_rel_pripojeni_main($multiContainerDOMid, $slave_hid, 0);
        } else {
            foreach ($this->data as $hidRel_id => $data) {
                $html .= CDO_print_hid_rel_pripojeni_main($multiContainerDOMid, $slave_hid, $hidRel_id, $data, $archive);
            }
        }
        $html .= "</div>";
        $html .= "<div class='tlacitkoRemField'>";
        $html .= addFields_butt_ajax('hidRel', $multiContainerDOMid, array('hidRel_type' => 'CDO_prilink_slave', 'slave_hid' => $slave_hid));
        $html .= "</div>";
        return $html;
    }

    public function print_hid_rel_pripojeni_slaves($main_hid, $place, $place_id)
    {
        $multiContainerDOMid = "hidRel-pripojeni-slaves--$place-$place_id";
        $html = "";
        $html .= "<table class='w100 AZsortOuter'><tr><td></td><td class='AZsort clickable'></td><tr></table>";
        $html .= "<div id='$multiContainerDOMid' class='multiContainer floatLeft'>";
        if (empty($this->data)) {
            $html .= CDO_print_hid_rel_pripojeni_slave($multiContainerDOMid, $main_hid, $place, $place_id, 0);
        } else {
            foreach ($this->data as $id => $hidRel_id) {
                $html .= CDO_print_hid_rel_pripojeni_slave($multiContainerDOMid, $main_hid, $place, $place_id, $id, $hidRel_id);
            }
        }
        $html .= "</div>";
        $html .= "<div class='tlacitkoRemField'>";
        $html .= addFields_butt_ajax('hidRel', $multiContainerDOMid, array('hidRel_type' => 'CDO_prilink_main', 'main_hid' => $main_hid, 'place' => $place, 'place_id' => $place_id));
        $html .= "</div>";
        return $html;
    }

    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            if ($this->master) {
                if ($this->parent == "vyznam") {
                    $this->data = dibi::query('
                    SELECT DISTINCT hr.id as id, hr.spojeno as spojeno, hr.rel_type, hr.main_hid, hr.main_str, hr.slave_hid, hr.slave_str, hr.sortorder, hr.sortorder2, hr.komentar, hr.poznamka
                    FROM %n hr LEFT JOIN %n hrp ON hr.id=hrp.hesla_rel_id
                    LEFT JOIN %n hrps ON hr.id = hrps.hesla_rel_id
                    LEFT JOIN %n hrl ON hr.id = hrl.hesla_rel_id
                    WHERE hr.main_hid = %i AND hr.rel_type IN %in AND (hrp.place = "syn" OR hrp.place = "ant" OR hrp.place = "vyznam") AND hrp.place_id=%i ORDER BY sortorder ASC
                    ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_places', $prefix . 'hesla_rel_slave_places', $prefix . 'hesla_rel_link_type_vyber', $this->hid, $this->type, $this->parentVal)->fetchAssoc('id');
                } elseif ($this->typeString == "odk") {
                    $this->data = dibi::query('
                    SELECT DISTINCT hr.id as id, hr.spojeno as spojeno, hr.rel_type, hr.main_hid, hr.main_str, hr.slave_hid, hr.slave_str, hr.sortorder, hr.sortorder2, hr.komentar, hr.poznamka
                    FROM %n hr LEFT JOIN %n hrp ON hr.id=hrp.hesla_rel_id
                    LEFT JOIN %n hrps ON hr.id = hrps.hesla_rel_id
                    LEFT JOIN %n hrl ON hr.id = hrl.hesla_rel_id
                    WHERE hr.main_hid = %i AND hr.rel_type IN %in ORDER BY sortorder ASC
                    ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_places', $prefix . 'hesla_rel_slave_places', $prefix . 'hesla_rel_link_type_vyber', $this->hid, $this->type)->fetchAssoc('id');
                } else {
                    $this->data = dibi::query('
                    SELECT DISTINCT hr.id as id, hr.spojeno as spojeno, hr.rel_type, hr.main_hid, hr.main_str, hr.slave_hid, hr.slave_str, hr.sortorder, hr.sortorder2, hr.komentar, hr.poznamka
                    FROM %n hr LEFT JOIN %n hrp ON hr.id=hrp.hesla_rel_id
                    LEFT JOIN %n hrps ON hr.id = hrps.hesla_rel_id
                    LEFT JOIN %n hrl ON hr.id = hrl.hesla_rel_id
                    WHERE hr.main_hid = %i AND hr.rel_type IN %in AND hrp.place = %sN AND hrp.place_id=%i ORDER BY sortorder ASC
                    ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_places', $prefix . 'hesla_rel_slave_places', $prefix . 'hesla_rel_link_type_vyber', $this->hid, $this->type, $this->parent, $this->parentVal)->fetchAssoc('id');
                }
            } else {
                if ($this->typeString == "odk") {
                    $this->data = dibi::query('
                    SELECT DISTINCT hr.id as id, hr.spojeno as spojeno, hr.rel_type, hr.main_hid, hr.main_str, hr.slave_hid, hr.slave_str, hr.sortorder, hr.sortorder2, hr.komentar, hr.poznamka
                    FROM %n hr LEFT JOIN %n hrp ON hr.id=hrp.hesla_rel_id
                    LEFT JOIN %n hrps ON hr.id = hrps.hesla_rel_id
                    LEFT JOIN %n hrl ON hr.id = hrl.hesla_rel_id
                    WHERE hr.slave_hid = %i AND hr.rel_type IN %in ORDER BY sortorder2 ASC
                    ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_places', $prefix . 'hesla_rel_slave_places', $prefix . 'hesla_rel_link_type_vyber', $this->hid, $this->type)->fetchAssoc('id');
                } else {
                    if ($this->parent == "heslo") {
                        $this->data = dibi::query('
                        SELECT DISTINCT hr.id as id, hr.rel_type, hr.main_hid, hr.main_str, hr.slave_hid, hr.slave_str, hr.sortorder, hr.sortorder2, hr.komentar, hr.poznamka
                        FROM %n hr LEFT JOIN %n hrp ON hr.id=hrp.hesla_rel_id
                        LEFT JOIN %n hrps ON hr.id = hrps.hesla_rel_id
                        LEFT JOIN %n hrl ON hr.id = hrl.hesla_rel_id
                        WHERE hr.slave_hid = %i AND hr.rel_type IN %in AND ((hrps.place = %sN AND hrps.place_id=%i) OR hrps.place is NULL) ORDER BY sortorder2 ASC
                        ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_places', $prefix . 'hesla_rel_slave_places', $prefix . 'hesla_rel_link_type_vyber', $this->hid, $this->type, $this->parent, $this->parentVal)->fetchAssoc('id');
                    } else {
                        $this->data = dibi::query('
                        SELECT DISTINCT hr.id as id, hr.rel_type, hr.main_hid, hr.main_str, hr.slave_hid, hr.slave_str, hr.sortorder, hr.sortorder2, hr.komentar, hr.poznamka
                        FROM %n hr LEFT JOIN %n hrp ON hr.id=hrp.hesla_rel_id
                        LEFT JOIN %n hrps ON hr.id = hrps.hesla_rel_id
                        LEFT JOIN %n hrl ON hr.id = hrl.hesla_rel_id
                        WHERE hr.slave_hid = %i AND hr.rel_type IN %in AND hrps.place = %sN AND hrps.place_id=%i ORDER BY sortorder2 ASC
                        ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_places', $prefix . 'hesla_rel_slave_places', $prefix . 'hesla_rel_link_type_vyber', $this->hid, $this->type, $this->parent, $this->parentVal)->fetchAssoc('id');
                    }
                }
            }
            foreach ($this->data as $id => $d) {
                $this->localDomId[$id] = $this->localDomStr . '-' . $id;
                $d["slave_places"] = dibi::query('
                        SELECT id, hrps.place as place, hrps.place_id as place_id
                        FROM %n hrps
                        WHERE hrps.hesla_rel_id = %i
                        ', $prefix . 'hesla_rel_slave_places', $id)->fetchAssoc('id');
                $d["main_places"] = dibi::query('
                        SELECT id, hrp.place as place, hrp.place_id as place_id
                        FROM %n hrp
                        WHERE hrp.hesla_rel_id = %i
                        ', $prefix . 'hesla_rel_places', $id)->fetchAssoc('id');
                $d["link_types"] = dibi::query('
                        SELECT hrl.id, hrl.cis_id as link_type_id, type_str.link_type as link_type
                        FROM %n hrl JOIN %n type_str ON hrl.cis_id=type_str.id
                        WHERE hrl.hesla_rel_id = %i
                        ', $prefix . 'hesla_rel_link_type_vyber', "cis_hesla_rel_link_type", $id)->fetchAssoc('id');
                if(empty($d["link_types"])) {
                    $d["link_types"] = dibi::query('
                        SELECT hrl.id, "jine" AS link_type, "jine" as link_type_id, jine 
                        FROM %n hrl
                        WHERE hrl.hesla_rel_id = %i
                        ', $prefix . 'hesla_rel_link_type_vyber' , $id)->fetchAssoc('id');
                }
            }
        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }
        return new Response(OK);
    }

    public function updateLink($data)
    {
        $this->data = array();
        foreach ($data as $rel_id => $red_data) {
            $this->data[$rel_id] = array();
            foreach ($red_data as $val_id => $val_data) {
                if (is_string($val_data)) {
                    $this->data[$rel_id][$val_id] = trim($val_data);
                } else {
                    $this->data[$rel_id][$val_id] = $val_data;
                }
            }
        }
    }

    protected function saveToDB_places($archive = False, $typ = null, $rel_id = 0, $newdata = null, $originalData = null, $rel_type = null)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        $db_table = $typ == "slave_place" ? "slave_place" : "place";
        try {
            if (!empty($originalData[$typ . 's'])) {
                $originalDataKeys = array_combine(array_keys($originalData[$typ . 's']), array_keys($originalData[$typ . 's']));
            } else {
                $originalDataKeys = NULL;
            }
            $sqlData = array(
                'hesla_rel_id' => $rel_id
            );
            if ($rel_type == 2 || $rel_type == 3) {
                foreach ($newdata[$typ . 's'] as $id => $d) {
                    $sqlData["place"] = $d["place"];
                    $sqlData['place_id'] = $d['place_id'];
                    if (!empty($originalDataKeys)) {
                        $klic = array_keys($originalDataKeys);
                        dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . 'hesla_rel_' . $db_table . 's', $sqlData, $klic[0]);
                        unset($originalDataKeys[$klic[0]]);
                    } else {
                        dibi::query('
                         INSERT INTO %n
                     ', $prefix . 'hesla_rel_' . $db_table . 's', $sqlData);
                    }
                }
            } else {
                foreach ($newdata[$typ . 's'] as $id => $d) {
                    $sqlData["place"] = $d["place"];
                    if (isset($d['place_id'])) {
                        $sqlData['place_id'] = $d['place_id'];
                    } else {
                        $sqlData['place_id'] = 0;
                    }
                    if (isset($originalDataKeys[$id]) || $id > 0) {
                        dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . 'hesla_rel_' . $db_table . 's', $sqlData, $id);
                        unset($originalDataKeys[$id]);
                    } else {
                        dibi::query('
                         INSERT INTO %n
                     ', $prefix . 'hesla_rel_' . $db_table . 's', $sqlData);
                    }
                }
            }
            if (!empty($originalDataKeys)) {
                foreach ($originalDataKeys as $id => $d) {
                    //delete
                    dibi::query('
                        DELETE FROM %n WHERE id = %i
                    ', $prefix . 'hesla_rel_' . $db_table . 's', $id);
                }
            }
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }
    }

    protected function saveToDB_link_type($archive = False, $rel_id = 0, $newdata = null, $originalData = null)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {

            if (!empty($originalData['link_types'])) {
                $originalDataKeys = array_combine(array_keys($originalData['link_types']), array_keys($originalData['link_types']));
            } else {
                $originalDataKeys = NULL;
            }
            $sqlData = array(
                'hesla_rel_id' => $rel_id
            );
            if (!empty($newdata['link_types'])) {
                foreach ($newdata['link_types'] as $id => $d) {
                    if ($d["link_type_id"] != 'jine') {
                        $sqlData["cis_id"] = $d["link_type_id"];
                    } else {
                        $sqlData['jine'] = $d['jine_text'];
                    }
                    $sqlData['sortorder'] = 1;
                    if (isset($originalDataKeys[$id])) {
                        dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . 'hesla_rel_link_type_vyber', $sqlData, $id);
                        unset($originalDataKeys[$id]);
                    } else {
                        dibi::query('
                         INSERT INTO %n
                     ', $prefix . 'hesla_rel_link_type_vyber', $sqlData);
                    }
                }
            }
            if (!empty($originalDataKeys)) {
                foreach ($originalDataKeys as $id => $d) {
                    //delete
                    dibi::query('
                        DELETE FROM %n WHERE id = %i
                    ', $prefix . 'hesla_rel_link_type_vyber', $id);
                }
            }
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }
    }

    protected function saveToDB($archive = False, $originalData = array())
    {
        $spec_data = array();
        $spec_data[0] = 0;
        /*  var_dump("---data---");
          var_dump($this->data);*/
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }
        /* var_dump("---originalData---");
         var_dump($originalData);*/

        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            dibi::begin();
            $ord = 1;
            $originalDataKeys = array();
            if($originalData){
                $originalDataKeys = array_combine(array_keys($originalData), array_keys($originalData));
            }
            foreach ($this->data as $id => $d) {
                if ($id > 0 || !is_numeric($id)) {
                    $sqlData = array(
                        'rel_type' => $d["rel_type"],
                        'main_hid' => $d["main_hid"] == 0 ? NULL : $d["main_hid"]
                    );
                    if ($d["slave_hid"] > 0) {
                        $sqlData['slave_hid'] = $d["slave_hid"] == 0 ? NULL : $d["slave_hid"];
                    }
                    if (is_numeric($d["spojeno"])) {
                        $sqlData['spojeno'] = $d["spojeno"];
                    } else {
                        $sqlData['spojeno'] = 0;
                    }
                    $sqlData['main_str'] = $d["main_str"];
                    if ($sqlData['main_str'] == false) {
                        $sqlData['main_str'] = $d["main_str"];
                    }
                    $sqlData['slave_str'] = $d["slave_str"];
                    if ($sqlData['slave_str'] == false) {
                        $sqlData['slave_str'] = $d["slave_str"];
                    }
                    $sqlData['komentar'] = $d["komentar"];
                    if ($sqlData['komentar'] == false) {
                        $sqlData['komentar'] = $d["komentar"];
                    }
                    $sqlData['poznamka'] = $d["poznamka"];
                    if ($sqlData['poznamka'] == false) {
                        $sqlData['poznamka'] = $d["poznamka"];
                    }
                    if (!empty($d["sortorder"])) $sqlData["sortorder"] = $d["sortorder"];
                    if (!empty($d["sortorder2"])) $sqlData["sortorder2"] = $d["sortorder2"];
                    /*var_dump("sqlData");
                    var_dump($sqlData);
                    var_dump("d");
                    var_dump($d);*/
                    if (isset($originalDataKeys[$id])) {
                        dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . 'hesla_rel', $sqlData, $id);
                        unset($originalDataKeys[$id]);
                        $this->saveToDB_places($archive, "slave_place", $id, $d, $originalData[$id], $d["rel_type"]);
                        $this->saveToDB_places($archive, "main_place", $id, $d, $originalData[$id], $d["rel_type"]);
                        $this->saveToDB_link_type($archive, $id, $d, $originalData[$id]);
                    } else {
                        dibi::query('
                        INSERT INTO %n
                    ', $prefix . 'hesla_rel', $sqlData);
                        $dbid = $this->newId(True);
                        $spec_data[$dbid] = $d["spojeno"];
                        $this->saveToDB_places($archive, "slave_place", $dbid, $d, NULL, $d["rel_type"]);
                        $this->saveToDB_places($archive, "main_place", $dbid, $d, NULL, $d["rel_type"]);
                        $this->saveToDB_link_type($archive, $dbid, $d, NULL);
                    }
                }
            }
            foreach ($originalDataKeys as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . 'hesla_rel', $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }

        return new Response(OK, $spec_data);
    }

    protected function loadCorrections($archive = False)
    {
        $domList = array();
        $transl = array();
        if (!empty($this->data)) {
            foreach ($this->data as $hidRel_DBid => $odkazy_info) {
                if ($this->master) {
                    switch ($this->type[0]) {
                        case HREL_TYPE_syn:
                            array_push($domList, 'hidRel-synonymum-slaves-' . $hidRel_DBid);
                            break;
                        case HREL_TYPE_ant:
                            array_push($domList, 'hidRel-antonymum-slaves-' . $hidRel_DBid);
                            break;
                        case HREL_TYPE_pripojovani:
                            array_push($domList, 'nest-' . $hidRel_DBid);
                            break;
                        case HREL_TYPE_odkazy:
                            array_push($domList, 'link-' . $hidRel_DBid);
                            break;
                    }
                } else {
                    switch ($this->type[0]) {
                        case HREL_TYPE_syn:
                            array_push($domList, 'hidRel-synonymum-slaves-' . $hidRel_DBid);
                            break;
                        case HREL_TYPE_ant:
                            array_push($domList, 'hidRel-antonymum-slaves-' . $hidRel_DBid);
                            break;
                        case HREL_TYPE_pripojovani:
                            array_push($domList, 'nest-' . $hidRel_DBid);
                            break;
                        case HREL_TYPE_odkazy:
                            array_push($domList, 'link-' . $hidRel_DBid);
                            break;
                    }
                }
            }
            $this->corr = $this->loadCorrectionsByDom($domList, $transl);
        }
    }

    protected function printPripojeni($archive = False)
    {
        $html = '';
        if (explode("-", $this->parentDomId)[0] == "heslo") {
            if (!$this->master) $html .= "<div class='label'>Propojování</div>";
            $html .= '
                 <div class="floatLeft" style="margin-left: 25px">';
            if (!$this->master) $html .= '<div class="label">Toto heslo je připojeno/přihnízdováno k těmto hlavním heslům:</div>';
            $html .= '<div></div>
                   <div class=" hidRel  " >  ';
            foreach ($this->data as $idn => $spojeni) {
                $id = "nest-" . $spojeni["id"];
                if ($archive) {
                    $corr = $this->emptyCorrectionArray();
                } else {
                    $corr = $this->displayCorrection($this->corr[$id]['field_DOMid'], $this->corr[$id]['id'], $this->corr[$id]['field'], $this->corr[$id]['nova']);
                }
                $html .= $this->print_this_hidRel_sign($spojeni) . "&nbsp; ";
                if (!$this->master) {
                    if ($spojeni['main_hid'] > 0) {
                        $disabled_cls = get_disabled_cls($spojeni['main_hid']);
                        $html .= "<span class='correctableFieldContainer " . $corr['class'] . "' id='$id' data-ldomid='" . $this->localDomId[$idn] . "'>" . $corr['flag'] . "<a href='formular_hesla_new.php?hid=" . $spojeni['main_hid'] . "' class='$disabled_cls'>" . print_heslo_str($spojeni['main_hid']) . "</a></span>";
                    } else {
                        $html .= "<span class='correctableFieldContainer " . $corr['class'] . "' id='$id' data-ldomid='" . $this->localDomId[$idn] . "'>" . $corr['flag'] . $spojeni['main_str'] . "</span>";
                    }
                } else {
                    if ($spojeni['slave_hid'] > 0) {
                        $disabled_cls = get_disabled_cls($spojeni['slave_hid']);
                        $html .= "<span class='correctableFieldContainer " . $corr['class'] . "' id='$id' data-ldomid='" . $this->localDomId[$idn] . "'>" . $corr['flag'] . "<a href='formular_hesla_new.php?hid=" . $spojeni['slave_hid'] . "' class='$disabled_cls'>" . print_heslo_str($spojeni['slave_hid']) . "</a></span>";
                    } else {
                        $html .= "<span class='correctableFieldContainer " . $corr['class'] . "' id='$id' data-ldomid='" . $this->localDomId[$idn] . "'>" . $corr['flag'] . $spojeni['slave_str'] . "</span>";
                    }
                }
                $html .= "<br />";
            }
            if (!$archive) {
                if ($this->master) $html .= '<span class="hidRel_popup_link clickable" onclick="show_link_popup(\'heslo-' . $this->parentVal . '--propojeni\',\'propojeni\')">Uprav připojená/přihnízdovaná hesla k "heslo"</span>';
                else $html .= '<span class="hidRel_popup_link" onclick="show_link_popup(\'heslo-' . $this->parentVal . '--propojeni_s\',\'propojeni_s\')">Uprav</span>';
            }
            $html .= '</div>';
            if (!$this->master) $html .= '<div id="heslo-' . $this->parentVal . '--propojeni-hidRelPopUp" hid="' . $this->hid . '" place="heslo" place_id="' . $this->hid . '" parentDomId="' . $this->parentDomId . '" title="Heslo připojeno/přihnízdováno k:"></div>';
            else $html .= '<div id="heslo-' . $this->parentVal . '--propojeni_s-hidRelPopUp" hid="' . $this->hid . '" place="heslo" place_id="' . $this->hid . '" parentDomId="' . $this->parentDomId . '" ></div>';
            $html .= '</div>';
            $html .= "<div class='clear'></div>";
        } else {
            foreach ($this->data as $key => $rel) {
                $hidRel_id = 'nest-' . $key;
                if ($archive) {
                    $corr = $this->emptyCorrectionArray();
                } else {
                    $corr = $this->displayCorrection($this->corr[$hidRel_id]['field_DOMid'], $this->corr[$hidRel_id]['id'], $this->corr[$hidRel_id]['field'], $this->corr[$hidRel_id]['nova']);
                }
                $html .= '<span id="' . $this->parentDomId . '--' . $hidRel_id . '" class="hidRel_slaves_placeholder correctableFieldContainer ' . $corr["class"] . '" data-ldomid="' . $this->localDomId[$key] . '">' . $this->print_this_hidRel_sign($rel) . '&nbsp';
                if ($rel['slave_hid'] > 0) {
                    $html .= $corr["flag"] . '<a href="formular_hesla_new.php?hid=' . $rel['slave_hid'] . '" class="$disabled_cls ">' . print_heslo_str($rel['slave_hid']) . '</a>';
                } else {
                    $html .= $corr["flag"] . $rel['slave_str'];
                }
                //$html.=" ".print_hidRel_vyznam_nums($cdo, $rel["id"], true);
                $html .= "</span><br/>";
            }
            if (!$archive) {
                $p = $this->parent;
                if($p == "vyznam"){
                    $p = "význam";
                }
                $html .= '
                    <span class="clickable hidRel hidRel_popup_link" onclick="show_link_popup(\'' . $this->parentDomId . '--propojeni\',\'propojeni\')">Uprav připojená/přihnízdovaná hesla k "' . $p . ' ' . $this->parentOrder . '"</span>
                    <div id="' . $this->parentDomId . '--propojeni-hidRelPopUp" class="popUp hidRel_popUp" place_id="' . $this->parentVal . '" place="' . $this->parent . '" hid="' . $this->hid . '"  parentDomId="' . $this->parentDomId . '"  style="" title="Připojeno/Přihnízdováno k ' . $p . ' ' . $this->parentOrder . '"></div>
            ';
            }
        }
        return $html;
    }

    protected function getCorrectionId($hidRel_DBid)
    {
        if ($this->master) {
            switch ($this->type[0]) {
                case HREL_TYPE_syn:
                    $corr_hidRel_id = 'hidRel-synonymum-slaves-' . $hidRel_DBid;
                    break;
                case HREL_TYPE_ant:
                    $corr_hidRel_id = 'hidRel-antonymum-slaves-' . $hidRel_DBid;
                    break;
            }
        } else {
            switch ($this->type[0]) {
                case HREL_TYPE_syn:
                    $corr_hidRel_id = 'hidRel-synonymum-slaves-' . $hidRel_DBid;
                    break;
                case HREL_TYPE_ant:
                    $corr_hidRel_id = 'hidRel-antonymum-slaves-' . $hidRel_DBid;
                    break;
            }
        }
        return $corr_hidRel_id;
    }

    protected function printAntSyn($typString, $archive = False)
    {
        if(!$archive)
        {
            $odkaz_click_tpl = '<div style="clear:both;float: right;" class="clickable hidRel hmargin" data-hid_rel_popup_id="'.$this->parentDomId.'--'.$this->typeString.'onyma_'.(($this->master)?'main':'slave').'">%content%</div>';
            $odkazy = str_replace('%content%', '<span class="hidRel_popup_link" onclick="show_link_popup(\''.$this->parentDomId.'--'.$this->typeString.'onyma_'.(($this->master)?'main':'slave').'\',\'ant_syn\')">Přidat '.$typString.' &hellip;</span>', $odkaz_click_tpl);
        }
        if ($this->master == true) {
            if ($typString == 'antonymum') {
                $nadpis = "<b>Antonyma</b>";
            } else {
                $nadpis = "<b>Synonyma</b>";
            }
            if (!empty($this->data)) {
                //var_dump($this->data);
                $odkazy = "";
                foreach ($this->data as $hidRel_DBid => $odkazy_info) {
                    $corr_hidRel_id = $this->getCorrectionId($hidRel_DBid);
                    if ($archive) {
                        $corr = $this->emptyCorrectionArray();
                    } else {
                        $corr = $this->displayCorrection($this->corr[$corr_hidRel_id]['field_DOMid'], $this->corr[$corr_hidRel_id]['id'], $this->corr[$corr_hidRel_id]['field'], $this->corr[$corr_hidRel_id]['nova']);
                    }
                    $id = $this->parentDomId . "--hidRel-$typString-slaves-$hidRel_DBid";
                    $odkazy .= '<span class="correctableFieldContainer ' . $corr["class"] . '" id="' . $id . '" data-ldomid="' . $this->localDomId[$hidRel_DBid] . '">' . $corr["flag"];
                    if (empty($odkazy_info['slave_str'])) {
                        $odkazy .= "<span class='disabled'>smazáno</span>";
                    } else if ($odkazy_info['slave_hid'] > 0) {
                        $odkazy .= "<a href='formular_hesla_new.php?hid=" . $odkazy_info['slave_hid'] . "'>" . print_heslo_str($odkazy_info['slave_hid']) . print_vyznam_str($hidRel_DBid, false) . "</a>";
                    } else {
                        $odkazy .= $odkazy_info['slave_str'];
                    }
                    $odkazy .= "</span><br/>";
                }
                $odkazy .= str_replace('%content%', '<span class="hidRel_popup_link" onclick="show_link_popup(\'' . $this->parentDomId . '--' . $this->typeString . 'onyma_' . (($this->master) ? 'main' : 'slave') . '\',\'ant_syn\')">Uprav</span>', $odkaz_click_tpl);
            }
            $startTag = "<fieldset style='clear:left;' class='antSynContainer' ><legend>$nadpis</legend>";
            $endTag = "</fieldset>";
        } else {
            $existuji = false;
            $odkazy = '<fieldset style="clear:left;" class="antSynContainer" ><legend><b>Připojeno jako ' . $typString . '</b></legend>';
            //var_dump($this->data);
            if (!empty($this->data)) {
                foreach ($this->data as $hidRel_DBid => $odkazy_info) {
                    $existuji = true;
                    $corr_hidRel_id = $this->getCorrectionId($hidRel_DBid);
                    if ($archive) {
                        $corr = $this->emptyCorrectionArray();
                    } else {
                        $corr = $this->displayCorrection($this->corr[$corr_hidRel_id]['field_DOMid'], $this->corr[$corr_hidRel_id]['id'], $this->corr[$corr_hidRel_id]['field'], $this->corr[$corr_hidRel_id]['nova']);
                    }
                    $id = $this->parentDomId . "--hidRel-$typString-slaves-$hidRel_DBid";
                    $odkazy .= '<span class="correctableFieldContainer ' . $corr["class"] . '" id="' . $id . '" data-ldomid="' . $this->localDomId[$hidRel_DBid] . '">' . $corr["flag"];
                    if ($odkazy_info['main_hid'] > 0) {
                        $odkazy .= '<a href="formular_hesla_new.php?hid=' . $odkazy_info['main_hid'] . '">' . print_heslo_str($odkazy_info['main_hid']) . print_vyznam_str($hidRel_DBid) . "</a>";
                    } else {
                        $odkazy .= $odkazy_info['main_str'];
                    }
                    $odkazy .= '</span>';
                    $odkazy .= "<br/>";
                }
            }
            $p = $this->parent;
            if($p == "vyznam"){
                $p = "význam";
            }
            if ($existuji) {
                $odkazy .= str_replace('%content%', '<span class="hidRel_popup_link" onclick="show_link_popup(\'' . $this->parentDomId . '--' . $this->typeString . 'onyma_' . (($this->master) ? 'main' : 'slave') . '\',\'ant_syn\')">Uprav</span>', $odkaz_click_tpl);
            } else {
                $odkazy .= str_replace('%content%', '<span class="hidRel_popup_link" onclick="show_link_popup(\'' . $this->parentDomId . '--' . $this->typeString . 'onyma_' . (($this->master) ? 'main' : 'slave') . '\',\'ant_syn\')">Přidat vazby na ' . $p . ' jako ' . $typString . '</span>', $odkaz_click_tpl);
            }
            $odkazy .= "</fieldset>";
            $startTag = "<div class='ant_syn_outer_wraper'>";
            $startTag .= "<div class='ant_syn_container2' style='display: block;'>";
            $endTag = "</div></div>";
            //var_dump($odkazy);
        }
        //print html
        $html = $startTag;
        $html .= '<div class="hidRel_slaves_placeholder">' . $odkazy . '</div>';
        $html .= '<div id="' . $this->parentDomId . '--' . $this->typeString . 'onyma_' . (($this->master) ? 'main' : 'slave') . '-hidRelPopUp" place="' . $this->parent . '" title="Odkazy na ' . $typString . '" class="popUp hidRel_popUp" hid="' . $this->hid . '" typ="' . $typString . '" vyz_id="' . $this->parentVal . '" master="' . (($this->master) ? "true" : "false") . '"></div>';
        $html .= $endTag;
        return $html;
    }

    protected function printOdkazy($archive)
    {
        $html = "";
        if (!$this->master) $html .= "<div class='label'>Odkazy</div>";
        $html .= '
            <div class="floatLeft topCont" style="margin-left: 25px">';
        if (!$this->master) $html .= '<div class="label">Na toto heslo odkazujeme z těchto hlavních hesel:</div>';
        $html .= '<div id="' . $this->parentDomId . '--odk_' . (($this->master) ? 'main' : 'slave') . '-hidRelPopUp" hid="' . $this->hid . '" place="heslo"  place_id="' . $this->hid . '" master="' . (($this->master) ? 'slave' : 'mains') . '"></div>
              <div class="clickable hidRel">';
        foreach ($this->data as $id => $odkaz) {
            $idn = $id;
            $odkaz_types = $odkaz['link_types'];
            $id = "link-" . $odkaz["id"];
            if ($archive) {
                $corr = $this->emptyCorrectionArray();
            } else {
                $corr = $this->displayCorrection($this->corr[$id]['field_DOMid'], $this->corr[$id]['id'], $this->corr[$id]['field'], $this->corr[$id]['nova']);
            }
            $html .= '<span class="correctableFieldContainer ' . $corr['class'] . '" id="' . $id . '" data-ldomid="' . $this->localDomId[$idn] . '">' . $corr['flag'];
            $je_odkaz = false;
            if (($key = array_search("odkaz", $odkaz_types)) !== false) {
                $je_odkaz = true;
                unset($odkaz_types[$key]);
            }
            $sarr = array();
            if (!empty($odkaz_types)) {
                $html .= $this->print_this_hidRel_sign($odkaz) . " ";
                foreach ($odkaz_types as $id_o => $odkaz_typ) {
                    $sarr[] = $odkaz_typ["link_type"];
                }
                $s = implode(", ", $sarr);
                if ($s != "verb. subst./adj." && $s != "vícesl.") {
                    $html .= $s . "&nbsp;";
                }

                if (!$this->master) {
                    if ($odkaz['main_hid'] > 0) {
                        $disabled_cls = get_disabled_cls($odkaz['main_hid']);
                        $html .= "<a href='formular_hesla_new.php?hid=" . $odkaz['main_hid'] . "' class='$disabled_cls'>" . print_heslo_str($odkaz['main_hid']) . "&nbsp;" . "</a>";
                    } else {
                        $html .= $odkaz['main_str'] . "&nbsp;";
                    }
                }
                $html .= CDO_print_hidRel_vyznam_nums($odkaz["main_places"], $odkaz["main_hid"], false, $archive);
            }
            $html .= " &rarr; ";
            $note = text_html_preprocess($odkaz['poznamka']);
            if (implode("", $sarr) == "verb. subst./adj.") {
                $html .= "<span class='petit'>i</span> ";
            }
            if (!empty($note)) {
                $html .= "<span class='petit'>$note</span> ";
            }

            if ($this->master) {
                if ($odkaz['slave_hid'] > 0) {
                    $disabled_cls = get_disabled_cls($odkaz['slave_hid']);
                    $html .= "<a href='formular_hesla_new.php?hid=" . $odkaz['slave_hid'] . "' class='$disabled_cls'>" . print_heslo_str($odkaz['slave_hid'], $je_odkaz) . "&nbsp;" . "</a>";
                } else {
                    $html .= $odkaz['slave_str'] . "&nbsp;";
                }
                $html .= " " . CDO_print_hidRel_vyznam_nums($odkaz["slave_places"], $odkaz["slave_hid"], true, $archive); //true: slave places
            }
            $html .= "</span><br/>";
        }
        if (!$archive) {
            if ($this->master) $html .= ' <span class="hidRel_popup_link clickable" onclick="show_link_popup(\'' . $this->parentDomId . '--odk_' . (($this->master) ? 'main' : 'slave') . '\',\'odkazy\')">Uprav odkazy na hesla</span>';
            else $html .= ' <span class="hidRel_popup_link clickable" onclick="show_link_popup(\'' . $this->parentDomId . '--odk_' . (($this->master) ? 'main' : 'slave') . '\',\'odkazy\')">Uprav</span>';
        }
        $html .= '</div>
              <div id="' . $this->parentDomId . '--odk_' . (($this->master) ? 'main' : 'slave') . '-hidRelPopUp" title="Odkazy na heslo"></div>
            </div>
          ';
        $html .= "<div class='clear'></div>";
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '';
        switch ($this->type[0]) {
            case HREL_TYPE_syn:
                $html .= $this->printAntSyn("synonymum", $archive);
                break;
            case HREL_TYPE_ant:
                $html .= $this->printAntSyn("antonymum", $archive);
                break;
            case HREL_TYPE_pripojovani:
                $html .= $this->printPripojeni($archive);
                break;
            case HREL_TYPE_odkazy:
                $html .= $this->printOdkazy($archive);

        }
        return $html;
    }

    protected function print_this_hidRel_sign($jedenOdkaz)
    {
        $sign = "";
        if ($jedenOdkaz['rel_type'] == HREL_TYPE_pripojovani) {
            $varianty = get("varianty_hesla", $jedenOdkaz['slave_hid'], array("sortorder"));
            $typy_hesla = array();
            foreach ($varianty as $var_id) {
                $_typy_hesla = get_vybrane_hodnoty_ciselniku(TYP_HESLA, "var", $var_id);
                $typy_hesla = array_merge($typy_hesla, array_keys($_typy_hesla));
            }
            if (empty($jedenOdkaz['slave_hid'])) $sign = "&times;";//obrateny vykricnik   //"&#9675"; //empty circle
            elseif (in_array(HREL_SLAVE_TYPE_vicesl, $typy_hesla)) $sign = "&#9633;"; //kocečka prazdna
            elseif (in_array(HREL_SLAVE_TYPE_souslovi, $typy_hesla)) $sign = "&#9633;"; //kocečka prazdna
            elseif (in_array(HREL_SLAVE_TYPE_frazem, $typy_hesla)) $sign = "&loz;"; //kosostvorec prazdny
        } //prihnizdeni a odkaz
        elseif ($jedenOdkaz['rel_type'] == HREL_TYPE_prihnizdovani || $jedenOdkaz['rel_type'] == HREL_TYPE_odkazy) {
            $sign = "<span class='x-small'>&#9658;</span>"; //play sign
        }
        return $sign;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';
        foreach ($this->colsStructured as $col) {
            $xml .= $this->{$col}->printXML;
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return Response
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        foreach ($this->data as $id => $d) {
            $mainPl = $d->main_places;
            $slavePl = $d->slave_places;
            $sqlData = array(
                'main_id' => is_numeric($id) ? $id : 0,
                'main_hid' => $this->master ? $archiveHid : $d['main_hid'],
                'slave_hid' => $this->master ? $d['slave_hid'] : $archiveHid,
                'main_str' => $d['main_str'],
                'slave_str' => $d['slave_str'],
                'rel_type' => $d['rel_type'],
                'archived' => $this->master ? 'main' : 'slave',
                'spojeno' => $d['spojeno'],
                'sortorder' => $d['sortorder'],
                'sortorder2' => $d['sortorder2'],
                'komentar' => $d['komentar'],
                'poznamka' => $d['poznamka'],
                'archived_id' => $archiveHid
            );
            $tmp = $this->archiveInsert('hesla_rel', 'archive_hesla_rel', $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
            $arelId = $tmp->data;
            foreach ($mainPl as $mid => $mpl) {
                $msqlData = array(
                    'hesla_rel_id' => $arelId,
                    'place' => $mpl['place'],
                    'place_id' => $this->master ? $archiveParentVal : $mpl['place_id']
                );
                $tmp = $this->archiveInsert('hesla_rel_places', 'archive_hesla_rel_places', $msqlData);
                if ($tmp->status !== OK) {
                    return $tmp;
                }
            }

            foreach ($slavePl as $sid => $spl) {
                $ssqlData = array(
                    'hesla_rel_id' => $arelId,
                    'place' => $spl['place'],
                    'place_id' => $this->master ? $spl['place_id'] : $archiveParentVal
                );
                $tmp = $this->archiveInsert('hesla_rel_slave_places', 'archive_hesla_rel_slave_places', $ssqlData);
                if ($tmp->status !== OK) {
                    return $tmp;
                }
            }
        }
        return new Response(OK);
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $headers = array();
        $tmp = array();
        $typeArr = array(
            HREL_TYPE_ant => ' (n)',
            HREL_TYPE_ant_przn => ' (p)',
            HREL_TYPE_syn => ' (n)',
            HREL_TYPE_syn_przn => ' (p)'
        );
        if ($this->master && ($this->typeString == 'syn' || $this->typeString == 'ant')) {
            foreach ($this->data as $id => $d) {
                $tmp[] = wrap_output($this->getDomId($id), $d->slave_str . $typeArr[$d->rel_type], $wrap);
            }
            $data[count($data) - 1][] = implode(', ', $tmp);
        }
        return new Response(OK, array('headers' => $headers));
    }

    public function getSQL($fields = False, $parentTable = False)
    {
        // hesla_rel inner join hesla_rel_places on id = hesla_rel_id
        $tsuff = rand(1, 100);
        $hr = 'hr' . $tsuff;
        $hrp = 'hrp' . $tsuff;
        $return = array(
            'columns' => array(
                '[' . $hr . '].[id] AS [' . $this->alias . '--id]',
                '[' . $hr . '].[slave_str] AS [' . $this->alias . '--' . $this->typeString . ']'
            ),
            'tables' => array(
                '([hesla_rel] AS [' . $hr . ']
                    INNER JOIN [hesla_rel_places] AS [' . $hrp . ']
                    ON [' . $hr . '].[id] = [' . $hrp . '].[hesla_rel_id])' => array(
                    'main' => array(
                        '[' . $hrp . '].[place_id]',
                        '[' . $parentTable . '].[id] AND [' . $hrp . '].[place]',
                        '"' . $this->typeString . '" AND [' . $hr . '].[rel_type] IN (' . implode(',', $this->type) . ')'
                    ),
                    'child' => array()
                )
            )
        );

        return new Response(OK, $return);
    }

    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        $tmp = array();
        foreach ($data as $did => $entry) {
            $tmp[] = $entry[$this->typeString];
        }
        return implode(', ', $tmp);
    }
}
