<?php
/**
 * Soubor podhesli.php obsahujici definici tridy CDO_Podhesli.
 */

/**
 * Trida pro skupinu podhesli vyznamu.
 */
class CDO_Podhesli extends CDO_Base
{

    /**
     * Konstruktor tridy CDO_Podhesli.
     * @param int $input
     * @param bool $parentVal
     * @param bool $parentDomId
     */
    function __construct($input = CDOin_DB, $parentVal = False, $parentDomId = False)
    {
        $this->input = $input;
        $this->parentVal = $parentVal;
        $this->parentFk = 'vyz_id';
        $this->parentDomId = $parentDomId;
        $this->baseDom = 'podhesli';
        $this->colsStructured = array('vyslovnost', 'slov_druh', 'tvar_char', 'tvar_char_sl', 'vazebnost', 'styl_kval', 'gram_info', 'tematicke_okruhy', 'obor_uziti');
        $this->colsSimple = array('komentar_pred', 'podhesli', 'komentar_za', 'komentar_tvarchar', 'komentar_tvarchar_slovesa');
        $this->xmlWrapper = 'sublemmas';
        $this->xmlElem = 'sublemma';
        $this->tbl = 'podhesli_vyznamu';
        $this->alias = 'vyznam-podhesli';
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {
            $this->loadStructured('temp');
        }
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array();
        foreach ($this->domId as $id => $dom) {
            array_push($domList, $dom . '--komentar_podhesli');
            array_push($domList, $dom . '--tvar_podhesli');
            array_push($domList, $dom . '--komentar_podhesli_za');
            array_push($domList, $dom . '--komentar_tvarchar');
            array_push($domList, $dom . '--komentar_tvarchar_slovesa');
        }
        $this->corr = $this->loadCorrectionsByDom($domList);
    }

    public function add()
    {
        $newId = $this->newId();
        $this->domId[$newId] = 'podhesli-' . $newId;
        $this->data[$newId] = new DibiRow(array(
            'komentar_pred' => '',
            'podhesli' => '',
            'komentar_za' => '',
            'komentar_tvarchar' => '',
            'komentar_tvarchar_slovesa' => '',
            'vyslovnost' => new CDO_TvarKom(CDOin_EMPTY, VYSLOVNOST, PODHESLI, $newId, $this->domId[$newId]),
            'vazebnost' => new CDO_TvarKom(CDOin_EMPTY, VAZEBNOST, PODHESLI, $newId, $this->domId[$newId]),
            'styl_kval' => new CDO_Ciselnik(CDOin_EMPTY, STYL_KVAL, PODHESLI, $newId, $this->domId[$newId]),
            'tematicke_okruhy' => new CDO_Ciselnik(CDOin_EMPTY, TEMATICKE_OKRUHY, PODHESLI, $newId, $this->domId[$newId]),
            'obor_uziti' => new CDO_Ciselnik(CDOin_EMPTY, OBOR_UZITI, PODHESLI, $newId, $this->domId[$newId]),
            'gram_info' => new CDO_Ciselnik(CDOin_EMPTY, GRAM_INFO, PODHESLI, $newId, $this->domId[$newId]),
            'slov_druh' => new CDO_SlovDruh(CDOin_EMPTY, PODHESLI, $newId, $this->domId[$newId]),
            'tvar_char' => new CDO_TvarChar(CDOin_EMPTY, False, PODHESLI, $newId, $this->domId[$newId]),
            'tvar_char_sl' => new CDO_TvarChar(CDOin_EMPTY, True, PODHESLI, $newId, $this->domId[$newId])
        ));
        $this->maximized[$newId] = True;
        $_SESSION['podh_vyz'][$newId] = $this->parentVal;
        return new Response(OK, $newId);
    }

    private function loadStructured($id)
    {
        $this->domId[$id] = 'podhesli-' . $id;
        $this->data[$id]['vyslovnost'] = new CDO_TvarKom($this->input, VYSLOVNOST, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['vazebnost'] = new CDO_TvarKom($this->input, VAZEBNOST, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['styl_kval'] = new CDO_Ciselnik($this->input, STYL_KVAL, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['tematicke_okruhy'] = new CDO_Ciselnik($this->input, TEMATICKE_OKRUHY, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['obor_uziti'] = new CDO_Ciselnik($this->input, OBOR_UZITI, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['gram_info'] = new CDO_Ciselnik($this->input, GRAM_INFO, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['slov_druh'] = new CDO_SlovDruh($this->input, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['tvar_char'] = new CDO_TvarChar($this->input, False, PODHESLI, $id, $this->domId[$id]);
        $this->data[$id]['tvar_char_sl'] = new CDO_TvarChar($this->input, True, PODHESLI, $id, $this->domId[$id]);
        $this->maximized[$id] = False;
    }

    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::query('
                SELECT id, komentar_pred, podhesli, komentar_za, komentar_tvarchar, komentar_tvarchar_slovesa
                FROM %n
                WHERE vyz_id = %i
                ORDER BY id ASC
            ', $prefix . $this->tbl, $this->parentVal)->fetchAssoc('id');

        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }

        foreach ($this->data as $id => $d) {
            $this->loadStructured($id);
        }
        return new Response(OK);
    }

    protected function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        try {
            dibi::begin();
            $sqlData = array(
                'vyz_id' => $this->parentVal
            );
            $ord = 1;
            $originalDataKeys = array_combine(array_keys($originalData), array_keys($originalData));
            foreach ($this->data as $id => $d) {
                //$sqlData['sortorder'] = $ord++;
                foreach ($this->colsSimple as $col) {
                    $sqlData[$col] = $this->data[$id][$col];
                }
                if (isset($originalDataKeys[$id])) {
                    // update
                    dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . $this->tbl, $sqlData, $id);
                    unset($originalDataKeys[$id]);
                    foreach ($this->colsStructured as $col) {
                        $this->data[$id][$col]->saveToDB($archive, $originalData[$id][$col]->data);
                    }
                } else {
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . $this->tbl, $sqlData);
                    $dbid = $this->newId(True);
                    foreach ($this->colsStructured as $col) {
                        $this->data[$id][$col]->setParentVal($dbid);
                        $this->data[$id][$col]->saveToDB($archive);
                    }
                }
            }

            foreach ($originalDataKeys as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . $this->tbl, $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }

        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param int $id
     * @param bool $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        $dom = $this->domId[$id];
        $html = '
            <div id="' . $dom . '" class="podhesli_vyznamu">
            <fieldset><legend>Podheslí významu</legend>
                <div style="clear:both;padding:10px;">
                    <div class="floatRight podhesli_keepshow">
                        <img src="css/images/minimize.png" class="clickable minimize_podhesli" alt="Minimalizovat podheslí" />
                    </div>
                    <table class="podhesli_keepshow">
                        <tr>
                            <td>
                                <label for="' . $dom . '--komentar_podhesli-input">komentář</label><br />
                                ' . $this->printInput($dom . '--komentar_podhesli', $archive, $this->data[$id]->komentar_pred) . '
                            </td>
                            <td>
                                <label for="' . $dom . '--tvar_podhesli-input">podheslí</label><br />
                                ' . $this->printInput($dom . '--tvar_podhesli', $archive, $this->data[$id]->podhesli) . '
                            </td>
                            <td>
                                <label for="' . $dom . '--komentar_podhesli_za-input">komentář</label><br />
                                ' . $this->printInput($dom . '--komentar_podhesli_za', $archive, $this->data[$id]->komentar_za) . '
                            </td>
                        </tr>
                    </table>
                    <table style="width:470px;">
                        <tr>
                            <td><span class="label">výslovnost:</span></td>
                            <td>' . $this->data[$id]->vyslovnost->printForm($archive) . '</td>
                        </tr>
                    </table>
                    <table style="width:470px;">
                        <tr>
                            <td>
                                <span class="label"> valence/vazebnost:</span>
                                ' . $this->data[$id]->vazebnost->printForm($archive) . '
                            </td>
                        </tr>
                    </table>
                    <table class="w100">
                        <tr>
                            <td>
                                <span class="label"> stylistický kvalifikátor:</span>
                                ' . $this->data[$id]->styl_kval->printForm($archive) . '
                            </td>
                            <td>
                                <span class="label">tematické okruhy:</span>
                                ' . $this->data[$id]->tematicke_okruhy->printForm($archive) . '
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <span class="label">obor/oblast užití:</span>
                                ' . $this->data[$id]->obor_uziti->printForm($archive) . '
                            </td>
                            <td></td>
                        </tr>
                        <tr>
                            <td> ' . $this->data[$id]->slov_druh->printForm($archive) . '</td>
                            <td>
                                <span class="label">upřesňující gram. inf.:</span>
                                ' . $this->data[$id]->gram_info->printForm($archive) . '
                            </td>
                        </tr>
                        <tr>
                            <td style="width: 50%;"> ' . $this->data[$id]->tvar_char->printForm($archive) . '</td>
                            <td style="width: 50%;"> ' . $this->data[$id]->tvar_char_sl->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td colspan="2"><span class="label"> komentář k tvaroslovné charakteristice:</span><br />
                                ' . $this->printInput($dom . '--komentar_tvarchar', $archive, $this->data[$id]->komentar_tvarchar, array('w100fixSml', 'textField', 'padd', 'flatBox', 'lightBG')) . '
                            </td>
                        </tr>
                        <tr>
                            <td colspan="2"><span class="label"> komentář k tvaroslovné charakteristice slovesa:</span><br />
                                ' . $this->printInput($dom . '--komentar_tvarchar_slovesa', $archive, $this->data[$id]->komentar_tvarchar_slovesa, array('w100fixSml', 'textField', 'padd', 'flatBox', 'lightBG')) . '
                            </td>
                        </tr>
                    </table>
                    ' . ($archive ? '' : '<input value="Smazat podheslí" class="redButtBox buttBox floatRight" type="button" onclick="rm_field(\'' . $dom . '\')" />') . '
                </div>
            </fieldset></div>
        ';
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '<div id="vyznam-' . $this->parentVal . '--podhesli" class="skupina_podhesli_wrapper">';
        foreach ($this->data as $id => $d) {
            $html .= $this->printOne($id, $archive);
        }
        $html .= '</div>';
        if (!$archive) {
            $html .= '
                <div class="add_podhesli_btn" style="margin-top: 10px;">
                <input value="Přidej podheslí" class="greenButtBox buttBox floatRight" type="button"
                    onclick="add_fields(\'vyznam-' . $this->parentVal . '--podhesli\')" />
                </div>
                ';
        }
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';
        $ids = array_keys($this->data);
        foreach ($ids as $id) {
            $xml .= '
                <' . $this->xmlElem . ' id="' . $id . '">
                    <comment-before>' . $this->data[$id]->komentar_pred . '</comment-before>
                    <word>' . $this->data[$id]->podhesli . '</word>
                    <comment-after>' . $this->data[$id]->komentar_za . '</comment-after>
            ';
            foreach ($this->colsStructured as $col) {
                $xml .= $this->data[$id]->{$col}->printXML();
            }
            $xml .= '</' . $this->xmlElem . '>';
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $tbl = $this->tbl;
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        $sqlData = array(
            'vyz_id' => $archiveParentVal
        );
        foreach ($this->data as $id => $d) {
            foreach ($this->colsSimple as $col) {
                $sqlData[$col] = $this->data[$id][$col];
            }
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
            foreach ($this->colsStructured as $col) {
                $this->data[$id][$col]->saveToArchive($tmp->data);
            }
        }
        return new Response(OK);
    }

    protected function addTabData($id)
    {
        return array(
            'podhesli' => 'Podheslí ' . getPodhesliNum($id)
        );
    }
}