<?php

/**
 * Trida pro semafor.
 */
class CDO_Semafor extends CDO_Base
{
    /** @var int $createdBy tvurce hesla */
    private $createdBy;
    /** @var array $zpracovatele zpracovatele hesla */
    private $zpracovatele;
    /** @var array $zpacKeys zpracovatele - ulozene jako priznaky */
    private $zpacKeys;

    function __construct($input = CDOin_DB, $parentVal = False, $parentDomId = False, $createdBy = False)
    {
        $this->input = $input;
        $this->parentVal = $parentVal;
        $this->parentDomId = $parentDomId;
        $this->baseDom = $parentDomId . '--' . 'semafor';
        $this->domId = $this->baseDom;
        $this->colsStructured = array();
        $this->colsSimple = array();
        $this->xmlWrapper = 'status';
        $this->xmlElem = 'status';
        $this->tbl = 'heslo_stav';
        $this->createdBy = $createdBy;
        $this->zpracovatele = array();
        $this->zpacKeys = array();
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->zpracovatele = dibi::fetchPairs('SELECT uid FROM zpracovatel WHERE hid = %i', $this->parentVal);
            $this->zpacKeys = array_flip($this->zpracovatele);
//            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array(
                'stav' => 1
            );
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        }
    }

    public function updateSemafor($data)
    {
        $this->data['stav'] = $data['stav'];
        foreach ($data['checks'] as $code => $val) {
            $this->data['checks'][$code]['value'] = $val;
        }
    }


    /**
     * Nacteni dat z databaze do objektu.
     * @param bool|False $archive nacist z archivu
     * @return Response navratova hodnota
     */
    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::fetch('
                SELECT id, stav
                FROM %n
                WHERE hid = %i
                ORDER BY id DESC LIMIT 1
            ', $prefix . $this->tbl, $this->parentVal);
            if (!$this->data) {
                $this->data = array(
                    'stav' => 1
                );
            }
            $this->data['checks'] = dibi::query('
                SELECT code, label, IFNULL(`value`, 0) AS `value`, c.id AS cid, IFNULL(hs.id, 0) AS relid, `required`, `stav`, `uts`
                FROM cis_heslo_check c LEFT JOIN %n hs ON hs.checkid = c.id AND hs.hid = %i
            ', $prefix . 'heslo_check_stav', $this->parentVal)->fetchAssoc('code');
        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                } else {
                    return handle_db_error($e);
                }
            } else {
                return handle_db_error($e);
            }
        }
        return new Response(OK);
    }

    /**
     * Ulozeni dat z objektu do DB.
     * @param bool|False $archive ulozit do archivu
     * @param array $originalData puvodni data v DB
     * @return Response navratova hodnota
     */
    public function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        try {
            dibi::begin();
            $sqlData = array(
                'hid' => $this->parentVal,
                'stav' => $this->data['stav'],
                'uid' => get_logged_user()
            );
            dibi::query('INSERT INTO %n', $prefix . $this->tbl, $sqlData);
            $carr = array(
                'hid' => $this->parentVal
            );
             foreach ($this->data['checks'] as $c) {
                $carr['checkid'] = $c['cid'];
                $carr['value'] = $c['value'];
                if ($c['relid']) {
                    dibi::query('UPDATE %n SET %a WHERE id = %i', $prefix . 'heslo_check_stav', $carr, $c['relid']);
                } else {
                    dibi::query('INSERT INTO %n', $prefix . 'heslo_check_stav', $carr);
                    $c['relid'] = dibi::getInsertId();
                }
                if($c['code']=="web" && $c['value']=="0"){
                    $manager = new CacheManager();
                    $manager->removeFromWeb($this->parentVal);
                }
                if(is_numeric($this->parentVal) && $this->parentVal > 0){
                    update_heslo_uts($this->parentVal);
                }
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }

        return new Response(OK);
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return Response navratova hodnota
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $tbl = $this->tbl;
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        $sqlData = array(
            'hid' => $archiveParentVal,
            'stav' => $this->data['stav'],
            'uid' => get_logged_user()
        );
        if (isset($this->data['id'])) {
            $sqlData['main_id'] = $this->data['id'];
        }
        $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
        if ($tmp->status !== OK) {
            return $tmp;
        }
        $date = date('Y-m-d H:i:s');
        foreach ($this->data['checks'] as $c) {
            $carr = array(
                'hid' => $archiveParentVal,
                'checkid' => $c['cid'],
                'value' => $c['value'],
                'uts' => (empty($c['uts']->date)?$date:$c['uts']->date)
            );
            if ($c['relid']) {
                $carr['main_id'] = $c['relid'];
            }
            $tmp = $this->archiveInsert('heslo_check_stav', ARCHIVE_PREFIX . 'heslo_check_stav', $carr);
            if ($tmp->status !== OK) {
                return $tmp;
            }
        }
        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $onclick = $archive ? '' : 'show_semafor_editor(\'' . $this->domId . '\', this)';
        $html = '
            <div id="semafor_wrapper_'.$this->parentVal.'">
                <div id="' . $this->domId . '"
                    class="floatRight heslo_semafor heslo_semafor_' . $this->data['stav'] . ' active"
                    title="' . getSemaforTitle($this->data['stav']) . '" onclick="' . $onclick . '">
                    ' . $this->data['stav'] . '
                </div>
                <div id="' . $this->domId . '_popUp"></div>
            </div>
        ';

        return $html;
    }

    /**
     * Vygeneruje editacni dialog.
     * @param bool|False $archive
     * @return string
     */
    public function printEditor($archive = False)
    {
        $html = '';
        $stat = $this->getSemaforStatus();
        foreach ($stat['stavy'] as $i => $stav) {
            $act = $i == $this->data['stav'] ? ' active' : '';
            if (!$stav) $act .= ' disabled';
            $html .= '
                <div class="floatLeft heslo_semafor heslo_semafor_' . $i . ' heslo_semafor_edit' . $act . '"
                data-dom="' . $this->domId . '"
                title="' . getSemaforTitle($i) . '"
                data-stav="' . $i . '">' . $i . '</div>
            ';
        }
        $web = $this->data['checks']['web'];
        $html .= '
            <div class="heslo_semafor_chb_web' . ($stat['webhidden'] ? ' hidden' : '') . '">
                <div class="heslo_semafor_chbox_wrap">
                <label><input type="checkbox"
                    class="heslo_semafor_chbox_web"
                    ' . ($web['value'] ? ' checked="checked"' : '') . '
                    data-code="web"
                    ' . ($stat['webchecked'] ? ' disabled="disabled" checked="checked"' : '') . ' />' . $web['label'] . '</label>
                </div>
            </div>
        ';
        $html .= '<div class="heslo_semafor_chb' . ($stat['chhidden'] ? ' hidden' : '') . '">';
        foreach ($this->data['checks'] as $code => $v) {
            if ($v['stav'] != 3) continue;
            $html .= '
                <div class="heslo_semafor_chbox_wrap">
                <label><input type="checkbox"
                    class="heslo_semafor_chbox"
                    ' . ($v['value'] ? ' checked="checked"' : '') . '
                    onchange="check_sem_req(\'' . $this->domId . '\', this)"
                    data-code="' . $code . '"
                    ' . ($stat['chdisabled'] ? ' disabled="disabled"' : '') . ' />' . $v['label'] . '</label>
                <span>' . ($v['required'] ? 'P' : 'N') . '</span>
                </div>
            ';
        }
        $html .= '
            </div>
            <input id="semafor_stav_input" type="hidden" value="' . $this->data['stav'] . '"/>
        ';
        return $html;
    }

    public function getSemaforStatus($curStav = False, $checks = False)
    {
        if (!$curStav) {
            $curStav = $this->data['stav'];
        }
        if (!$checks) {
            $checks = array();
            foreach ($this->data['checks'] as $code => $ch) {
                $checks[$code] = $ch['value'];
            }
        }
        $hlred = is_hlred();
        $uziv = is_role_uzivatel();
        $return = array(
            'stavy' => array(),
            'chhidden' => $curStav < 3,
            'chdisabled' => !$uziv || $curStav > 3,
            'webhidden' => $curStav < 5,
            'webchecked' => $curStav == 7
        );
        for ($i = 1; $i < 8; $i++) {
            $return['stavy'][$i] = $this->checkRequirements($curStav, $i, $hlred, $uziv, $checks);
        }
        return $return;
    }

    private function checkRequirements($fromStav, $toStav, $hlred, $uziv, $ch)
    {
        if ($uziv && $fromStav < 4 && $toStav < 4) return True;
        if ($hlred && $fromStav > 3 && $toStav > 2) return True;
        if ($hlred && $toStav > 3) {
            foreach ($ch as $code => $val) {
                if ($this->data['checks'][$code]['required'] && !$val) {
                    return False;
                }
            }
            return True;
        }
        return False;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        return '';
    }
}