<?php
/**
 * Soubor slovdruh.php obsahujici definici tridy CDO_SlovDruh.
 */

/**
 * Trida pro skupinu slovnich druhu
 */
class CDO_SlovDruh extends CDO_Base
{
    /**
     * @var int $parent typ nadrazeneho zaznamu
     */
    private $parent;

    /**
     * Konstruktor tridy CDO_SlovDruh.
     * @param int $input
     * @param bool|False $parent
     * @param bool|False $parentVal
     * @param bool|False $parentDomId
     */
    function __construct($input = CDOin_DB, $parent = False, $parentVal = False, $parentDomId = False)
    {
        $this->input = $input;
        $this->parent = $parent;
        $this->parentVal = $parentVal;
        $this->parentDomId = $parentDomId;
        $this->colsStructured = array('vyber');
        $this->colsSimple = array('komentar');
        $this->xmlWrapper = 'parts-of-speech';
        $this->xmlElem = 'part-of-speech';
        if ($parent && $parentVal) {
            $this->translateQuery();
        }
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {
            $this->loadStructured('temp');
        }
    }

    /**
     * Vyplni informace o zdrojove DB.
     */
    private function translateQuery()
    {
        if ($this->parent == VARIANTA) {
            $this->tbl = 'var_slov_druhy_vyber_rel';
            $this->parentFk = 'vid';
            $this->alias = 'varianta-slov_druh';
        } elseif ($this->parent == VYZNAM) {
            $this->tbl = 'vyz_slov_druhy_vyber_rel';
            $this->parentFk = 'vyz_id';
            $this->alias = 'vyznam-slov_druh';
        } elseif ($this->parent == PODHESLI) {
            $this->tbl = 'podh_vyz_slov_druhy_vyber_rel';
            $this->parentFk = 'podh_id';
            $this->alias = 'vyznam-podhesli-slov_druh';
        }
        $this->domStr = 'slovnyDruh';
        $this->baseDom = $this->parentDomId . '--slovnyDruh';
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array();
        foreach ($this->domId as $id => $dom) {
            array_push($domList, $dom . '--komentar');
        }
        $this->corr = $this->corr = $this->loadCorrectionsByDom($domList);
    }

    /**
     * Pridani noveho slovniho druhu.
     * @return Response
     */
    public function add()
    {
        $newId = $this->newId();
        $this->domId[$newId] = $this->parentDomId . '--slovnyDruh-' . $newId;
        $this->data[$newId] = new DibiRow(array(
            'komentar' => '',
            'vyber' => new CDO_Ciselnik(CDOin_EMPTY, SLOV_DRUHY, $this->parent, $newId, $this->domId[$newId]),
        ));
        return new Response(OK, $newId);
    }

    private function loadStructured($id)
    {
        $this->domId[$id] = $this->baseDom . '-' . $id;
        $this->data[$id]['vyber'] = new CDO_Ciselnik($this->input, SLOV_DRUHY, $this->parent, $id, $this->domId[$id]);
    }

    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::query('
                SELECT id, komentar FROM %n WHERE %n = %i
                ORDER BY id ASC
            ', $prefix . $this->tbl, $this->parentFk, $this->parentVal)->fetchAssoc('id');

        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }
        foreach ($this->data as $id => $kom) {
            $this->loadStructured($id);
        }
        return new Response(OK);
    }

    protected function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        try {
            dibi::begin();
            $sqlData = array(
                $this->parentFk => $this->parentVal,
            );
            $originalDataKeys = array_combine(array_keys($originalData), array_keys($originalData));
            foreach ($this->data as $id => $d) {
                $sqlData['komentar'] = $d['komentar'];
                if (isset($originalDataKeys[$id])) {
                    // update
                    dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . $this->tbl, $sqlData, $id);
                    $this->data[$id]['vyber']->saveToDB($archive, $originalData[$id]['vyber']->data);
                    unset($originalDataKeys[$id]);
                } else {
                    // insert - nejspíš nebude potřeba
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . $this->tbl, $sqlData);
                    $dbid = $this->newId(True);
                    $this->data[$id]['vyber']->setParentVal($dbid);
                    $this->data[$id]['vyber']->saveToDB($archive);
                }
            }

            foreach ($originalDataKeys as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . $this->tbl, $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }
        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param int $id
     * @param bool $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        $komDom = $this->domId[$id] . '--komentar';
        if ($archive) {
            $ro = ' readonly="readonly"';
            $komCorr = $this->emptyCorrectionArray();
        } else {
            $ro = '';
            $komCorr = $this->displayCorrection($this->domId[$id], $this->corr[$komDom]['id'], $this->corr[$komDom]['field'], $this->corr[$komDom]['nova']);
        }
        $html = '';
        $html .= '<div class="multiInput" id="' . $this->domId[$id] . '">';
        if (!$archive) {
            //$html .= $this->rmFieldsBtn($this->domId[$id]);
        }
        $html .=
            '
                <div class="floatLeft" style="width: 120px;">
                    ' . $this->data[$id]->vyber->printForm($archive) . '
                </div>
                <span id="' . $komDom . '" class="correctableFieldContainer">
                  ' . $komCorr['flag'] . '
                  <input type="text"
                  value="' . $this->data[$id]->komentar . '"
                  class="textField clearingInput ' . $komCorr['class'] . '" style="width:240px"
                  ' . $ro . ' onblur="' . $this->intputOnBlur($komDom, $archive) . '" />
                </span>
                <div class="hspace"></div>
            </div>
            ';
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '';
        $html .= '
            <div class="label floatLeft">slovní druh</div>
            <div class="label floatLeft" style="margin-left: 80px;">komentář slovního druhu</div>
            <div id="' . $this->baseDom . '" class="multiContainer floatLeft">
        ';
        if (!$this->data) {
            $this->add();
        }
        foreach ($this->data as $id => $d) {
            $html .= $this->printOne($id, $archive);
        }

        $html .= '</div>';

        /*
        if (!$archive) {
            $html .= '
                <div class="tlacitkoRemField">
                    ' . $this->addFieldsBtn($this->baseDom, 'slovní druh') . '
                </div>
            ';
        }
        */
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';
        $ids = array_keys($this->data);
        foreach ($ids as $id) {
            $xml .= '<' . $this->xmlElem . ' id="' . $id . '">';
            $xml .= '<comment>' . $this->data[$id]->komentar . '</comment>';
            foreach ($this->colsStructured as $col) {
                $xml .= $this->data[$id]->{$col}->printXML();
            }
            $xml .= '</' . $this->xmlElem . '>';
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $tbl = $this->tbl;
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        $sqlData = array(
            $this->parentFk => $archiveParentVal,
        );

        foreach ($this->data as $id => $d) {
            $sqlData['komentar'] = $d['komentar'];
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
            $this->data[$id]['vyber']->saveToArchive($tmp->data);
        }
        return new Response(OK);
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $strArr = array();
        foreach ($this->data as $id => $d) {
            $strArr[] = wrap_output($d['vyber']->getDomId(), $d['vyber']->getValsList(0, ', '), $wrap) .
                ($d['komentar'] ? ' (' . wrap_output($this->getDomId($id, 'komentar'), $d['komentar'], $wrap) . ')' : '');
        }
        $data[count($data) - 1][] = implode(', ', $strArr);
        $headers = array($this->tbl);

        return new Response(OK, array('headers' => $headers));
    }

    protected function getSQLFields($fields)
    {
        return array('vyber' => 1, 'komentar' => 1);
    }


    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        $tmp = array();
        foreach ($data as $did => $entry) {
            $id = $entry['id'];
            $kom = $entry['komentar'];
            foreach ($entry['vyber'] as $vid => $vyb) {
                if (isset($vyb['jine']) && !is_null($vyb['jine']) && $vyb['jine']) {
                    $tmp[] = $vyb['jine'];
                } else {
                    $tmp[] = $vyb['vyber'];
                }
            }
            $vyber = wrap_output($parentStr . '--' . $this->domStr . '-' . $id, implode(', ', $tmp), WRAP_CORRECTION);
            $komentar = wrap_output($parentStr . '--' . $this->domStr . '-' . $id . '--komentar', $kom, WRAP_CORRECTION);
            return $vyber . ($kom ? ' (' . $komentar . ')' : '');
        }
        return '';
    }
}