<?php
/**
 * Soubor tvarchar.php obsahujici definici tridy CDO_TvarChar.
 */

/**
 * Trida pro skupinu tvaroslovnych charakteristik.
 */
class CDO_TvarChar extends CDO_Base
{
    /**
     * @var int $parent typ nadrazeneho zaznamu
     */
    private $parent;
    /**
     * @var bool|False $sloveso tvaroslovna charakteristika slovesa
     */
    private $sloveso;
    /**
     * @var int $cisType typ podrazeneho ciselniku (TVAR_CHAR nebo TVAR_CHAR_SL)
     */
    private $cisType;
    /**
     * @var int $tvarkomType typ podrazene skupiny dvojic tvar-komentar (KONCOVKA_TVARCHAR nebo KONCOVKA_TVARCHAR_SL)
     */
    private $tvarkomType;

    /**
     * Konstruktor tridy CDO_TvarChar.
     * @param int $input
     * @param bool|False $sloveso
     * @param bool|False $parent
     * @param bool|False $parentVal
     * @param bool|False $parentDomId
     */
    function __construct($input = CDOin_DB, $sloveso = False, $parent = False, $parentVal = False, $parentDomId = False)
    {
        $this->input = $input;
        $this->sloveso = $sloveso;
        $this->parent = $parent;
        $this->parentVal = $parentVal;
        $this->parentDomId = $parentDomId;
        $this->colsStructured = array('vyber', 'tvary');
        if ($sloveso) {
            $this->xmlWrapper = 'morph-chars-verb';
            $this->xmlElem = 'morph-char-verb';
        } else {
            $this->xmlWrapper = 'morph-chars';
            $this->xmlElem = 'morph-char';
        }
        if ($parent && $parentVal) {
            $this->translateQuery();
        }
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {
            $this->loadStructured('temp');
        }
    }

    /**
     * Vyplni informace o zdrojove DB.
     */
    private function translateQuery()
    {
        if (!$this->sloveso) {
            //tvaroslovná charakteristika
            $this->cisType = TVAR_CHAR;
            $this->tvarkomType = KONCOVKA_TVARCHAR;
            if ($this->parent == VARIANTA) {
                $this->tbl = 'var_tvar_char_vyber_rel';
                $this->parentFk = 'vid';
                $this->alias = 'varianta-';
            } elseif ($this->parent == VYZNAM) {
                $this->tbl = 'vyz_tvar_char_vyber_rel';
                $this->parentFk = 'vyz_id';
                $this->alias = 'vyznam-';
            } elseif ($this->parent == PODHESLI) {
                $this->tbl = 'podh_tvar_char_vyber_rel';
                $this->parentFk = 'podh_id';
                $this->alias = 'vyznam-podhesli-';
            }
            $this->alias .= 'tvar_char';
            $this->domStr = 'tvarChar';
        } else {
            // tvaroslovná charakteristika slovesa
            $this->cisType = TVAR_CHAR_SL;
            $this->tvarkomType = KONCOVKA_TVARCHAR_SL;
            if ($this->parent == VARIANTA) {
                $this->tbl = 'var_tvar_char_sl_vyber_rel';
                $this->parentFk = 'vid';
                $this->alias = 'varianta-';
            } elseif ($this->parent == VYZNAM) {
                $this->tbl = 'vyz_tvar_char_sl_vyber_rel';
                $this->parentFk = 'vyz_id';
                $this->alias = 'vyznam-';
            } elseif ($this->parent == PODHESLI) {
                $this->tbl = 'podh_tvar_char_sl_vyber_rel';
                $this->parentFk = 'podh_id';
                $this->alias = 'vyznam-podhesli-';
            }
            $this->alias .= 'tvar_char_sl';
            $this->domStr = 'tvarCharSl';
        }
        $this->baseDom = $this->parentDomId . '--' . $this->domStr;
    }

    public function add()
    {
        $newId = $this->newId();
        $this->domId[$newId] = $this->baseDom . '-' . $newId;
        $this->data[$newId] = new DibiRow(array(
            'vyber' => new CDO_Ciselnik(CDOin_EMPTY, $this->cisType, $this->parent, $newId, $this->domId[$newId]),
            'tvary' => new CDO_TvarKom(CDOin_EMPTY, $this->tvarkomType, $this->parent, $newId, $this->domId[$newId])
        ));
        return new Response(OK, $newId);
    }

    private function loadStructured($id)
    {
        $this->domId[$id] = $this->baseDom . '-' . $id;
        $this->data[$id] = new DibiRow(array(
            'vyber' => new CDO_Ciselnik($this->input, $this->cisType, $this->parent, $id, $this->domId[$id]),
            'tvary' => new CDO_TvarKom($this->input, $this->tvarkomType, $this->parent, $id, $this->domId[$id])
        ));
    }

    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::fetchPairs('
                SELECT id, sortorder FROM %n WHERE %n = %i
                ORDER BY sortorder ASC
            ', $prefix . $this->tbl, $this->parentFk, $this->parentVal);

        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }
        foreach ($this->data as $id => $sort) {
            $this->loadStructured($id);
        }
        return new Response(OK);
    }

    protected function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        $ord = 1;
        try {
            dibi::begin();
            $sqlData = array(
                $this->parentFk => $this->parentVal
            );
            $originalDataKeys = array_combine(array_keys($originalData), array_keys($originalData));
            foreach ($this->data as $id => $d) {
                $sqlData['sortorder'] = $ord++;
                if (isset($originalDataKeys[$id])) {
                    // update
                    dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . $this->tbl, $sqlData, $id);
                    $this->data[$id]['vyber']->saveToDB($archive, $originalData[$id]['vyber']->data);
                    $this->data[$id]['tvary']->saveToDB($archive, $originalData[$id]['tvary']->data);
                    unset($originalDataKeys[$id]);
                } else {
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . $this->tbl, $sqlData);
                    $dbid = $this->newId(True);
                    $this->data[$id]['vyber']->setParentVal($dbid);
                    $this->data[$id]['tvary']->setParentVal($dbid);
                    $this->data[$id]['vyber']->saveToDB($archive);
                    $this->data[$id]['tvary']->saveToDB($archive);
                }
            }

            foreach ($originalDataKeys as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . $this->tbl, $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }
        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param $id
     * @param bool|False $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        $html = '';
        $html .= '<div class="multiInput" id="' . $this->domId[$id] . '">';
        if (!$archive) {
            $html .= $this->rmFieldsBtn($this->domId[$id]);
        }
        $html .=
            '
                <div class="floatLeft" style="width: 120px;">
                    <div class="smallFont">typ</div>
                    <div class="clear"></div>
                    ' . $this->data[$id]->vyber->printForm($archive) . '
                </div>
                <div class="floatLeft margin4addButt">
                    ' . $this->data[$id]->tvary->printForm($archive) . '
                </div>
                <div class="hspace"></div>
            </div>
            ';
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $suffix = $this->sloveso ? ' slovesa' : '';
        $html = '';
        if (!$archive) {
            $html .= '
            <img src="css/images/mini_reorder.png" style="border:0; margin-left: 5px" class="clickable floatLeft"
               onClick="show_order_editor(\'' . $this->baseDom . '\')" alt="Uspořádat" />
            <div id="' . $this->baseDom . '_orderPopUp"></div>
            ';
        }
        $html .= '
            <div class="label floatLeft">tvaroslovná charakteristika' . $suffix . '</div>
            <div id="' . $this->baseDom . '" class="multiContainer floatLeft margin4addButt">
        ';
        if (!$this->data) {
            $this->add();
        }
        foreach ($this->data as $id => $d) {
            $html .= $this->printOne($id, $archive);
        }

        $html .= '</div>';
        if (!$archive) {
            $html .= '
                <div class="tlacitkoRemField">
                    ' . $this->addFieldsBtn($this->baseDom, 'charakteristiku') . '
                </div>
            ';
        }
        return $html;
    }

    /**
     * Vygeneruje editor poradi.
     * @return string
     */
    public function printOrderEdit()
    {
        $html = '
            <table>
                <tr>
                    <td rowspan="2">
                        <select id="' . $this->baseDom . '--orderer" class="ordererSB" multiple="multiple">
        ';
        foreach ($this->data as $id => $d) {
            $html .= '<option value="' . $id . '" data-dom="' . $this->domId[$id] . '">' . $this->data[$id]->vyber->getValsList(2, ';') . '</option>';
        }
        $html .= '
                        </select>
                    </td>
                    <td>
                        <input type="button" class="sortUp_butt" value="&nbsp;Up&nbsp;" onClick="Selectbox.moveOptionUp(document.getElementById(\'' . $this->baseDom . '--orderer\'))" />
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="button" class="sortDown_butt" value="&nbsp;Down&nbsp;" onClick="Selectbox.moveOptionDown(document.getElementById(\'' . $this->baseDom . '--orderer\'))" />
                    </td>
                </tr>
            </table>
        ';
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';
        $ids = array_keys($this->data);
        foreach ($ids as $id) {
            $xml .= '<' . $this->xmlElem . ' id="' . $id . '">';
            foreach ($this->colsStructured as $col) {
                $xml .= $this->data[$id]->{$col}->printXML();
            }
            $xml .= '</' . $this->xmlElem . '>';
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $tbl = $this->tbl;
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        $sqlData = array(
            $this->parentFk => $archiveParentVal,
        );

        $ord = 1;
        foreach ($this->data as $id => $d) {
            $sqlData['sortorder'] = $ord++;
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
            $this->data[$id]['vyber']->saveToArchive($tmp->data);
            $this->data[$id]['tvary']->saveToArchive($tmp->data);
        }
        return new Response(OK);
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        if ($this->sloveso) {
            $tbl = 'cis_tvar_char_sl';
            $col = 'tvar_char_sl';
        } else {
            $tbl = 'cis_tvar_char';
            $col = 'tvar_char';
        }
        $headers = dibi::fetchPairs('SELECT id, %n FROM %n WHERE %n IN %in ORDER BY sortorder ASC', $col, $tbl, $col, array_keys($fields));
        if (isset($fields['jine'])) {
            $headers['jine'] = 'jine';
        }
        $headersf = array_flip($headers);
        foreach ($headersf as $i => $h) {
            $headersf[$i] = array();
        }
        foreach ($this->data as $id => $d) {
            foreach ($headers as $hid => $h) {
                $vyber = $this->data[$id]['vyber']->isPresent($hid);
                if ($vyber) {
                    if ($hid == 'jine') {
                        $prefix = $vyber . ' ';
                    } else {
                        $prefix = '';
                    }
                    $headersf[$h] = array_merge($headersf[$h], $this->data[$id]['tvary']->getTvarKomList($prefix, $wrap));
                }
            }
        }
        foreach ($headersf as $h) {
            $data[count($data) - 1][] = implode(', ', $h);
        }
        return new Response(OK, array('headers' => $headers));
    }

    protected function getSQLFields($fields)
    {
        return array('vyber' => 1, 'tvary' => 1);
    }

    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        if ($this->sloveso) {
            $tbl = 'cis_tvar_char_sl';
            $col = 'tvar_char_sl';
        } else {
            $tbl = 'cis_tvar_char';
            $col = 'tvar_char';
        }
        if ($fields && is_array($fields)) {
            $headers = dibi::fetchPairs('SELECT %n, "" AS Str FROM %n WHERE %n IN %in ORDER BY sortorder ASC', $col, $tbl, $col, array_keys($fields));
        } else {
            $headers = dibi::fetchPairs('SELECT %n, "" AS Str FROM %n ORDER BY sortorder ASC', $col, $tbl);
        }
        if (!$headers) {
            return '';
        }
        $return = $headers;
        if (isset($fields['jine'])) {
            $return['jine'] = array();
            $jine = True;
        } else {
            $jine = False;
        }
        foreach ($data as $did => $entry) {
            // $did == tvar_char-<id>
            $tvchid = explode('-', $did);
            $vyber = $entry['vyber'];
            $tvary = $entry['tvary'];
            $domBase = $parentStr . '--' . $this->domStr . '-' . $tvchid[1];

            $tmp = array();
            foreach ($tvary as $tid => $tvar) {
                // $tid == id tvarkomu
                $id = explode('-', $tid);
                $tv = wrap_output($domBase . '--koncovka-' . $id[1] . '--tvar', $tvar['tvar'], WRAP_CORRECTION);
                $komentar = wrap_output($domBase . '--koncovka-' . $id[1] . '--komentar', $tvar['komentar'], WRAP_CORRECTION);
                $tmp[] = $tv . ($tvar['komentar'] ? ' (' . $komentar . ')' : '');
            }
            $tmp = implode(', ', $tmp);

            foreach ($vyber as $vid => $v) {
                if ($jine && isset($v['jine']) && !is_null($v['jine']) && $v['jine']) {
                    if (!isset($return['jine'][$v['jine']])) {
                        $return['jine'][$v['jine']] = array();
                    }
                    $return['jine'][$v['jine']][] = $tmp;
                } elseif (isset($return[$v['vyber']])) {
                    $return[$v['vyber']][] = $tmp;
                }
            }
        }
        if ($jine) {
            foreach ($return['jine'] as $id => $rj) {
                $return['jine'][$id] = $id . ': ' . implode(', ', $rj);
            }
        }
        foreach ($return as $id => $r) {
            if (is_array($r)) {
                $return[$id] = implode(', ', $r);
            }
        }
        return $return;
    }
}