<?php
/**
 * Soubor tvarkom.php obsahujici definici tridy CDO_TvarKom.
 */

/**
 * Trida pro skupinu dvojic tvar-komentar.
 */
class CDO_TvarKom extends CDO_Base
{
    /**
     * @var int $type typ dvojice tvar-komentar
     */
    private $type;
    /**
     * @var int $parent typ nadrazeneho zaznamu
     */
    private $parent;
    /**
     * @var int $parentLinked je nadrazeny zaznam dedeny
     */
    private $parentLinked;
    /**
     * @var string $defTbl
     */
    private $defTbl;
    /**
     * @var string $tvarCol nazev sloupce v DB, ktery obsahuje tvar
     */
    private $tvarCol;
    /**
     * @var string $komCol nazev sloupce v DB, ktery obsahuje komentar
     */
    private $komCol;
    /**
     * @var bool $isSortable daji se dvojice tvar-komentar radit
     */
    private $isSortable = True;
    /**
     * @var array $htmlMeta informace pro html výstup
     */
    private $htmlMeta = array();
    /**
     * @var string $linkOld
     */
    private $linkOld;

    /**
     * Konstruktor tridy CDO_TvarKom.
     * @param int $input
     * @param bool|False $type
     * @param bool $parent
     * @param bool $parentVal
     * @param bool $parentDomId
     * @param int $linked
     */
    function __construct($input = CDOin_DB, $type = False, $parent = False, $parentVal = False, $parentDomId = False, $linked = 0)
    {
        $this->input = $input;
        $this->parent = $parent;
        $this->parentVal = $parentVal;
        $this->parentLinked = $linked;
        $this->linkOld = $linked;
        $this->parentDomId = $parentDomId;
        $this->type = $type;
        $this->xmlWrapper = 'expressions';
        $this->xmlElem = 'expression';
        if ($type && $parent && $parentVal) {
            $this->translateQuery($type);
        }
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {

        }
    }

    /**
     * Setter pro parentLinked.
     * @param int $parentLinked
     */
    public function setParentLinked($parentLinked)
    {
        $this->parentLinked = $parentLinked;
    }


    /**
     * Vyplni informace o zdrojove DB dle typu dvojice tvar-komentar.
     * @param int $q typ dvojice tvar-komentar
     */
    private function translateQuery($q)
    {
        $this->isSortable = True;
        $this->htmlMeta = array(
            'komLabel' => '',
            'wrapperClass' => array('multiContainer'),
            'tvarStyle' => array('width' => '50px'),
            'komStyle' => array('width' => '150px'),
            'addBtnStyle' => array()
        );
        $this->alias = '';
        switch ($q) {
            case VYSLOVNOST:
                $this->tvarCol = 'vyslovnost';
                $this->komCol = 'komentar';
                $this->isSortable = False;
                if ($this->parent == VARIANTA) {
                    $this->tbl = 'vyslovnost_var';
                    $this->parentFk = 'vid';
                    $this->alias = 'varianta-vyslovnost';
                    $this->htmlMeta['komStyle']['width'] = '150px';
                } elseif ($this->parent == PODHESLI) {
                    $this->tbl = 'podh_vyz_vyslovnost';
                    $this->parentFk = 'podh_id';
                    $this->alias = 'vyznam-podhesli-vyslovnost';
                    $this->htmlMeta['komStyle']['width'] = '150px';
                }
                $this->domStr = 'vyslovnost';
                $this->htmlMeta['komLabel'] = 'kom.:';
                $this->htmlMeta['addBtnStyle']['left'] = '-27px';
                $this->xmlWrapper = 'pronunciations';
                break;
            case KONCOVKA_TVARCHAR:
                $this->tvarCol = 'koncovka';
                $this->komCol = 'komentar';
                if ($this->parent == VARIANTA) {
                    $this->tbl = 'koncovky_tvar_char';
                    $this->parentFk = 'var_tvar_char_rel_id';
                    $this->alias = 'varianta-tvar_char-tvary';
                } elseif ($this->parent == VYZNAM) {
                    $this->tbl = 'koncovky_tvar_char_vyz';
                    $this->parentFk = 'vyz_tvar_char_rel_id';
                    $this->alias = 'vyznam-tvar_char-tvary';
                } elseif ($this->parent == PODHESLI) {
                    $this->tbl = 'podh_vyz_koncovky_tvar_char';
                    $this->parentFk = 'podh_tvch';
                    $this->alias = 'vyznam-podhesli-tvar_char-tvary';
                }
                $this->domStr = 'koncovka';
                $this->htmlMeta['addBtnStyle']['left'] = '-27px';
                $this->xmlWrapper = 'suffixes';
                break;
            case KONCOVKA_TVARCHAR_SL:
                $this->tvarCol = 'koncovka';
                $this->komCol = 'komentar';
                if ($this->parent == VARIANTA) {
                    $this->tbl = 'koncovky_tvar_char_sl';
                    $this->parentFk = 'var_tvar_char_sl_rel_id';
                    $this->alias = 'varianta-tvar_char_sl-tvary';
                } elseif ($this->parent == VYZNAM) {
                    $this->tbl = 'koncovky_tvar_char_sl_vyz';
                    $this->parentFk = 'vyz_tvar_char_sl_rel_id';
                    $this->alias = 'vyznam-tvar_char_sl-tvary';
                } elseif ($this->parent == PODHESLI) {
                    $this->tbl = 'podh_vyz_koncovky_tvar_char_sl';
                    $this->parentFk = 'podh_tvch_sl';
                    $this->alias = 'vyznam-podhesli-tvar_char_sl-tvary';
                }
                $this->domStr = 'koncovka';
                $this->htmlMeta['addBtnStyle']['left'] = '-27px';
                $this->xmlWrapper = 'suffixes';
                break;
            case VAZEBNOST:
                $this->isSortable = False;
                $this->tvarCol = 'vazebnost';
                $this->komCol = 'komentar';
                if ($this->parent == VARIANTA) {
                    $this->tbl = 'vazebnost_var';
                    $this->parentFk = 'vid';
                    $this->alias = 'varianta-vazebnost';
                    $this->htmlMeta['tvarStyle']['width'] = '350px';
                    $this->htmlMeta['komStyle']['width'] = '310px';
                } elseif ($this->parent == VYZNAM) {
                    $this->tbl = 'vazebnost_vyz';
                    $this->parentFk = 'vyz_id';
                    $this->alias = 'vyznam-vazebnost';
                    $this->htmlMeta['tvarStyle']['width'] = '775px';
                    $this->htmlMeta['komStyle']['width'] = '780px';
                } elseif ($this->parent == PODHESLI) {
                    $this->tbl = 'podhesli_vazebnost';
                    $this->parentFk = 'podh_id';
                    $this->alias = 'vyznam-podhesli-vazebnost';
                    $this->htmlMeta['tvarStyle']['width'] = '775px';
                    $this->htmlMeta['komStyle']['width'] = '310px';
                }
                $this->htmlMeta['komStyle']['margin'] = '5px 0 5px 0';
                //$this->htmlMeta['wrapperClass'][] = 'margin4addButt';
                $this->domStr = 'vazebnost';
                $this->htmlMeta['komLabel'] = 'kom.:';
                $this->xmlWrapper = 'valencies';
                break;
            case PREDVYKLAD:
                if ($this->parentLinked) {
                    $this->tbl = 'vyrazy_predvykladu_m';
                    $this->domStr = 'predvyklad_master';
                    $this->defTbl = 'vyrazy_predvykladu';
                } else {
                    $this->tbl = 'vyrazy_predvykladu';
                    $this->domStr = 'predvyklad';
                }
                $this->tvarCol = 'vyraz';
                $this->komCol = 'komentar';
                $this->parentFk = 'vyz_id';
                $this->alias = 'vyznam-predvyklad';
                $this->htmlMeta['addBtnStyle']['left'] = '-27px';
                $this->xmlWrapper = 'prel-meaning';
                break;
        }
        if (!isset($this->defTbl)) {
            $this->defTbl = $this->tbl;
        }
        $this->baseDom = $this->parentDomId . '--' . $this->domStr;
        if ($this->htmlMeta['komLabel']) {
            $this->htmlMeta['komLabel'] = '<span class="label">' . $this->htmlMeta['komLabel'] . '</span>';
        }

        $this->htmlMeta['tvarStyleStr'] = implode(';', array_map(function ($k, $v) {
            return $k . ':' . $v;
        }, array_keys($this->htmlMeta['tvarStyle']), $this->htmlMeta['tvarStyle']));


        $this->htmlMeta['komStyleStr'] = implode(';', array_map(function ($k, $v) {
            return $k . ':' . $v;
        }, array_keys($this->htmlMeta['komStyle']), $this->htmlMeta['komStyle']));

        $this->htmlMeta['addBtnStyleStr'] = implode(';', array_map(function ($k, $v) {
            return $k . ':' . $v;
        }, array_keys($this->htmlMeta['addBtnStyle']), $this->htmlMeta['addBtnStyle']));

        $this->htmlMeta['tvarStyleStr'] = 'style="' . $this->htmlMeta['tvarStyleStr'] . '"';
        $this->htmlMeta['komStyleStr'] = 'style="' . $this->htmlMeta['komStyleStr'] . '"';
        $this->htmlMeta['addBtnStyleStr'] = 'style="' . $this->htmlMeta['addBtnStyleStr'] . '"';
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array();
        $transl = array();
        foreach ($this->domId as $id => $dom) {
            array_push($domList, $dom . '--tvar', $dom . '--komentar');
        }
        if ($this->parentLinked) {
            foreach ($domList as $l) {
                $transl[$l] = str_replace($this->parentVal, $this->parentLinked, $l);
            }
        }
        $this->corr = $this->loadCorrectionsByDom($domList, $transl);
    }

    /**
     * Prepne mezi dedenym/nededenym
     * @param $linked
     * @param bool $load
     */
    public function switchLinked($linked, $load = False)
    {
        $this->corr = array();
        $this->domId = array();
        $this->parentLinked = $linked;
        $this->translateQuery($this->type);
        foreach ($this->data as $id => $d) {
            $this->domId[$id] = $this->baseDom . '-' . $id;
        }
        $this->loadCorrections();
        if ($load) {
            $this->loadFromDB();
        }
    }

    /**
     * Pridani nove dvojice tvar-komentar.
     * @param string $tvar tvar
     * @param string $kom komentar
     * @return Response
     */
    public function add($tvar = '', $kom = '')
    {
        $tvar = trim($tvar);
        $kom = trim($kom);
        $newId = $this->newId();
        $this->data[$newId] = new DibiRow(array(
            'tvar' => $tvar,
            'komentar' => $kom
        ));
        $this->domId[$newId] = $this->baseDom . '-' . $newId;
        return new Response(OK, $newId);
    }

    /**
     * Nacteni dat z databaze do objektu.
     * @param bool|False $archive nacist z archivu
     * @return int navratova hodnota
     */
    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        $sortcol = $this->isSortable ? 'sortorder' : 'id';
        if ($this->parentLinked && !is_numeric($this->parentLinked)) {
            $tbl = $this->defTbl;
            $fk = substr($this->parentLinked, 3);
        } else {
            $tbl = $this->tbl;
            $fk = $this->parentLinked ? $this->parentLinked : $this->parentVal;
        }
        try {
            $this->data = dibi::query('
                SELECT id, %n AS [tvar], %n AS [komentar]
                FROM %n
                WHERE %n = %i
                ORDER BY %n ASC
            ', $this->tvarCol, $this->komCol, $prefix . $tbl, $this->parentFk, $fk, $sortcol)->fetchAssoc('id');
        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }

        foreach ($this->data as $id => $d) {
            $this->domId[$id] = $this->baseDom . '-' . $id;
        }
        return new Response(OK);
    }

    /**
     * Ulozeni dat z objektu do DB.
     * @param bool|False $archive ulozit do archivu
     * @param array $originalData puvodni data v DB
     * @return int navratova hodnota
     */
    protected function saveToDB($archive = False, $originalData = array())
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        $newlyLinked = $this->parentLinked && !$this->linkOld;
        $newlyUnlinked = !$this->parentLinked && $this->linkOld;

        if (!$newlyLinked && !$newlyUnlinked && CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        $ord = 1;
        try {
            dibi::begin();
            // pokud dojde k přidání / odebrání ze skupiny, vymažeme data z neskupinové tabulky
            if ($newlyUnlinked || $newlyLinked) {
                $originalData = array();
                dibi::query('DELETE FROM %n WHERE %n = %i', $this->defTbl, $this->parentFk, $this->parentVal);
                if ($newlyLinked) {
                    // jako original data musíme vzít data ze skupiny
                    $originalData = dibi::fetchPairs('
                        SELECT id, 1 AS tmp FROM %n WHERE %n = %i
                    ', $prefix . $this->tbl, $this->parentFk, $this->parentLinked);
                }
            }
            foreach ($this->data as $id => $d) {
                $sqlData = array(
                    $this->parentFk => $this->parentLinked ? $this->parentLinked : $this->parentVal,
                    $this->tvarCol => $d['tvar'],
                    $this->komCol => $d['komentar']
                );
                if ($this->isSortable) {
                    $sqlData['sortorder'] = $ord++;
                }
                if (isset($originalData[$id])) {
                    // update
                    dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . $this->tbl, $sqlData, $id);
                    unset($originalData[$id]);
                } else {
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . $this->tbl, $sqlData);
                }
            }

            dibi::query('
                DELETE FROM %n WHERE id IN %in
            ', $prefix . $this->tbl, array_keys($originalData));

            dibi::commit();
        } catch (DibiException $e) {
            dibi::rollback();
            return handle_db_error($e);
        }

        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param int $id
     * @param bool $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        $tvar = $this->data[$id]->tvar;
        $kom = $this->data[$id]->komentar;
        $html = '';
        $tvarDom = $this->domId[$id] . '--tvar';
        $komDom = $this->domId[$id] . '--komentar';
        if ($archive) {
            $ro = ' readonly="readonly"';
            $tvarCorr = $this->emptyCorrectionArray();
            $komCorr = $this->emptyCorrectionArray();
        } else {
            $ro = '';
            $tvarCorr = $this->displayCorrection($this->corr[$tvarDom]['field_DOMid'], $this->corr[$tvarDom]['id'], $this->corr[$tvarDom]['field'], $this->corr[$tvarDom]['nova']);
            $komCorr = $this->displayCorrection($this->corr[$komDom]['field_DOMid'], $this->corr[$komDom]['id'], $this->corr[$komDom]['field'], $this->corr[$komDom]['nova']);
        }
        if ($this->parent == VYZNAM && $this->parentLinked) {
            $onclick = 'varovaniSkup(this, \'' . $this->parentVal . '\')';
        } else {
            $onclick = '';
        }
        if ($this->type == VAZEBNOST) {
            $html .= '<div id="' . $this->domId[$id] . '" class="multiInput">';
            if (!$archive) {
                $html .= '
                    <div class="center butt blueBG symbolButt" onclick="insert_symbol(\'|\', \'' . $tvarDom . '--helperCpy\')">|</div>
                    <div class="center butt blueBG symbolButt" onclick="insert_symbol(\'~\', \'' . $tvarDom . '--helperCpy\')">~</div>
                    <div class="center butt blueBG symbolButt" onclick="insert_symbol(\'{}\', \'' . $tvarDom . '--helperCpy\')">{}</div>
                    <div class="center butt blueBG symbolButt" onclick="insert_symbol(\'„“\', \'' . $tvarDom . '--helperCpy\')">„“</div>
                ';
            }
            $html .= '

                    <div>&nbsp;</div>
                    <div id="' . $tvarDom . '" class="correctableFieldContainer clear">
                        ' . $tvarCorr['flag'] . '
                        <textarea id="' . $tvarDom . '--helperCpy" class="flatBox lightBG padd oneRowTextarea autosize ' . $tvarCorr['class'] . '"
                            ' . $this->htmlMeta['tvarStyleStr'] . $ro . ' onblur="' . $this->intputOnBlur($tvarDom, $archive) . '">' . $tvar . '</textarea>
                    </div>
                    <span class="label">kom.:</span><br/>
                    <span id="' . $komDom . '" class="correctableFieldContainer">' . $komCorr['flag'] . '
                        <input type="text" value="' . $kom . '" class="textField ' . $komCorr['class'] . '"
                            ' . $this->htmlMeta['komStyleStr'] . $ro . ' onblur="' . $this->intputOnBlur($komDom, $archive) . '" />
                    </span>
                </div>
            ';
        } else {
            $html .= '
                <div id="' . $this->domId[$id] . '" class="multiInput">
                    ' . ($archive ? '' : $this->rmFieldsBtn($this->domId[$id])) . '
                    <span id="' . $tvarDom . '" class="correctableFieldContainer">
                        ' . $tvarCorr['flag'] . '
                        <input type="text" value="' . $this->data[$id]['tvar'] . '"
                        class="textField' . ($this->isSortable ? ' value2sort' : '') . ' ' . $tvarCorr['class'] . '" ' . $this->htmlMeta['tvarStyleStr'] . $ro . '
                        onclick="' . $onclick . '"
                        onblur="' . $this->intputOnBlur($tvarDom, $archive) . '" />
                    </span>
                    ' . $this->htmlMeta['komLabel'] . '
                    <span id="' . $komDom . '" class="correctableFieldContainer">
                        ' . $komCorr['flag'] . '
                        <input type="text" value="' . $this->data[$id]['komentar'] . '" class="textField ' . $komCorr['class'] . '" ' . $this->htmlMeta['komStyleStr'] . $ro . '
                        onclick="' . $onclick . '"
                        onblur="' . $this->intputOnBlur($komDom, $archive) . '" />
                    </span>
                </div>
            ';
        }

        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '';
        $html .= '
                <div id="' . $this->baseDom . '" class="' . implode(' ', $this->htmlMeta['wrapperClass']) . '">
            ';
        if ($this->isSortable) {
            $s = "tvar";
            if($this->defTbl == "vyrazy_predvykladu"){
                $s = "výraz";
            }
            $html .= '
                <div class="smallFont">
                    <div class="floatLeft" style="width: 75px">'.$s;
            

            if (!$archive) {
                $html .= '
                    <img src="css/images/mini_reorder.png" style="border:0; margin-left: 5px" class="clickable"
                            onClick="show_order_editor(\'' . $this->baseDom . '\')" alt="Uspořádat" />
                ';
            }

            $html .= '
                    </div>
                    <div class="floatLeft">komentáře</div>
                </div>
                <div class="clear"></div>
                <div id="' . $this->baseDom . '_orderPopUp"></div>
            ';
        }
        if (!$this->data) {
            $this->add();
        }
        foreach ($this->data as $id => $d) {
            $html .= $this->printOne($id, $archive);
        }

        $html .= '</div>';
        if (!$archive) {
            if ($this->type != VAZEBNOST) {
                $html .= '
                  <div class="tlacitkoRemField" ' . $this->htmlMeta['addBtnStyleStr'] . '>
                      ' . $this->addFieldsBtn($this->baseDom, 'tvar') . '
                  </div>
              ';
            }
        }

        return $html;
    }

    /**
     * Vygeneruje editor poradi.
     * @return string
     */
    public function printOrderEdit()
    {
        $html = '
            <table>
                <tr>
                    <td rowspan="2">
                        <select id="' . $this->baseDom . '--orderer" class="ordererSB" multiple="multiple">
        ';
        foreach ($this->data as $id => $d) {
            $html .= '<option value="' . $id . '" data-dom="' . $this->domId[$id] . '">' . $d['tvar'] . '</option>';
        }
        $html .= '
                        </select>
                    </td>
                    <td>
                        <input type="button" class="sortUp_butt" value="&nbsp;Up&nbsp;" onClick="Selectbox.moveOptionUp(document.getElementById(\'' . $this->baseDom . '--orderer\'))" />
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="button" class="sortDown_butt" value="&nbsp;Down&nbsp;" onClick="Selectbox.moveOptionDown(document.getElementById(\'' . $this->baseDom . '--orderer\'))" />
                    </td>
                </tr>
            </table>
        ';
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';

        foreach ($this->data as $id => $d) {
            $xml .= '
                <' . $this->xmlElem . ' id="' . $id . '">
                    <text>' . $d->tvar . '</text>
                    <comment>' . $d->komentar . '</comment>
                </' . $this->xmlElem . '>
            ';
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        // abychom neukládali do _master tabulek
        $tbl = $this->defTbl;
        $atbl = ARCHIVE_PREFIX . $this->defTbl;
        $ord = 1;
        foreach ($this->data as $id => $d) {
            $sqlData = array(
                $this->parentFk => $archiveParentVal,
                $this->tvarCol => $d['tvar'],
                $this->komCol => $d['komentar'],
            );
            if (is_numeric($id)) {
                $sqlData['main_id'] = $id;
            }
            if ($this->isSortable) {
                $sqlData['sortorder'] = $ord++;
            }
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }

        }
        return new Response(OK);
    }

    public function getTvarKomList($prefix = '', $wrap = WRAP_NONE)
    {
        $strArr = array();
        foreach ($this->data as $id => $d) {
            $strArr[] = $prefix . wrap_output($this->getDomId($id, 'tvar'), $d['tvar'], $wrap) .
                ($d['komentar'] ? ' (' . wrap_output($this->getDomId($id, 'komentar'), $d['komentar'], $wrap) . ')' : '');
        }
        return $strArr;
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $data[count($data) - 1][] = implode(', ', $this->getTvarKomList(False, $wrap));
        $headers = array($this->tbl);

        return new Response(OK, array('headers' => $headers));
    }

    public function getSQL($fields = False, $parentTable = False)
    {
        $return = array(
            'columns' => array(
                '[' . $this->tbl . '].[id] AS [' . $this->alias . '--id]',
                '[' . $this->tbl . '].[' . $this->tvarCol . '] AS [' . $this->alias . '--tvar]',
                '[' . $this->tbl . '].[' . $this->komCol . '] AS [' . $this->alias . '--komentar]'),
            'tables' => array(
                '[' . $this->tbl . ']' => array(
                    'main' => array('[' . $parentTable . '].[id]', '[' . $this->tbl . '].[' . $this->parentFk . ']'),
                    'child' => array()
                )
            )
        );
        return new Response(OK, $return);
    }

    public function tabFormat($data, $fields = array(), $parentStr = '')
    {
        $linked = False;
        $tmp = array();
        $domtmp = array();
        if ($this->type == PREDVYKLAD) {
            $domtmp = explode('-', $parentStr);
            $vyz_id = $domtmp[count($domtmp) - 1];
            unset($domtmp[count($domtmp) - 1]);
            $linked = dibi::fetchSingle('
              SELECT vyklad_vyznamu_link FROM vyznamy
              WHERE vyklad_vyznamu_link IS NOT NULL AND predvyklad_link = 1 AND id = %i
            ', $vyz_id);
        }
        if ($linked) {
            $data = dibi::query('
                SELECT CONCAT("predvyklad-", id) AS id, %n AS [tvar], %n AS [komentar]
                FROM vyrazy_predvykladu_m
                WHERE vyz_id = %i
                ORDER BY sortorder ASC
            ', $this->tvarCol, $this->komCol, $linked)->fetchAssoc('id');
            $parentStr = implode('-', $domtmp) . '-' . $linked;
        }
        foreach ($data as $did => $entry) {
            //$did == 'heslo-6508' např.
            $id = explode('-', $did);
            $tvar = wrap_output($parentStr . '--' . $this->domStr . '-' . $id[1] . '--tvar', $entry['tvar'], WRAP_CORRECTION);
            $komentar = wrap_output($parentStr . '--' . $this->domStr . '-' . $id[1] . '--komentar', $entry['komentar'], WRAP_CORRECTION);
            $tmp[] = $tvar . ($entry['komentar'] ? ' (' . $komentar . ')' : '');
        }
        return implode(', ', $tmp);
    }
}