<?php
/**
 * Soubor varianta.php obsahujici definici tridy CDO_Varianta.
 */

/**
 * Trida pro skupinu variant.
 */
class CDO_Varianta extends CDO_Base
{
    /**
     * Konstruktor tridy CDO_Varianta.
     * @param int $input
     * @param bool $parentVal
     */
    function __construct($input = CDOin_DB, $parentVal = False)
    {
        $this->input = $input;
        $this->parentVal = $parentVal;
        $this->alias = 'varianta';
        $this->tbl = 'varianty';
        $this->parentFk = 'hid';
        $this->colsStructured = array('typ_hesla', 'vyslovnost', 'slov_druh', 'tvar_char', 'tvar_char_sl', 'vazebnost', 'styl_kval', 'gram_info', 'tematicke_okruhy', 'puvod', 'obor_uziti');
        $this->colsSimple = array('varianta', 'neosemantismus', 'homonymie', 'komentar', 'komentar_za', 'prvni_reg_vyskyt',
            'komentar_tvarchar', 'komentar_tvarchar_slovesa', 'varianty_bez_zavor', 'varianta_bd', 'f_varianta', 'komentar_tvaru', 'komentar_tvaru_bz');
        $this->domStr = 'varianta';
        $this->baseDom = 'varianta';
        $this->xmlWrapper = 'variants';
        $this->xmlElem = 'variant';
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {
            $this->loadStructured('temp');
        }
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array();
        foreach ($this->domId as $id => $dom) {
            array_push($domList, $dom . '--homonymie');
            array_push($domList, $dom . '--tvar');
            array_push($domList, $dom . '--komentar');
            array_push($domList, $dom . '--komentarza');
            array_push($domList, $dom . '--komentar_tvaru');
            array_push($domList, $dom . '--tvar_podhesli');
            array_push($domList, $dom . '--komentar_podhesli');
            array_push($domList, $dom . '--komentar_tvarchar');
            array_push($domList, $dom . '--komentar_tvarchar_slovesa');
            array_push($domList, $dom . '--prvni_registr_vyskyt');
        }
        $this->corr = $this->loadCorrectionsByDom($domList);
    }

    /**
     * Prida novou variantu.
     */
    public function add()
    {
        $newId = $this->newId();
        $this->domId[$newId] = 'varianta-' . $newId;
        $this->data[$newId] = new DibiRow(array(
            'varianta' => '',
            'neosemantismus' => 0,
            'homonymie' => '',
            'komentar' => '',
            'komentar_za' => '',
            'prvni_reg_vyskyt' => '',
            'komentar_tvarchar' => '',
            'komentar_tvarchar_slovesa' => '',
            'varianty_bez_zavor' => '',
            'f_varianta' => 1,
            'komentar_tvaru' => '',
            'typ_hesla' => new CDO_Ciselnik(CDOin_EMPTY, TYP_HESLA, VARIANTA, $newId, $this->domId[$newId]),
            'vyslovnost' => new CDO_TvarKom(CDOin_EMPTY, VYSLOVNOST, VARIANTA, $newId, $this->domId[$newId]),
            'slov_druh' => new CDO_SlovDruh(CDOin_EMPTY, VARIANTA, $newId, $this->domId[$newId]),
            'tvar_char' => new CDO_TvarChar(CDOin_EMPTY, False, VARIANTA, $newId, $this->domId[$newId]),
            'tvar_char_sl' => new CDO_TvarChar(CDOin_EMPTY, True, VARIANTA, $newId, $this->domId[$newId]),
            'vazebnost' => new CDO_TvarKom(CDOin_EMPTY, VAZEBNOST, VARIANTA, $newId, $this->domId[$newId]),
            'styl_kval' => new CDO_Ciselnik(CDOin_EMPTY, STYL_KVAL, VARIANTA, $newId, $this->domId[$newId]),
            'gram_info' => new CDO_Ciselnik(CDOin_EMPTY, GRAM_INFO, VARIANTA, $newId, $this->domId[$newId]),
            'tematicke_okruhy' => new CDO_Ciselnik(CDOin_EMPTY, TEMATICKE_OKRUHY, VARIANTA, $newId, $this->domId[$newId]),
            'puvod' => new CDO_Ciselnik(CDOin_EMPTY, PUVOD, VARIANTA, $newId, $this->domId[$newId]),
            'obor_uziti' => new CDO_Ciselnik(CDOin_EMPTY, OBOR_UZITI, VARIANTA, $newId, $this->domId[$newId]),
        ));
        $this->data[$newId]->typ_hesla->data[1] = "jednoslovné";       
        return new Response(OK, $newId);
    }

    public function update($id, $field, $data, $data2 = False)
    {
        $resp = parent::update($id, $field, $data);
        if ($resp->status == OK) {
            if ($field == 'varianta') {
                $this->data[$id]->varianty_bez_zavor = bz($this->data[$id]->varianta);
                $this->data[$id]->varianta_bd = bd($this->data[$id]->varianty_bez_zavor);
            } elseif ($field == 'komentar_tvaru') {
                $this->data[$id]->komentar_tvaru_bz = bz($this->data[$id]->komentar_tvaru);
            }
        }
        return $resp;
    }


    public function copyLast()
    {
        $tmp = array_keys($this->data);
        $lastId = $tmp[count($tmp) - 1];
        $clone = $this->getClone(False, $lastId);
        $k = array_keys($clone->data);
        $this->data[$k[0]] = $clone->data[$k[0]];
        $this->recreateDomIds($this->parentVal);
        return new Response(OK, $k[0]);
    }

    private function loadStructured($id)
    {
        $this->domId[$id] = $this->domStr . '-' . $id;
        $this->data[$id]['typ_hesla'] = new CDO_Ciselnik($this->input, TYP_HESLA, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['vyslovnost'] = new CDO_TvarKom($this->input, VYSLOVNOST, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['slov_druh'] = new CDO_SlovDruh($this->input, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['tvar_char'] = new CDO_TvarChar($this->input, False, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['tvar_char_sl'] = new CDO_TvarChar($this->input, True, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['vazebnost'] = new CDO_TvarKom($this->input, VAZEBNOST, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['styl_kval'] = new CDO_Ciselnik($this->input, STYL_KVAL, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['gram_info'] = new CDO_Ciselnik($this->input, GRAM_INFO, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['tematicke_okruhy'] = new CDO_Ciselnik($this->input, TEMATICKE_OKRUHY, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['puvod'] = new CDO_Ciselnik($this->input, PUVOD, VARIANTA, $id, $this->domId[$id]);
        $this->data[$id]['obor_uziti'] = new CDO_Ciselnik($this->input, OBOR_UZITI, VARIANTA, $id, $this->domId[$id]);
    }

    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::query('
				SELECT id, varianta, neosemantismus, homonymie, komentar, komentar_za, prvni_reg_vyskyt, sortorder, uts, 
					komentar_tvarchar, komentar_tvarchar_slovesa, varianty_bez_zavor, f_varianta, komentar_tvaru, varianta_bd
				FROM %n WHERE %n = %i
				ORDER BY sortorder ASC
			', $prefix . $this->tbl, $this->parentFk, $this->parentVal)->fetchAssoc('id');

        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }

        foreach ($this->data as $id => $d) {
            $this->loadStructured($id);
        }
        return new Response(OK);
    }

    protected function saveToDB($archive = False, $originalData = array())
    {

        $prefix = $archive ? ARCHIVE_PREFIX : '';
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }

        try {
            dibi::begin();
            $ord = 1;
            $originalDataKeys = array_combine(array_keys($originalData), array_keys($originalData));
            foreach ($this->data as $id => $d) {
                $sqlData = array(
                    'hid' => $this->parentVal,
                    'sortorder' => $ord++
                );
                foreach ($this->colsSimple as $col) {
                    $sqlData[$col] = $this->data[$id][$col];
                }
                if (isset($originalDataKeys[$id])) {
                    // update
                    dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . $this->tbl, $sqlData, $id);
                    unset($originalDataKeys[$id]);
                    foreach ($this->colsStructured as $col) {
                        $this->data[$id][$col]->saveToDB($archive, $originalData[$id][$col]->data);
                    }
                } else {
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . $this->tbl, $sqlData);
                    $dbid = $this->newId(True);
                    foreach ($this->colsStructured as $col) {
                        $this->data[$id][$col]->setParentVal($dbid);
                        $this->data[$id][$col]->saveToDB($archive);
                    }
                }
            }

            foreach ($originalDataKeys as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . $this->tbl, $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }

        return new Response(OK);
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param int $id
     * @param bool $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        $podh_vystupArr = array('podheslí ... číselníky', 'číselníky ... podheslí');
        $dom = $this->domId[$id];
        if ($archive) {
            $corr = $this->emptyCorrectionArray();
        } else {
            $tvarDomId = $dom . "--komentar_tvaru";
            $corr = $this->displayCorrection($this->corr[$tvarDomId]['field_DOMid'], $this->corr[$tvarDomId]['id'], $this->corr[$tvarDomId]['field'], $this->corr[$tvarDomId]['nova']);
        }

        $html = '
        <div class="varianta w50 floatLeft " id="' . $dom . '" rel="' . $id . '">
            <div class="orangeBox">
                <table>
                    <thead></thead>
                    <tbody>
                        <tr>
                            <td>
                                ' . $this->printCheckboxImg($dom . '--f_varianta', $archive, $this->data[$id]->f_varianta, 'butt_varianta_switch_%val.png', array('var_podh_switch')) . '
                            </td>
                            <td>
                                <div>
                                    <span class="label">neosémantismus:</span>
                                    ' . $this->printCheckbox($dom . '--neosemantismus', $archive, $this->data[$id]->neosemantismus, array('noBorder')) . '
                                </div>
                                <div>
                                    <span class="label">homonymie:</span>
                                    ' . $this->printInput($dom . '--homonymie', $archive, $this->data[$id]->homonymie, array('short_input')) . '
                                </div>
                            </td>
                        </tr>
                        <tr>
                            <td><span class="label">heslo (var.):</span></td>
                            <td>' . $this->printInput($dom . '--tvar', $archive, $this->data[$id]->varianta) . '</td>
                        </tr>
                        <tr>
                            <td><span class="label">komentář před:</span></td>
                            <td>' . $this->printInput($dom . '--komentar', $archive, $this->data[$id]->komentar) . '</td>
                        </tr>
                        <tr>
                            <td><span class="label">komentář za:</span></td>
                            <td>' . $this->printInput($dom . '--komentarza', $archive, $this->data[$id]->komentar_za) . '</td>
                        </tr>
                    </tbody>
                    <tbody>
                        <tr>
                            <td><span class="label">typ hesla:</span></td>
                            <td>' . $this->data[$id]->typ_hesla->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td><span class="label">výslovnost:</span></td>
                            <td>' . $this->data[$id]->vyslovnost->printForm($archive) . '</td>
                        </tr>';
        $style = "";
        if (!is_field_enabled($this->parentVal, 'varianta--komentar_tvaru')) {
            $style = "style='display:none'";
        }
        $html .= '
              <tr ' . $style . '><td colspan="2">okomentované tvary:</td></tr>
              <tr ' . $style . '><td colspan="2">
                 <div class="correctableFieldContainer">
                  ' . $corr['flag'] . '
                  <textarea id="' . $dom . '--komentar_tvaru" class="lightBG cdoEditor ' . $corr['class'] . '" rows="3">
              ' . $this->data[$id]->komentar_tvaru . '
                  </textarea>
                ' . $this->initTinyMCE($dom . '--komentar_tvaru', $archive, KOM_TVARU, $id) . '
              </div>
            </td></tr>';


        $html .= '<tr>
                            <td colspan="2">' . $this->data[$id]->slov_druh->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td colspan="2">' . $this->data[$id]->tvar_char->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td colspan="2"><span class="label">komentář k tvaroslovné charakteristice:</span><br/>
                            ' . $this->printInput($dom . '--komentar_tvarchar', $archive, $this->data[$id]->komentar_tvarchar, array('w100', 'textField')) . '
                            </td>
                        </tr>
                        <tr>
                            <td colspan="2">' . $this->data[$id]->tvar_char_sl->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td colspan="2"><span class="label">komentář k tvaroslovné charakteristice slovesa:</span><br/>
                            ' . $this->printInput($dom . '--komentar_tvarchar_slovesa', $archive, $this->data[$id]->komentar_tvarchar_slovesa, array('w100', 'textField')) . '
                            </td>
                        </tr>
                        <tr>
                            <td colspan="2">
                                <span class="label">valence/vazebnost:</span>
                                ' . $this->data[$id]->vazebnost->printForm($archive) . '
                            </td>
                        </tr>
                        <tr>
                            <td><span class="label">stylistický kvalifikátor:</span></td>
                            <td>' . $this->data[$id]->styl_kval->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td><div class="label">upřesňující gram. inf.</div>' . $this->data[$id]->gram_info->printForm($archive) . '</td>
                            <td><div class="label">tematické okruhy</div>' . $this->data[$id]->tematicke_okruhy->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td><div class="label">původ</div>' . $this->data[$id]->puvod->printForm($archive) . '</td>
                            <td><div class="label">obor/oblast užití</div>' . $this->data[$id]->obor_uziti->printForm($archive) . '</td>
                        </tr>
                        <tr>
                            <td><span class="label">první registr. výskyt:</span></td>
                            <td>' . $this->printInput($dom . '--prvni_registr_vyskyt', $archive, $this->data[$id]->prvni_reg_vyskyt, array('w100')) . '</td>

                        </tr>
                    </tbody>
                </table>
            ';
        if (!$archive) {
            $html .= '
            <div class="hspaceM"></div>
            <div><input type="button" onClick="rm_field(\'' . $dom . '\')" value="smazat tuto variantu" class="buttBox redButtBox needToConfirmOnClick" /></div>
            ';
        }
        $html .= '
            </div>
        </div>
        ';

        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '
            <div id="hlavickaVariant">
                <h2 class="floatLeft">Varianty hesla</h2>
        ';

        if (!$archive) {
            $html .= '
                <div class="floatRight">
                    <input type="button" onClick="add_fields(\'varianta\')" value="přidej variantu" class="buttBox greenButtBox clickable" />
                    <input type="button" onClick="copy_last_var()" value="přidej variantu jako kopii poslední" class="buttBox greenButtBox clickable" />
                    <input type="button" id="seradVarianty" onClick="show_order_editor(\'varianta\')" value="pořadí" class="buttBox yellowButtBox clickable" />
                    <input value="uložit" class="blueButtBox buttBox floatRight" type="submit" />
                </div>
            ';
        }
        $html .= '
            </div>
            <div class="hspace"></div>
            <div id="varianta_orderPopUp"></div>
            <div id="varianta">
        ';
        foreach ($this->data as $id => $d) {
            $html .= $this->printOne($id, $archive);
        }
        $html .= '</div>';
        return $html;
    }

    /**
     * Vygeneruje editor poradi.
     * @return string
     */
    public function printOrderEdit()
    {
        $html = '
            <table>
                <tr>
                    <td rowspan="2">
                        <select id="varianta--orderer" class="ordererSB" multiple="multiple">
        ';
        foreach ($this->data as $id => $d) {
            $html .= '<option value="' . $id . '" data-dom="' . $this->domId[$id] . '">' . $d['varianta'] . '</option>';
        }
        $html .= '
                        </select>
                    </td>
                    <td>
                        <input type="button" class="sortUp_butt" value="&nbsp;Up&nbsp;" onClick="Selectbox.moveOptionUp(document.getElementById(\'varianta--orderer\'))" />
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="button" class="sortDown_butt" value="&nbsp;Down&nbsp;" onClick="Selectbox.moveOptionDown(document.getElementById(\'varianta--orderer\'))" />
                    </td>
                </tr>
            </table>
        ';
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';
        $ids = array_keys($this->data);
        foreach ($ids as $id) {
            $xml .= '
                <' . $this->xmlElem . ' id="' . $id . '">
                    <type>' . $this->data[$id]->f_varianta . '</type>
                    <lemma-variant>' . $this->data[$id]->varianta . '</lemma-variant>
                    <neosemantism>' . $this->data[$id]->neosemantismus . '</neosemantism>
                    <homonymie>' . $this->data[$id]->homonymie . '</homonymie>
                    <comment-before>' . $this->data[$id]->komentar . '</comment-before>
                    <comment-after>' . $this->data[$id]->komentar_za . '</comment-after>
                    <first-occurrence>' . $this->data[$id]->prvni_reg_vyskyt . '</first-occurrence>
                    <morph-char-comment>' . $this->data[$id]->komentar_tvarchar . '</morph-char-comment>
                    <morph-char-verb-comment>' . $this->data[$id]->komentar_tvarchar_slovesa . '</morph-char-verb-comment>
            ';

            foreach ($this->colsStructured as $col) {
                $xml .= $this->data[$id]->{$col}->printXML();
            }
            $xml .= '</' . $this->xmlElem . '>';
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return Response
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $ord = 1;
        $atbl = ARCHIVE_PREFIX . $this->tbl;
        foreach ($this->data as $id => $d) {
            $sqlData = array(
                'hid' => $archiveParentVal,
                'sortorder' => $ord++
            );
            foreach ($this->colsSimple as $col) {
                $sqlData[$col] = $this->data[$id][$col];
            }
            if (is_numeric($id)) {
                $sqlData['main_id'] = $id;
            }
            $tmp = $this->archiveInsert($this->tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
            $dbid = $tmp->data;
            foreach ($this->colsStructured as $col) {
                $this->data[$id][$col]->saveToArchive($dbid);
            }
        }
        return new Response(OK);
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $headers = array();
        $i = 0;
        $len = count($data[0]);

        foreach ($this->data as $id => $d) {
            $tmp = array();
            foreach ($fields as $c => $j) {
                if (isset($this->data[$id][$c])) {
                    if (get_parent_class($this->data[$id][$c]) != 'CDO_Base') {
                        $headers[] = $c;
                        if ($c == 'varianta') {
                            $tmpStr = $this->data[$id][$c] . ($this->data[$id]['f_varianta'] ? ' (v)' : ' (p)');
                        } else {
                            $tmpStr = $this->data[$id][$c];
                        }
                        $tmp[] = wrap_output($this->getDomId($id, $c), $tmpStr, $wrap);
                    }
                }
            }
            $this->insertTabRow($data, $i, $tmp, $len);
            $i++;
            foreach ($fields as $c => $j) {
                if (get_parent_class($this->data[$id][$c]) == 'CDO_Base') {
                    $headers[] = $c;
                    $response = $this->data[$id]->{$c}->tabExport($data, $fields[$c], $wrap);
                }
            }
        }

        return new Response(OK, array('headers' => $headers));
    }
}