<?php
/**
 * Soubor vyznam.php obsahujici definici tridy CDO_Vyznam.
 */

/**
 * Trida pro skupinu vyznamu.
 */
class CDO_Vyznam extends CDO_Base
{
    /**
     * @var array $linkOld
     */
    private $linkOld;

    /**
     * Konstruktor tridy CDO_Vyznam.
     * @param int $input
     * @param bool $parentVal
     */
    function __construct($input = CDOin_DB, $parentVal = False)
    {
        $this->input = $input;
        $this->parentVal = $parentVal;
        $this->alias = 'vyznam';
        $this->tbl = 'vyznamy';
        $this->parentFk = 'hid';
        $this->colsStructured = array('podhesli', 'vazebnost', 'styl_kval', 'tematicke_okruhy', 'obor_uziti', 'gram_info', 'slov_druh', 'tvar_char', 'tvar_char_sl', 'exemplifikace',
            'ext_pozn', 'metavyklad1', 'predvyklad', 'synonyma_slave', 'synonyma_main', 'antonyma_slave', 'antonyma_main', 'propojeni');
        $this->colsSimple = array('vystup', 'komentar_tvarchar', 'komentar_tvarchar_slovesa', 'zacatek_predvykladu', 'konec_predvykladu',
            'vyklad_vyznamu', 'vyklad_vyznamu_bz', 'metavyklad2', 'typ_vystupu');
        $this->xmlWrapper = 'meanings';
        $this->xmlElem = 'meaning';
        $this->baseDom = 'vyznam';
        if ($this->input === CDOin_DB) {
            $this->loadFromDB();
            $this->loadCorrections();
        } elseif ($this->input === CDOin_EMPTY) {
            $this->data = array();
            $this->add();
        } elseif ($this->input === CDOin_ARCHIVE) {
            $this->loadFromDB(True);
        } elseif ($this->input == CDOin_SQL) {
            $this->loadStructured('temp', array(), 0);
        }
    }

    /**
     * Nacteni korektur.
     */
    private function loadCorrections()
    {
        $domList = array();
        $transl = array();
        foreach ($this->domId as $id => $dom) {
            array_push($domList, $dom . '--komentar_tvarchar');
            array_push($domList, $dom . '--komentar_tvarchar_slovesa');

            array_push($domList, $dom . '--metavyklad2');
            $lnk = $this->data[$id]->vyklad_vyznamu_link;
            if ($lnk) {
                array_push($domList, $dom . '--vyklad_vyznamu_master');
                $transl[$dom . '--vyklad_vyznamu_master'] = str_replace('-' . $id, '-' . $lnk, $dom . '--vyklad_vyznamu_master');
            } else {
                array_push($domList, $dom . '--vyklad_vyznamu');
            }

            if ($lnk && $this->data[$id]->predvyklad_link) {
                array_push($domList, $dom . '--zacatek_predvykladu_master');
                array_push($domList, $dom . '--konec_predvykladu_master');
                $transl[$dom . '--zacatek_predvykladu_master'] = str_replace('-' . $id, '-' . $lnk, $dom . '--zacatek_predvykladu_master');
                $transl[$dom . '--konec_predvykladu_master'] = str_replace('-' . $id, '-' . $lnk, $dom . '--konec_predvykladu_master');
            } else {
                array_push($domList, $dom . '--zacatek_predvykladu');
                array_push($domList, $dom . '--konec_predvykladu');
            }

            if ($lnk && $this->data[$id]->metavyklad2_link) {
                array_push($domList, $dom . '--metavyklad2_master');
                $transl[$dom . '--metavyklad2_master'] = str_replace('-' . $id, '-' . $lnk, $dom . '--metavyklad2_master');
            } else {
                array_push($domList, $dom . '--metavyklad2');
            }

            foreach ($this->data[$id]->zakl_slovo_from as $idz => $dz) {
                array_push($domList, 'vyznam-link-' . $idz);
            }
        }
        $this->corr = $this->loadCorrectionsByDom($domList, $transl);
    }

    /**
     * Prida novy vyznam.
     * @param bool $subvyznam
     * @return Response
     */
    public function add($subvyznam = False)
    {
        $newId = $this->newId();
        $this->domId[$newId] = 'vyznam-' . $newId;
        $this->data[$newId] = new DibiRow(array(
            'komentar' => '',
            'vystup' => 0,
            'typ_vystupu' => 2,
            'komentar_tvarchar' => '',
            'komentar_tvarchar_slovesa' => '',
            'predvyklad_link' => 0,
            'metavyklad1_link' => 0,
            'metavyklad2_link' => 0,
            'zakl_slovo_from' => array(),
            'sortorder' => count(array_keys($this->data)),
            'podhesli' => new CDO_Podhesli(CDOin_EMPTY, $newId),
            'vazebnost' => new CDO_TvarKom(CDOin_EMPTY, VAZEBNOST, VYZNAM, $newId, $this->domId[$newId]),
            'styl_kval' => new CDO_Ciselnik(CDOin_EMPTY, STYL_KVAL, VYZNAM, $newId, $this->domId[$newId]),
            'tematicke_okruhy' => new CDO_Ciselnik(CDOin_EMPTY, TEMATICKE_OKRUHY, VYZNAM, $newId, $this->domId[$newId]),
            'obor_uziti' => new CDO_Ciselnik(CDOin_EMPTY, OBOR_UZITI, VYZNAM, $newId, $this->domId[$newId]),
            'gram_info' => new CDO_Ciselnik(CDOin_EMPTY, GRAM_INFO, VYZNAM, $newId, $this->domId[$newId]),
            'slov_druh' => new CDO_SlovDruh(CDOin_EMPTY, VYZNAM, $newId, $this->domId[$newId]),
            'tvar_char' => new CDO_TvarChar(CDOin_EMPTY, False, VYZNAM, $newId, $this->domId[$newId]),
            'tvar_char_sl' => new CDO_TvarChar(CDOin_EMPTY, True, VYZNAM, $newId, $this->domId[$newId]),
            'exemplifikace' => new CDO_Exempl(CDOin_EMPTY, VYZNAM, $newId, $this->domId[$newId]),
            'ext_pozn' => new CDO_ExtPozn(CDOin_EMPTY, VYZNAM, $newId, $this->domId[$newId]),
            'predvyklad' => new CDO_TvarKom(CDOin_EMPTY, PREDVYKLAD, VYZNAM, $newId, $this->domId[$newId]),
            'metavyklad1' => new CDO_Ciselnik(CDOin_EMPTY, METAVYKLAD1, VYZNAM, $newId, $this->domId[$newId]),
            'synonyma_slave' => new CDO_Link(CDOin_EMPTY, HREL_TYPE_syn, false, $newId, $this->domId[$newId], $this->parentVal),
            'synonyma_main' => new CDO_Link(CDOin_EMPTY, HREL_TYPE_syn, true, $newId, $this->domId[$newId], $this->parentVal),
            'antonyma_slave' => new CDO_Link(CDOin_EMPTY, HREL_TYPE_ant, false, $newId, $this->domId[$newId], $this->parentVal),
            'antonyma_main' => new CDO_Link(CDOin_EMPTY, HREL_TYPE_ant, true, $newId, $this->domId[$newId], $this->parentVal),
            'propojeni' => new CDO_Link(CDOin_EMPTY, HREL_TYPE_pripojovani, true, $newId, $this->domId[$newId], $this->parentVal, 'vyznam')
        ));
        if ($subvyznam) {
            $this->data[$newId]['pid'] = $subvyznam;
            $this->data[$subvyznam]->subvyznamy[] = $newId;
            $this->fixOrder();
        } else {
            $this->data[$newId]['subvyznamy'] = array();
        }
        $this->maximized[$newId] = True;
        return new Response(OK, $newId);
    }

    public function update($id, $field, $data, $data2 = False)
    {
        $resp = parent::update($id, $field, $data);
        if ($resp->status == OK) {
            $html = '';
            if ($field == 'predvyklad_link') {
                $linked = ($this->data[$id]->vyklad_vyznamu_link && (int)$data) ? $this->data[$id]->vyklad_vyznamu_link : 0;
                $this->data[$id]->predvyklad->switchLinked($linked);
                $html = $this->printPredvyklad($id);
            } elseif ($field == 'metavyklad1_link') {
                $linked = ($this->data[$id]->vyklad_vyznamu_link && (int)$data) ? $this->data[$id]->vyklad_vyznamu_link : 0;
                $this->data[$id]->metavyklad1->switchLinked($linked);
                $html = $this->printMetavyklad1($id);
            } elseif ($field == 'metavyklad2_link') {
                $html = $this->printMetavyklad2($id);
            } elseif ($field == 'vyklad_vyznamu') {
                $this->data[$id]->vyklad_vyznamu_bz = bz($this->data[$id]->vyklad_vyznamu);
            }
            return new Response(OK, $html);
        } else {
            return $resp;
        }
    }

    public function remove($id)
    {
        $links = array('synonyma_slave', 'synonyma_main', 'antonyma_slave', 'antonyma_main', 'propojeni');
        $hasLinks = False;
        foreach ($links as $l) {
            if ($this->data[$id][$l]->hasData()) {
                $hasLinks = True;
            }
        }
        if ($hasLinks) {
            return new Response(NOT_OK, array(), 'Význam obsahuje odkazy.');
        } else {
            foreach ($this->data[$id]['exemplifikace']->data as $exd) {
                if ($exd->propojeni->hasData()) {
                    return new Response(NOT_OK, array(), 'Exemplifikace v tomto významu obsahuje odkazy.');
                }
            }
            $pid = $this->data[$id]['pid'];
            if ($pid) {
                $idx = array_search($id, $this->data[$pid]->subvyznamy);
                unset($this->data[$pid]->subvyznamy[$idx]);
            }
            return parent::remove($id);
        }
    }

    /**
     * Seradit vyznamy a subvyznamy.
     */
    public function fixOrder()
    {
        $ids = array_keys($this->data);
        $ids = array_combine($ids, $ids);
        $list = array();
        foreach ($ids as $id) {
            if (!$this->isSub($id)) {
                $list[] = $id;
                foreach ($this->data[$id]['subvyznamy'] as $subid) {
                    $list[] = $subid;
                }
            }
        }
        $this->reorder($list);
    }

    public function moveUp($id)
    {
        $m = $this->isSub($id);
        if ($m) {
            // pokud je subvyznam, nejdrive posuneme
            // v seznamu subvyznamu
            $list = $this->data[$m]['subvyznamy'];
            $pos = array_search($id, $list);
            if ($pos > 0 && count($list) > 1) {
                $tmp = $list[$pos];
                $list[$pos] = $list[$pos - 1];
                $list[$pos - 1] = $tmp;
                $this->data[$m]['subvyznamy'] = $list;
                // a posuneme v datove slozce
                $this->fixOrder();
                return new Response(OK, $list);
            } else {
                return new Response(NOT_OK, $list);
            }
        } else {
            return parent::moveUp($id);
        }
    }

    public function moveDown($id)
    {
        $m = $this->isSub($id);
        if ($m) {
            // pokud je subvyznam, nejdrive posuneme
            // v seznamu subvyznamu
            $list = $this->data[$m]['subvyznamy'];
            $pos = array_search($id, $list);
            if ($pos < count($list) - 1 && count($list) > 1) {
                $tmp = $list[$pos];
                $list[$pos] = $list[$pos + 1];
                $list[$pos + 1] = $tmp;
                $this->data[$m]['subvyznamy'] = $list;
                // a posuneme v datove slozce
                parent::moveDown($id);
                return new Response(OK, $list);
            }
            return new Response(NOT_OK, $list);
        } else {
            return parent::moveDown($id);
        }
    }


    /**
     * Kontroluje, jestli je subvyznam.
     * @param int $vyz id vyznamu
     * @return int id nadrazeneho vyznamu nebo False
     */
    public function isSub($vyz)
    {
        return $this->data[$vyz]['pid'] ? $this->data[$vyz]['pid'] : False;
    }

    /**
     * Cislo vyznamu.
     * @param $id
     * @return int
     */
    public function getNum($id)
    {
        if ($this->isSub($id)) {
            $pid = $this->data[$id]['pid'];
            return array_search($id, $this->data[$pid]['subvyznamy']) + 1;
        } else {
            $num = 1;
            foreach ($this->data as $idv => $v) {
                if ($this->isSub($idv)) continue;
                if ($idv == $id) break;
                $num++;

            }
            return $num;
        }
    }

    private function loadStructured($id, $d, $counter)
    {
        $this->domId[$id] = 'vyznam-' . $id;
        $this->data[$id]['podhesli'] = new CDO_Podhesli($this->input, $id, $this->domId[$id]);
        $this->data[$id]['vazebnost'] = new CDO_TvarKom($this->input, VAZEBNOST, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['styl_kval'] = new CDO_Ciselnik($this->input, STYL_KVAL, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['tematicke_okruhy'] = new CDO_Ciselnik($this->input, TEMATICKE_OKRUHY, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['obor_uziti'] = new CDO_Ciselnik($this->input, OBOR_UZITI, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['gram_info'] = new CDO_Ciselnik($this->input, GRAM_INFO, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['slov_druh'] = new CDO_SlovDruh($this->input, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['tvar_char'] = new CDO_TvarChar($this->input, False, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['tvar_char_sl'] = new CDO_TvarChar($this->input, True, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['predvyklad'] = new CDO_TvarKom($this->input, PREDVYKLAD, VYZNAM, $id, $this->domId[$id], $d['predvyklad_link'] * $d['vyklad_vyznamu_link']);
        $this->data[$id]['metavyklad1'] = new CDO_Ciselnik($this->input, METAVYKLAD1, VYZNAM, $id, $this->domId[$id], $d['metavyklad1_link'] * $d['vyklad_vyznamu_link']);
        $this->data[$id]['exemplifikace'] = new CDO_Exempl($this->input, VYZNAM, $id, $this->domId[$id], $this->parentVal);
        $this->data[$id]['ext_pozn'] = new CDO_ExtPozn($this->input, VYZNAM, $id, $this->domId[$id]);
        $this->data[$id]['synonyma_slave'] = new CDO_Link($this->input, HREL_TYPE_syn, false, $id, $this->domId[$id], $this->parentVal, 'vyznam', $counter);
        $this->data[$id]['synonyma_main'] = new CDO_Link($this->input, HREL_TYPE_syn, true, $id, $this->domId[$id], $this->parentVal, 'vyznam', $counter);
        $this->data[$id]['antonyma_slave'] = new CDO_Link($this->input, HREL_TYPE_ant, false, $id, $this->domId[$id], $this->parentVal, 'vyznam', $counter);
        $this->data[$id]['antonyma_main'] = new CDO_Link($this->input, HREL_TYPE_ant, true, $id, $this->domId[$id], $this->parentVal, 'vyznam', $counter);
        $this->data[$id]['propojeni'] = new CDO_Link($this->input, HREL_TYPE_pripojovani, true, $id, $this->domId[$id], $this->parentVal, 'vyznam', $counter);
    }

    protected function loadFromDB($archive = False)
    {
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            $this->data = dibi::query('
			SELECT v.id, pid, uts, komentar, vystup, vyklad_vyznamu_link, predvyklad_link, metavyklad1_link, metavyklad2_link,
				IF(predvyklad_link = 1 AND vyklad_vyznamu_link IS NOT NULL, vm.predvyklad, v.predvyklad) AS predvyklad_old,
				IF(predvyklad_link = 1 AND vyklad_vyznamu_link IS NOT NULL, vm.zacatek_predvykladu, v.zacatek_predvykladu) AS zacatek_predvykladu,
				IF(predvyklad_link = 1 AND vyklad_vyznamu_link IS NOT NULL, vm.konec_predvykladu, v.konec_predvykladu) AS konec_predvykladu,
				IF(metavyklad1_link = 1 AND vyklad_vyznamu_link IS NOT NULL, vm.metavyklad1, v.metavyklad1) AS metavyklad1_txt,
				IF(vyklad_vyznamu_link IS NULL, v.vyklad_vyznamu, vm.vyklad_vyznamu) AS vyklad_vyznamu,
				IF(vyklad_vyznamu_link IS NULL, v.vyklad_vyznamu_bz, vm.vyklad_vyznamu_bz) AS vyklad_vyznamu_bz,
				IF(metavyklad2_link = 1 AND vyklad_vyznamu_link IS NOT NULL, vm.metavyklad2, v.metavyklad2) AS metavyklad2,
				koncovka_gram_info, sortorder, typ_vystupu, komentar_tvarchar, komentar_tvarchar_slovesa
			FROM %n v LEFT JOIN vyznamy_master vm ON v.vyklad_vyznamu_link=vm.id
			WHERE hid = %i ORDER BY sortorder ASC
		', $prefix . 'vyznamy', $this->parentVal)->fetchAssoc('id');
        } catch (DibiException $e) {
            if ($archive) {
                if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $this->data = array();
                }
            } else {
                return handle_db_error($e);
            }
        }

        $counter = 0;
        foreach ($this->data as $id => $d) {
            $this->linkOld[$id] = $d['vyklad_vyznamu_link'];
            $this->domId[$id] = 'vyznam-' . $id;
            $this->data[$id]['subvyznamy'] = dibi::fetchPairs('SELECT id FROM %n WHERE hid = %i AND pid = %i ORDER BY sortorder ', $prefix . 'vyznamy', $this->parentVal, $id);
            if (!$this->isSub($id)) $counter++;
            $this->loadStructured($id, $d, $counter);
            $this->maximized[$id] = False;

            // odkazy na základové slovo
            if ($archive) {
                $this->data[$id]['zakl_slovo_to'] = array();
                $this->data[$id]['zakl_slovo_from'] = array();
            } else {
                $this->data[$id]['zakl_slovo_to'] = dibi::query('
                    SELECT pv.id, pv.vyz_from, pv.vyz_m_from, pv.heslo_to, pv.heslo_id, pv.vyz_to, v.hid
                    FROM [propojeni_vyznamu] pv LEFT JOIN [vyznamy] v ON pv.vyz_from = v.id
                    WHERE [vyz_to] = %i
                ', $id)->fetchAssoc('id');

                if ($this->data[$id]->vyklad_vyznamu_link) {
                    $this->data[$id]['zakl_slovo_from'] = dibi::query('
                        SELECT pv.id, pv.vyz_from, pv.vyz_m_from, pv.heslo_to, pv.heslo_id, pv.vyz_to, v.hid
                        FROM [propojeni_vyznamu] pv LEFT JOIN [vyznamy] v ON pv.vyz_to = v.id
                        WHERE [vyz_m_from] = %i
                    ', $this->data[$id]->vyklad_vyznamu_link)->fetchAssoc('id');
                } else {
                    $this->data[$id]['zakl_slovo_from'] = dibi::query('
                        SELECT pv.id, pv.vyz_from, pv.vyz_m_from, pv.heslo_to, pv.heslo_id, pv.vyz_to, v.hid
                        FROM [propojeni_vyznamu] pv LEFT JOIN [vyznamy] v ON pv.vyz_to = v.id
                        WHERE [vyz_from] = %i
                    ', $id)->fetchAssoc('id');
                }
            }
        }

        return new Response(OK);
    }

    protected function saveToDB($archive = False, $originalData = array())
    {
        if (CDO_compare($this->data, $originalData)) {
            return new Response(OK);
        }
        $this->fixOrder();

        $colsSimple = array('komentar', 'vystup', 'vyklad_vyznamu_link', 'predvyklad_link', 'metavyklad1_link', 'metavyklad2_link', 'komentar_tvarchar', 'komentar_tvarchar_slovesa', 'typ_vystupu');
        $prefix = $archive ? ARCHIVE_PREFIX : '';
        try {
            dibi::begin();
            $sqlData = array(
                'hid' => $this->parentVal
            );
            $ord = 1;
            $originalDataKeys = array_combine(array_keys($originalData), array_keys($originalData));
            foreach ($this->data as $id => $d) {
//                if (isset($originalData[$id]) && CDO_compare($this->data[$id], $originalData[$id])) {
//                    return new Response(OK);
//                }
                // jeden význam
                if ($this->data[$id]->vyklad_vyznamu_link) {
                    // význam je ve skupině
                    if (!$this->linkOld[$id]) {
                        // význam je ve skupině a předtím nebyl
                        if (!is_numeric($this->data[$id]->vyklad_vyznamu_link)) {
                            // nová skupina - id významu vzoru
                            $vzor = substr($this->data[$id]->vyklad_vyznamu_link, 3);
                            $skupId = vytvorit_vzor($vzor);
                            $this->data[$id]->vyklad_vyznamu_link = $skupId;

                            // update linků pro předvýklad a metavýklad 1
                            if ($this->data[$id]->predvyklad_link) {
                                $this->data[$id]->predvyklad->setParentLinked($skupId);
                            }
                            if ($this->data[$id]->metavyklad1_link) {
                                $this->data[$id]->metavyklad1->setParentLinked($skupId);
                            }
                        }
                        //existující skupina
                        $skupArr = array(
                            'vyklad_vyznamu' => $this->data[$id]->vyklad_vyznamu,
                            'vyklad_vyznamu_bz' => $this->data[$id]->vyklad_vyznamu_bz
                        );
                        if ($this->data[$id]->metavyklad2_link) {
                            $skupArr['metavyklad2'] = $this->data[$id]->metavyklad2;
                        } else {
                            $colsSimple[] = 'metavyklad2';
                        }
                        if ($this->data[$id]->predvyklad_link) {
                            $skupArr['zacatek_predvykladu'] = $this->data[$id]->zacatek_predvykladu;
                            $skupArr['konec_predvykladu'] = $this->data[$id]->konec_predvykladu;
                        } else {
                            $colsSimple[] = 'zacatek_predvykladu';
                            $colsSimple[] = 'konec_predvykladu';
                        }
                        dibi::query('UPDATE vyznamy_master SET %a WHERE id=%i', $skupArr, $this->data[$id]->vyklad_vyznamu_link);

                    } else {
                        // význam již byl ve skupině
                        $skupArr = array(
                            'vyklad_vyznamu' => $this->data[$id]->vyklad_vyznamu,
                            'vyklad_vyznamu_bz' => $this->data[$id]->vyklad_vyznamu_bz
                        );
                        if ($this->data[$id]->metavyklad2_link) {
                            $skupArr['metavyklad2'] = $this->data[$id]->metavyklad2;
                        } else {
                            $colsSimple[] = 'metavyklad2';
                        }
                        if ($this->data[$id]->predvyklad_link) {
                            $skupArr['zacatek_predvykladu'] = $this->data[$id]->zacatek_predvykladu;
                            $skupArr['konec_predvykladu'] = $this->data[$id]->konec_predvykladu;
                        } else {
                            $colsSimple[] = 'zacatek_predvykladu';
                            $colsSimple[] = 'konec_predvykladu';
                        }
                        dibi::query('UPDATE vyznamy_master SET %a WHERE id=%i', $skupArr, $this->data[$id]->vyklad_vyznamu_link);
                    }
                } else {
                    if ($this->linkOld[$id]) {
                        // význam byl ve skupině a už není
                        foreach ($this->data[$id]->zakl_slovo_from as $idz => $dz) {
                            $this->data[$id]->zakl_slovo_from[$idz]->vyz_m_from = null;
                            $this->data[$id]->zakl_slovo_from[$idz]->vyz_from = $id;
                        }
                        $this->data[$id]->vyklad_vyznamu_link = null;
                    }

                    $colsSimple[] = 'vyklad_vyznamu';
                    $colsSimple[] = 'vyklad_vyznamu_bz';
                    $colsSimple[] = 'zacatek_predvykladu';
                    $colsSimple[] = 'konec_predvykladu';
                    $colsSimple[] = 'metavyklad2';
                }
                $sqlData['sortorder'] = $ord++;
                foreach ($colsSimple as $col) {
                    $sqlData[$col] = $this->data[$id][$col];
                }
                if (!$this->data[$id]->vyklad_vyznamu_link) {
                    unset($this->data[$id]->vyklad_vyznamu_link);
                }
                $dbid = $id;
                if (isset($originalDataKeys[$id])) {
                    // update
                    dibi::query('
                        UPDATE %n SET %a WHERE id = %i
                    ', $prefix . 'vyznamy', $sqlData, $id);
                    unset($originalDataKeys[$id]);
                    foreach ($this->colsStructured as $col) {
                        $this->data[$id][$col]->saveToDB($archive, $originalData[$id][$col]->data);
                    }
                } else {
                    dibi::query('
                        INSERT INTO %n
                    ', $prefix . 'vyznamy', $sqlData);
                    $dbid = $this->newId(True);
                    foreach ($this->colsStructured as $col) {
                        $this->data[$id][$col]->setParentVal($dbid);
                        $this->data[$id][$col]->saveToDB($archive);
                    }
                }

                // uložení odkazů na základové slovo
                $vv = $this->data[$id]['vyklad_vyznamu'];
                $origKeys = isset($originalData[$id]['zakl_slovo_from']) ? array_combine(array_keys($originalData[$id]['zakl_slovo_from']), array_keys($originalData[$id]['zakl_slovo_from'])) : array();
                foreach ($this->data[$id]->zakl_slovo_from as $idz => $dz) {
                    unset($dz['id'], $dz['hid']);
                    if (isset($origKeys[$idz])) {
                        // update
                        dibi::query('
                            UPDATE [propojeni_vyznamu] SET %a WHERE id = %i
                        ', $dz, $idz);
                        unset($origKeys[$idz]);
                    } else {
                        dibi::query('
                            INSERT INTO [propojeni_vyznamu]
                        ', $dz);
                        $vv = str_replace('id="link-' . $idz . '"', 'id="link-' . $this->newId(True) . '"', $vv);
                    }
                }
                if ($this->data[$id]->vyklad_vyznamu_link) {
                    dibi::query('UPDATE vyznamy_master SET vyklad_vyznamu = %s WHERE id = %i', $vv, $this->data[$id]->vyklad_vyznamu_link);
                } else {
                    dibi::query('UPDATE vyznamy SET vyklad_vyznamu = %s WHERE id = %i', $vv, $dbid);
                    if ($this->linkOld[$id]) {
                        smaz_skupinu($this->linkOld[$id]);
                    }
                }

                $col = $this->data[$id]->vyklad_vyznamu_link ? 'vyz_m_from' : 'vyz_from';
                dibi::query('
                    DELETE FROM [propojeni_vyznamu] WHERE id IN %in AND %n = %i
                ', array_keys($origKeys), $col, $id);
            }

            foreach ($originalDataKeys as $id => $d) {
                //delete
                dibi::query('
                    DELETE FROM %n WHERE id = %i
                ', $prefix . 'vyznamy', $id);
            }
            dibi::commit();
        } catch (DibiException $e) {
            /* ERROR HANDLING */
            dibi::rollback();
            return handle_db_error($e);
        }

        return new Response(OK);
    }

    /**
     * Vygeneruje blok s predvykladem.
     * @param $id
     * @param $archive
     * @return string
     */
    public function printPredvyklad($id, $archive = False)
    {
        $dom = $this->domId[$id];
        if ($this->data[$id]->vyklad_vyznamu_link && $this->data[$id]->predvyklad_link) {
            $predvykladLinked = $this->data[$id]->vyklad_vyznamu_link;
        } else {
            $predvykladLinked = 0;
        }
        $suff = $predvykladLinked ? '_master' : '';
        $zDom = $dom . '--zacatek_predvykladu' . $suff;
        $kDom = $dom . '--konec_predvykladu' . $suff;

        $html = '<div class="label">Předvýklad</div>';
        $onclick = '';
        if ($this->data[$id]->vyklad_vyznamu_link && !$archive) {
            $html .= '<div class="predvyklad-checkbox_div">Převzít předvýklad ' . $this->printCheckbox($dom . '--predvyklad_link', $archive, $this->data[$id]->predvyklad_link) . '</div>';
            $onclick = 'varovaniSkup(this, \'' . $id . '\')';
        }
        $html .= '
            <table class="predvyklad_table">
                <tr>
                    <td>
                        <label>začátek předvýkladu</label>
                        ' . $this->printInput($zDom, $archive, $this->data[$id]->zacatek_predvykladu, array(), $onclick) . '
                    </td>
                    <td>' . $this->data[$id]->predvyklad->printForm($archive) . '</td>
                    <td>
                        <label>konec předvýkladu</label>
                        ' . $this->printInput($kDom, $archive, $this->data[$id]->konec_predvykladu, array(), $onclick) . '
                    </td>
                </tr>
            </table>
        ';

        return $html;
    }

    /**
     * Vygeneruje blok s metavykladem1.
     * @param $id
     * @param $archive
     * @return string
     */
    public function printMetavyklad1($id, $archive = False)
    {
        $dom = $this->domId[$id];
        if ($this->data[$id]->vyklad_vyznamu_link && $this->data[$id]->metavyklad1_link) {
            $meta1Linked = $this->data[$id]->vyklad_vyznamu_link;
        } else {
            $meta1Linked = 0;
        }
        $html = '<div class="label">metavýklad 1</div>';
        if ($this->data[$id]->vyklad_vyznamu_link && !$archive) {
            $html .= '<div class="metavyklad1-checkbox_div">Převzít ' . $this->printCheckbox($dom . '--metavyklad1_link', $archive, $this->data[$id]->metavyklad1_link) . '</div>';
        }
        $html .= $this->data[$id]->metavyklad1->printForm($archive);

        return $html;
    }

    private function printVyznamLinks($data)
    {
        $links = array();
        $ltmp = array();
        foreach ($data as $l) {
            if ($l['skup']) { //skupina
                $tmp = array();
                foreach ($l['data'] as $grp) {
                    $links[] = $grp['hid'];
                    $tmp[] = '<a target="_blank" href="' . LEMMA_FORM_ROUTER . '.php?hid=' . $grp['hid'] . '">' . $grp['varianta'] . ' ' . get_vyznam_num($grp['vyz_id']) . '</a>';
                }
                $ltmp[] = 'skupina <span class="nobold">(' . implode(', ', $tmp) . ')</span>';
                unset($tmp);
            } else {
                $links[] = $l['hid'];
                $ltmp[] = '<a target="_blank" href="' . LEMMA_FORM_ROUTER . '.php?hid=' . $l['hid'] . '">' . $l['varianta'] . ' ' . get_vyznam_num($l['vyz_id']) . '</a>';
            }
        }
        $return = implode(', ', $ltmp);
        $return .= '. <a target="_blank" href="heslar.php?force_entries=' . implode(',', $links) . '" class="showall">Zobrazit vše</a>';
        return $return;
    }

    /**
     * Vygeneruje blok s vykladem vyznamu.
     * @param $id
     * @param $archive
     * @return string
     */
    public function printVyklad($id, $archive = False)
    {
        $dom = $this->domId[$id] . '--vyklad_vyznamu';
        $suff = $this->data[$id]->vyklad_vyznamu_link ? '_master' : '';
        $dom .= $suff;
        $corrLink = array();
        $corrLinkNew = False;
        $corrLinkFlags = '';
        if ($archive) {
            $corr = $this->emptyCorrectionArray();
            foreach ($this->data[$id]->zakl_slovo_from as $idz => $dz) {
                $corrLink[$id] = $this->emptyCorrectionArray();
            }
        } else {
            $corr = $this->displayCorrection($this->corr[$dom]['field_DOMid'], $this->corr[$dom]['id'], $this->corr[$dom]['field'], $this->corr[$dom]['nova']);
            $left = 20;
            foreach ($this->data[$id]->zakl_slovo_from as $idz => $dz) {
                $domL = 'vyznam-link-' . $idz;
                if ($this->corr[$domL]['nova']) {
                    $corrLinkNew = True;
                    $corr['class'] = 'corrected';
                }
                $corrLink[$id] = $this->displayCorrection($this->corr[$domL]['field_DOMid'], $this->corr[$domL]['id'], $this->corr[$domL]['field'], $this->corr[$domL]['nova'], $left);
                if ($corrLink[$id]['flag']) {
                    $left += 20;
                    $corrLinkFlags .= $corrLink[$id]['flag'];
                }
            }
        }

        if ((isset($corr[$dom]) && $corr[$dom]['nova']) || $corrLinkNew) {
            $btncls = 'greyButtBox';
            $dis = ' disabled="disabled"';
        } else {
            $btncls = 'greenButtBox';
            $dis = '';
        }

        $html = '
            <table class="static w100">
                <tr>
                    <td>
                        <label>výklad významu</label>
        ';

        if (!$archive) {
            if ($this->data[$id]->vyklad_vyznamu_link) {
                $vyz_ids = $this->getSkupIds($id);
                $arr = array();
                foreach ($vyz_ids as $vi) {
                    $arr[] = getVyznamNum($vi, True, True, True);
                }
                $pocet = count($vyz_ids);
                $str = '';
                if ($pocet > 2) {
                    $pocet -= 2;
                    if ($pocet < 5) {
                        $pocet_str = ' další &gt;&gt;';
                    } else {
                        $pocet_str = ' dalších &gt;&gt;';
                    }
                    $str .= '
                        <div id="' . $this->domId[$id] . '--div_nahled_skupiny">
                            ' . implode(', ', array_slice($arr, 0, 2)) . '
                            <span class="clickable" onclick="rozbal_skupinku(\'' . $id . '\')"> a ' . $pocet . $pocet_str . '</span>
                        </div>
                        <div id="' . $this->domId[$id] . '--div_cela_skupina" style="display: none;">
                            ' . implode(', ', $arr) . '
                        </div>
                    ';
                } else {
                    $str .= implode(', ', $arr);
                }
                $html .= '
                        <div id="vyznam-' . $id . '--div_skupina">
                            <div style="float:left">Heslo je součástí skupiny hesel:<br />
                                ' . $str . '
                            </div>
                        </div>
                    </td>
                    <td>
                        <input id="vyznam-' . $id . '--tlacitko_skupiny" value="Odebrat ze skupiny" class="buttBox floatRight ' . $btncls . '" type="button"
                        onclick="odebrat_ze_skupiny(\'' . $id . '\')"' . $dis . ' />
                    </td>
                ';
            } else {
                $html .= '
                    </td>
                    <td>
                        <input id="vyznam-' . $id . '--tlacitko_skupiny" value="Přidat do skupiny" class="buttBox floatRight ' . $btncls . '" type="button"
                        onclick="pridat_do_skupiny_dlg(\'' . $id . '\')" />
                    </td>
                ';
            }
            $html .= '<tr><td colspan="2">';
            // upozornění na odkazy na základové slovo
            if ($this->data[$id]->zakl_slovo_to) {
                $zzArray = array();
                foreach ($this->data[$id]->zakl_slovo_to as $idzz => $zz) {
                    if ($zz->vyz_m_from) {
                        $zzRow = array('skup' => 1);
                        $zzRow['data'] = dibi::fetchAll('
                            SELECT vz.hid, varianta, vz.id AS vyz_id
                            FROM (vyznamy_master vm INNER JOIN vyznamy vz ON vm.id = vz.vyklad_vyznamu_link)
                              INNER JOIN varianty vr ON vz.hid=vr.hid AND vr.sortorder=1
                            WHERE vm.id = %i
                        ', $zz->vyz_m_from);
                        $zzArray[] = $zzRow;
                    } else {
                        $zzArray[] = dibi::fetch('
                            SELECT vz.hid, varianta, vz.id AS vyz_id, 0 AS skup
                            FROM vyznamy vz INNER JOIN varianty vr ON vz.hid=vr.hid AND vr.sortorder=1
                            WHERE vz.id = %i
                        ', $zz->vyz_from);
                    }
                }
                $html .= '<div class="varovani">Významy odkazující na tento význam: <p>' . $this->printVyznamLinks($zzArray) . '</p></div>';
            }
            if ($this->data[$id]->zakl_slovo_from) {
                $html .= print_vyznam_links_to($id, $this->data[$id]->vyklad_vyznamu_link);
            }
            $html .= '</td></tr>';
        } else {
            $html .= '
                </td><td></td></tr>
            ';
        }
        $html .= '
            <tr><td colspan="2">
                <div id="' . $dom . '---helperIdCpy" class="correctableFieldContainer">
                    ' . $corr['flag'] . $corrLinkFlags . '
                    <textarea id="' . $dom . '" class="lightBG cdoEditor ' . $corr['class'] . '" rows="3">
                        ' . $this->data[$id]->vyklad_vyznamu . '
                    </textarea>
                    ' . $this->initTinyMCE($dom, $archive, VYKLAD_VYZNAMU, $id) . '
                </div>
                <input type="hidden" id="' . $this->domId[$id] . '--vyklad_vyznamu_link" class="vyklad_vyznamu_link" value="' . intval($this->data[$id]->vyklad_vyznamu_link) . '"/>
            </td></tr>
        </table>
        ';

        return $html;
    }

    /**
     * Vygeneruje blok s metavykladem2.
     * @param $id
     * @param $archive
     * @return string
     */
    public function printMetavyklad2($id, $archive = False)
    {
        $dom = $this->domId[$id];
        if ($this->data[$id]->vyklad_vyznamu_link && $this->data[$id]->metavyklad2_link) {
            $meta2Linked = $this->data[$id]->vyklad_vyznamu_link;
            $onclick = 'varovaniSkup(this, \'' . $id . '\')';
        } else {
            $meta2Linked = 0;
            $onclick = '';
        }

        $suff = $meta2Linked ? '_master' : '';
        $html = '<div class="label">metavýklad 2</div>';
        if ($this->data[$id]->vyklad_vyznamu_link && !$archive) {
            $html .= '<div class="metavyklad2-checkbox_div">Převzít ' . $this->printCheckbox($dom . '--metavyklad2_link', $archive, $this->data[$id]->metavyklad2_link) . '</div>';
        }

        $html .= $this->printTextarea($dom . '--metavyklad2' . $suff, $archive, $this->data[$id]->metavyklad2,
            array('ant_syn_check', 'lightBG', 'flatBox', 'w100fix', 'padd', 'oneRowTextarea', 'metavyklad2txt'), $onclick);

        return $html;
    }

    public function printSkupinaBlok($id, $archive = False)
    {
        $dom = $this->domId[$id];
        $html = '
            <tr>
                <td colspan="2">
                    <div id="' . $dom . '--predvyklad--wrapper">
                        ' . $this->printPredvyklad($id, $archive) . '
                    </div>
                </td>
            </tr>
            <tr>
                <td colspan="2">
                    <div id="' . $dom . '--metavyklad1--wrapper">
                        ' . $this->printMetavyklad1($id, $archive) . '
                    </div>
                </td>
            </tr>
            <tr class="static">
                <td colspan="2">
                    ' . $this->printVyklad($id, $archive) . '
                <input id="vyznam--' . $id . '--link_text-hid" style="float:right" type="hidden" name="link_text_hid" value="0" class="vyklad_vyznamu_link_hid"  />
                </td>
            </tr>
            <tr>
                <td colspan="2">
                    <div id="' . $dom . '--metavyklad2--wrapper">
                        ' . $this->printMetavyklad2($id, $archive) . '
                    </div>
                </td>
            </tr>
        ';
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro jednu polozku.
     * @param int $id
     * @param bool $archive
     * @return string
     */
    public function printOne($id, $archive = False)
    {
        if (!$archive) {
            $this->checkLinkKonzistence($id);
        }
        $vystupArr = array(NEZARAZENO => 'nezařazeno', TISTENY => 'tištěný + elektronický', ELEKTRONICKY => 'elektronický');
        $antSynMaster = !(empty($this->data[$id]->synonyma_main->data) && empty($this->data[$id]->antonyma_main)) ? ("Hesla, u nichž je tento význam uveden jako synonymum/antonymum") : ("Tento význam není připojen jako synonymum k žádnému heslu");
        $dom = $this->domId[$id];
        if ($this->isSub($id)) {
            $str = 'podvýznamu';
            $classes = 'subvyznam box greyBox';
        } else {
            $str = 'významu';
            $classes = 'vyznam orangeBox';
        }
        $num = $this->getNum($id);
        $html = '';
        if ($this->isSub($id)) {
            $html .= '<div id="vyznam-' . $this->data[$id]->pid . '--subvyznamy-' . $id . '">';
        } else {
            $html .= '<div id="' . $this->domId[$id] . '--outerWrap">';
        }

        $html .= '
            <div id="' . $dom . '-wrapper" class="vyznam-wrapper">
            <a name="vyznam-' . $id . '"></a>
            <div id="' . $dom . '" class="' . $classes . '">
                <h3 class="floatLeft">Výklad ' . $str . ' ' . $num . '</h3>
                <div id="' . $dom . '--minimax" class="floatRight">' . $this->printMinimax($id) . '</div>
        ';
        if ($this->isSub($id)) {
            $html .= '
                <table class="small_updown_arrows floatLeft hmargin">
                    <tr><td class="up clickable" onClick="moveItem(\'vyznam-' . $this->data[$id]->pid . '--subvyznamy\', \'' . $id . '\', \'up\')"></td></tr>
                    <tr><td class="down clickable" onClick="moveItem(\'vyznam-' . $this->data[$id]->pid . '--subvyznamy\', \'' . $id . '\', \'down\')"></td></tr>
                </table>
            ';
        }
        $html .= '
                <div class="clear"></div>
                <div>
                    <span class="label floatLeft">do výstupu:</span>
                    ' . $this->printRadioGroup($dom . '--typ_vystupu', $archive, $vystupArr, $this->data[$id]->typ_vystupu) . '
                </div>
                ' . $this->data[$id]->podhesli->printForm($archive) . '
                <table class="vyznam_tbl w100 clear">
                    <tr>
                        <td colspan="2">
                            <span class="label"> valence/vazebnost:</span>
                            ' . $this->data[$id]->vazebnost->printForm($archive) . '
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <span class="label"> stylistický kvalifikátor:</span>
                            ' . $this->data[$id]->styl_kval->printForm($archive) . '
                        </td>
                        <td>
                            <span class="label"> tematické okruhy:</span>
                            ' . $this->data[$id]->tematicke_okruhy->printForm($archive) . '
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <span class="label"> obor/oblast užití:</span>
                            ' . $this->data[$id]->obor_uziti->printForm($archive) . '
                        </td>
                        <td></td>
                    </tr>
                    <tr>
                        <td> ' . $this->data[$id]->slov_druh->printForm($archive) . '</td>
                        <td>
                            <span class="label"> upřesňující gram. inf.:</span>
                            ' . $this->data[$id]->gram_info->printForm($archive) . '
                        </td>
                    </tr>
                    <tr>
                        <td style="width: 50%;"> ' . $this->data[$id]->tvar_char->printForm($archive) . '</td>
                        <td style="width: 50%;"> ' . $this->data[$id]->tvar_char_sl->printForm($archive) . '</td>
                    </tr>
                    <tr>
                        <td colspan="2"><span class="label"> komentář k tvaroslovné charakteristice:</span><br />
                            ' . $this->printInput($dom . '--komentar_tvarchar', $archive, $this->data[$id]->komentar_tvarchar, array('w100fix', 'textField', 'padd', 'flatBox', 'lightBG')) . '
                        </td>
                    </tr>
                    <tr>
                        <td colspan="2"><span class="label"> komentář k tvaroslovné charakteristice slovesa:</span><br />
                            ' . $this->printInput($dom . '--komentar_tvarchar_slovesa', $archive, $this->data[$id]->komentar_tvarchar_slovesa, array('w100fix', 'textField', 'padd', 'flatBox', 'lightBG')) . '
                        </td>
                    </tr>
                    <tbody id="' . $dom . '--skupina_blok">' . $this->printSkupinaBlok($id, $archive) . '</tbody>
                    <tr>
                        <td id="' . $dom . '-as" colspan="2" class="clickable label " onclick="show_ant_syn(\'' . $dom . '\')">
                            <img alt="Rozbalit" src="css/images/roll_out.png" >
                            ' . $antSynMaster . '
                        </td>
                    </tr>
                    <tr id="' . $dom . '-asm" class="asm" style="display: none;">
                        <td id="vyznam-' . $id . '-s_syn_wrapper" >
                            ' . $this->data[$id]->synonyma_slave->printForm($archive) . '
                        </td>
                        <td id="vyznam-' . $id . '-s_ant_wrapper">
                            ' . $this->data[$id]->antonyma_slave->printForm($archive) . '
                        </td>
                    </tr>
                    <tr  id="' . $dom . '-ash" class="asm" style="display: none;">
                        <td colspan="2" class="clickable label"  onclick="hide_ant_syn(\'' . $dom . '\')">
                            <img alt="Skrýt" src="css/images/roll_in.png" onclick="$(\'.hide_ant_syn\').show">
                            Skrýt
                        </td>
                    </tr>
                    <tr>
                     <td id="vyznam-' . $id . '-syn_wrapper">' . $this->data[$id]->synonyma_main->printForm($archive) . '</td>
                     <td id="vyznam-' . $id . '-ant_wrapper">' . $this->data[$id]->antonyma_main->printForm($archive) . '</td>
                    </tr>
                    <tr>
                        <td colspan="2">' . $this->data[$id]->exemplifikace->printForm($archive) . '</td>
                    </tr>
                    <tr>
                        <td colspan="2">' . $this->data[$id]->ext_pozn->printForm($archive) . '</td>
                    </tr>
                    <tr>
                        <td colspan="2" id="vyznam-' . $id . '-propojeni_wrapper">' . $this->data[$id]->propojeni->printForm($archive) . '</td>
                    </tr>
                </table>
        ';
        if (!$archive) {
            $html .= '
                <div class="static" style="overflow: hidden; padding: 5px;">
                    <input type="button" onClick="rm_field(\'' . $dom . '\')" value="smazat význam" class="buttBox redButtBox needToConfirmOnClick floatRight" />
                </div>
            ';
        }
        $html .= '
            </div>
            <div id="' . $dom . '--skupinaPopUp"></div>
            </div>
        ';
        if ($this->isSub($id)) {
            $html .= '</div>';
        } else {
            $subvyz_count = count($this->data[$id]->subvyznamy);
            $html .= '
                <div class="subvyznamy_outer_wraper" style="overflow: hidden;">
                    <div class="show_subvyznamy clickable label floatRight"><img src="css/images/roll_out.png" alt="Zobrazit podvýznamy" />
                        Ukaž podvýznamy významu ' . $num . ' (' . $subvyz_count . ')
                    </div>
                    <div class="clear"></div>
                    <div class="subvyznamy_container">
                        <div id="' . $dom . '--subvyznamy">
            ';
            $html .= '';
            if ($subvyz_count) {
                foreach ($this->data[$id]->subvyznamy as $sub) {
                    $html .= $this->printOne($sub, $archive);
                }
            } else {
                $html .= '<div class="nosubvyznam greyInfo box">Význam ' . $num . ' nemá podvýznamy.</div>';
            }
            $html .= '
                        </div>
                        <div class="hide_subvyznamy label clickable floatRight"><img src="css/images/roll_in.png" alt="Skrýt podvýznamy" />Skrýt</div>
            ';
            if (!$archive) {
                $html .= '
                    <div class="add_subvyznam label clickable floatLeft" onClick="add_fields(\'' . $dom . '--subvyznamy\')">
                        <img src="css/images/green_plus.png" alt="Přidat" />&nbsp;Přidej podvýznam k významu ' . $num . '
                    </div>
                ';
            }
            $html .= '
                    </div>
                </div>
            </div>
            ';
        }
        return $html;
    }

    /**
     * Vygeneruje cast formulare pro dany objekt.
     * @param bool $archive
     * @return string
     */
    public function printForm($archive = False)
    {
        $html = '
            <h2 class="floatLeft">Významy</h2>
            <div class="minimax_vyznam_all floatLeft">
               [ <a href="#" class="maximize_vyznam_all clickable">Rozbalit</a> :: <a href="#" class="minimize_vyznam_all clickable">Sbalit</a> ]
            </div>
        ';
        if (!$archive) {
            $html .= '
                <div class="floatRight">
                    <input type="button" id="seradVyznamy"  onClick="show_order_editor(\'vyznam\')" value="pořadí" class="buttBox yellowButtBox clickable" />
                </div>
            ';
        }
        $html .= '
            <div class="hspace"></div>
            <div id="heslo_ant_syn_wrapper"></div>
            <div class="clear"></div>
            <div id="vyznam_orderPopUp"></div>
            <div id="vyznamy">
        ';
        foreach ($this->data as $id => $d) {
            if ($this->isSub($id)) continue;
            $html .= $this->printOne($id, $archive);
        }
        $html .= '
            </div>
        ';
        if (!$archive) {
            $upravy_as_cnt = getLinkCount();
            $redbox = $upravy_as_cnt > 0 ? printRedBox() : '';
            $html .= '
            <div class="padd clear">
                <input type="button" id="pridejVyznam_butt" onClick="add_fields(\'vyznam\')" value="přidej význam" class="buttBox greenButtBox padd vmargin" />
                <div class="clear"></div>
            </div>
            <div id="vyznamlinkPopUp"></div>
            <div id="upravy_ant_syn_container" class="ant_syn_container_cor">' . $redbox . '</div>
            <div id="popup_uprava_ant_syn" class="popUp"></div>
            ';
        }
        return $html;
    }

    public function saveOdkZakl($id, $odkId, $target_vyznam, $target_hid, $target_heslo)
    {
        //pv.id, pv.vyz_from, pv.vyz_m_from, pv.heslo_to, pv.heslo_id, pv.vyz_to, v.hid
        if ($odkId == '0') {
            $odkId = $this->newId();
            $resp = array('type' => 'new');
        } else {
            unset($this->data[$id]->zakl_slovo_from[$odkId]);
            $resp = array('type' => 'edit');
        }
        $resp['dom'] = $this->domId[$id] . '--vyklad_vyznamu';
        $suff = $this->data[$id]->vyklad_vyznamu_link ? '_master' : '';
        $resp['dom'] .= $suff;
        $delete = False;
        $arr = array();
        if ($this->data[$id]->vyklad_vyznamu_link) {
            $arr['vyz_m_from'] = $this->data[$id]->vyklad_vyznamu_link;
        } else {
            $arr['vyz_from'] = $id;
        }
        if ($target_vyznam > 0) {
            // k existujícímu významu
            $arr['vyz_to'] = $target_vyznam;
            $arr['heslo_to'] = '';
            $arr['heslo_id'] = null;
            $arr['hid'] = $target_hid;
        } elseif ($target_hid == -1 && $target_heslo != '' && $target_vyznam == -1) {
            // k neexistujícímu heslu
            $arr['heslo_to'] = $target_heslo;
            $arr['heslo_id'] = null;
            $arr['hid'] = null;
        } elseif ($target_vyznam == 0) {
            // odebrání
            $resp['type'] = 'delete';
            $delete = True;
        }
        if (!$delete) {
            $this->data[$id]->zakl_slovo_from[$odkId] = new DibiRow($arr);
            $resp['link_id'] = $odkId;
        }

        return new Response(OK, $resp);
    }

    public
    function printOdkZaklForm($id, $odkId, $text)
    {
        $options = '';
        if ($odkId) {
            $trg = $this->data[$id]->zakl_slovo_from[$odkId]->vyz_to;
            if ($trg) {
                $hid = $this->data[$id]->zakl_slovo_from[$odkId]->hid;
                $var = getHlavniVarianta($hid);
                $vzs = dibi::fetchAll('SELECT id, vyklad_vyznamu, sortorder, pid FROM vyznamy v WHERE v.hid = %i ORDER BY sortorder ASC', $hid);
                $vyzn = 1;
                $sub_vyzn = 1;
                foreach ($vzs as $vz) {
                    if ($vz['pid']) {
                        $txt = 'Subvyznam ' . $sub_vyzn;
                        $sub_vyzn++;
                    } else {
                        $txt = 'Vyznam ' . $vyzn;
                        $sub_vyzn = 1;
                        $vyzn++;
                    }
                    $options .= '<option class="ozs_select_opt" value="' . $vz['id'] . '"' . ($vz['id'] == $trg ? ' selected="selected"' : '') . '>' . $txt . '</option>';
                }
            } else {
                $var = $this->data[$id]->zakl_slovo_from[$odkId]->heslo_to;
                $hid = -1;
            }
        } else {
            $var = '';
            $hid = -1;
        }
        $html = '
            <div class="varovani2">Heslo vyberte kliknutím na položku v našeptávači.</div>
            <table class="multiInput">
                <tr>
                    <td>' . $this->convert($text) . '</td>
                    <td><input class="ac_vyznam_text" value="' . $var . '" hid="' . $hid . '" /></td>
                    <td>
                        <select class="ozs_select" multiple="multiple">
                            <option value="0" style="color:gray; font-style: italic" class="option_not_rem ozs_select_opt">- nepropojovat -</option>
                            ' . $options . '
                        </select>
                    </td>
                </tr>
            </table>
        ';

        return $html;
    }

    /**
     * Pridani vyznamu do skupiny.
     * @param $id
     * @param $skup
     * @return string
     */
    public function addToSkup($id, $skup)
    {
        $this->data[$id]->vyklad_vyznamu_link = $skup;
        $this->data[$id]->predvyklad_link = 1;
        $this->data[$id]->metavyklad1_link = 1;
        $this->data[$id]->metavyklad2_link = 1;
        $this->data[$id]->predvyklad->switchLinked($skup, True);
        $this->data[$id]->metavyklad1->switchLinked($skup, True);
        if (is_numeric($skup)) {
            // existující skupina
            $tmp = dibi::fetch('SELECT vyklad_vyznamu, vyklad_vyznamu_bz, zacatek_predvykladu, konec_predvykladu, metavyklad2 FROM vyznamy_master WHERE id=%i', $skup);
            $this->data[$id]['zakl_slovo_from'] = dibi::query('
                SELECT pv.id, pv.vyz_from, pv.vyz_m_from, pv.heslo_to, pv.heslo_id, pv.vyz_to, v.hid
                FROM [propojeni_vyznamu] pv LEFT JOIN [vyznamy] v ON pv.vyz_to = v.id
                WHERE [vyz_m_from] = %i
            ', $skup)->fetchAssoc('id');
        } else {
            // nová skupina
            $vyz = substr($skup, 3);
            $tmp = dibi::fetch('SELECT vyklad_vyznamu, vyklad_vyznamu_bz, zacatek_predvykladu, konec_predvykladu, metavyklad2 FROM vyznamy WHERE id=%i', $vyz);
            $this->data[$id]['zakl_slovo_from'] = dibi::query('
                SELECT pv.id, NULL AS vyz_from, %s AS vyz_m_from, pv.heslo_to, pv.heslo_id, pv.vyz_to, v.hid
                FROM [propojeni_vyznamu] pv LEFT JOIN [vyznamy] v ON pv.vyz_to = v.id
                WHERE [vyz_from] = %i
            ', $skup, $vyz)->fetchAssoc('id');
        }
        $this->data[$id]->vyklad_vyznamu = $tmp['vyklad_vyznamu'];
        $this->data[$id]->vyklad_vyznamu_bz = $tmp['vyklad_vyznamu_bz'];
        $this->data[$id]->metavyklad2 = $tmp['metavyklad2'];
        $this->data[$id]->zacatek_predvykladu = $tmp['zacatek_predvykladu'];
        $this->data[$id]->konec_predvykladu = $tmp['konec_predvykladu'];
        return $this->printSkupinaBlok($id);
    }

    /**
     * Odebrani vyznamu ze skupiny.
     * @param $id
     * @return string
     */
    public function skupRemove($id)
    {
        $this->data[$id]->vyklad_vyznamu_link = 0;
        $this->data[$id]->predvyklad_link = 0;
        $this->data[$id]->metavyklad1_link = 0;
        $this->data[$id]->metavyklad2_link = 0;
        $this->data[$id]->predvyklad->switchLinked(0);
        $this->data[$id]->metavyklad1->switchLinked(0);
        // transformace odkazů na základové slovo
        foreach ($this->data[$id]->zakl_slovo_from as $idz => $d) {
            $newId = $this->newId();
            $this->data[$id]->zakl_slovo_from[$newId] = $d;
            $this->data[$id]->zakl_slovo_from[$newId]->id = $newId;
            $this->data[$id]->zakl_slovo_from[$newId]->vyz_from = $id;
            $this->data[$id]->zakl_slovo_from[$newId]->vyz_m_from = null;
            $this->data[$id]->vyklad_vyznamu = str_replace('id="link-' . $idz . '"', 'id="link-' . $newId . '"', $this->data[$id]->vyklad_vyznamu);
            unset($this->data[$id]->zakl_slovo_from[$idz]);
        }

        return $this->printSkupinaBlok($id);
    }

    /**
     * Vygenerovani dialogu pro pridani do skupiny.
     * @param $id
     * @return string
     */
    public function printGroupDlg($id)
    {
        $str = $this->isSub($id) ? 'podvýznam' : 'význam';
        $num = $this->getNum($id);
        $html = '
            <div title="Úprava skupiny pro ' . $str . ' ' . $num . '" class="popUp skupina-vyklad" id="skupina-vyklad-' . $id . '">
                <span class="lable">Zdrojové slovo: </span>
                <div class="selectize-control" style="margin-bottom: 10px; overflow:hidden;">
                    <input id="vyznam-' . $id . '--link_text" style="float:left" type="text" name="link_text" value="" class="ac_link_text" size="40" />
                </div>
                <span class="lable">Výběr významu: </span>
                <select class="vyznam_link_select" style="float:left" id="select-vyklad-' . $id . '" placeholder="Linkování významu"></select>
            </div>
        ';

        return $html;
    }

    /**
     * Vygeneruje editor poradi.
     * @return string
     */
    public function printOrderEdit()
    {
        $html = '
            <table>
                <tr>
                    <td rowspan="2">
                        <select id="vyznam--orderer" class="ordererSB" multiple="multiple">
        ';
        foreach ($this->data as $id => $d) {
            if ($d['pid']) continue;
            $html .= '<option value="' . $id . '" data-dom="' . $this->domId[$id] . '">Význam ' . $this->getNum($id) . '</option>';
        }
        $html .= '
                        </select>
                    </td>
                    <td>
                        <input type="button" class="sortUp_butt" value="&nbsp;Up&nbsp;" onClick="Selectbox.moveOptionUp(document.getElementById(\'vyznam--orderer\'))" />
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="button" class="sortDown_butt" value="&nbsp;Down&nbsp;" onClick="Selectbox.moveOptionDown(document.getElementById(\'vyznam--orderer\'))" />
                    </td>
                </tr>
            </table>
        ';
        return $html;
    }

    /**
     * Kontrola konzistence odkazu na zakladove slovo.
     * @param $id
     * @return Response
     */
    private function checkLinkKonzistence($id)
    {
        $str = $this->data[$id]->vyklad_vyznamu;
        // kontrola "je v textu ale není v objektu"
        $num = preg_match_all('/id=\"link-n?[0-9]+\"/', $this->data[$id]->vyklad_vyznamu, $matches);
        $ids = array_combine(array_keys($this->data[$id]->zakl_slovo_from), array_keys($this->data[$id]->zakl_slovo_from));
        if ($num) {
            foreach ($matches[0] as $m) {
                $lid = substr(substr($m, 9), 0, -1);
                unset($ids[$lid]);
                if (!isset($this->data[$id]->zakl_slovo_from[$lid])) {
                    $stringArr = array();
                    $span_start = strpos($str, '<span id="link-' . $lid . '"');
                    $text_start = strpos($str, '>', $span_start);
                    $text_end = strpos($str, '</span>', $text_start);
                    $stringArr[0] = substr($str, 0, $span_start); // text před
                    $stringArr[1] = substr($str, $span_start, ($text_start - $span_start) + 1); // otevírací span
                    $stringArr[2] = substr($str, $text_start + 1, ($text_end - $text_start) - 1); // text uvnitř
                    $stringArr[3] = substr($str, $text_end, 7); // zavírací span
                    $stringArr[4] = substr($str, $text_end + 7); // text za
                    unset($stringArr[1], $stringArr[3]);
                    $new = implode('', $stringArr);
                    $this->data[$id]->vyklad_vyznamu = $new;
                }
            }
        }

        // kontrola "je v objektu ale není v textu"
        foreach ($ids as $idz) {
            unset($this->data[$id]->zakl_slovo_from[$idz]);
        }
        return new Response(OK);
    }

    public function printMinimax($id)
    {
        if ($this->maximized[$id]) {
            $html = '<img class="mini-max-ico clickable minimize_vyznam" alt="Minimalizovat význam" src="css/images/minimize.png">';
        } else {
            $html = '<img class="mini-max-ico clickable maximize_vyznam" alt="Maximalizovat význam" src="css/images/maximize.png">';
        }
        return $html;
    }

    /**
     * XML export
     * @return string
     */
    public function printXML()
    {
        $xml = '<' . $this->xmlWrapper . '>';
        $ids = array_keys($this->data);
        foreach ($ids as $id) {
            if ($this->isSub($id)) continue;
            $xml .= '
                <' . $this->xmlElem . ' id="' . $id . '">
                    <output>' . $this->data[$id]->typ_vystupu . '</output>
                    <morph-char-comment>' . $this->data[$id]->komentar_tvarchar . '</morph-char-comment>
                    <morph-char-verb-comment>' . $this->data[$id]->komentar_tvarchar_slovesa . '</morph-char-verb-comment>
                    <meaning-text>' . $this->data[$id]->vyklad_vyznamu_bz . '</meaning-text>
                    <meaning-text-html><![CDATA[' . $this->data[$id]->vyklad_vyznamu . ']]></meaning-text-html>
                    <meta-meaning-2>' . $this->data[$id]->metavyklad2 . '</meta-meaning-2>
            ';
            foreach ($this->colsStructured as $col) {
                $xml .= $this->data[$id]->{$col}->printXML();
            }
            if ($this->data[$id]->subvyznamy) {
                $xml .= '<sub-meanings>';
                foreach ($this->data[$id]->subvyznamy as $sid) {
                    $xml .= '
                        <' . $this->xmlElem . ' id="' . $sid . '">
                            <output>' . $this->data[$sid]->typ_vystupu . '</output>
                            <morph-char-comment>' . $this->data[$sid]->komentar_tvarchar . '</morph-char-comment>
                            <morph-char-verb-comment>' . $this->data[$sid]->komentar_tvarchar_slovesa . '</morph-char-verb-comment>
                            <meaning-text>' . $this->data[$sid]->vyklad_vyznamu_bz . '</meaning-text>
                            <meaning-text-html><![CDATA[' . $this->data[$sid]->vyklad_vyznamu . ']]></meaning-text-html>
                            <meta-meaning-2>' . $this->data[$sid]->metavyklad2 . '</meta-meaning-2>
                    ';
                    foreach ($this->colsStructured as $col) {
                        $xml .= $this->data[$sid]->{$col}->printXML();
                    }
                    $xml .= '</' . $this->xmlElem . '>';
                }
                $xml .= '</sub-meanings>';
            }
            $xml .= '</' . $this->xmlElem . '>';
        }
        $xml .= '</' . $this->xmlWrapper . '>';
        return $xml;
    }

    public function getSkupIds($id)
    {
        if ($this->data[$id]->vyklad_vyznamu_link) {
            if (is_numeric($this->data[$id]->vyklad_vyznamu_link)) {
                $vyz_ids = dibi::fetchPairs('SELECT id FROM vyznamy WHERE vyklad_vyznamu_link = %i', $this->data[$id]->vyklad_vyznamu_link);
            } else {
                // nově vytvořená skupina
                $vyz_ids = array(substr($this->data[$id]->vyklad_vyznamu_link, 3));
            }
            return $vyz_ids;
        } else {
            return array();
        }
    }

    /**
     * Ulozeni dat do archivu.
     * @param $archiveParentVal
     * @param int $archiveHid
     * @return mixed
     */
    public function saveToArchive($archiveParentVal, $archiveHid = 0)
    {
        $ord = 1;
        $tbl = 'vyznamy';
        $atbl = ARCHIVE_PREFIX . 'vyznamy';
        foreach ($this->data as $id => $d) {
            $sqlData = array(
                'hid' => $archiveParentVal,
                'sortorder' => $ord++
            );
            foreach ($this->colsSimple as $col) {
                $sqlData[$col] = $this->data[$id][$col];
            }
            if (is_numeric($id)) {
                $sqlData['main_id'] = $id;
            }
            $tmp = $this->archiveInsert($tbl, $atbl, $sqlData);
            if ($tmp->status !== OK) {
                return $tmp;
            }
            $dbid = $tmp->data;
            foreach ($this->colsStructured as $col) {
                $this->data[$id][$col]->saveToArchive($dbid, $archiveHid);
            }
        }
        return new Response(OK);
    }

    protected function getSQLFields($fields)
    {
        if (isset($fields['vystup'])) {
            unset($fields['vystup']);
            $fields['typ_vystupu'] = 1;
        }
        return $fields;
    }


    public function getSQL($fields = False, $parentTable = False)
    {
        // doplňková pole
        $fields = $this->getSQLFields($fields);
        // vždy načteme z DB alespoň ID
        $return = array(
            'columns' => array('[' . $this->tbl . '].[id] AS [' . $this->alias . '--id]'),
        );
        // pro každý vybraný sloupec (jednoduché string sloupce)
        $colsSimple = $this->colsSimple;
        foreach ($colsSimple as $c) {
            if (isset($fields[$c])) {
                if ($c == 'vyklad_vyznamu') {
                    $return['columns'][] = '
                        IF ([' . $this->tbl . '].[vyklad_vyznamu_link] IS NOT NULL,
                            [vyznamy_master].[vyklad_vyznamu],
                            [' . $this->tbl . '].[vyklad_vyznamu]
                        ) AS [' . $this->alias . '--vyklad_vyznamu]';
                } elseif ($c == 'zacatek_predvykladu' || $c == 'konec_predvykladu') {
                    $return['columns'][] = '
                        IF ([' . $this->tbl . '].[vyklad_vyznamu_link] IS NOT NULL AND [' . $this->tbl . '].[predvyklad_link] = 1,
                            [vyznamy_master].[' . $c . '],
                            [' . $this->tbl . '].[' . $c . ']
                        ) AS [' . $this->alias . '--' . $c . ']';
                } elseif ($c == 'metavyklad2') {
                    $return['columns'][] = '
                        IF ([' . $this->tbl . '].[vyklad_vyznamu_link] IS NOT NULL AND [' . $this->tbl . '].[metavyklad2_link] = 1,
                            [vyznamy_master].[metavyklad2],
                            [' . $this->tbl . '].[metavyklad2]
                        ) AS [' . $this->alias . '--metavyklad2]';
                } elseif ($c == 'typ_vystupu') {
                    $return['columns'][] = '[' . $this->tbl . '].[' . $c . '] AS [' . $this->alias . '--vystup_vyznam]';
                } else {
                    $return['columns'][] = '[' . $this->tbl . '].[' . $c . '] AS [' . $this->alias . '--' . $c . ']';
                }
            }
        }
        $firstId = array_keys($this->data);
        $firstId = $firstId[0];
        $childTbl = array();
        // pro každý vybraný strukturovaný sloupec (subclass CDO_Base)
        foreach ($this->colsStructured as $c) {
            if ($fields === False || isset($fields[$c])) {
                $childFields = isset($fields[$c]) ? $fields[$c] : False;
                $resp = $this->data[$firstId][$c]->getSQL($childFields, $this->tbl);
                $return['columns'] = array_merge($return['columns'], $resp->data['columns']);
                $childTbl[] = $resp->data['tables'];
            }
        }
        // struktura, která se pak využije v JOINech - spojíme přes id nadřazené tabulky a FK aktuální tabulky
        $return['tables'] = array(
            '([' . $this->tbl . '] LEFT JOIN [vyznamy_master]
                ON [' . $this->tbl . '].[vyklad_vyznamu_link] = [vyznamy_master].[id])' => array(
                'main' => array('[' . $parentTable . '].[id]', '[' . $this->tbl . '].[' . $this->parentFk . ']'),
                'child' => $childTbl
            )
        );

        return new Response(OK, $return);
    }

    /**
     * Export do pole pro html tabulku / csv
     * @param array $data
     * @param array $fields pole pro export
     * @param int $wrap
     * @return Response
     */
    public function tabExport(&$data, $fields = array(), $wrap = WRAP_NONE)
    {
        $headers = array('cislo_vyznamu');
        $i = 0;
        $len = count($data[0]);
        $cis = 0;
        foreach ($this->data as $id => $d) {
            $tmp = array(
                ($this->isSub($id) ? 'sub-' . $cis : ++$cis)
            );
            foreach ($fields as $c => $j) {
                if (isset($this->data[$id][$c])) {
                    if (get_parent_class($this->data[$id][$c]) != 'CDO_Base') {
                        $headers[] = $c;
                        $tmp[] = wrap_output($this->getDomId($id, $c), $this->data[$id][$c], $wrap);
                    }
                }
            }
            $this->insertTabRow($data, $i, $tmp, $len);
            $i++;
            foreach ($fields as $c => $j) {
                if (get_parent_class($this->data[$id][$c]) == 'CDO_Base') {
                    $headers[] = $c;
                    $response = $this->data[$id]->{$c}->tabExport($data, $fields[$c], $wrap);
                }
            }
        }

        return new Response(OK, array('headers' => $headers));
    }

    protected function addTabData($id)
    {
        $num = getVyznamNum($id);
        $str = $num ? 'Význam ' . $num : 'Podvýznam';
        return array(
            'vyznam' => $str
        );
    }
}