<?php
define('WRAP_NONE', 0);
define('WRAP_CORRECTION', 1);
function wrap_output($domId, $value, $type = WRAP_NONE)
{
    if ($type == WRAP_CORRECTION) {
        return '<a onclick="handle_correction(\'' . $domId . '\')">' . $value . '</a>';
    }

    return $value;
}

function handle_db_error($e, $archive = False)
{
    var_dump($e);
    return new Response(DB_ERROR, $e, $e->getMessage());
}

/**
 * Seznam cizích klíčů definovaných na sloupcích tabulky $table
 * @param $table
 * @return mixed
 */
function CDO_get_keys($table)
{
    $keys = dibi::fetchAll('
		SELECT COLUMN_NAME, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, CONSTRAINT_NAME
		FROM information_schema.KEY_COLUMN_USAGE
		WHERE TABLE_SCHEMA=%s AND TABLE_NAME=%s AND REFERENCED_TABLE_NAME IS NOT NULL
		ORDER BY REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME
	', DB, $table);
    return $keys;
}

/**
 * Vrací informace o cizím klíči na sloupci $column v tabulce $table
 * @param $table
 * @param $column
 * @return mixed
 */
function CDO_check_key($table, $column)
{
    $key = dibi::fetch('
		SELECT REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME, CONSTRAINT_NAME
		FROM information_schema.KEY_COLUMN_USAGE
		WHERE TABLE_SCHEMA=%s AND TABLE_NAME=%s AND COLUMN_NAME=%s AND REFERENCED_TABLE_NAME IS NOT NULL
		ORDER BY REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME
	', DB, $table, $column);
    return $key;
}


function handle_archive_error($e, $table)
{
//                vložení záznamu se nepodařilo, zde se zjišťuje důvod
//                tabulka neexistuje: archivujeme část hesla, která je v nové tabulce, ta se musí vytvořit
//                sloupec neexistuje: do tabulky byl přidán nový sloupec, je potřeba ho přidat i do archivní
//                (toto se může opakovat víckrát)
//                zároveň se musí navíc vytvořit i FK vazby
    $atable = ARCHIVE_PREFIX . $table;
    if ($e->getCode() == MYSQL_TABLE_NOT_EXISTS) { //tabulka neexistuje
        try {
            dibi::query('CREATE TABLE %n LIKE %n', $atable, $table);
            if ($table == 'heslo') {
                dibi::query('ALTER TABLE %n ADD COLUMN `main_heslo` INT NULL DEFAULT NULL', $atable);
                dibi::query('
                CREATE TABLE `version` (
                    `id` int(11) NOT NULL,
                    `uid` int(11) NOT NULL,
                    `user_save` tinyint(4) NOT NULL DEFAULT "0",
                    `comment` varchar(255) COLLATE utf8_czech_w1 DEFAULT NULL,
                    `stamp` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
                    PRIMARY KEY (`id`),
                    KEY `uid` (`uid`),
                    CONSTRAINT `fk_version_uziv` FOREIGN KEY (`uid`) REFERENCES `uzivatel` (`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
                ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_czech_w1
            ');
            } else {
                dibi::query('ALTER TABLE %n ADD COLUMN `main_id` INT NULL DEFAULT NULL', $atable);
            }
            $keys = CDO_get_keys($table);
            foreach ($keys as $k) {
                dibi::query('ALTER TABLE %n ADD INDEX (%n)', $atable, $k['COLUMN_NAME']);
                if (strpos($k['REFERENCED_TABLE_NAME'], 'cis_') === 0 || $k['REFERENCED_TABLE_NAME'] == 'uzivatel' || $k['REFERENCED_TABLE_NAME'] == 'heslare') {
                    dibi::query('ALTER TABLE %n ADD FOREIGN KEY (%n) REFERENCES %n (%n) ON UPDATE CASCADE ON DELETE CASCADE', $atable, $k['COLUMN_NAME'], $k['REFERENCED_TABLE_NAME'], $k['REFERENCED_COLUMN_NAME']);
                } else {
                    dibi::query('ALTER TABLE %n ADD FOREIGN KEY (%n) REFERENCES %n (%n) ON UPDATE CASCADE ON DELETE CASCADE', $atable, $k['COLUMN_NAME'], ARCHIVE_PREFIX . $k['REFERENCED_TABLE_NAME'], $k['REFERENCED_COLUMN_NAME']);
                }
            }
            return new Response(OK, False, 'Tabulka ' . $atable . ' vytvořena');
        } catch (DibiException $e) {
            return handle_db_error($e);
        }
    } elseif ($e->getCode() == MYSQL_UNKNOWN_COLUMN) { //sloupec neexistuje
        $msg = $e->getMessage();
        $col = substr($msg, strpos($msg, '\'') + 1);
        $col = substr($col, 0, strpos($col, '\'')); //$col teď obsahuje název sloupce
        try {
            $col_info = dibi::fetch('SELECT COLUMN_TYPE, IS_NULLABLE FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=%s AND TABLE_NAME=%s AND COLUMN_NAME=%s', 'neoled', $table, $col);
            dibi::query('ALTER TABLE %n ADD COLUMN %n %sql %sql', $atable, $col, $col_info['COLUMN_TYPE'], ($col_info['IS_NULLABLE'] == 'YES' ? 'NULL' : 'NOT NULL'));
            $key = CDO_check_key($table, $col);
            if ($key) {
                dibi::query('ALTER TABLE %n ADD INDEX (%n)', $atable, $col);
                dibi::query('ALTER TABLE %n ADD FOREIGN KEY (%n) REFERENCES %n (%n) ON UPDATE CASCADE ON DELETE CASCADE', $atable, $col, ARCHIVE_PREFIX . $key['REFERENCED_TABLE_NAME'], $key['REFERENCED_COLUMN_NAME']);
            }
            return new Response(OK, False, 'Tabulka ' . $table . ': přidán sloupec ' . $col);
        } catch (DibiException $e) {
            return handle_db_error($e);
        }
    } else {
//                    pokud nastala jiná chyba, proces končí
        return handle_db_error($e);
    }
}

function bz($str, $typ = '')
{
    return strip_tags($str);
}

function bd($str)
{
    $prevodni_tabulka = array(
        'ä' => 'a', 'Ä' => 'A', 'á' => 'a', 'Á' => 'A', 'â' => 'a', 'Â' => 'A',
        'č' => 'c', 'Č' => 'C', 'ć' => 'c', 'Ć' => 'C',
        'ď' => 'd', 'Ď' => 'D',
        'ě' => 'e', 'Ě' => 'E', 'é' => 'e', 'É' => 'E', 'ë' => 'e', 'Ë' => 'E',
        'í' => 'i', 'Í' => 'I', 'î' => 'i', 'Î' => 'I',
        'ľ' => 'l', 'Ľ' => 'L', 'ĺ' => 'l', 'Ĺ' => 'L',
        'ń' => 'n', 'Ń' => 'N', 'ň' => 'n', 'Ň' => 'N',
        'ó' => 'o', 'Ó' => 'O', 'ö' => 'o', 'Ö' => 'O', 'ô' => 'o', 'Ô' => 'O', 'ő' => 'o', 'Ő' => 'O',
        'ř' => 'r', 'Ř' => 'R', 'ŕ' => 'r', 'Ŕ' => 'R',
        'š' => 's', 'Š' => 'S', 'ś' => 's', 'Ś' => 'S',
        'ť' => 't', 'Ť' => 'T',
        'ú' => 'u', 'Ú' => 'U', 'ů' => 'u', 'Ů' => 'U', 'ü' => 'u', 'Ü' => 'U',
        'ý' => 'y', 'Ý' => 'Y',
        'ž' => 'z', 'Ž' => 'Z', 'ź' => 'z', 'Ź' => 'Z'
    );
    return strtr($str, $prevodni_tabulka);
}

function CDO_compare($data1, $data2)
{
    return (md5(serialize($data1)) == md5(serialize($data2)));
}

function getNextSortorder($tbl, $parentFk, $parentVal)
{
    try {
        return intval(dibi::fetchSingle('SELECT MAX(sortorder) FROM %n WHERE %n = %i', $tbl, $parentFk, $parentVal)) + 1;
    } catch (DibiException $e) {
        return handle_db_error($e);
    }
}

function getHlavniVarianta($hid, $homonymie = False)
{
    $tmp2 = dibi::fetch('SELECT [varianta], [homonymie] FROM [varianty] WHERE [hid] = %i AND [sortorder] = 1', $hid);
    $var = $tmp2['varianta'];
    if ($homonymie && $tmp2['homonymie']) {
        $var .= ' [' . $tmp2['homonymie'] . ']';
    }
    return $var;
}

function getVyznamNum($vyz_id, $varianta = False, $homonymie = False, $url = False)
{
    try {
        $tmp = dibi::fetch('SELECT [hid], [sortorder] FROM [vyznamy] WHERE [id] = %i', $vyz_id);
        //$num = intval(dibi::fetchSingle('SELECT COUNT([id]) FROM [vyznamy] WHERE [hid] = %i AND [sortorder] < %i', $tmp['hid'], $tmp['sortorder'])) + 1;
        $num = get_vyznam_num($vyz_id);
        if ($varianta) {
            $var = getHlavniVarianta($tmp['hid'], $homonymie);
            $num = $var . ' ' . $num;
        }
        if ($url) {
            $num = '<a style="text-decoration: underline;" href="formular_hesla_new.php?hid=' . $tmp['hid'] . '#vyznam-' . $vyz_id . '" target="_blank">' . $num . '</a>';
        }
        return $num;
    } catch (DibiException $e) {
        return handle_db_error($e);
    }
}

function getPodhesliNum($podh_id)
{
    $vyz = dibi::fetchSingle('SELECT vyz_id FROM podhesli_vyznamu WHERE id = %i', $podh_id);
    return dibi::fetchSingle('SELECT COUNT(id) FROM podhesli_vyznamu WHERE vyz_id = %i AND id < %i', $vyz, $podh_id) + 1;
}

function checkBlokSkup($hid)
{
    $skups = dibi::fetchPairs('SELECT vyklad_vyznamu_link FROM vyznamy WHERE hid = %i AND vyklad_vyznamu_link IS NOT NULL', $hid);
    $hids = dibi::fetchPairs('SELECT hid FROM vyznamy WHERE vyklad_vyznamu_link IN %in', $skups);
    $onl = dibi::fetch('SELECT uid, cts FROM hesla_online WHERE hid IN %in AND uid <> %i AND DATE_SUB(NOW(), INTERVAL 1 MINUTE) <= uts', $hids, get_logged_user());
    return $onl;
}

function getSemaforTitle($stav)
{
    if (!is_numeric($stav) || $stav < 1 || $stav > 7) {
        return '';
    }
    $titles = array('',
        'Základní zpracování',
        'Předredakční revize',
        'Ukončená vzájemná revize',
        'Redakce ukončená hlavním redaktorem',
        'Odsouhlasené hlavním redaktorem',
        'Recenze uzavřená hlavním redaktorem',
        'Uzavřené heslo'
    );
    return $titles[$stav];
}

function getVystup($code)
{
    $vystup = array(
        NEZARAZENO => 'nezařazeno',
        NEZARAZENO_FRAZ => 'nezařazeno - hyperlemma',
        ELEKTRONICKY => 'elektronický',
        TISTENY => 'tištěný + elektronický'
    );
    return isset($vystup[$code]) ? $vystup[$code] : '';
}

define('ARCHIVE_PREFIX', 'archive_');
define('SESS_DEFAULT_CDO', 'dCDO');
define('SESS_ORIG_CDO', 'oCDO');
define('SESS_CURR_CDO', 'cCDO');
define('SESS_ARCH_CDO', 'aCDO');
define('SESS_TISK_CDO', 'tCDO');

// MÍSTA
define('SOURCE_FORM', 'form');
define('SOURCE_TISK', 'tisk');

// TYPY VSTUPU
define('CDOin_DB', 1);
define('CDOin_ARCHIVE', 2);
define('CDOin_POST', 3);
define('CDOin_XML', 4);
define('CDOin_EMPTY', 5);
define('CDOin_SQL', 6);

// TYPY VÝSTUPU
define('CDOout_DB', 1);
define('CDOout_ARCHIVE', 2);
define('CDOout_XML', 3);
define('CDOout_PDF', 4);
define('CDOout_HTML', 5);
define('CDOout_WORD', 6);

//CHYBOVÉ KÓDY
define('OK', -1000);
define('NOT_OK', -1001);
define('DB_ERROR', -100);
define('BAD_INPUT_TYPE', -1);
define('BAD_OUTPUT_TYPE', -2);
define('ADDED_ID_EXISTS', -3);
define('NO_ID_GIVEN', -4);
define('WRONG_ID_GIVEN', -5);

//DEFAULTNÍ HODNOTY
define('EXT_POZN_EMPTY', 0);
define('DEF_EXT_POZN_TYP', 7); // typ externí poznámky = obec.
define('DEF_EXEMPL_VYSTUP', 2); // typ výstupu exemplifikace = tištěný + elektronický

//TYP NADŘAZENÉHO ZÁZNAMU
define('VARIANTA', 1);
define('VYZNAM', 2);
define('PODHESLI', 3);
define('HESLO', 4);

//DVOJICE TVAR-KOMENTÁŘ
define('VYSLOVNOST', 1);
define('KONCOVKA_TVARCHAR', 2);
define('KONCOVKA_TVARCHAR_SL', 3);
define('VAZEBNOST', 4);
define('PREDVYKLAD', 5);
define('PREDVYKLAD_M', 6);

// RICH EDITOR
define('VYKLAD_VYZNAMU', 'vyklad');
define('EXT_POZN', 'extpozn');
define('EXEMPLIFIKACE', 'exempl');
define('KOM_TVARU', 'komtv');

function getDefaultCDO($source)
{
    $defaultCDO = array(
        SOURCE_FORM => SESS_CURR_CDO,
        SOURCE_TISK => SESS_TISK_CDO
    );
    return $defaultCDO[$source];
}

require_once(__DIR__ . '/cdo/response.php');
require_once(__DIR__ . '/cdo/abs.base.php');
require_once(__DIR__ . '/cdo/ciselnik.php');
require_once(__DIR__ . '/cdo/tvarkom.php');
require_once(__DIR__ . '/cdo/slovdruh.php');
require_once(__DIR__ . '/cdo/tvarchar.php');
require_once(__DIR__ . '/cdo/extpozn.php');
require_once(__DIR__ . '/cdo/exempl.php');
require_once(__DIR__ . '/cdo/podhesli.php');
require_once(__DIR__ . '/cdo/varianta.php');
//require_once(__DIR__ . '/cdo/odkzakl.php');
require_once(__DIR__ . '/cdo/vyznam.php');
require_once(__DIR__ . '/cdo/semafor.php');
require_once(__DIR__ . '/cdo/heslo.php');
require_once(__DIR__ . '/cdo/link.php');
require_once(__DIR__ . '/cdo/CDOList.php');
