<?php
function setAndTrue($var, $default)
{
    return (isset($var) && $var ? $var : $default);
}

/**
 * poznĂˇmka k dÄ›dÄ›nĂ­ (aĹľ se bude Ĺ™eĹˇit uklĂˇdĂˇnĂ­):
 * kdyĹľ vyberu vĂ˝znam k dÄ›dÄ›nĂ­, musĂ­m si nÄ›kam schovat pĹŻvodnĂ­ hodnoty,
 * kdybych se rozhodl pĹ™ed uloĹľenĂ­m nedÄ›dit, tak aby se to vrĂˇtilo zpÄ›t
 */
class Heslo
{
    private $data;
    public $hid;
    private $is_archive;
    private $prefix;
    private $archive_id;
    private $podh_vyz;
    private $is_loaded;

    function __construct($hid = False, $archive = False)
    {
        $this->is_archive = $archive;
        $this->prefix = $archive ? 'archive_' : '';
        $this->archive_id = False;
        $this->data = new DibiRow(array());
        $this->loaded = False;
        if ($hid) {
            $this->hid = $hid;
        }
    }

    /**
     * VracĂ­ data celĂ©ho CDO
     * @return DibiRow
     */
    public function get()
    {
        return $this->data;
    }

    /**
     * VracĂ­ id hesla, kterĂ© je naÄŤtenĂ© v CDO
     * @return int
     */
    public function get_hid()
    {
        return $this->hid;
    }

    /**
     * Je naÄŤtenĂ© heslo verzĂ­ z archivu?
     * @return bool
     */
    public function is_archive()
    {
        return $this->is_archive;
    }

    /**
     * Id nĂˇsledujĂ­cĂ­ verze v archivu
     * @return mixed
     */
    public function next_arch_entry()
    {
        return dibi::fetchSingle('
            SELECT MAX(id) FROM archive_heslo
            WHERE main_heslo = %i AND id > %i
        ', $this->hid, $this->archive_id);
    }

    /**
     * Id pĹ™edchozĂ­ verze v archivu
     * @return mixed
     */
    public function prev_arch_entry()
    {
        return dibi::fetchSingle('
            SELECT MAX(id) FROM archive_heslo
            WHERE main_heslo = %i AND id < %i
        ', $this->hid, $this->archive_id);
    }

    /**
     * NaÄŤtenĂ­ ÄŤĂ­selnĂ­ku do CDO
     * @param string $vyber_tbl - tabulka vĂ˝bÄ›ru
     * @param string $cis_tbl - tabulka celĂ©ho ÄŤĂ­selnĂ­ku
     * @param string $txt_col - sloupec obsahujĂ­cĂ­ text poloĹľky
     * @param string $fk_name - sloupec s cizĂ­m klĂ­ÄŤem v tabulce vĂ˝bÄ›ru
     * @param int $fk_val - hodnota cizĂ­ho klĂ­ÄŤe v tabulce vĂ˝bÄ›ru
     * @return array - vĂ˝bÄ›r ÄŤĂ­selnĂ­ku
     */
    private function get_cis($vyber_tbl, $cis_tbl, $txt_col, $fk_name, $fk_val)
    {
        $vyber_tbl = $this->prefix . $vyber_tbl;
        try {
//            vracĂ­ pole s vĂ˝bÄ›rem ÄŤĂ­selnĂ­ku ve formĂˇtu:
//              id => text,
//              "jine" => text
            return dibi::fetchPairs('
				SELECT IFNULL(cis_id, "jine"), IF(cis_id IS NULL, jine, c.%n)
				FROM %n v LEFT JOIN %n c ON v.cis_id=c.id
				WHERE v.%n = %i
				ORDER BY v.sortorder ASC
			', $txt_col, $vyber_tbl, $cis_tbl, $fk_name, $fk_val);
        } catch (DibiException $e) {
//            pokud tabulka vĂ˝bÄ›ru jeĹˇtÄ› neexistuje v archivu, vracĂ­me prĂˇzdnĂ˝ vĂ˝bÄ›r
            if ($this->is_archive && $e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                return array();
            } else {
                var_dump($e);
                die();
            }
        }
    }

    /**
     * NaÄŤtenĂ­ vĂ˝slovnosti do CDO
     * @param string $parent - varianta (var) / podheslĂ­ (podh)
     * @param int $fk_val - id nadĹ™azenĂ©ho zĂˇznamu (varianta nebo podheslĂ­)
     * @param bool|False $copyTo - data nejdĹ™Ă­ve uloĹľit do novĂ© vĂ˝slovnosti
     * @return array - vĂ˝slovnosti (id => [vyslovnost, komentar])
     */
    private function load_vyslovnost($parent, $fk_val, $copyTo = False)
    {
        if ($parent == 'var') {
            $tbl = 'vyslovnost_var';
            $fk_name = 'vid';
        } elseif ($parent == 'podh') {
            $tbl = 'podh_vyz_vyslovnost';
            $fk_name = 'podh_id';
        } else {
            die();
        }

        $tbl = $this->prefix . $tbl;
        try {
            $data = dibi::query('
				SELECT id, vyslovnost, komentar
				FROM %n WHERE %n = %i
			', $tbl, $fk_name, $fk_val)->fetchAssoc('id');

            if ($copyTo) {
                $newData = array();
                foreach ($data as $vid => $d) {
                    $newid = $this->add_empty('vyslovnost', $parent, $copyTo);
                    $newData[$newid] = $d;
                }
                $data = $newData;
            }

            return $data;
        } catch (DibiException $e) {
            if ($this->is_archive && $e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                return array();
            } else {
                var_dump($e);
                die();
            }
        }
    }

    /**
     * NaÄŤtenĂ­ externĂ­ poznĂˇmky do CDO
     * @param string $parent - heslo (hes) / vĂ˝znam (vyz)
     * @param int $fk_val - id nadĹ™azenĂ©ho zĂˇznamu (heslo nebo vĂ˝znam)
     * @return array - externĂ­ poznĂˇmky (id => [id typu, poznamka, uts, text typu])
     */
    private function load_externi_poznamka($parent, $fk_val)
    {
        if ($parent == 'hes') {
            $tbl = 'ext_poznamky_hes';
            $fk_name = 'hid';
        } elseif ($parent == 'vyz') {
            $tbl = 'ext_poznamky_vyz';
            $fk_name = 'vyz_id';
        } else {
            die();
        }
        $tbl = $this->prefix . $tbl;
        try {
            return dibi::query('
				SELECT v.id, v.cis_id, v.poznamka, v.uts, c.typ
				FROM %n v LEFT JOIN cis_typ_ext_pozn c ON v.cis_id = c.id
				WHERE v.%n = %i
			', $tbl, $fk_name, $fk_val)->fetchAssoc('id');
        } catch (DibiException $e) {
            if ($this->is_archive && $e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                return array();
            } else {
                var_dump($e);
                die();
            }
        }
    }

    /**
     * NaÄŤtenĂ­ vazebnostĂ­ do CDO
     * @param string $parent - varianta (var) / vĂ˝znam (vyz) / podheslĂ­ (podh)
     * @param int $fk_val - id nadĹ™azenĂ©ho zĂˇznamu (var/vyz/podh)
     * @param bool|False $copyTo - data nejdĹ™Ă­ve uloĹľit do novĂ© vazebnosti
     * @return array - vazebnosti (id => [vazebnost, komentar])
     */
    private function load_vazebnost($parent, $fk_val, $copyTo = False)
    {
        if ($parent == 'var') {
            $tbl = 'vazebnost_var';
            $fk_name = 'vid';
        } elseif ($parent == 'vyz') {
            $tbl = 'vazebnost_vyz';
            $fk_name = 'vyz_id';
        } elseif ($parent == 'podh') {
            $tbl = 'podhesli_vazebnost';
            $fk_name = 'podh_id';
        } else {
            die();
        }

        $tbl = $this->prefix . $tbl;
        try {
            $data = dibi::query('
				SELECT id, vazebnost, komentar
				FROM %n WHERE %n = %i
			', $tbl, $fk_name, $fk_val)->fetchAssoc('id');

            if ($copyTo) {
                $newData = array();
                foreach ($data as $vid => $d) {
                    $newid = $this->add_empty('vazebnost', $parent, $copyTo);
                    $newData[$newid] = $d;
                }
                $data = $newData;
            }

            return $data;
        } catch (DibiException $e) {
            if ($this->is_archive && $e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                return array();
            } else {
                var_dump($e);
                die();
            }
        }
    }

    /**
     * NaÄŤtenĂ­ tvaroslovnĂ˝ch charakteristik do CDO
     * @param string $type - tvar. char. (tvar_char) / tvar. char. slovesa (tvar_char_sl)
     * @param string $parent - varianta (var) / vĂ˝znam (vyz) / podheslĂ­ (podh)
     * @param int $fk_val - id nadĹ™azenĂ©ho zĂˇznamu (var/vyz/podh)
     * @param bool|False $copyTo - data nejdĹ™Ă­ve uloĹľit do novĂ© tvar. char.
     * @return array
     */
    private function load_tvar_char($type, $parent, $fk_val, $copyTo = False)
    {
        if ($type == 'tvar_char') {
            $add = '';
            $nm = 'tvarchar';
        } elseif ($type == 'tvar_char_sl') {
            $add = '_sl';
            $nm = 'tvarchar_sl';
        } else {
            die();
        }
        if ($parent == 'var') {
            $tbl = 'var_tvar_char' . $add . '_vyber_rel';
            $fk_name = 'vid';
            $vyber_tbl = 'tvar_char' . $add . '_vyber';
            $vyber_fk = 'var_tvar_char' . $add . '_vyber_rel_id';
            $konc_tbl = 'koncovky_tvar_char' . $add;
            $konc_fk = 'var_tvar_char' . $add . '_rel_id';
        } elseif ($parent == 'vyz') {
            $tbl = 'vyz_tvar_char' . $add . '_vyber_rel';
            $fk_name = 'vyz_id';
            $vyber_tbl = 'tvar_char' . $add . '_vyber_vyz';
            $vyber_fk = 'vyz_tvar_char' . $add . '_vyber_rel_id';
            $konc_tbl = 'koncovky_tvar_char' . $add . '_vyz';
            $konc_fk = 'vyz_tvar_char' . $add . '_rel_id';
        } elseif ($parent == 'podh') {
            $tbl = 'podh_tvar_char' . $add . '_vyber_rel';
            $fk_name = 'podh_id';
            $vyber_tbl = 'podh_tvar_char' . $add . '_vyber';
            $vyber_fk = 'podh_tvch' . $add . '_rel_id';
            $konc_tbl = 'podh_vyz_koncovky_tvar_char' . $add;
            $konc_fk = 'podh_tvch' . $add;
        } else {
            die();
        }

        $tbl = $this->prefix . $tbl;
        try {
            $tmp = dibi::query('
				SELECT id, sortorder FROM %n WHERE %n = %i
				ORDER BY sortorder ASC
			', $tbl, $fk_name, $fk_val)->fetchAssoc('id');
            $transl_ids = array();
            if ($copyTo) {
                $newData = array();
                foreach ($tmp as $vid => $d) {
                    $newid = $this->add_empty($nm, $parent, $copyTo);
                    $newData[$newid] = $d;
                    $transl_ids[$newid] = $vid;
                }
                $tmp = $newData;
            }

        } catch (DibiException $e) {
            if ($this->is_archive && $e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                return array();
            } else {
                var_dump($e);
                die();
            }
        }
        foreach ($tmp as $rid => $tvch) {
            if ($copyTo) {
                $inid = $transl_ids[$rid];
            } else {
                $inid = $rid;
            }
            $tmp[$rid]['vyber'] = $this->get_cis($vyber_tbl, 'cis_tvar_char' . $add, 'tvar_char' . $add, $vyber_fk, $inid);
            try {
                $tmp[$rid]['koncovky'] = dibi::query('
					SELECT id, koncovka, komentar FROM %n WHERE %n = %i
					ORDER BY sortorder ASC
				', $konc_tbl, $konc_fk, $inid)->fetchAssoc('id');

                if ($copyTo) {
                    $newtmp = array();
                    foreach ($tmp[$rid]['koncovky'] as $idk => $konc) {
                        $newid = $this->add_empty('koncovka_' . $nm, $parent, $rid);
                        $newtmp[$newid] = $konc;
                    }
                    $tmp[$rid]['koncovky'] = $newtmp;
                }

            } catch (DibiException $e) {
                if ($this->is_archive && $e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                    $tmp[$rid]['koncovky'] = array();
                } else {
                    var_dump($e);
                    die();
                }
            }
        }

        return $tmp;
    }

    /**
     * NaÄŤtenĂ­ slovnĂ­ho druhu do CDO
     * @param string $parent - varianta (var) / vĂ˝znam (vyz) / podheslĂ­ (podh)
     * @param int $fk_val - id nadĹ™azenĂ©ho zĂˇznamu (var/vyz/podh)
     * @param bool|False $copyTo - data nejdĹ™Ă­ve uloĹľit do novĂ©ho slovnĂ­ho druhu
     * @return array
     */
    private function load_slovni_druh($parent, $fk_val, $copyTo = False)
    {
        if ($parent == 'var') {
            $tbl = 'var_slov_druhy_vyber_rel';
            $fk_name = 'vid';
            $vyber_tbl = 'slov_druhy_vyber';
            $vyber_fk = 'var_slov_druhy_vyber_rel_id';
        } elseif ($parent == 'vyz') {
            $tbl = 'vyz_slov_druhy_vyber_rel';
            $fk_name = 'vyz_id';
            $vyber_tbl = 'slov_druhy_vyber_vyz';
            $vyber_fk = 'vyz_slov_druhy_vyber_rel_id';
        } elseif ($parent == 'podh') {
            $tbl = 'podh_vyz_slov_druhy_vyber_rel';
            $fk_name = 'podh_id';
            $vyber_tbl = 'podh_vyz_slov_druhy';
            $vyber_fk = 'podh_rel_id';
        } else {
            die();
        }

        try {
            $tmp = dibi::query('
				SELECT id, komentar FROM %n WHERE %n = %i
			', $tbl, $fk_name, $fk_val)->fetchAssoc('id');
            $transl_ids = array();
            if ($copyTo) {
                $newData = array();
                foreach ($tmp as $vid => $d) {
                    $newid = $this->add_empty('slovni_druh', $parent, $copyTo);
                    $newData[$newid] = $d;
                    $transl_ids[$newid] = $vid;
                }
                $tmp = $newData;
            }

        } catch (DibiException $e) {
            if ($this->is_archive && $e->getCode() == MYSQL_TABLE_NOT_EXISTS) {
                return array();
            } else {
                var_dump($e);
                die();
            }
        }
        foreach ($tmp as $rid => $sldr) {
            if ($copyTo) {
                $inid = $transl_ids[$rid];
            } else {
                $inid = $rid;
            }
            $tmp[$rid]['vyber'] = $this->get_cis($vyber_tbl, 'cis_slov_druhy', 'slovny_druh', $vyber_fk, $inid);
        }
        return $tmp;
    }

    /**
     * NaÄŤtenĂ­ vĂ˝razu pĹ™edvĂ˝kladu
     * @param int $vyz_id - id nadĹ™azenĂ©ho vĂ˝znamu
     * @param int $link_id - id skupiny, pokud je vĂ˝znam dÄ›dÄ›nĂ˝
     * @param bool $predv_link - dÄ›dÄ›nĂ˝ pĹ™edvĂ˝klad
     * @return mixed
     */
    public function load_vyraz_predvykladu($vyz_id, $link_id, $predv_link)
    {
        if ($link_id && $predv_link) {
            return dibi::query('
				SELECT id, sortorder, vyraz, komentar
				FROM vyrazy_predvykladu_m
				WHERE vyz_id = %i
				ORDER BY sortorder ASC
			', $link_id)->fetchAssoc('id');
        } else {
            return dibi::query('
				SELECT id, sortorder, vyraz, komentar
				FROM vyrazy_predvykladu
				WHERE vyz_id = %i
				ORDER BY sortorder ASC
			', $vyz_id)->fetchAssoc('id');
        }
    }

    /**
     * VytvoĹ™enĂ­ kopie varianty
     * @param int $vid - id vzorovĂ© varianty
     * @param int $hid - id hesla se vzorovou variantou
     * @return mixed - id novĂ© varianty
     */
    public function load_varianta_copy($vid, $hid)
    {
        $this->hid = $hid;
        $this->loaded = True;
        $vid_new = $this->add_empty('varianta', 'hes', $hid);
        $this->data->varianty[$vid_new] = dibi::fetch('
			SELECT hid, varianta, neosemantismus, homonymie, komentar, komentar_za, prvni_reg_vyskyt,
				 komentar_tvarchar, komentar_tvarchar_slovesa, varianty_bez_zavor, f_varianta, (sortorder+1) AS sortorder
			FROM varianty WHERE id = %i
		', $vid);
        $this->data->varianty[$vid_new]['vazebnost'] = $this->load_vazebnost('var', $vid, $vid_new);
        $this->data->varianty[$vid_new]['vyslovnost'] = $this->load_vyslovnost('var', $vid, $vid_new);
        $this->data->varianty[$vid_new]['typ_hesla'] = $this->get_cis('typ_hesla_var_vyber', 'cis_typ_hesla', 'typ_hesla', 'vid', $vid);
        $this->data->varianty[$vid_new]['styl_kval'] = $this->get_cis('styl_kval_var_vyber', 'cis_styl_kval', 'styl_kval', 'vid', $vid);
        $this->data->varianty[$vid_new]['tematicke_okruhy'] = $this->get_cis('tematicke_okruhy_var_vyber', 'cis_tematicke_okruhy', 'tematicke_okruhy', 'vid', $vid);
        $this->data->varianty[$vid_new]['obor_uziti'] = $this->get_cis('obor_uziti_var_vyber', 'cis_obor_uziti', 'obor_uziti', 'vid', $vid);
        $this->data->varianty[$vid_new]['gram_info'] = $this->get_cis('gram_info_var_vyber', 'cis_gram_info', 'gram_info', 'vid', $vid);
        $this->data->varianty[$vid_new]['puvod'] = $this->get_cis('puvod_var_vyber', 'cis_puvod', 'puvod', 'vid', $vid);
        $this->data->varianty[$vid_new]['tvar_char'] = $this->load_tvar_char('tvar_char', 'var', $vid, $vid_new);
        $this->data->varianty[$vid_new]['tvar_char_sl'] = $this->load_tvar_char('tvar_char_sl', 'var', $vid, $vid_new);
        $this->data->varianty[$vid_new]['slov_druhy'] = $this->load_slovni_druh('var', $vid, $vid_new);
        return $vid_new;
    }

    /**
     * NaÄŤtenĂ­ pouze zĂˇkladnĂ­ch informacĂ­ o variantĂˇch
     * @param $main_pk - id hesla s variantou
     */
    private function load_varianty_main($main_pk)
    {
        $this->data['varianty'] = dibi::query('
			SELECT id, varianta, neosemantismus, homonymie, komentar, komentar_za, prvni_reg_vyskyt, sortorder, uts,
			 komentar_tvarchar, komentar_tvarchar_slovesa, varianty_bez_zavor, f_varianta, komentar_tvaru
			FROM %n WHERE hid = %i
			ORDER BY sortorder ASC
		', $this->prefix . 'varianty', $main_pk)->fetchAssoc('id');
    }

    /**
     * NaÄŤtenĂ­ varianty do CDO
     * @param int $id - id varianty
     * @param bool|False $load_main - naÄŤtenĂ­ i zĂˇkladnĂ­ch informacĂ­
     * @return int
     */
    private function load_varianta($id, $load_main = False)
    {
        if ($load_main) {
            $this->data['varianty'] = dibi::query('
				SELECT id, varianta, neosemantismus, homonymie, komentar, komentar_za, prvni_reg_vyskyt, sortorder, uts,
					 komentar_tvarchar, komentar_tvarchar_slovesa, varianty_bez_zavor, f_varianta, komentar_tvaru
				FROM %n WHERE id = %i
				ORDER BY sortorder ASC
			', $this->prefix . 'varianty', $id)->fetchAssoc('id');
        }
        $this->data->varianty[$id]['vazebnost'] = $this->load_vazebnost('var', $id);
        $this->data->varianty[$id]['vyslovnost'] = $this->load_vyslovnost('var', $id);
        $this->data->varianty[$id]['typ_hesla'] = $this->get_cis('typ_hesla_var_vyber', 'cis_typ_hesla', 'typ_hesla', 'vid', $id);
        $this->data->varianty[$id]['styl_kval'] = $this->get_cis('styl_kval_var_vyber', 'cis_styl_kval', 'styl_kval', 'vid', $id);
        $this->data->varianty[$id]['tematicke_okruhy'] = $this->get_cis('tematicke_okruhy_var_vyber', 'cis_tematicke_okruhy', 'tematicke_okruhy', 'vid', $id);
        $this->data->varianty[$id]['obor_uziti'] = $this->get_cis('obor_uziti_var_vyber', 'cis_obor_uziti', 'obor_uziti', 'vid', $id);
        $this->data->varianty[$id]['gram_info'] = $this->get_cis('gram_info_var_vyber', 'cis_gram_info', 'gram_info', 'vid', $id);
        $this->data->varianty[$id]['puvod'] = $this->get_cis('puvod_var_vyber', 'cis_puvod', 'puvod', 'vid', $id);
        $this->data->varianty[$id]['tvar_char'] = $this->load_tvar_char('tvar_char', 'var', $id);
        $this->data->varianty[$id]['tvar_char_sl'] = $this->load_tvar_char('tvar_char_sl', 'var', $id);
        $this->data->varianty[$id]['slov_druhy'] = $this->load_slovni_druh('var', $id);
        return 1;
    }

    /**
     * NaÄŤtenĂ­ pouze zĂˇkladnĂ­ch informacĂ­ o vĂ˝znamech
     * @param $main_pk - id hesla s vĂ˝znamem
     */
    private function load_vyznamy_main($main_pk)
    {

    }

    /**
     * NaÄŤtenĂ­ vĂ˝znamu do CDO
     * @param $id - id vĂ˝znamu
     * @param bool|False $load_main - naÄŤtenĂ­ i zĂˇkladnĂ­ch informacĂ­
     */
    private function load_vyznam($id, $load_main = False)
    {

    }

    /**
     * NaÄŤtenĂ­ pouze zĂˇkladnĂ­ch informacĂ­ o podheslĂ­ch vĂ˝znamu
     * @param $main_pk - id vĂ˝znamu s podheslĂ­mi
     */
    private function load_podhesli_main($main_pk)
    {
        $this->data->vyznamy[$main_pk]->podhesli = dibi::query('
				SELECT id, komentar_pred, podhesli, komentar_za, komentar_tvarchar, komentar_tvarchar_slovesa
				FROM %n WHERE vyz_id = %i
			', $this->prefix . 'podhesli_vyznamu', $main_pk)->fetchAssoc('id');
    }

    /**
     * NaÄŤtenĂ­ podheslĂ­ vĂ˝znamu do CDO
     * @param int $vyz_id - id vĂ˝znamu
     * @param int $id - id podheslĂ­
     * @param bool|False $load_main - naÄŤtenĂ­ i zĂˇkladnĂ­ch informacĂ­
     */
    private function load_podhesli($vyz_id, $id, $load_main = False)
    {
        if ($load_main) {
            dibi::query('
				SELECT id, komentar_pred, podhesli, komentar_za, komentar_tvarchar, komentar_tvarchar_slovesa
				FROM %n WHERE vyz_id = %i AND id = %i
			', $this->prefix . 'podhesli_vyznamu', $vyz_id, $id)->fetchAssoc('id');
        }
        $this->podh_vyz[$id] = $vyz_id;
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['vyslovnost'] = $this->load_vyslovnost('podh', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['vazebnost'] = $this->load_vazebnost('podh', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['styl_kval'] = $this->get_cis('styl_kval_podh_vyber', 'cis_styl_kval', 'styl_kval', 'podh_id', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['tematicke_okruhy'] = $this->get_cis('tematicke_okruhy_podh_vyber', 'cis_tematicke_okruhy', 'tematicke_okruhy', 'podh_id', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['obor_uziti'] = $this->get_cis('obor_uziti_podh_vyber', 'cis_obor_uziti', 'obor_uziti', 'podh_id', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['gram_info'] = $this->get_cis('podh_vyz_gram_info', 'cis_gram_info', 'gram_info', 'podh_id', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['tvar_char'] = $this->load_tvar_char('tvar_char', 'podh', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['tvar_char_sl'] = $this->load_tvar_char('tvar_char_sl', 'podh', $id);
        $this->data->vyznamy[$vyz_id]->podhesli[$id]['slov_druhy'] = $this->load_slovni_druh('podh', $id);
    }

    private function load_propojovani($type, $rel_place, $id, $place = False)
    {
        if ($type == 'odk') {
            $rel_types = array(HREL_TYPE_odkazy);
        } elseif ($type == 'prip') {
            $rel_types = array(HREL_TYPE_pripojovani, HREL_TYPE_prihnizdovani);
        } elseif ($type == 'syn') {
            $rel_types = array(HREL_TYPE_syn, HREL_TYPE_syn_przn);
        } elseif ($type == 'ant') {
            $rel_types = array(HREL_TYPE_ant, HREL_TYPE_ant_przn);
        } else {
            return array();
        }
        if ($rel_place == 'main') {
            $hidcol = 'main_hid';
            $reltable = 'hesla_rel_places';
            $sortcol = 'sortorder';
            if ($type == 'ant' || $type == 'syn') {
                $place = $type;
            }
        } else { //slave
            $hidcol = 'slave_hid';
            $reltable = 'hesla_rel_slave_places';
            $sortcol = 'sortorder2';
        }
        if ($this->is_archive) {
            $reltable = 'archive_' . $reltable;
            if ($type == 'odk' || ($type == 'prip' && $rel_place == 'slave')) {
                $data = dibi::nativeQuery('
                    SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id, cis.link_type
                    FROM `archive_hesla_rel` hr LEFT JOIN ' . $reltable . ' hrp ON hr.id = hrp.hesla_rel_id
                    LEFT JOIN (`archive_hesla_rel_link_type_vyber` hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id)
                        ON hr.id = hrlt.hesla_rel_id
                    WHERE  `archived`="' . $rel_place . '" AND `archived_id` = ' . $this->archive_id . '  AND `rel_type` IN  (' . implode($rel_types, ",") . ')
                    ORDER BY hr.' . $sortcol)->fetchAssoc('id');
            } else {
                if (empty($id)) $id = 0;
                if ($place == 'heslo') {
                    $data = dibi::nativeQuery('
                        SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id, cis.link_type
                        FROM `archive_hesla_rel` hr INNER JOIN ' . $reltable . ' hrp ON hr.id = hrp.hesla_rel_id
                        LEFT JOIN (`archive_hesla_rel_link_type_vyber` hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id)
                            ON hr.id = hrlt.hesla_rel_id
                        WHERE `archived`="' . $rel_place . '" AND `archived_id` = ' . $this->archive_id . ' AND `rel_type` IN (' . implode($rel_types, ",") . ') AND `place` = "' . $place . '"
                        ORDER BY hr.' . $sortcol)->fetchAssoc('id');
                } else {
                    $data = dibi::nativeQuery('
                        SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id, cis.link_type
                        FROM `archive_hesla_rel` hr INNER JOIN ' . $reltable . ' hrp ON hr.id = hrp.hesla_rel_id
                        LEFT JOIN (`archive_hesla_rel_link_type_vyber` hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id)
                            ON hr.id = hrlt.hesla_rel_id
                        WHERE `archived`="' . $rel_place . '" AND `archived_id` = ' . $this->archive_id . ' AND `rel_type` IN (' . implode($rel_types, ",") . ') AND `place` = "' . $place . '" AND `place_id` = ' . $id . '
                        ORDER BY hr.' . $sortcol)->fetchAssoc('id');
                }
            }
        } else {
            if ($type == 'odk' || ($type == 'prip' && $rel_place == 'slave')) {
                $data = dibi::nativeQuery('
                    SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id, cis.link_type
                    FROM `hesla_rel` hr LEFT JOIN ' . $reltable . ' hrp ON hr.id = hrp.hesla_rel_id
                    LEFT JOIN (`hesla_rel_link_type_vyber` hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id)
                        ON hr.id = hrlt.hesla_rel_id
                    WHERE  ' . $hidcol . '=' . $this->hid . '  AND `rel_type` IN  (' . implode($rel_types, ",") . ')
                    ORDER BY hr.' . $sortcol)->fetchAssoc('id');
                /*$data = dibi::query('
                    SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id
                    FROM `hesla_rel` hr LEFT JOIN %n hrp ON hr.id = hrp.hesla_rel_id
              LEFT JOIN `hesla_rel_link_type_vyber` hrlt ON hr.id = hrlt.hesla_rel_id
                    WHERE %n=%i AND `rel_type` IN %in
                    ORDER BY hr.%n
                ', $reltable, $hidcol, $this->hid, $rel_types, $sortcol)->fetchAssoc('id');*/
            } else {
                if (empty($id)) $id = 0;
                if ($place == 'heslo') {
                    $data = dibi::nativeQuery('
                        SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id, cis.link_type
                        FROM `hesla_rel` hr INNER JOIN ' . $reltable . ' hrp ON hr.id = hrp.hesla_rel_id
                        LEFT JOIN (`hesla_rel_link_type_vyber` hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id)
                            ON hr.id = hrlt.hesla_rel_id
                        WHERE ' . $hidcol . '=' . $this->hid . ' AND `rel_type` IN (' . implode($rel_types, ",") . ') AND `place` = "' . $place . '"
                        ORDER BY hr.' . $sortcol)->fetchAssoc('id');
                } else {
                    $data = dibi::nativeQuery('
                        SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id, cis.link_type
                        FROM `hesla_rel` hr INNER JOIN ' . $reltable . ' hrp ON hr.id = hrp.hesla_rel_id
                        LEFT JOIN (`hesla_rel_link_type_vyber` hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id)
                            ON hr.id = hrlt.hesla_rel_id
                        WHERE ' . $hidcol . '=' . $this->hid . ' AND `rel_type` IN (' . implode($rel_types, ",") . ') AND `place` = "' . $place . '" AND `place_id` = ' . $id . '
                        ORDER BY hr.' . $sortcol)->fetchAssoc('id');
                }

                /*$data = dibi::query('
                    SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str, cis_id
                    FROM `hesla_rel` hr INNER JOIN %n hrp ON hr.id = hrp.hesla_rel_id
              LEFT JOIN `hesla_rel_link_type_vyber` hrlt ON hr.id = hrlt.hesla_rel_id
                    WHERE %n=%i AND `rel_type` IN %in AND `place` = %s AND `place_id` = %i
                    ORDER BY hr.%n
                ', $reltable, $hidcol, $this->hid, $rel_types, $place, $id, $sortcol)->fetchAssoc('id'); */
            }
        }
        $mtbl = $this->is_archive ? 'archive_hesla_rel_places' : 'hesla_rel_places';
        $stbl = $this->is_archive ? 'archive_hesla_rel_slave_places' : 'hesla_rel_slave_places';
        foreach ($data as $rel_id => $d) {
            $data[$rel_id]->places = array(
                'main' => dibi::nativeQuery('SELECT id, place, place_id FROM ' . $mtbl . ' WHERE hesla_rel_id = ' . $rel_id)->fetchAssoc('id'),
                'slave' => dibi::nativeQuery('SELECT id, place, place_id FROM ' . $stbl . ' WHERE hesla_rel_id = ' . $rel_id)->fetchAssoc('id')
            );
        }

        return $data;
    }

    /*private function load_syn_ant($type, $rel_place, $id, $place=False){
        // $rel_place ... 'main' nebo 'slave'
        // $place ... main: 'syn', 'ant' / slave: 'vyznam', 'heslo'
        if ($type == 'syn'){
            $rel_types = array(HREL_TYPE_syn, HREL_TYPE_syn_przn);
        }elseif ($type == 'ant'){
            $rel_types = array(HREL_TYPE_ant, HREL_TYPE_ant_przn);
        }else{
            return array();
        }
        if ($rel_place == 'main'){
            $hidcol = 'main_hid';
            $reltable = 'hesla_rel_places';
            $sortcol = 'sortorder';
            $place = $type;
        }else{ //slave
            $hidcol = 'slave_hid';
            $reltable = 'hesla_rel_slave_places';
            $sortcol = 'sortorder2';
        }
        $data = dibi::query('
            SELECT hr.id, komentar, poznamka, main_hid, slave_hid, main_str, slave_str
            FROM `hesla_rel` hr INNER JOIN %n hrp ON hr.id = hrp.hesla_rel_id
            WHERE %n=%i AND `rel_type` IN %in AND `place` = %s AND `place_id` = %i
            ORDER BY hr.%n
        ', $reltable, $hidcol, $this->hid, $rel_types, $place, $id, $sortcol)->fetchAssoc('id');
        return $data;
    }              */
    /**
     * Korektura na danĂ©m elementu
     * @param string $domid - atribut id elementu
     * @return array
     */
    public function getCorr($domid)
    {
        if (isset($this->data['korektury'][$domid])) {
            return $this->data['korektury'][$domid];
        } else {
            return array(
                'id' => 0,
                'field' => '',
                'stavy' => -1,
            );
        }
    }

    /**
     * NaÄŤtenĂ­ celĂ©ho hesla do CDO
     * @param int $hid - id hesla
     * @param bool|False $archive - id verze v archivu
     */
    public function load($hid, $archive = False)
    {
        $this->loaded = True;
        $this->hid = $hid;
        $this->data = array();
        if (!$archive) {
            $this->is_archive = False;
            $this->prefix = '';
            $main_pk = $hid;
        } else {
            $this->is_archive = True;
            $this->prefix = 'archive_';
            $main_pk = $archive;
        }
        $ver_id = $archive;
        if ($this->is_archive) {
            $this->data = dibi::fetch('
				SELECT id, cts, uts, stav, vystup, lexgraf_pozn, pozn_odbornik, pid, redukce, heslar_id
				FROM `archive_heslo` WHERE id = %i AND `main_heslo` = %i
			', $main_pk, $hid);
        } else {
            $this->data = dibi::fetch('
				SELECT id, cts, uts, stav, vystup, lexgraf_pozn, pozn_odbornik, pid, redukce, heslar_id
				FROM `heslo` WHERE id = %i
			', $hid);
        }

        if ($archive === False) {
            $this->data['korektury'] = dibi::query(
                'SELECT k1.field_DOMid, k1.id, k1.field, COUNT(ks.id) AS stavy
				FROM korektura k1 LEFT JOIN korektura k2
							ON (k1.field_DOMid = k2.field_DOMid AND k1.id < k2.id)
						LEFT JOIN korektura_stavy ks ON ks.kor_id=k1.id
				WHERE k2.id IS NULL AND k1.hid=%i
				GROUP BY k1.id
			', $hid)->fetchAssoc('field_DOMid');
        } else {
            $this->data['korektury'] = array();
        }


        if ($this->data['pid'] && !$this->is_archive) {
            $this->data['parent_heslo'] = dibi::fetchSingle('SELECT `varianta` FROM `varianty` WHERE `hid` = %i AND `sortorder` = 1', $this->data['pid']);
        } else {
            $this->data['parent_heslo'] = null;
        }

        $this->data['ext_poznamky'] = $this->load_externi_poznamka('hes', $main_pk);

        $this->data['zpracovatel'] = dibi::fetchPairs('
			SELECT v.uid, u.login
			FROM zpracovatel v INNER JOIN uzivatel u ON v.uid = u.id
			WHERE v.hid = %i
			ORDER BY sortorder ASC
		', $hid);

        $this->data['syn_slave_h'] = $this->load_propojovani('syn', 'slave', $hid, 'heslo');
        $this->data['ant_slave_h'] = $this->load_propojovani('ant', 'slave', $hid, 'heslo');
        $this->data['odk_slave_h'] = $this->load_propojovani('odk', 'slave', $hid, 'heslo');
        $this->data['prip_slave_h'] = $this->load_propojovani('prip', 'slave', $hid, 'heslo');
        $this->data['odk_main_h'] = $this->load_propojovani('odk', 'main', $hid, 'heslo');
        $this->data['prip_main_h'] = $this->load_propojovani('prip', 'main', $hid, 'heslo');

        $this->data['odk_all_h'] = $this->data['odk_main_h'] + $this->data['odk_slave_h'];

        // VARIANTY
        $this->load_varianty_main($main_pk);
        foreach ($this->data->varianty as $id => $var) {
            $this->load_varianta($id);
        }
        // KONEC VARIANT

        // VYZNAMY

        $this->data['vyznamy'] = dibi::query('
			SELECT v.id, pid, uts, komentar, vystup, vyklad_vyznamu_link, predvyklad_link, metavyklad1_link, metavyklad2_link,
				IF(predvyklad_link = 1 AND vyklad_vyznamu_link > 0, vm.predvyklad, v.predvyklad) AS predvyklad,
				IF(predvyklad_link = 1 AND vyklad_vyznamu_link > 0, vm.zacatek_predvykladu, v.zacatek_predvykladu) AS zacatek_predvykladu,
				IF(predvyklad_link = 1 AND vyklad_vyznamu_link > 0, vm.konec_predvykladu, v.konec_predvykladu) AS konec_predvykladu,
				IF(metavyklad1_link = 1 AND vyklad_vyznamu_link > 0, vm.metavyklad1, v.metavyklad1) AS metavyklad1_txt,
				IF(vyklad_vyznamu_link IS NULL OR vyklad_vyznamu_link = 0, v.vyklad_vyznamu, vm.vyklad_vyznamu) AS vyklad_vyznamu,
				IF(vyklad_vyznamu_link IS NULL OR vyklad_vyznamu_link = 0, v.vyklad_vyznamu_bz, vm.vyklad_vyznamu_bz) AS vyklad_vyznamu_bz,
				IF(metavyklad2_link = 1 AND vyklad_vyznamu_link > 0, vm.metavyklad2, v.metavyklad2) AS metavyklad2,
				koncovka_gram_info, sortorder, typ_vystupu, komentar_tvarchar, komentar_tvarchar_slovesa
			FROM %n v LEFT JOIN vyznamy_master vm ON v.vyklad_vyznamu_link=vm.id
			WHERE hid = %i ORDER BY sortorder ASC
		', $this->prefix . 'vyznamy', $main_pk)->fetchAssoc('id');

        foreach ($this->data->vyznamy as $id => $vyz) {
            $this->data->vyznamy[$id]['subvyznamy'] = dibi::fetchPairs('
				SELECT id FROM %n WHERE hid = %i AND pid = %i
			', $this->prefix . 'vyznamy', $main_pk, $id);

            $this->data->vyznamy[$id]['vazebnost'] = $this->load_vazebnost('vyz', $id);


            $this->data->vyznamy[$id]['exemplifikace'] = dibi::query('
				SELECT id, exemplifikace, exemplifikace_bz, vystup, sortorder FROM %n WHERE vyz_id = %i ORDER BY sortorder ASC
			', $this->prefix . 'exemplifikace', $id)->fetchAssoc('id');

            $this->data->vyznamy[$id]['vyrazy_predvykladu'] = $this->load_vyraz_predvykladu($id, $this->data->vyznamy[$id]['vyklad_vyznamu_link'], $this->data->vyznamy[$id]['predvyklad_link']);

            $this->data->vyznamy[$id]['styl_kval'] = $this->get_cis('styl_kval_vyz_vyber', 'cis_styl_kval', 'styl_kval', 'vyz_id', $id);
            $this->data->vyznamy[$id]['tematicke_okruhy'] = $this->get_cis('tematicke_okruhy_vyz_vyber', 'cis_tematicke_okruhy', 'tematicke_okruhy', 'vyz_id', $id);
            $this->data->vyznamy[$id]['obor_uziti'] = $this->get_cis('obor_uziti_vyz_vyber', 'cis_obor_uziti', 'obor_uziti', 'vyz_id', $id);
            $this->data->vyznamy[$id]['gram_info'] = $this->get_cis('gram_info_vyz_vyber', 'cis_gram_info', 'gram_info', 'vyz_id', $id);

            if ($this->data->vyznamy[$id]['vyklad_vyznamu_link'] && $this->data->vyznamy[$id]['metavyklad1_link']) {
                $this->data->vyznamy[$id]['metavyklad1'] = $this->get_cis('metavyklad1_vyz_vyber_m', 'cis_metavyklad1', 'metavyklad1', 'vyz_id', $this->data->vyznamy[$id]['vyklad_vyznamu_link']);
            } else {
                $this->data->vyznamy[$id]['metavyklad1'] = $this->get_cis('metavyklad1_vyz_vyber', 'cis_metavyklad1', 'metavyklad1', 'vyz_id', $id);
            }

            $this->data->vyznamy[$id]['tvar_char'] = $this->load_tvar_char('tvar_char', 'vyz', $id);
            $this->data->vyznamy[$id]['tvar_char_sl'] = $this->load_tvar_char('tvar_char_sl', 'vyz', $id);

            $this->data->vyznamy[$id]['slov_druhy'] = $this->load_slovni_druh('vyz', $id);

            $this->data->vyznamy[$id]->ext_poznamky = $this->load_externi_poznamka('vyz', $id);

            $this->load_podhesli_main($id);
            foreach ($this->data->vyznamy[$id]['podhesli'] as $phid => $podhesli) {
                $this->load_podhesli($id, $phid);
            }

            // PROPOJENĂŤ VĂťZNAMĂ™
            if ($this->data->vyznamy[$id]['vyklad_vyznamu_link']) {
                $this->data->vyznamy[$id]['rel_to'] = dibi::query('
					SELECT l.id, l.vyz_to, l.heslo_to, l.heslo_id, l.uts
					FROM propojeni_vyznamu l
					WHERE vyz_m_from = %i
				', $this->data->vyznamy[$id]['vyklad_vyznamu_link']);
            } else {
                $this->data->vyznamy[$id]['rel_to'] = dibi::query('
					SELECT l.id, l.vyz_to, l.heslo_to, l.heslo_id, l.uts
					FROM propojeni_vyznamu l
					WHERE vyz_from = %i
				', $id);
            }

            $this->data->vyznamy[$id]['rel_from'] = dibi::query('
				SELECT l.id, l.vyz_from, l.vyz_m_from
				FROM propojeni_vyznamu l
				WHERE vyz_to = %i
			', $id);

            //SYN/ANT
            $this->data->vyznamy[$id]['syn_main'] = $this->load_propojovani('syn', 'main', $id);
            $this->data->vyznamy[$id]['ant_main'] = $this->load_propojovani('ant', 'main', $id);
            $this->data->vyznamy[$id]['syn_slave'] = $this->load_propojovani('syn', 'slave', $id, 'vyznam');
            $this->data->vyznamy[$id]['ant_slave'] = $this->load_propojovani('ant', 'slave', $id, 'vyznam');
            $this->data->vyznamy[$id]['prip_main_v'] = $this->load_propojovani('prip', 'main', $id, 'vyznam');
            foreach ($this->data->vyznamy[$id]['exemplifikace'] as $ex_id => $exemplifikace) {
                $this->data->vyznamy[$id]['exemplifikace'][$ex_id]['prip_main_e'] = $this->load_propojovani('prip', 'main', $ex_id, 'exemplifikace');
            }
        }
        // KONEC VYZNAMU
    }

    private function loadAllRels()
    {
        $prefix = $this->is_archive ? 'archive_' : '';

        $hid = $this->hid;
        $this->data->all_rels_main = dibi::query('
            SELECT hr.id, rel_type, main_hid, main_str, slave_hid, slave_str, hr.sortorder, sortorder2, komentar, poznamka, spojeno,
              hrlt.cis_id, IFNULL(hrlt.cis_id, "jine") AS cis_id, IF(cis_id IS NULL, hrlt.jine, cis.link_type) AS link_type
            FROM %n hr
            LEFT JOIN (%n hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id) ON hr.id = hrlt.hesla_rel_id
            WHERE main_hid = %i
        ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_link_type_vyber', $hid)->fetchAssoc('id');
        $this->data->all_rels_slave = dibi::query('
            SELECT hr.id, rel_type, main_hid, main_str, slave_hid, slave_str, hr.sortorder, sortorder2, komentar, poznamka, spojeno,
              hrlt.cis_id, IFNULL(hrlt.cis_id, "jine") AS cis_id, IF(cis_id IS NULL, hrlt.jine, cis.link_type) AS link_type
            FROM %n hr
            LEFT JOIN (%n hrlt INNER JOIN cis_hesla_rel_link_type cis ON hrlt.cis_id=cis.id) ON hr.id = hrlt.hesla_rel_id
            WHERE slave_hid = %i
        ', $prefix . 'hesla_rel', $prefix . 'hesla_rel_link_type_vyber', $hid)->fetchAssoc('id');

        foreach ($this->data->all_rels_main as $rel_id => $d) {
            $this->data->all_rels_main[$rel_id]->places = new DibiRow(array(
                'main' => dibi::query('SELECT id, place, place_id FROM %n WHERE hesla_rel_id = %i', $prefix . 'hesla_rel_places', $rel_id)->fetchAssoc('id'),
                'slave' => dibi::query('SELECT id, place, place_id FROM %n WHERE hesla_rel_id = %i', $prefix . 'hesla_rel_slave_places', $rel_id)->fetchAssoc('id')
            ));
        }

        foreach ($this->data->all_rels_slave as $rel_id => $d) {
            $this->data->all_rels_slave[$rel_id]->places = new DibiRow(array(
                'main' => dibi::query('SELECT id, place, place_id FROM %n WHERE hesla_rel_id = %i', $prefix . 'hesla_rel_places', $rel_id)->fetchAssoc('id'),
                'slave' => dibi::query('SELECT id, place, place_id FROM %n WHERE hesla_rel_id = %i', $prefix . 'hesla_rel_slave_places', $rel_id)->fetchAssoc('id')
            ));
        }
    }

    /**
     * NaÄŤtenĂ­ verze z archivu do CDO
     * @param int $hid - id hesla
     * @param int $ver_id - id verze
     */
    public function loadFromArchive($hid, $ver_id)
    {
        $this->archive_id = $ver_id;
        $this->load($hid, $ver_id);
    }

    /**
     * Text hlavnĂ­ varianty naÄŤtenĂ©ho hesla
     * @return mixed
     */
    public function get_hlavni_varianta()
    {
        reset($this->data->varianty);
        $vid = key($this->data->varianty);
        return $this->data->varianty[$vid]['varianta'];
    }

    /**
     * Text homonymie naÄŤtenĂ©ho hesla
     * @return string
     */
    public function get_homonymie_str()
    {
        $hom = array();
        foreach ($this->data->varianty as $v) {
            if ($v['homonymie']) {
                array_push($hom, $v['homonymie']);
            }
        }
        if ($hom) {
            return '&nbsp;[' . implode(', ', $hom) . ']';
        } else {
            return '';
        }
    }

    /**
     * Informace do hlaviÄŤky naÄŤtenĂ©ho hesla
     * @return array
     */
    public function get_header_data()
    {
        return array(
            'cts' => $this->data->cts->format('d.m.Y'),
            'uts' => $this->data->uts->format('d.m.Y'),
            'stav' => $this->data->stav,
            'vystup' => $this->data->vystup,
            'lexgraf_pozn' => $this->data->lexgraf_pozn,
            'pozn_odbornik' => $this->data->pozn_odbornik,
            'redukce' => $this->data->redukce,
            'hlavni_varianta' => $this->get_hlavni_varianta(),
            'homonymie_str' => $this->get_homonymie_str(),
            'pid' => $this->data->pid,
            'parent_heslo' => $this->data->parent_heslo,
            'heslar' => $this->data->heslar_id
        );
    }

    /**
     * VybranĂ© hodnoty ÄŤĂ­selnĂ­ku
     * @param int $cis - oznaÄŤenĂ­ ÄŤĂ­selnĂ­ku
     * @param string $parent - varianta (var) / vĂ˝znam (vyz) / podheslĂ­ (podh)
     * @param int $pid - id nadĹ™azenĂ©ho zĂˇznamu (var/vyz/podh)
     * @param bool|False $pid2 - id spojovacĂ­ho zĂˇznamu (napĹ™. pro tvar. char.)
     * @return array|bool
     */
    public function get_vybrane_hodnoty_ciselniku($cis, $parent, $pid, $pid2 = False)
    {
        if (!$this->loaded) return array();
        switch ($cis) {
            case ZPRACOVATELE:
                $return = $this->data->zpracovatel;
                break;
            case TYP_HESLA:
                $return = $this->data->varianty[$pid]->typ_hesla;
                break;
            case SLOV_DRUHY:
                if ($parent == 'vyz') {
                    $return = $this->data->vyznamy[$pid]->slov_druhy[$pid2]['vyber'];
                } elseif ($parent == 'var') {
                    $return = $this->data->varianty[$pid]->slov_druhy[$pid2]['vyber'];
                } elseif ($parent == 'podh') {
                    $vyz = $this->podh_vyz[$pid];
                    $return = $this->data->vyznamy[$vyz]->podhesli[$pid]->slov_druhy[$pid2]['vyber'];
                } else {
                    $return = False;
                }
                break;
            case TVAR_CHAR:
                if ($parent == 'vyz') {
                    $return = $this->data->vyznamy[$pid]->tvar_char[$pid2]['vyber'];
                } elseif ($parent == 'var') {
                    $return = $this->data->varianty[$pid]->tvar_char[$pid2]['vyber'];
                } elseif ($parent == 'podh') {
                    $vyz = $this->podh_vyz[$pid];
                    $return = $this->data->vyznamy[$vyz]->podhesli[$pid]->tvar_char[$pid2]['vyber'];
                } else {
                    $return = False;
                }
                break;
            case TVAR_CHAR_SL:
                if ($parent == 'vyz') {
                    $return = $this->data->vyznamy[$pid]->tvar_char_sl[$pid2]['vyber'];
                } elseif ($parent == 'var') {
                    $return = $this->data->varianty[$pid]->tvar_char_sl[$pid2]['vyber'];
                } elseif ($parent == 'podh') {
                    $vyz = $this->podh_vyz[$pid];
                    $return = $this->data->vyznamy[$vyz]->podhesli[$pid]->tvar_char_sl[$pid2]['vyber'];
                } else {
                    $return = False;
                }
                break;
            case STYL_KVAL:
                if ($parent == 'vyz') {
                    $return = $this->data->vyznamy[$pid]->styl_kval;
                } elseif ($parent == 'var') {
                    $return = $this->data->varianty[$pid]->styl_kval;
                } elseif ($parent == 'podh') {
                    $vyz = $this->podh_vyz[$pid];
                    $return = $this->data->vyznamy[$vyz]->podhesli[$pid]->styl_kval;
                } else {
                    $return = False;
                }
                break;
            case GRAM_INFO:
                if ($parent == 'vyz') {
                    $return = $this->data->vyznamy[$pid]->gram_info;
                } elseif ($parent == 'var') {
                    $return = $this->data->varianty[$pid]->gram_info;
                } elseif ($parent == 'podh') {
                    $vyz = $this->podh_vyz[$pid];
                    $return = $this->data->vyznamy[$vyz]->podhesli[$pid]->gram_info;
                } else {
                    $return = False;
                }
                break;
            case TEMATICKE_OKRUHY:
                if ($parent == 'vyz') {
                    $return = $this->data->vyznamy[$pid]->tematicke_okruhy;
                } elseif ($parent == 'var') {
                    $return = $this->data->varianty[$pid]->tematicke_okruhy;
                } elseif ($parent == 'podh') {
                    $vyz = $this->podh_vyz[$pid];
                    $return = $this->data->vyznamy[$vyz]->podhesli[$pid]->tematicke_okruhy;
                } else {
                    $return = False;
                }
                break;
            case PUVOD:
                $return = $this->data->varianty[$pid]->puvod;
                break;
            case OBOR_UZITI:
                if ($parent == 'vyz') {
                    $return = $this->data->vyznamy[$pid]->obor_uziti;
                } elseif ($parent == 'var') {
                    $return = $this->data->varianty[$pid]->obor_uziti;
                } elseif ($parent == 'podh') {
                    $vyz = $this->podh_vyz[$pid];
                    $return = $this->data->vyznamy[$vyz]->podhesli[$pid]->obor_uziti;
                } else {
                    $return = False;
                }
                break;
            case METAVYKLAD1:
                $return = $this->data->vyznamy[$pid]->metavyklad1;
                break;
            case METAVYKLAD1_M:
                $return = $this->data->vyznamy[$pid]['metavyklad1_m'];
                break;
            case TYP_LINKOVANI:
                $return = $this->data->odk_all_h[$pid];
                break;
            default:
                $return = False;
                break;
        }
        if ($return) {
            return $return;
        }
        return array();
    }

    /**
     * NaÄŤtenĂ­ specifickĂ© ÄŤĂˇsti hesla do CDO
     * @param string $part - nĂˇzev ÄŤĂˇsti
     * @param bool|False $id
     * @param bool|False $id2
     * @param bool|False $id3
     */
    public function load_part($part, $id = False, $id2 = False, $id3 = False)
    {
        $this->loaded = True;
        switch ($part) {
            case 'predvyklad':
                $this->data['vyznamy'] = dibi::query('
					SELECT id, predvyklad, zacatek_predvykladu, konec_predvykladu
					FROM vyznamy WHERE id=%i
				', $id)->fetchAssoc('id');
                $this->data->vyznamy[$id]['vyrazy_predvykladu'] = $this->load_vyraz_predvykladu($id, 0, 0);
                break;
            case 'predvyklad_m':
                $this->data['vyznamy'] = dibi::query('
					SELECT %i AS id, predvyklad, zacatek_predvykladu, konec_predvykladu
					FROM vyznamy_master WHERE Id=%i
				', $id, $id2)->fetchAssoc('id');
                $this->data->vyznamy[$id]['vyrazy_predvykladu'] = $this->load_vyraz_predvykladu($id, $id2, 1);
                break;
            case 'metavyklad1':
                $this->data->vyznamy[$id] = new DibiRow(array(
                    'metavyklad1' => $this->get_cis('metavyklad1_vyz_vyber', 'cis_metavyklad1', 'metavyklad1', 'vyz_id', $id)
                ));
                break;
            case 'metavyklad1_m':
                $this->data->vyznamy[$id] = new DibiRow(array(
                    'metavyklad1' => $this->get_cis('metavyklad1_vyz_vyber_m', 'cis_metavyklad1', 'metavyklad1', 'vyz_id', $id2)
                ));
                break;
            case 'metavyklad2':
                $this->data['vyznamy'] = dibi::query('
					SELECT id, metavyklad2
					FROM vyznamy WHERE id=%i
				', $id)->fetchAssoc('id');
                break;
            case 'metavyklad2_m':
                $this->data['vyznamy'] = dibi::query('
					SELECT %i AS id, metavyklad2
					FROM vyznamy_master WHERE Id=%i
				', $id, $id2)->fetchAssoc('id');
                break;
            case 'vyklad_vyznamu':
                $this->data['vyznamy'] = dibi::query('
					SELECT id, vyklad_vyznamu, vyklad_vyznamu_bz
					FROM vyznamy WHERE Id=%i
				', $id)->fetchAssoc('id');
                break;
            case 'vyklad_vyznamu_m':
                $this->data['vyznamy'] = dibi::query('
					SELECT %i AS id, vyklad_vyznamu, vyklad_vyznamu_bz
					FROM vyznamy_master WHERE Id=%i
				', $id, $id2)->fetchAssoc('id');
                break;
            case 'syn_main':
                $this->data->vyznamy[$id] = new DibiRow(array(
                    'syn_main' => $this->load_propojovani('syn', 'main', $id)
                ));
                break;
            case 'ant_main':
                $this->data->vyznamy[$id] = new DibiRow(array(
                    'ant_main' => $this->load_propojovani('ant', 'main', $id)
                ));
                break;
            case 'synant_slave':
                $this->data->vyznamy[$id] = new DibiRow(array(
                    'syn_slave' => $this->load_propojovani('syn', 'slave', $id, 'vyznam'),
                    'ant_slave' => $this->load_propojovani('ant', 'slave', $id, 'vyznam')
                ));
                break;
                break;
            case 'synant_slave_h':
                $this->data = new DibiRow(array(
                    'syn_slave_h' => $this->load_propojovani('syn', 'slave', 0, 'heslo'),
                    'ant_slave_h' => $this->load_propojovani('ant', 'slave', 0, 'heslo')
                ));
                break;
            case 'propojeni_odkazy_h':
                $this->data = new DibiRow(array(
                    "odk_slave_h" => $this->load_propojovani('odk', 'slave', $id, 'heslo'),
                    "prip_slave_h" => $this->load_propojovani('prip', 'slave', $id, 'heslo'),
                    "odk_main_h" => $this->load_propojovani('odk', 'main', $id, 'heslo'),
                    "prip_main_h" => $this->load_propojovani('prip', 'main', $id, 'heslo')
                ));
                $this->data['odk_all_h'] = $this->data['odk_main_h'] + $this->data['odk_slave_h'];
                break;
            case 'propojeni_main_h':
                $this->data = new DibiRow(array(
                    "prip_main_h" => $this->load_propojovani('prip', 'main', $id, 'heslo')
                ));
                break;
            case 'propojeni_main_v':
                $this->data->vyznamy[$id2] = new DibiRow(array(
                    "prip_main_v" => $this->load_propojovani('prip', 'main', $id2, 'vyznam')
                ));
                break;
            case 'propojeni_main_e':
                $this->data->vyznamy[$id3]->exemplifikace[$id2] = new DibiRow(array(
                    "prip_main_e" => $this->load_propojovani('prip', 'main', $id2, 'exemplifikace')
                ));
                break;
            default:
                # code...
                break;
        }
    }

    /**
     * SpecifickĂˇ ÄŤĂˇst CDO
     * @param string $part - nĂˇzev ÄŤĂˇsti
     * @param bool|False $id
     * @param bool|False $id2
     * @param bool|False $id3
     * @return array
     */
    public function get_part($part, $id = False, $id2 = False, $id3 = False)
    {
        if (!$this->loaded) return array();
        if (
            ($id !== False && !is_numeric($id)) ||
            ($id2 !== False && !is_numeric($id2)) ||
            ($id3 !== False && !is_numeric($id3))
        ) return array();
        switch ($part) {
            case 'vyznamy':
                $return = array();
                foreach ($this->data->vyznamy as $id => $v) {
                    if (!$v['pid']) {
                        array_push($return, $id);
                    }
                }
                //$return = array_keys($this->data->vyznamy);
                break;
            case 'vyznam':
                $return = $this->data->vyznamy[$id];
                break;
            case 'subvyznamy':
                $return = $this->data->vyznamy[$id]->subvyznamy;
                break;
            case 'podhesli_vyznamu':
                $return = array_keys($this->data->vyznamy[$id]->podhesli);
                break;
            case 'jedno_podhesli':
                $id1 = $this->podh_vyz[$id];
                $return = $this->data->vyznamy[$id1]->podhesli[$id];
                break;
            case 'varianty':
                $return = array_keys($this->data->varianty);
                break;
            case 'varianta':
                $return = $this->data->varianty[$id];
                break;
            case 'vyslovnosti_var':
                $return = array_keys($this->data->varianty[$id]->vyslovnost);
                break;
            case 'vyslovnost_var':
                $return = $this->data->varianty[$id]->vyslovnost[$id2];
                break;
            case 'vyslovnosti_podh':
                $id1 = $this->podh_vyz[$id];
                $return = array_keys($this->data->vyznamy[$id1]->podhesli[$id]->vyslovnost);
                break;
            case 'vyslovnost_podh':
                $id1 = $this->podh_vyz[$id];
                $return = $this->data->vyznamy[$id1]->podhesli[$id]->vyslovnost[$id2];
                break;
            case 'slov_druhy_var':
                $return = array_keys($this->data->varianty[$id]->slov_druhy);
                break;
            case 'slov_druhy_vyz':
                $return = array_keys($this->data->vyznamy[$id]->slov_druhy);
                break;
            case 'slov_druhy_podh':
                $id1 = $this->podh_vyz[$id];
                $return = array_keys($this->data->vyznamy[$id1]->podhesli[$id]->slov_druhy);
                break;
            case 'slov_druh_var':
                $return = $this->data->varianty[$id]->slov_druhy[$id2];
                break;
            case 'slov_druh_komentar_var':
                $return = $this->data->varianty[$id]->slov_druhy[$id2]->komentar;
                break;
            case 'slov_druh_komentar_vyz':
                $return = $this->data->vyznamy[$id]->slov_druhy[$id2]->komentar;
                break;
            case 'slov_druh_komentar_podh':
                $id1 = $this->podh_vyz[$id];
                $return = $this->data->vyznamy[$id1]->podhesli[$id]->slov_druhy[$id2]->komentar;
                break;
            case 'tvarchary_var':
                $return = array_keys($this->data->varianty[$id]->tvar_char);
                break;
            case 'tvarchary_vyz':
                $return = array_keys($this->data->vyznamy[$id]->tvar_char);
                break;
            case 'tvarchary_podh':
                $id1 = $this->podh_vyz[$id];
                $return = array_keys($this->data->vyznamy[$id1]->podhesli[$id]->tvar_char);
                break;
            case 'koncovky_tvarchar_podh':
                $id1 = $this->podh_vyz[$id];
                if (!isset($this->data->vyznamy[$id1]->podhesli[$id]->tvar_char[$id2])) return array();
                $return = array_keys($this->data->vyznamy[$id1]->podhesli[$id]->tvar_char[$id2]->koncovky);
                break;
            case 'koncovky_tvarchar_vyz':
                if (!isset($this->data->vyznamy[$id]->tvar_char[$id2])) return array();
                $return = array_keys($this->data->vyznamy[$id]->tvar_char[$id2]->koncovky);
                break;
            case 'koncovky_tvarchar_var':
                if (!isset($this->data->varianty[$id]->tvar_char[$id2])) return array();
                $return = array_keys($this->data->varianty[$id]->tvar_char[$id2]->koncovky);
                break;
            case 'koncovka_tvarchar_podh':
                $id1 = $this->podh_vyz[$id];
                $return = $this->data->vyznamy[$id1]->podhesli[$id]->tvar_char[$id2]->koncovky[$id3];
                break;
            case 'koncovka_tvarchar_vyz':
                $return = $this->data->vyznamy[$id]->tvar_char[$id2]->koncovky[$id3];
                break;
            case 'koncovka_tvarchar_var':
                $return = $this->data->varianty[$id]->tvar_char[$id2]->koncovky[$id3];
                break;
            case 'tvarchary_sl_var':
                $return = array_keys($this->data->varianty[$id]->tvar_char_sl);
                break;
            case 'tvarchary_sl_vyz':
                $return = array_keys($this->data->vyznamy[$id]->tvar_char_sl);
                break;
            case 'tvarchary_sl_podh':
                $id1 = $this->podh_vyz[$id];
                $return = array_keys($this->data->vyznamy[$id1]->podhesli[$id]->tvar_char_sl);
                break;
            case 'koncovky_tvarchar_sl_podh':
                $id1 = $this->podh_vyz[$id];
                if (!isset($this->data->vyznamy[$id1]->podhesli[$id]->tvar_char_sl[$id2])) return array();
                $return = array_keys($this->data->vyznamy[$id1]->podhesli[$id]->tvar_char_sl[$id2]->koncovky);
                break;
            case 'koncovky_tvarchar_sl_vyz':
                if (!isset($this->data->vyznamy[$id]->tvar_char_sl[$id2])) return array();
                $return = array_keys($this->data->vyznamy[$id]->tvar_char_sl[$id2]->koncovky);
                break;
            case 'koncovky_tvarchar_sl_var':
                if (!isset($this->data->varianty[$id]->tvar_char_sl[$id2])) return array();
                $return = array_keys($this->data->varianty[$id]->tvar_char_sl[$id2]->koncovky);
                break;
            case 'koncovka_tvarchar_sl_podh':
                $id1 = $this->podh_vyz[$id];
                $return = $this->data->vyznamy[$id1]->podhesli[$id]->tvar_char_sl[$id2]->koncovky[$id3];
                break;
            case 'koncovka_tvarchar_sl_vyz':
                $return = $this->data->vyznamy[$id]->tvar_char_sl[$id2]->koncovky[$id3];
                break;
            case 'koncovka_tvarchar_sl_var':
                $return = $this->data->varianty[$id]->tvar_char_sl[$id2]->koncovky[$id3];
                break;
            case 'vazebnosti_var':
                $return = array_keys($this->data->varianty[$id]->vazebnost);
                break;
            case 'vazebnosti_vyz':
                $return = array_keys($this->data->vyznamy[$id]->vazebnost);
                break;
            case 'vazebnosti_podh':
                $id1 = $this->podh_vyz[$id];
                $return = array_keys($this->data->vyznamy[$id1]->podhesli[$id]->vazebnost);
                break;
            case 'vazebnost_var':
                $return = $this->data->varianty[$id]->vazebnost[$id2];
                break;
            case 'vazebnost_vyz':
                $return = $this->data->vyznamy[$id]->vazebnost[$id2];
                break;
            case 'vazebnost_podh':
                $id1 = $this->podh_vyz[$id];
                $return = $this->data->vyznamy[$id1]->podhesli[$id]->vazebnost[$id2];
                break;
            case 'ext_poznamky_hes':
                $return = array_keys($this->data->ext_poznamky);
                break;
            case 'ext_poznamka_hes':
                $return = $this->data->ext_poznamky[$id];
                break;
            case 'ext_poznamky_vyz':
                $return = array_keys($this->data->vyznamy[$id]->ext_poznamky);
                break;
            case 'ext_poznamka_vyz':
                $return = $this->data->vyznamy[$id2]->ext_poznamky[$id];
                break;
            case 'vyrazy_predvykladu':
                $return = array_keys($this->data->vyznamy[$id]->vyrazy_predvykladu);
                break;
            case 'vyraz_predvykladu':
                $return = $this->data->vyznamy[$id2]->vyrazy_predvykladu[$id];
                break;
            case 'exemplifikace':
                $return = array_keys($this->data->vyznamy[$id]->exemplifikace);
                break;
            case 'jedna_exemplifikace':
                $return = $this->data->vyznamy[$id2]->exemplifikace[$id];
                break;
            case 'syn_main':
                $return = $this->data->vyznamy[$id]->syn_main;
                break;
            case 'ant_main':
                $return = $this->data->vyznamy[$id]->ant_main;
                break;
            case 'syn_slave':
                $return = $this->data->vyznamy[$id]->syn_slave;
                break;
            case 'ant_slave':
                $return = $this->data->vyznamy[$id]->ant_slave;
                break;
            case 'syn_slave_h':
                $return = $this->data->syn_slave_h;
                break;
            case 'ant_slave_h':
                $return = $this->data->ant_slave_h;
                break;
            case 'propojeni_main_h':
                //var_dump($this->data['prip_main_h']);
                $return = $this->data->prip_main_h;
                break;
            case 'pripojeni_slave':
                //var_dump($this->data['prip_slave_h']);
                $return = $this->data->prip_slave_h;
                break;
            case 'odkazy_main':
                $return = $this->data->odk_main_h;
                break;
            case 'odkazy_slave':
                $return = $this->data->odk_slave_h;
                break;
            case 'propojeni_main_v':
                $return = $this->data->vyznamy[$id2]->prip_main_v;
                break;
            case 'propojeni_main_e':
                $return = $this->data->vyznamy[$id3]->exemplifikace[$id2]->prip_main_e;
                break;
            default:
                # code...
                break;
        }
        if ($return) {
            return $return;
        }
        return array();
    }

    /**
     * VytvoĹ™enĂ­ prĂˇzdnĂ© poloĹľky
     * @param string $part - nĂˇzev poloĹľky
     * @param string $parent - varianta (var) / vĂ˝znam (vyz) / podheslĂ­ (podh)
     * @param int $pid - id nadĹ™azenĂ©ho zĂˇznamu (var/vyz/podh)
     * @param bool|False $pid2 - id spojovacĂ­ho zĂˇznamu (napĹ™. pro tvar. char.)
     * @return int - id novÄ› vytvoĹ™enĂ© poloĹľky
     */
    public function add_empty($part, $parent, $pid, $pid2 = False)
    {
        if ($this->is_archive) {
            return False;
        }
        $id = False;
        switch ($part) {
            case 'varianta':
                $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM varianty WHERE hid=%i', $pid);
                if (!$ord) $ord = 1;
                dibi::query('INSERT INTO varianty', array('hid' => $pid, 'sortorder' => $ord));
                $id = dibi::insertId();
                $this->data->varianty[$id] = new DibiRow(array(
                    'vazebnost' => array(),
                    'vyslovnost' => array(),
                    'tvar_char' => array(),
                    'tvar_char_sl' => array(),
                    'slov_druhy' => array()
                ));
                return $id;
                break;
            case 'vyznam':
                $arr = array('hid' => $pid);
                if ($pid2) {
                    $arr['pid'] = $pid2;
                    $arr['sortorder'] = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM vyznamy WHERE hid=%i AND id=%i', $pid, $pid2);
                    dibi::query('UPDATE vyznamy SET sortorder=sortorder+1 WHERE hid=%i AND sortorder >= %i', $pid, $arr['sortorder']);
                } else {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM vyznamy WHERE hid=%i', $pid);
                    if (!$ord) $ord = 1;
                    $arr['sortorder'] = $ord;
                }
                dibi::query('INSERT INTO vyznamy', $arr);
                $id = dibi::insertId();
                $this->data->vyznamy[$id] = new DibiRow(array());
                if ($pid2) {
                    $this->data->vyznamy[$pid2]['subvyznamy'][] = $id;
                } else {
                    $pid2 = null;
                }
                $this->data->vyznamy[$id] = new DibiRow(array(
                    'id' => $id,
                    'sortorder' => $ord,
                    'pid' => $pid2,
                    'vystup' => 0,
                    'typ_vystupu' => 2,
                    'vyklad_vyznamu_link' => 0,
                    'predvyklad_link' => 1,
                    'metavyklad1_link' => 1,
                    'metavyklad2_link' => 1,
                    'podhesli' => array(),
                    'zacatek_predvykladu' => '',
                    'konec_predvykladu' => '',
                    'vyrazy_predvykladu' => array(),
                    'komentar_tvarchar' => '',
                    'komentar_tvarchar_slovesa' => '',
                    'vazebnost' => array(),
                    'styl_kval' => array(),
                    'tematicke_okruhy' => array(),
                    'obor_uziti' => array(),
                    'gram_info' => array(),
                    'tvar_char' => array(),
                    'tvar_char_sl' => array(),
                    'slov_druhy' => array(),
                    'vyklad_vyznamu' => '',
                    'metavyklad1' => array(),
                    'metavyklad2' => '',
                    'exemplifikace' => array(),
                    'ext_poznamky' => array(),
                    'subvyznamy' => array()
                ));
                return $id;
                break;
            case 'podhesli':
                dibi::query('INSERT INTO podhesli_vyznamu', array('vyz_id' => $pid));
                $id = dibi::insertId();
                $this->data->vyznamy[$pid]->podhesli[$id] = new DibiRow(array(
                    'id' => $id,
                    'komentar_pred' => '',
                    'podhesli' => '',
                    'komentar_za' => '',
                    'komentar_tvarchar' => '',
                    'komentar_tvarchar_slovesa' => '',
                    'vyslovnost' => array(),
                    'vazebnost' => array(),
                    'styl_kval' => array(),
                    'tematicke_okruhy' => array(),
                    'obor_uziti' => array(),
                    'gram_info' => array(),
                    'tvar_char' => array(),
                    'tvar_char_sl' => array(),
                    'slov_druhy' => array()
                ));
                $this->podh_vyz[$id] = $pid;
                return $id;
                break;
            case 'vyslovnost':
                if ($parent == 'var') {
                    dibi::query('INSERT INTO vyslovnost_var', array('vid' => $pid));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid]->vyslovnost[$id] = new DibiRow(array(
                        'id' => $id,
                        'vyslovnost' => '',
                        'komentar' => ''
                    ));
                } elseif ($parent == 'podh') {
                    dibi::query('INSERT INTO podh_vyz_vyslovnost', array('podh_id' => $pid));
                    $id = dibi::insertId();
                    $vyz = $this->podh_vyz[$pid];
                    $this->data->vyznamy[$vyz]->podhesli[$pid]->vyslovnost[$id] = new DibiRow(array(
                        'id' => $id,
                        'vyslovnost' => '',
                        'komentar' => ''
                    ));
                }
                return $id;
                break;
            case 'slovni_druh':
                if ($parent == 'var') {
                    dibi::query('INSERT INTO var_slov_druhy_vyber_rel', array('vid' => $pid));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid]->slov_druhy[$id] = new DibiRow(array(
                        'id' => $id,
                        'komentar' => '',
                        'vyber' => array()
                    ));
                } elseif ($parent == 'vyz') {
                    dibi::query('INSERT INTO vyz_slov_druhy_vyber_rel', array('vyz_id' => $pid));
                    $id = dibi::insertId();
                    $this->data->vyznamy[$pid]->slov_druhy[$id] = new DibiRow(array(
                        'id' => $id,
                        'komentar' => '',
                        'vyber' => array()
                    ));
                } elseif ($parent == 'podh') {
                    dibi::query('INSERT INTO podh_vyz_slov_druhy_vyber_rel', array('podh_id' => $pid));
                    $id = dibi::insertId();
                    $vyz = $this->podh_vyz[$pid];
                    $this->data->vyznamy[$vyz]->podhesli[$pid]->slov_druhy[$id] = new DibiRow(array(
                        'id' => $id,
                        'komentar' => '',
                        'vyber' => array()
                    ));
                }
                return $id;
                break;
            case 'vazebnost':
                if ($parent == 'var') {
                    dibi::query('INSERT INTO vazebnost_var', array('vid' => $pid, 'vazebnost' => ''));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid]->vazebnost[$id] = new DibiRow(array(
                        'id' => $id,
                        'vazebnost' => '',
                        'komentar'
                    ));
                } elseif ($parent == 'vyz') {
                    dibi::query('INSERT INTO vazebnost_vyz', array('vyz_id' => $pid, 'vazebnost' => ''));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid]->vazebnost[$id] = new DibiRow(array(
                        'id' => $id,
                        'vazebnost' => '',
                        'komentar'
                    ));
                } elseif ($parent == 'podh') {
                    dibi::query('INSERT INTO podhesli_vazebnost', array('podh_id' => $pid, 'vazebnost' => ''));
                    $id = dibi::insertId();
                    $vyz = $this->podh_vyz[$pid];
                    $this->data->vyznamy[$vyz]->podhesli[$pid]->vazebnost[$id] = new DibiRow(array(
                        'id' => $id,
                        'vazebnost' => '',
                        'komentar'
                    ));
                }
                return $id;
                break;
            case 'ext_poznamka':
                if ($parent == 'hes') {
                    dibi::query('INSERT INTO ext_poznamky_hes', array('hid' => $pid, 'cis_id' => 7, 'poznamka' => ''));
                    $id = dibi::insertId();
                    $this->data->ext_poznamky[$id] = array('cis_id' => 7, 'poznamka' => '');
                } elseif ($parent == 'vyz') {
                    dibi::query('INSERT INTO ext_poznamky_vyz', array('vyz_id' => $pid, 'cis_id' => 7, 'poznamka' => ''));
                    $id = dibi::insertId();
                    $this->data->vyznamy[$pid]->ext_poznamky[$id] = array('cis_id' => 7, 'poznamka' => '');
                }
                return $id;
                break;
            case 'vyraz_predvykladu':
                $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM vyrazy_predvykladu WHERE vyz_id=%i', $pid);
                if (!$ord) $ord = 0;
                dibi::query('INSERT INTO vyrazy_predvykladu', array('vyz_id' => $pid, 'sortorder' => $ord));
                $id = dibi::insertId();
                $this->data->vyznamy[$pid]->vyrazy_predvykladu[$id] = array();
                return $id;
                break;
            case 'vyraz_predvykladu_master':
                $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM vyrazy_predvykladu_m WHERE vyz_id=%i', $pid);
                if (!$ord) $ord = 0;
                dibi::query('INSERT INTO vyrazy_predvykladu_m', array('vyz_id' => $pid, 'sortorder' => $ord));
                $id = dibi::insertId();
                $this->data->vyznamy[$pid]->vyrazy_predvykladu[$id] = array();
                return $id;
                break;
            case 'tvarchar':
                if ($parent == 'var') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM var_tvar_char_vyber_rel WHERE vid=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO var_tvar_char_vyber_rel', array('vid' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid]->tvar_char[$id] = new DibiRow(array(
                        'id' => $id,
                        'sortorder' => $ord,
                        'vyber' => array(),
                        'koncovky' => array()
                    ));
                } elseif ($parent == 'vyz') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM vyz_tvar_char_vyber_rel WHERE vyz_id=%i', $pid);
                    dibi::query('INSERT INTO vyz_tvar_char_vyber_rel', array('vyz_id' => $pid, 'sortorder' => $ord));
                    if (!$ord) $ord = 1;
                    $id = dibi::insertId();
                    $this->data->vyznamy[$pid]->tvar_char[$id] = new DibiRow(array(
                        'id' => $id,
                        'sortorder' => $ord,
                        'vyber' => array(),
                        'koncovky' => array()
                    ));
                } elseif ($parent == 'podh') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM podh_tvar_char_vyber_rel WHERE podh_id=%i', $pid);
                    dibi::query('INSERT INTO podh_tvar_char_vyber_rel', array('podh_id' => $pid, 'sortorder' => $ord));
                    if (!$ord) $ord = 1;
                    $id = dibi::insertId();
                    $vyz = $this->podh_vyz[$pid];
                    $this->data->vyznamy[$vyz]->podhesli[$pid]->tvar_char[$id] = new DibiRow(array(
                        'id' => $id,
                        'sortorder' => $ord,
                        'vyber' => array(),
                        'koncovky' => array()
                    ));
                }
                return $id;
                break;
            case 'koncovka_tvarchar':
                if ($parent == 'var') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM koncovky_tvar_char WHERE var_tvar_char_rel_id=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO koncovky_tvar_char', array('var_tvar_char_rel_id' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid2]->tvar_char[$pid]->koncovky[$id] = new DibiRow(array(
                        'id' => $id,
                        'koncovka' => '',
                        'komentar' => ''
                    ));
                } elseif ($parent == 'vyz') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM koncovky_tvar_char_vyz WHERE vyz_tvar_char_rel_id=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO koncovky_tvar_char_vyz', array('vyz_tvar_char_rel_id' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $this->data->vyznamy[$pid2]->tvar_char[$pid]->koncovky[$id] = new DibiRow(array(
                        'id' => $id,
                        'koncovka' => '',
                        'komentar' => ''
                    ));
                } elseif ($parent == 'podh') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM podh_vyz_koncovky_tvar_char WHERE podh_tvch=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO podh_vyz_koncovky_tvar_char', array('podh_tvch' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $vyz = $this->podh_vyz[$pid2];
                    $this->data->vyznamy[$vyz]->podhesli[$pid2]->tvar_char[$pid]->koncovky[$id] = new DibiRow(array(
                        'id' => $id,
                        'koncovka' => '',
                        'komentar' => ''
                    ));
                }
                return $id;
                break;
            case 'tvarchar_sl':
                if ($parent == 'var') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM var_tvar_char_sl_vyber_rel WHERE vid=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO var_tvar_char_sl_vyber_rel', array('vid' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid]->tvar_char_sl[$id] = new DibiRow(array(
                        'id' => $id,
                        'sortorder' => $ord,
                        'vyber' => array(),
                        'koncovky' => array()
                    ));
                } elseif ($parent == 'vyz') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM vyz_tvar_char_sl_vyber_rel WHERE vyz_id=%i', $pid);
                    dibi::query('INSERT INTO vyz_tvar_char_sl_vyber_rel', array('vyz_id' => $pid, 'sortorder' => $ord));
                    if (!$ord) $ord = 1;
                    $id = dibi::insertId();
                    $this->data->vyznamy[$pid]->tvar_char_sl[$id] = new DibiRow(array(
                        'id' => $id,
                        'sortorder' => $ord,
                        'vyber' => array(),
                        'koncovky' => array()
                    ));
                } elseif ($parent == 'podh') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM podh_tvar_char_sl_vyber_rel WHERE podh_id=%i', $pid);
                    dibi::query('INSERT INTO podh_tvar_char_sl_vyber_rel', array('podh_id' => $pid, 'sortorder' => $ord));
                    if (!$ord) $ord = 1;
                    $id = dibi::insertId();
                    $vyz = $this->podh_vyz[$pid];
                    $this->data->vyznamy[$vyz]->podhesli[$pid]->tvar_char_sl[$id] = new DibiRow(array(
                        'id' => $id,
                        'sortorder' => $ord,
                        'vyber' => array(),
                        'koncovky' => array()
                    ));
                }
                return $id;
                break;
            case 'koncovka_tvarchar_sl':
                if ($parent == 'var') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM koncovky_tvar_char_sl WHERE var_tvar_char_sl_rel_id=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO koncovky_tvar_char_sl', array('var_tvar_char_sl_rel_id' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $this->data->varianty[$pid2]->tvar_char_sl[$pid]->koncovky[$id] = new DibiRow(array(
                        'id' => $id,
                        'koncovka' => '',
                        'komentar' => ''
                    ));
                } elseif ($parent == 'vyz') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM koncovky_tvar_char_sl_vyz WHERE vyz_tvar_char_sl_rel_id=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO koncovky_tvar_char_sl_vyz', array('vyz_tvar_char_sl_rel_id' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $this->data->vyznamy[$pid2]->tvar_char_sl[$pid]->koncovky[$id] = new DibiRow(array(
                        'id' => $id,
                        'koncovka' => '',
                        'komentar' => ''
                    ));
                } elseif ($parent == 'podh') {
                    $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM podh_vyz_koncovky_tvar_char_sl WHERE podh_tvch_sl=%i', $pid);
                    if (!$ord) $ord = 1;
                    dibi::query('INSERT INTO podh_vyz_koncovky_tvar_char_sl', array('podh_tvch_sl' => $pid, 'sortorder' => $ord));
                    $id = dibi::insertId();
                    $vyz = $this->podh_vyz[$pid2];
                    $this->data->vyznamy[$vyz]->podhesli[$pid2]->tvar_char_sl[$pid]->koncovky[$id] = new DibiRow(array(
                        'id' => $id,
                        'koncovka' => '',
                        'komentar' => ''
                    ));
                }
                return $id;
                break;
            case 'exemplifikace':
                $ord = dibi::fetchSingle('SELECT MAX(sortorder)+1 FROM exemplifikace WHERE vyz_id=%i', $pid);
                if (!$ord) $ord = 1;
                dibi::query('INSERT INTO exemplifikace', array('vyz_id' => $pid, 'sortorder' => $ord, 'vystup' => 2, 'exemplifikace' => ''));
                $id = dibi::insertId();
                $this->data->vyznamy[$pid]->exemplifikace[$id] = array('sortorder' => $ord, 'vystup' => 2, 'exemplifikace' => '');
                return $id;
                break;
            default:
                # code...
                break;
        }
    }

    public static function save_part($part, $data)
    {
        switch ($part) {
            case 'value':
                # code...
                break;

            default:
                # code...
                break;
        }
    }

    private function getVarFromVyz($id)
    {
        return dibi::fetchSingle('
            SELECT v.varianta
            FROM varianty v INNER JOIN vyznamy vy ON v.hid=vy.hid
            WHERE vy.id = %i AND v.sortorder = 1
        ', $id);
    }

    public function ciselnik_xmlExport($part, $name = 'choices')
    {
        $xml = '<' . $name . '>';
        foreach ($part as $id => $str) {
            $xml .= '<choice id="' . $id . '">' . $str . '</choice>';
        }

        $xml .= '</' . $name . '>';

        return $xml;
    }

    public function vyslovnost_xmlExport($data)
    {
        $xml = '<pronunciations>';
        foreach ($data as $id => $d) {
            $xml .= '
            <pronunciation id="' . $id . '">
                <pron>' . $d['vyslovnost'] . '</pron>
                <comment>' . $d['komentar'] . '</comment>
            </pronunciation>
            ';
        }
        $xml .= '</pronunciations>';
        return $xml;
    }

    public function slovni_druh_xmlExport($id, $data)
    {
        $xml = '<part-of-speech id="' . $id . '">';
        $xml .= $this->ciselnik_xmlExport($data->vyber);
        $xml .= '<comment>' . $data->komentar . '</comment>';
        $xml .= '</part-of-speech>';
        return $xml;
    }

    public function slovni_druhy_xmlExport($data)
    {
        $xml = '<parts-of-speech>';
        foreach ($data as $id => $d) {
            $xml .= $this->slovni_druh_xmlExport($id, $d);
        }
        $xml .= '</parts-of-speech>';
        return $xml;
    }

    public function konc_tvarchar_xmlExport($id, $data)
    {
        $xml = '
            <suffixes id="' . $id . '">
            <suffix>' . $data['koncovka'] . '</suffix>
            <comment>' . $data['komentar'] . '</comment>
            </suffixes>
        ';
        return $xml;
    }

    public function tvarchar_xmlExport($id, $data, $suff)
    {
        $xml = '<morph-char' . $suff . ' id="' . $id . '">';
        $xml .= $this->ciselnik_xmlExport($data->vyber);
        foreach ($data->koncovky as $id => $d) {
            $xml .= $this->konc_tvarchar_xmlExport($id, $d);
        }
        $xml .= '</morph-char' . $suff . '>';
        return $xml;
    }

    public function tvarchary_xmlExport($data, $sl = False)
    {
        $suff = $sl ? '-verb' : '';
        $xml = '<morph-chars' . $suff . '>';
        foreach ($data as $id => $d) {
            $xml .= $this->tvarchar_xmlExport($id, $d, $suff);
        }
        $xml .= '</morph-chars' . $suff . '>';

        return $xml;
    }

    public function vazebnosti_xmlExport($data)
    {
        $xml = '<valencies>';
        foreach ($data as $id => $d) {
            $xml .= '<valency id="' . $id . '"><word>' . $d['vazebnost'] . '</word><comment>' . $d['komentar'] . '</comment></valency>';
        }
        $xml .= '</valencies>';
        return $xml;
    }

    public function varianta_xmlExport($id, $data)
    {
        $xml = '
            <variant id="' . $id . '" order="' . $data->sortorder . '">
            <type>' . $data->f_varianta . '</type>
            <lemma-variant>' . $data->varianta . '</lemma-variant>
            <neosemantism>' . $data->neosemantismus . '</neosemantism>
            <homonymie>' . $data->homonymie . '</homonymie>
            <comment-before>' . $data->komentar . '</comment-before>
            <comment-after>' . $data->komentar_za . '</comment-after>
            <first-occurrence>' . $data->prvni_reg_vyskyt . '</first-occurrence>
            <morph-char-comment>' . $data->komentar_tvarchar . '</morph-char-comment>
            <morph-char-verb-comment>' . $data->komentar_tvarchar_slovesa . '</morph-char-verb-comment>
        ';
        $xml .= $this->ciselnik_xmlExport($data->typ_hesla, 'typy-hesla');
        $xml .= $this->vyslovnost_xmlExport($data->vyslovnost);
        $xml .= $this->slovni_druhy_xmlExport($data->slov_druhy);
        $xml .= $this->tvarchary_xmlExport($data->tvar_char);
        $xml .= $this->tvarchary_xmlExport($data->tvar_char_sl, True);
        $xml .= $this->vazebnosti_xmlExport($data->vazebnost);
        $xml .= $this->ciselnik_xmlExport($data->styl_kval, 'styl-qualifier');
        $xml .= $this->ciselnik_xmlExport($data->gram_info, 'gram-info');
        $xml .= $this->ciselnik_xmlExport($data->tematicke_okruhy, 'them-area');
        $xml .= $this->ciselnik_xmlExport($data->puvod, 'origin');
        $xml .= $this->ciselnik_xmlExport($data->obor_uziti, 'field-of-use');

        $xml .= '</variant>';
        return $xml;
    }

    public function podhesli_xmlExport($id, $data)
    {
        $xml = '
            <sublemma id="' . $id . '">
                <comment-before>' . $data->komentar_pred . '</comment-before>
                <word>' . $data->podhesli . '</word>
                <comment-after>' . $data->komentar_za . '</comment-after>
        ';
        $xml .= $this->vyslovnost_xmlExport($data->vyslovnost);
        $xml .= $this->vazebnosti_xmlExport($data->vazebnost);
        $xml .= $this->ciselnik_xmlExport($data->styl_kval, 'styl-qualifier');
        $xml .= $this->ciselnik_xmlExport($data->tematicke_okruhy, 'them-area');
        $xml .= $this->ciselnik_xmlExport($data->obor_uziti, 'field-of-use');
        $xml .= $this->ciselnik_xmlExport($data->gram_info, 'gram-info');
        $xml .= $this->slovni_druhy_xmlExport($data->slov_druhy);
        $xml .= $this->tvarchary_xmlExport($data->tvar_char);
        $xml .= $this->tvarchary_xmlExport($data->tvar_char_sl, True);
        $xml .= '
                <morph-char-comment>' . $data->komentar_tvarchar . '</morph-char-comment>
                <morph-char-verb-comment>' . $data->komentar_tvarchar_slovesa . '</morph-char-verb-comment>
            ';
        $xml .= '</sublemma>';
        return $xml;
    }

    public function predvyklad_xmlExport($b, $e, $data)
    {
        $xml = '<prel-meaning>';
        $xml .= '<beginning>' . $b . '</beginning>';
        $xml .= '<words>';
        foreach ($data as $id => $d) {
            $xml .= '
                <expression id="' . $id . '">
                    <word>' . $d->vyraz . '</word>
                    <comment>' . $d->komentar . '</comment>
                </expression>
            ';
        }
        $xml .= '</words>';
        $xml .= '<end>' . $e . '</end>';
        $xml .= '</prel-meaning>';
        return $xml;
    }

    public function exemplifikace_xmlExport($data)
    {
        //exemplifikace, exemplifikace_bz, vystup, sortorder
        $xml = '<exemplifications>';
        foreach ($data as $id => $d) {
            $xml .= '
                <exemplification id="' . $id . '">
                    <exemplification-text-html><![CDATA[' . $d->exemplifikace . ']]></exemplification-text-html>
                    <exemplification-text>' . $d->exemplifikace_bz . '</exemplification-text>
                </exemplification>
            ';
        }

        $xml .= '</exemplifications>';
        return $xml;
    }

    public function vyznam_xmlExport($id, $data, $prefix = '')
    {
        $xml = '<' . $prefix . 'meaning id="' . $id . '">';
        $xml .= '<output>' . $data->vystup . '</output>';
        $xml .= '<sublemmas>';
        foreach ($data->podhesli as $pid => $d) {
            $xml .= $this->podhesli_xmlExport($pid, $d);
        }
        $xml .= '</sublemmas>';
        $xml .= $this->vazebnosti_xmlExport($data->vazebnost);
        $xml .= $this->ciselnik_xmlExport($data->styl_kval, 'styl-qualifier');
        $xml .= $this->ciselnik_xmlExport($data->tematicke_okruhy, 'them-area');
        $xml .= $this->ciselnik_xmlExport($data->obor_uziti, 'field-of-use');
        $xml .= $this->ciselnik_xmlExport($data->gram_info, 'gram-info');
        $xml .= $this->slovni_druhy_xmlExport($data->slov_druhy);
        $xml .= $this->tvarchary_xmlExport($data->tvar_char);
        $xml .= $this->tvarchary_xmlExport($data->tvar_char_sl, True);
        $xml .= '
                <morph-char-comment>' . $data->komentar_tvarchar . '</morph-char-comment>
                <morph-char-verb-comment>' . $data->komentar_tvarchar_slovesa . '</morph-char-verb-comment>
            ';
        $xml .= $this->predvyklad_xmlExport($data->zacatek_predvykladu, $data->konec_predvykladu, $data->vyrazy_predvykladu);
        $xml .= $this->ciselnik_xmlExport($data->metavyklad1, 'meta-meaning-1');
        $xml .= '
            <meaning-text>' . $data->vyklad_vyznamu_bz . '</meaning-text>
            <meaning-text-html><![CDATA[' . $data->vyklad_vyznamu . ']]></meaning-text-html>
            <meta-meaning-2>' . $data->metavyklad2 . '</meta-meaning-2>
        ';
        $xml .= '<links-to-this>';
        foreach ($data->rel_from as $d) {
            $xml .= '
                <link-to-this id="' . $d->id . '" meaning-id="' . $d->vyz_from . '">' . $this->getVarFromVyz($d->vyz_from) . '</link-to-this>
            ';
        }
        $xml .= '</links-to-this>';

        $xml .= '<links-from-this>';
        foreach ($data->rel_to as $d) {
            $xml .= '
                <link-from-this id="' . $d->id . '" meaning-id="' . $d->vyz_to . '">' . $this->getVarFromVyz($d->vyz_to) . '</link-from-this>
            ';
        }
        $xml .= '</links-from-this>';
        $xml .= $this->exemplifikace_xmlExport($data->exemplifikace);
        $xml .= $this->ext_pozn_xmlExport($data->ext_poznamky);
        if ($this->data->vyznamy[$id]->subvyznamy) {
            $xml .= '<sub-meanings>';
            foreach ($this->data->vyznamy[$id]->subvyznamy as $sid) {
                $xml .= $this->vyznam_xmlExport($sid, $this->data->vyznamy[$sid], 'sub-');
            }
            $xml .= '</sub-meanings>';
        }
        $xml .= '</' . $prefix . 'meaning>';
        return $xml;
    }

    public function ext_pozn_xmlExport($data)
    {
        //v.id, v.cis_id, v.poznamka, v.uts, c.typ
        $xml = '<ext-remarks>';
        foreach ($data as $id => $d) {
            $xml .= '
                <ext-remark id="' . $id . '">
                    <remark>' . strip_tags($d->poznamka) . '</remark>
                    <remark-html><![CDATA[' . $d->poznamka . ']]></remark-html>
                    <type id="' . $d->cis_id . '">' . $d->typ . '</type>
                </ext-remark>
            ';
        }
        $xml .= '</ext-remarks>';
        return $xml;
    }

    public function links_xmlExport($data, $main = True, $singular = 'link', $plural = 'links')
    {
        $place = $main ? 'slave' : 'main';
        $xml = '<' . $plural . '>';
        foreach ($data as $id => $d) {
            $xml .= '
                <' . $singular . ' id="' . $id . '" sortorder1="' . $d->sortorder . '" sortorder2="' . $d->sortorder2 . '">
                    <rel-type>' . $d->rel_type . '</rel-type>
                    <' . $place . '-lemma id="' . $d[$place . '_hid'] . '">' . $d[$place . '_str'] . '</' . $place . '-lemma>
                    <comment>' . $d->komentar . '</comment>
                    <remark>' . $d->poznamka . '</remark>
                    <connected>' . $d->spojeno . '</connected>
                    <main-places>
            ';
            foreach ($d->places->main as $id => $pl) {
                $xml .= '<place id="' . $pl->id . '" place-id="' . $pl->place_id . '">' . $pl->place . '</place>';
            }
            $xml .= '
                    </main-places>
                    <slave-places>
            ';
            foreach ($d->places->slave as $id => $pl) {
                $xml .= '<place id="' . $pl->id . '" place-id="' . $pl->place_id . '">' . $pl->place . '</place>';
            }
            $xml .= '
                </slave-places>
                </' . $singular . '>
            ';
        }
        $xml .= '</' . $plural . '>';
        return $xml;
    }

    public function xmlExport()
    {
        $this->loadAllRels();
        $xml = '<lemma id="' . $this->hid . '">';
        $xml .= '
            <created>' . $this->data->cts . '</created>
            <updated>' . $this->data->uts . '</updated>
            <processor>' . $this->ciselnik_xmlExport($this->data->zpracovatel) . '</processor>
            <lexgraf-note>' . $this->data->lexgraf_pozn . '</lexgraf-note>
            <status>' . $this->data->stav . '</status>
            <output>' . $this->data->vystup . '</output>
            <expert-note>' . $this->data->pozn_odbornik . '</expert-note>
            <dupl-reduction>' . $this->data->redukce . '</dupl-reduction>
            <parent-lemma id="' . $this->data->pid . '">' . $this->data->parent_heslo . '</parent-lemma>
        ';
        $xml .= $this->ext_pozn_xmlExport($this->data->ext_poznamky);
        $xml .= '<variants>';
        foreach ($this->data->varianty as $id => $v) {
            $xml .= $this->varianta_xmlExport($id, $v);
        }
        $xml .= '</variants>';
        $xml .= '<meanings>';
        foreach ($this->data->vyznamy as $id => $v) {
            if ($this->data->vyznamy[$id]->pid) {
                continue;
            }
            $xml .= $this->vyznam_xmlExport($id, $v);
        }
        $xml .= '</meanings>';
        $xml .= $this->links_xmlExport($this->data->all_rels_main, True, 'link-main', 'links-main');
        $xml .= $this->links_xmlExport($this->data->all_rels_slave, False, 'link-slave', 'links-slave');
        $xml .= '</lemma>';

        return $xml;
    }
}